using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Token;
using LotusCraft.Spell;

namespace LotusCraft.Ability
{
	class SpellContinuousAbility : thrts.AbilityTemplate
	{
		public IEffect Effect;
		Vector3D offset;
		bool stopEffectOnVanish;
		
		public bool StopEffectRootOnVanish;

		int counter = 0;
		int interval = 0;
		public Func<IUnit,bool> CastUnitAction;
		public Func<IBuilding,bool> CastBuildingAction;
		public Func<IBullet,bool> CastBulletAction;
		public bool FollowEffect;
		public SpellContinuousAbility(Func<IUnit,bool>castAction, IEffect effect, Vector3D offset, int duration, string name, int interval = 1, bool stopEffect = true)
		{
			this.Name = name;
			this.DurationTime = duration;
			this.interval = interval;
			this.Effect = effect;
			this.offset = offset;
			this.stopEffectOnVanish = stopEffect;
			this.CastUnitAction = castAction;
			this.CastBuildingAction = null;
			this.CastBulletAction = null;
			this.FollowEffect = true;
		}
		public SpellContinuousAbility(Func<IBuilding, bool> castAction, IEffect effect, Vector3D offset, int duration, string name, int interval = 1, bool stopEffect = true)
		{
			this.Name = name;
			this.DurationTime = duration;
			this.interval = interval;
			this.Effect = effect;
			this.offset = offset;
			this.stopEffectOnVanish = stopEffect;
			this.CastUnitAction = null;
			this.CastBuildingAction = castAction;
			this.CastBulletAction = null;
			this.FollowEffect = true;
		}
		public SpellContinuousAbility(Func<IBullet, bool> castAction, IEffect effect, Vector3D offset, int duration, string name, int interval = 1, bool stopEffect = true)
		{
			this.Name = name;
			this.DurationTime = duration;
			this.interval = interval;
			this.Effect = effect;
			this.offset = offset;
			this.stopEffectOnVanish = stopEffect;
			this.CastUnitAction = null;
			this.CastBuildingAction = null;
			this.CastBulletAction = castAction;
			this.FollowEffect = true;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.OtherExecs:
					if (Effect != null && FollowEffect)
						Effect.SetPosition(unit.Pos + offset);

					if (++counter >= interval)
					{
						if (CastUnitAction != null) CastUnitAction(unit);
						counter = 0;
					}
					break;
			}
			return true;
		}

		public override bool ExecBuildingAbility (thrts.IBuilding building, thrts.AbilityOrder order)
		{
			switch (order) {
			case thrts.AbilityOrder.OtherExecs:
				if(Effect != null && FollowEffect)
					Effect.SetPosition(building.Pos+offset);

				if (++counter >= interval) {
					if(CastBuildingAction!=null) CastBuildingAction(building);
					counter=0;
				}
				break;
			}
			return true;
		}
		public override bool ExecBulletAbility (thrts.IBullet bullet, thrts.AbilityOrder order)
		{
			switch (order) {
			case thrts.AbilityOrder.OtherExecs:
				if(Effect != null && FollowEffect)
					Effect.SetPosition(bullet.Position+offset);

				if (++counter >= interval) {
					if(CastBulletAction!=null) CastBulletAction(bullet);
					counter = 0;
				}
				break;
			}
			return true;
		}
		public override bool OnVanish(IBullet bullet)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else if(stopEffectOnVanish)
				{
					Effect.Stop();
				}
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else if (stopEffectOnVanish)
				{
					Effect.Stop();
				}
			}

			return true;
		}
		public override bool OnVanish(IBuilding building)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else if (stopEffectOnVanish)
				{
					Effect.Stop();
				}
			}

			return true;
		}
	}
	class EffectContinuousAbility : thrts.AbilityTemplate
	{
		public bool StopEffectRootOnVanish = false;
		public IEffect Effect;
		public EffectContinuousAbility()
		{
			this.Name = string.Empty;
			this.DurationTime = 0;
			this.Effect = null;
		}
		public EffectContinuousAbility(IEffect effect, int duration, string name)
		{
			this.Name = name;
			this.DurationTime = duration;
			this.Effect = effect;
		}
		public override bool OnVanish(IBullet bullet)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else
				{
					Effect.Stop();
				}
				Effect = null;
			}

			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else
				{
					Effect.Stop();
				}
				Effect = null;
			}

			return true;
		}
		public override bool OnVanish(IBuilding building)
		{
			if (Effect != null)
			{
				if (StopEffectRootOnVanish)
				{
					Effect.StopRoot();
				}
				else
				{
					Effect.Stop();
				}
				Effect = null;
			}

			return true;
		}
	}
	class EffectFollowContinuousAbility : thrts.AbilityTemplate
	{
		public IEffect Effect;
		Vector3D offset;
		public EffectFollowContinuousAbility()
		{
			this.Name = string.Empty;
			this.DurationTime = 0;
			this.Effect = null;
		}
		public EffectFollowContinuousAbility(IEffect effect, int duration, string name, Vector3D offset)
		{
			this.Name = name;
			this.DurationTime = duration;
			this.Effect = effect;
			this.offset = offset;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			if (Effect != null)
				Effect.SetPosition(unit.Pos + offset);
			return true;
		}
		public override bool ExecBuildingAbility(thrts.IBuilding building, thrts.AbilityOrder order)
		{
			if (Effect != null)
				Effect.SetPosition(building.Pos + offset);
			return true;
		}
		public override bool ExecBulletAbility(thrts.IBullet bullet, thrts.AbilityOrder order)
		{
			if (Effect != null)
				Effect.SetPosition(bullet.Position + offset);
			return true;
		}
		public override bool OnVanish(IBullet bullet)
		{
			if (Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}
			
			return true;
		}
		public override bool OnVanish(IUnit unit)
		{
			if (Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}
			
			return true;
		}
		public override bool OnVanish(IBuilding building)
		{
			if (Effect != null)
			{
				Effect.Stop();
				Effect = null;
			}
			
			return true;
		}
	}

	/// <summary>
	/// Stun ability.
	/// </summary>
	class StunAbility : LotusCraft.Ability.FollowEffectAbility
	{
		public StunAbility (IEffect effect, Vector3D offset, int duration, string name)
			:base(effect,offset,duration,name)
		{
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order) {
			case AbilityOrder.ConstDecParam:
				unit.IsForbiddenAttacking = true;
				unit.IsForbiddenSpelling = true;
				break;
			case AbilityOrder.RateDecParam:
				unit.Speed = 0;
				break;
			default:
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool OnVanish (IBuilding building)
		{
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (building);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	class SuicideAbility : LotusCraft.Ability.EffectContinuousAbility
	{
		public SuicideAbility (IEffect effect, int duration, string name)
			:base(effect,duration,name)
		{
		}
		public override bool OnVanish (IBuilding building)
		{
			if( 0 != building.HP )
				building.HP = 0;
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (building);
		}
		public override bool OnVanish (IUnit unit)
		{
			if( 0 != unit.HP )
				unit.HP = 0;
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	class SummonUnitAbility : LotusCraft.Ability.EffectContinuousAbility
	{
		bool isSuicideTimeUp = true;
		public const string AbilityName = "SummonedUnit";
		public IEffect effect = null;
		public SummonUnitAbility (IEffect effect, int duration, string name,bool isSuicideTimeUp = true)
			:base(effect,duration,name)
		{
			this.isSuicideTimeUp = isSuicideTimeUp;
			this.effect = effect;
		}
		public SummonUnitAbility (IEffect effect, int duration,bool isSuicideTimeUp = true)
			:base(effect,duration,AbilityName)
		{
			this.isSuicideTimeUp = isSuicideTimeUp;
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order) {
			case AbilityOrder.OtherExecs:
				unit.IsSummon = true;
				unit.CanStateNow = false;
				break;
			default:
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool OnVanish (IBuilding building)
		{
			if( isSuicideTimeUp &&  0 != building.HP )
				building.HP = 0;
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (building);
		}
		public override bool OnVanish (IUnit unit)
		{
			if( isSuicideTimeUp && 0 != unit.HP )
				unit.HP = 0;
			if (Effect != null) {
				Effect.Stop ();
				Effect = null;
			}
			return base.OnVanish (unit);
		}
	}

	class SendExpAndCollectTokenAbility : thrts.definedAbility.SendExpAbility
	{
		
		public SendExpAndCollectTokenAbility(IUnit holder,Func<IUnit,IUnit,bool>   isDamageMax)
			:base(holder,isDamageMax)
		{
			
		}
		
		public override void OnVanishHolder (IBuilding b)
		{
			base.OnVanishHolder(b);
		}
		
		public override void OnVanishHolder (IUnit u)
		{
			if( !u.IsSummon )
			{
				if (dist.UserValues.ContainsKey (CorpseToken.TokenName)) {
					var value = dist.UserValues [CorpseToken.TokenName] as CorpseToken;
					if (value != null)
						value.IncToken ();
				} else {
					dist.UserValues.Add (CorpseToken.TokenName, new CorpseToken (dist, 5));
				}
			}
			base.OnVanishHolder (u);
		}
	}
}

namespace LotusCraft.Token
{
	class CorpseToken
	{
		public const string TokenName = "r̃g[N";
		int counter = 0;
		int maxNum = 15;
		IUnit dist;
		public int Count { get { return counter; } }
		public int MaxCount { get { return maxNum; } }
		public CorpseToken(IUnit dist, int initCount = 0)
		{
			this.dist = dist;
			var spell = null != dist ? dist.GetSpellWithName(CorpseTokenDisplay.TokenName) : null;
			if (null != spell)
			{
				counter = spell.Level;
				maxNum = spell.MaxLevel;
			}
			if (null != dist && !dist.Exists() && initCount > 0)
				IncToken(initCount);

			if (null != dist && dist.IsMPBarShown)
			{
				dist.MPBarCurrent = counter;
				dist.MPBarMax = maxNum;
			}
		}
		public int IncToken(int inc = 1)
		{
			var spell = null != dist ? dist.GetSpellWithName(CorpseTokenDisplay.TokenName) : null;
			if (null != spell)
			{
				spell.LvUp(inc);
				counter = spell.Level;
			}
			else
			{
				counter += inc;
				if (counter > maxNum)
					counter = maxNum;
			}
			if (null != dist && dist.IsMPBarShown)
			{
				dist.MPBarCurrent = counter;
				dist.MPBarMax = maxNum;
			}
			return counter;
		}
		public int DecToken(int dec = 1)
		{
			var spell = null != dist ? dist.GetSpellWithName(CorpseTokenDisplay.TokenName) : null;
			if (null != spell)
			{
				spell.LvUp(-dec);
				counter = spell.Level;
			}
			else
			{
				counter -= dec;
				if (counter < 0)
					counter = 0;
			}
			if (null != dist && dist.IsMPBarShown)
			{
				dist.MPBarCurrent = counter;
				dist.MPBarMax = maxNum;
			}
			return counter;
		}
	}
}