﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using LotusCraft.Effect.Common;

namespace LotusCraft.AutoCast
{
	/// <summary>
	/// Dissolve auto cast.
	/// 5sec,
	/// </summary>
	public class DissolveAutoCast : thrts.AutoCastParameter<thrts.IBuilding>
	{
		float DissolveRate = 0.50f;
		float tickPerFrame = 1 / (5.0f * 30);
		float[] DissolveResource = new[] { 0.0f, 0.0f, 0.0f, 0.0f };
		public const string AutoCastName = "Dissolve";
		public DissolveAutoCast()
		{
			this.Name = AutoCastName;
			this.DefaultAutoCastEnable = false;
			this.CastAction = DissolveAutoCastAction;
			// cast all flames
			this.CastInterval = 1;
			/// set autocast button
			this.IsButtonSetup = true;
			this.ButtonPosX = thrts.ButtonPosition._4;
			this.ButtonPosY = thrts.ButtonPosition._3;

		}

		public bool DissolveAutoCastAction(thrts.IBuilding building)
		{
			// reduce HP and get resources

			building.HP -= (int)Math.Ceiling(building.MaxHP * tickPerFrame);
			for (int i = 0; i < building.Parameter.Costs.Length; ++i)
			{
				DissolveResource[i] += building.Parameter.Costs[i] * tickPerFrame * DissolveRate;
				while (DissolveResource[i] > 1.0)
				{
					DissolveResource[i] -= 1.0f;
					building.Faction.IncResource(i, 1);
				}
			}


			if (building.Faction.World.CurrentTime % 2 == 0)
			{
				// エフェクトを描画
				var faction = building.Faction;
				var world = faction.World;
				var effectPos = building.Pos;

				var size = building.Parameter.Size * 32.0f / 4.0f;

				var effect = world.EffectManager.CreateEffect(typeof(DissolveBuilding).FullName, effectPos);
				effect.SetScale(new thrts.Vector3D(size,size,size));
			}

			return true;
		}
	}
}
