using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;

namespace LotusCraft.Bomb
{
	public class Fast : BombParameter
	{
		const float affectRadius = 8;
		const int duration = 4*Def.FPS;
		const int interval = 10;
		const string actionAbilityName = "FastBombActionAbility";
		bool used = false;
		// 俊足 : 4秒、移動速度+60%
		public Fast ()
		{
			ImageOnSelection = "System/Bomb/bomb_fast";
			DescriptionOnSelection = "俊足「疾風迅雷」\n\n1stヒロインの周囲に居る範囲の味方ユニットの移動速度を大幅に上昇させる。\n1stヒロインが存在しない場合は使用できない。\n\nコスト\n　　酒200\n効果\n　　4秒間、味方ユニットの移動速度を1.6倍にする\n範囲\n　　1stヒロインの周囲、半径8。";

			Name = "俊足";
			Costs[0] = 200;
			Description = "4秒、移動速度+60%";
			this.ButtonParameter = new Button.BombButton(this,BombEvent,CanExec,"button_syunnsoku");
			used = false;

			BombCutin800.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/800/White/white_syunsoku");
			BombCutin1280.Add(typeof(Faction.White).FullName, "LotusCraft/Image/Bomb/1280/White/white_syunsoku");

			BombCutin800.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/800/Red/red_syunsoku");
			BombCutin1280.Add(typeof(Faction.Red).FullName, "LotusCraft/Image/Bomb/1280/Red/red_syunsoku");

			BombCutin800.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/800/Green/green_syunsoku");
			BombCutin1280.Add(typeof(Faction.Green).FullName, "LotusCraft/Image/Bomb/1280/Green/green_syunsoku");

#if !_TRIAL
			BombCutin800.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/800/Blue/blue_syunsoku");
			BombCutin1280.Add(typeof(Faction.Blue).FullName, "LotusCraft/Image/Bomb/1280/Blue/blue_syunsoku");

			BombCutin800.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/800/Black/black_syunsoku");
			BombCutin1280.Add(typeof(Faction.Black).FullName, "LotusCraft/Image/Bomb/1280/Black/black_syunsoku");
#endif
		}
		public ExecutableInfo CanExec (IFaction faction, BombButtonArguments args)
		{
			var cantExecuteMessage = new List<ColorTextLine> ();
			if (null == faction) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
			} else if(!args.CreatedOnceForGame) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
			} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインがマップに存在しません", new Color (1, 0, 0))));
			} else if(used) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("ボムは1回しか使えません", new Color (1, 0, 0))));
			} else if (Enumerable.Range (0, Costs.Length).Any (i => faction.GetResource (i) < Costs [i])) {
				cantExecuteMessage.Add (new ColorTextLine (new ColorText ("リソースが足りません", new Color (1, 0, 0))));
			}
			return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
		}
		public void BombEvent (IBomb bomb, BombButtonArguments args, bool flag)
		{
			if (!args.FirstHiroin.AbilityManager.ContainsAbility(actionAbilityName) && BombAction (args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction)) {
				if(null != bomb.Faction)
				{
					for(int i=0;i<Costs.Length;++i)
						bomb.Faction.DecResource(i,Costs[i]);
				}

				IEffect effect = null;
				Vector3D offset = new Vector3D(2, 2, -20.0f);
				// エフェクトを描画
				{
					var unit = args.FirstHiroin;
					var selfFaction = unit.Faction;
					var world = selfFaction.World;
					var e = world.EffectManager.CreateEffect (typeof(FastBombAreaEffect).FullName, unit.Pos + offset);
					var effectScale = 6.0f*affectRadius;
					e.SetScale (new Vector3D (effectScale, effectScale, effectScale));
					effect = e;
				}
				args.FirstHiroin.AbilityManager.AppendAbility(new SpellContinuousAbility((IUnit unit) => BombAction (args.CreatedOnceForGame, args.FirstHiroin, bomb.Faction),effect,offset,duration,actionAbilityName,interval));
				args.FirstHiroin.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Wind).FullName, args.FirstHiroin.Pos);

				bomb.ShowCutin();
				used = true;
			}
		}

		static public bool BombAction (bool has1stHero, IUnit unit, IFaction faction)
		{
			if(unit == null)
				return false;

			var selfFaction = unit.Faction;
			var world = selfFaction.World;
			// get targets
			var targets = world.GetObjectFromCircle(new Circle2D( affectRadius*Def.MapTipLengthOnField,new Vector2D( unit.Pos.X, unit.Pos.Y ))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>world.IsSameTeam(selfFaction,_.Faction)))
			{
				// 自分には効果無し
//				if(u.UniqueID == unit.UniqueID)
//					continue;

				if(!u.AbilityManager.ContainsAbility(FastAbility.AbilityName))
				{
					IEffect effect = null;
					Vector3D offset = new Vector3D (2.0f, 2.0f, 4.0f);
					if(u != unit)
					{
						// エフェクトを描画
						{
							var e = world.EffectManager.CreateEffect (typeof(FastBombEffect).FullName, unit.Pos + offset);
							e.SetScale (new Vector3D (8, 8, 8));
							effect = e;
						}
					}
					if(!u.AbilityManager.AppendAbility(new FastAbility(effect,offset,interval*2)))
					{
						if(effect != null)
						{
							effect.Stop();
							effect = null;
						}
					}
				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(interval,FastAbility.AbilityName,interval*2);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class FastAbility : FollowEffectAbility
	{
		public const float IncMove = 0.6f;
		public const string AbilityName = "FastBomb";
		public FastAbility(IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Speed += IncMove * UnitParameter.BasisSpeed;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish(IUnit unit)
		{
			return base.OnVanish(unit);
		}
	}
}