using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Unit.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Building.Black
{
	public class Factory : General.BaseStructure
	{
		public Factory ()
		{
			IconPath = "LotusCraft/Image/Button/Black/Building/Factory";
			Name = "上級魔法陣";
			ButtonDescription += "特殊ユニットを作成する魔法陣を建築する。\n";
			ShortcutKeyword = "BuildWorkshop";
			Size = 3;

			MakingTime = 65*30;
			Bullets = new string[] { };
			HP = 2500;
			Armor = 2;
			Supply = 5;
			Costs = new [] {150,25,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Black/Factory";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/MagicCircleHigh.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			
			RequireBuildings = new string[]{
				typeof(HomeBase).FullName,
			};
		}
		public override void OnLoad (IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Youmu).FullName,faction.GetUnitParameter(typeof(Youmu).FullName).MakingTime, ButtonPosition._0, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Satori).FullName,faction.GetUnitParameter(typeof(Satori).FullName).MakingTime, ButtonPosition._1, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Orin).FullName,faction.GetUnitParameter(typeof(Orin).FullName).MakingTime, ButtonPosition._2, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Oku).FullName,faction.GetUnitParameter(typeof(Oku).FullName).MakingTime, ButtonPosition._3, ButtonPosition._0),
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};
			/*
			var KaguyaSkillTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Black.YoumuSkillTech()),
			};
*/

			var reactorTech = new Technology.ReactorTech(typeof(Youmu).FullName,faction.GetUnitParameter(typeof(Youmu).FullName).Name);
			var ShortcutKeywords = new string[]
			{
				"CreateSpecial1",
				"CreateSpecial2",
				"CreateSpecial3",
				"CreateSpecial4",
			};
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				/*
				new ButtonParameterHolder<IBuilding> (
					KaguyaSkillTechButtons[0],KaguyaSkillTechButtons
					),
					*/
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[1].Name), AutocasterParams[1].ButtonPosX, AutocasterParams[1].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[2].Name), AutocasterParams[2].ButtonPosX, AutocasterParams[2].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[2]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[3].Name), AutocasterParams[3].ButtonPosX, AutocasterParams[3].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[3]
					)
					),
				new ButtonParameterHolder<IBuilding>(new LotusCraft.Button.TechnologyButton<IBuilding>(reactorTech,"BuildSpecial1")),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			TechnologyParams = new []{reactorTech};
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction (IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0,0,1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName,building.Faction,building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale,scale,scale));
			building.Completed += () => {if(effect != null){effect.Stop ();effect=null;}};
			building.Destoroyed += () => {
				if(effect != null)
				{
					effect.Stop ();
					effect=null;
				}
			};
		}
		public override void OnCompleted (IBuilding building)
		{
			IEffect effect = null;
			bool isReactor = false;
			building.Updating += () =>
			{
				// change reactor mode
				if(isReactor ^ building.UserValues.ContainsKey(BuildingParameter.ReactorKey))
				{
					effect.Stop();
					effect = null;
				}
				if (effect == null)
				{
					isReactor = building.UserValues.ContainsKey(BuildingParameter.ReactorKey);
					effect = building.Faction.World.EffectManager.CreateEffect(
						isReactor?typeof(Effect.Black.FactoriesReactor).FullName:typeof(Effect.Black.Factories).FullName,
						new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 16));
				}
			};

			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
				}
			};

			if(building.Faction.World.CurrentTime > 10)
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedBuilding).FullName, building.Pos);
			base.OnCompleted (building);
		}
	}
}
