﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Unit.Black;

namespace LotusCraft.Building.Black
{
	public class ProduceResType1 : General.ProduceResources<HomeBase,CollectPowerStone>
	{
		public ProduceResType1 ()
		{
			IconPath = "LotusCraft/Image/Button/ProduceResType1";
			Name = "霊力回収施設";
			ShortcutKeyword = "BuildOratory";
			Size = 2;

			Supply = 5;

			MakingTime = 30*Def.FPS;

//			Bullets = new string[] { typeof(ResType1Bullet).FullName };

			HP = 1000;

			Exp = 300;


			Costs = new [] {80,0,0,0};

			ButtonDescription += "霊石から資源を回収する建物を建築する。\nこの建物は陰陽玉(岩)を生産出来る。";

			EnableBuildingOnResourceKindSequence =  new []{1};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Black/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/PowerStoneCollector.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned2.mdl";

//			StationableUnits = new string[]{typeof(Farmer).FullName};
			Capacity = 6;
			DoesRequireWorkerToTouch = true;
		}
		
		public override void OnLoad (IFaction faction)
		{
			correctPerSec = 1.5f;
			incPerFarmer = 0.25f;

			AutocasterParams = new AutoCastParameter<IBuilding>[]{
				new AutoCast.EnqueueProductionUnitAutoCast(typeof(Low03).FullName,faction.GetUnitParameter(typeof(Low03).FullName).MakingTime, ButtonPosition._0, ButtonPosition._0),
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};


			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,shortcutKeyword:"CreateClay"
					)
					),
//				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton()),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, faction, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction (IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0,0,1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName,building.Faction,building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale,scale,scale));
			building.Completed += () => {if(effect != null){effect.Stop ();effect=null;}};
			building.Destoroyed += () => {
				if(effect != null)
				{
					effect.Stop ();
					effect=null;
				}
			};
		}
		public override void OnCompleted(IBuilding building)
		{
			List<IUnit> stayedUnits = new List<IUnit>();
			
			building.UnitStayed += (b, u) => 
			{
				var unitParam = building.Faction.GetUnitParameter(typeof(Low03).FullName);
				if(null == unitParam)
					return;

				stayedUnits.Add(u);
			};
			building.UnitReleased += (b, u) => 
			{
				if(stayedUnits.Contains(u))
				{
					stayedUnits.Remove(u);
					u.AbilityManager.RemoveAllAbility();
					u.HP = 0;
				}
			};
			building.Destoroyed += () => 
			{
				building.ReleaseAllUnit();

				stayedUnits.Clear();
			};


			if(building.Faction.World.CurrentTime > 10)
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.CreatedBuilding).FullName, building.Pos);
			
			base.OnCompleted (building);
		}
	}
}
