﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.AutoCast;

namespace LotusCraft.Building.Blue
{
	public class Airport : General.BaseStructure
	{
		public Airport()
		{
			IconPath = "LotusCraft/Image/Button/Black/Building/Airport";
			Name = "未知魔法陣";
			ButtonDescription += "飛行ユニットを作成することができる";
			ShortcutKeyword = "BuildAtelier";
			Size = 3;

			MakingTime = 45 * Def.FPS;
			Bullets = new string[] { };
			HP = 2500;
			Armor = 2;
			Supply = 15;
			Costs = new[] { 150, 100, 0, 0 };

			Animation = new TextureAnimation();
			Animation.Path = "LotusCraft/Image/Building/Blue/Factory";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/MagicCircleSky.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";


			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
				typeof(Factory).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(FlyBomber).FullName,faction.GetUnitParameter(typeof(FlyBomber).FullName).MakingTime, ButtonPosition._0, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(FlyFighter).FullName,faction.GetUnitParameter(typeof(FlyFighter).FullName).MakingTime, ButtonPosition._1, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(FlyTransporter).FullName,faction.GetUnitParameter(typeof(FlyTransporter).FullName).MakingTime, ButtonPosition._2, ButtonPosition._0),

				new EnqueueProductionUnitAutoCast(typeof(HighFairy).FullName,faction.GetUnitParameter(typeof(HighFairy).FullName).MakingTime, ButtonPosition._0, ButtonPosition._1),

				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			var ShortcutKeywords = new string[]
			{
				"CreateBomber",
				"CreateFighter",
				"CreateTransporter",

				"CreateSpecial5",
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[1].Name), AutocasterParams[1].ButtonPosX, AutocasterParams[1].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[2].Name), AutocasterParams[2].ButtonPosX, AutocasterParams[2].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[2]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[3].Name), AutocasterParams[3].ButtonPosX, AutocasterParams[3].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[3]
					)
					),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

		}
		public override void OnVanish(IBuilding building)
		{
			building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
			var faction = building.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
			var scale = new Vector3D(25, 25, 25);
			effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}
		public override void OnCompleted(IBuilding building)
		{
			IEffect effect = null;

			building.Updating += () =>
			{
				if (effect == null)
				{
					effect = building.Faction.World.EffectManager.CreateEffect(
							typeof(Effect.Blue.Airports).FullName,
							new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 40));
				}
			};

			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
				}
			};

			BuildingUtls.ShowComplete(building);
			base.OnCompleted(building);
		}
	}
}
