﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Blue;
using LotusCraft.Unit.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Building.Blue
{
	public class SukimaBuilding : General.BaseStructure
	{
		public SukimaBuilding ()
		{
			IconPath = "LotusCraft/Image/Button/Blue/Building/Tower";
			Name = "スキマ";
			ButtonDescription += "自軍ユニットが自由に出入りできる建物です。\n";
			Size = 2;
			ViewRange = 6*32;


			MakingTime = 1;
			Bullets = new string[] { };
			HP = 150;
			Armor = 0;
			Supply = 0;
			Costs = new [] {0,0,0,0};

			PathThrough = true;

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Blue/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Sukima.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			IsBuildedInOnlyFieldOfVision = true;

		}
		public override void OnLoad (IFaction faction)
		{
			///  農民以外の全ユニットが駐留可能
			StationableUnits = faction.Parameter.Units.Except(new []{typeof(Farmer).FullName,}).ToArray();
			Capacity =  100;

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton()),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			

			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
		}
		public override void OnStartConstruction(IBuilding building)
		{

			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
			BuildingUtls.ShowComplete(building);
			bool stationLock = false;
			int counter = 0;
			int stationDelay = Def.FPS/5;
			building.UnitStaying += (IBuilding b,IUnit u) => {
				if(u.IsEscaping)
					return true;
				
				if(stationLock)
				{
					u.Stop (false);
					u.Faction.World.UnitManager.SelectTarget(new[] {u},b,false);
					return false;
				}
				else
				{
					counter = 0;
					stationLock = true;
					return true;
				}
			};
			building.Updating += () => 
			{
				if(stationLock)
				{
					stationLock = ++counter < stationDelay;
				}
			};
			base.OnCompleted (building);
		}
 	}
}
