﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Red;

namespace LotusCraft.Building.Red
{
	public class BlackSmith : General.BaseStructure
	{
		static string defenceUnitKindName = typeof(Unit.Common.Patchouli).FullName;
		public BlackSmith ()
		{
			IconPath = "LotusCraft/Image/Button/Red/Building/BlackSmith";
			Name = "研究所";
			ButtonDescription += "ユニット・建築物を強化することができる。";
			ShortcutKeyword = "BuildLaboratory";
			Size = 3;

			MakingTime = 25*30;
			Bullets = new string[] { };
			HP = 1500;
			Armor = 2;
			Supply = 10;
			Costs = new [] {125,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/Red/BlackSmith";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Laboratory.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			RequireBuildings = new string[]{
				typeof(HomeBase).FullName
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

			var buildingHpTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.BuildingHpLv1Tech()),
			};
			var buildingArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.BuildingArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.BuildingArmorLv2Tech()),
			};
			var infantryArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.InfantryArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.InfantryArmorLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.InfantryArmorLv3Tech()),
			};
			var gunTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.GunLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.GunLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.GunLv3Tech()),
			};
			var cannonArmorTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonArmorLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonArmorLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonArmorLv3Tech()),
			};
			var cannonAtkTechButtons = new []{
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonAtkLv1Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonAtkLv2Tech()),
				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.Red.CannonAtkLv3Tech()),
			};

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding>(buildingArmorTechButtons[0],buildingArmorTechButtons),
				new ButtonParameterHolder<IBuilding>(buildingHpTechButtons[0],buildingHpTechButtons),
				new ButtonParameterHolder<IBuilding>(infantryArmorTechButtons[0],infantryArmorTechButtons),
				new ButtonParameterHolder<IBuilding>(gunTechButtons[0],gunTechButtons),
				new ButtonParameterHolder<IBuilding>(cannonAtkTechButtons[0],cannonAtkTechButtons),
				new ButtonParameterHolder<IBuilding>(cannonArmorTechButtons[0],cannonArmorTechButtons),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			

			TechnologyParams = buildingHpTechButtons
				.Concat(buildingArmorTechButtons)
				.Concat(infantryArmorTechButtons)
				.Concat(gunTechButtons)
				.Concat(cannonArmorTechButtons)
				.Concat(cannonAtkTechButtons)
				.Select(_=>_.TechParam);

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
			//研究所を一つ建てるたびに、防衛ユニットの生産可能最大数が増える。但し、6体を上限とする。また研究所がなんらかの行為によりその数を減らしたとしても、生産可能最大数の数は減らさない
			var defenceUnitParam = building.Faction.GetUnitParameter (defenceUnitKindName);
			if (null != defenceUnitParam) {
				int BlackSmithNum = building.Faction.World.BuildingManager.GetBuildings (_ => null != _ && _.IsComplete && _.Faction == building.Faction && _.Parameter.GetType () == building.Parameter.GetType ()).Count ();
				defenceUnitParam.MaximumExists = Math.Min (defenceUnitParam.MaximumExists+1,6);
			}
			BuildingUtls.ShowComplete(building);

			base.OnCompleted (building);
		}
	}
}
