﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.White;
using LotusCraft.Effect.White;
using LotusCraft.AutoCast;

namespace LotusCraft.Building.White
{
	public class Barrack : General.BaseStructure
	{
		public Barrack ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/Barrack";
			Name = "下級魔法陣";
			ButtonDescription += "下級ユニットを作成することができる。";
			ShortcutKeyword = "BuildBarrack";
			Size = 3;
			Armor = 2;

			MakingTime = 50*30;
			HP = 2200;
			Supply = 10;
			Costs = new [] {125,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/MagicCircleLow.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] {  };

			RequireBuildings = new string[]{
				typeof(HomeBase).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Low02).FullName,faction.GetUnitParameter(typeof(Low02).FullName).MakingTime, ButtonPosition._1, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Low03).FullName,faction.GetUnitParameter(typeof(Low03).FullName).MakingTime, ButtonPosition._2, ButtonPosition._0),
				new DissolveAutoCast(),
			};

			var reactorTech = new Technology.ReactorTech(typeof(Low03).FullName,faction.GetUnitParameter(typeof(Low03).FullName).Name);

			var ShortcutKeywords = new string[]
			{
				"CreateClay",
				"CreateWater",
			};
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[1].Name), AutocasterParams[1].ButtonPosX, AutocasterParams[1].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding>(new LotusCraft.Button.TechnologyButton<IBuilding>(reactorTech,"BuildSpecial1")),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			TechnologyParams = new []{ reactorTech };
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
			IEffect effect = null;
			bool isReactor = false;
			building.Updating += () =>
			{
				// change reactor mode
				if(isReactor ^ building.UserValues.ContainsKey(BuildingParameter.ReactorKey))
				{
					effect.Stop();
					effect = null;
				}
				if (effect == null)
				{
					isReactor = building.UserValues.ContainsKey(BuildingParameter.ReactorKey);
					effect = building.Faction.World.EffectManager.CreateEffect(
							isReactor?typeof(Effect.White.BarracksReactor).FullName:typeof(Effect.White.Barracks).FullName,
							new Vector3D(building.Pos.X, building.Pos.Y, building.Pos.Z + 16));
				}

			};

			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
				}
			};

			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted(building);
		}
	}
}
