﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.White;
using LotusCraft.Unit.White;
using LotusCraft.Bullet.White;

namespace LotusCraft.Building.White
{
	public class Bunker : General.BaseStructure
	{
		public Bunker ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/Bunker";
			Name = "博麗神社バンカー";
			ButtonDescription += "中に下級ユニットを駐留させ、ユニットを保護しつつ\n中から攻撃することができる。";
			ShortcutKeyword = "BuildBulwark";
			Size = 3;
			ViewRange += 1*Def.MapTipLengthOnField;


			MakingTime = 45*30;
			Bullets = new string[] { };
			HP = 750;
			Armor = 2;
			Supply = 0;
			Costs = new [] {125,0,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/HakureiShrineBuker.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Capacity = 4;
			StationableUnits = new string[] {
				typeof(Farmer).FullName,
				typeof(Low02).FullName,
				typeof(Low03).FullName,
			};
			CanStayedUnitsAttack = true;
			StayedUnitBonusRange = 2*Def.MapTipLengthOnField;

			RequireBuildings = new string[]{"LotusCraft.Building.White.Barrack"};
		}
		public override void OnLoad (IFaction faction)
		{
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.UnStationButton()),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
			

			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new LotusCraft.AutoCast.DissolveAutoCast(),
			};

		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}
		public override void OnStartConstruction(IBuilding building)
		{

			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted (IBuilding building)
		{
			BuildingUtls.ShowComplete(building);

			base.OnCompleted (building);
		}
 	}
}
