﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.AutoCast;
using LotusCraft.Unit.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Building.White
{
	public class Hospital : General.BaseStructure
	{
		const int healHP = 5;
		const int healInterval = 4*Def.FPS;
		const float healRadius = 4*Def.MapTipLengthOnField;
		public Hospital ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/Hospital";
			Name = "診療所";
			ButtonDescription += "周囲にいる味方ユニット1体のHPを回復する。\n鈴仙の生産に必要である。";
			ShortcutKeyword = "BuildExhibition3";
			Size = 3;

			MakingTime = 75*Def.FPS;
			HP = 1800;
			Armor = 2;
			Supply = 15;
			Costs = new [] {75,125,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Hospital.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned3.mdl";

			Bullets = new string[] {  };

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Reisen).FullName,faction.GetUnitParameter(typeof(Reisen).FullName).MakingTime, ButtonPosition._3, ButtonPosition._0),
				new DissolveAutoCast(),
			};
			
//			var ReisenSkillTechButtons = new []{
//				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.White.ReisenSkillTech(ButtonPosition._0,ButtonPosition._0)),
//			};

			var ShortcutKeywords = new string[]
			{
				"CreateSpecial4",
			};
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
//				new ButtonParameterHolder<IBuilding> (
//					ReisenSkillTechButtons[0],ReisenSkillTechButtons
//					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};

//			TechnologyParams = ReisenSkillTechButtons
//					.Select (_=>_.TechParam);
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
			int counter = 0;
			building.Updating += () =>
			{
				if(counter++ > healInterval)
				{
					counter = 0;
					var world = building.Faction.World;
					var targets = world.GetObjectFromCircle(new Circle2D(healRadius,new Vector2D(building.Pos.X,building.Pos.Y))).OfType<IUnit>().OrderBy(_=>_.HP/(float)_.MaxHP);
					foreach(var u in targets)
					{
						if(null == u || !u.Exists() || !world.IsSameTeam(u.Faction,building.Faction) || u.AbilityManager.ContainsAbility(this.Name+"HealEffect"))
							continue;

						// add effect
						var healEffect  = world.EffectManager.CreateEffect(typeof(HealEffect).FullName, u.Pos);
						if(!u.AbilityManager.AppendAbility(new FollowEffectAbility(healEffect, new Vector3D(0,0,1), healInterval, this.Name+"HealEffect")))
						{
							healEffect.Stop();
						}
						else
						{
							// HPが最も減っているユニット1体を選択
							u.HP += healHP;
							if(u.HP > u.MaxHP)
								u.HP = u.MaxHP;
							
							break;
						}
					}
				}
			};

			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
