﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Unit.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Building.White
{
	public class Houtou : General.BaseStructure
	{
		public Houtou ()
		{
			IconPath = "LotusCraft/Image/Button/White/Building/Houtou";
			Name = "宝塔";
			ButtonDescription += "広い範囲の視界を得る。\n寅丸星とナズーリンの生産に必要である。";
			ShortcutKeyword = "BuildExhibition1";
			Size = 2;

			MakingTime = 75*Def.FPS;
			HP = 1600;
			Armor = 2;
			Supply = 15;
			ViewRange = 18*Def.MapTipLengthOnField;
			Costs = new [] {125,75,0,0};

			Animation = new TextureAnimation ();
			Animation.Path = "LotusCraft/Image/Building/White/Barrack";
			Animation.Width = 128;
			Animation.Height = 128;
			Animation.Center.X = 0;
			Animation.Center.Y = -32;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 1;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;

			BuildingAnimation = Animation;
			ObjectPath = "LotusCraft/3D/Model/Houtou.mdl";
			BuildingObjectPath = "LotusCraft/3D/Model/None.mdl";
			ConstructionPlannedObjectPath = "LotusCraft/3D/Model/BuildingConstructionPlanned2.mdl";

			Bullets = new string[] {  };

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};

			ObjectEmissionColors = new Tuple<int, thrts.Color>[] { 
				new Tuple<int, thrts.Color>(2, new thrts.Color(2.0f,2.0f,0.0f, 0.0f))
			};
		}
		public override void OnLoad(IFaction faction)
		{
			AutocasterParams = new AutoCastParameter<IBuilding>[]
			{
				new EnqueueProductionUnitAutoCast(typeof(Toramaru).FullName,faction.GetUnitParameter(typeof(Toramaru).FullName).MakingTime, ButtonPosition._1, ButtonPosition._0),
				new EnqueueProductionUnitAutoCast(typeof(Nazurin).FullName,faction.GetUnitParameter(typeof(Nazurin).FullName).MakingTime, ButtonPosition._0, ButtonPosition._1),
				new DissolveAutoCast(),
			};
			
//			var ToramaruSkillTechButtons = new []{
//				new LotusCraft.Button.TechnologyButton<IBuilding>(new LotusCraft.Technology.White.ToramaruSkillTech(ButtonPosition._0,ButtonPosition._0)),
//			};

			var ShortcutKeywords = new string[]
			{
				"CreateSpecial2",
				"CreateSpecial5",
			};
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IBuilding>[]
			{
//				new ButtonParameterHolder<IBuilding> (
//					ToramaruSkillTechButtons[0],ToramaruSkillTechButtons
//					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[0].Name), AutocasterParams[0].ButtonPosX, AutocasterParams[0].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[0]
					)
					),
				new ButtonParameterHolder<IBuilding> (
					new LotusCraft.Button.CreateUnitByBuildingButton (
					faction.GetUnitParameter(AutocasterParams[1].Name), AutocasterParams[1].ButtonPosX, AutocasterParams[1].ButtonPosY,
					shortcutKeyword : ShortcutKeywords[1]
					)
					),
				new ButtonParameterHolder<IBuilding> (new LotusCraft.Button.DissolveBuildingButton()),
			};
//			TechnologyParams = ToramaruSkillTechButtons
//					.Select (_=>_.TechParam);
			
		}
		public override void OnVanish (IBuilding building)
		{
				building.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BreakBuildingSE).FullName, building.Pos);
				var faction = building.Faction;
				var offset = new Vector3D(-2, -2, 8);
				var effect = faction.World.EffectManager.CreateEffect(typeof(BreakBuilding).FullName, building.Pos + offset);
				var scale = new Vector3D(25, 25, 25);
				effect.SetScale(scale);
		}

		public override void OnStartConstruction(IBuilding building)
		{
			IEffect effect = null;
			var offset = new Vector3D(0, 0, 1);
			effect = building.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.CreateBuildingEffect).FullName, building.Pos + offset);
			var scale = 3.0f * building.Parameter.Size;
			effect.SetScale(new Vector3D(scale, scale, scale));
			building.Completed += () => { if (effect != null) { effect.StopRoot(); effect = null; } };
			building.Destoroyed += () =>
			{
				if (effect != null)
				{
					effect.Stop();
					effect = null;
				}
			};
		}

		public override void OnCompleted(IBuilding building)
		{
			BuildingUtls.ShowComplete(building);
			
			base.OnCompleted (building);
		}
	}
}
