﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Button
{
	public class MercenaryButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public MercenaryButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/AttackState";
			ButtonName = "増援";
			ShortcutKeyword = "";

			int [] costs = new int[4];
			costs[0] = 200;
			costs[1] = 0;
			costs[2] = 0;
			costs[3] = 0;

			var buttonDescription = "映姫と数体の増援ユニットを呼び出します。\n増援ユニットは一定時間で消滅します。";
			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();

			MouseOverTextFunc = faction => 
			{
				ColorText alert;
				var spells = faction.GetSpellWithName(typeof(Spell.Common.EikiSkill).FullName);
				if(spells == null || spells.IsEmpty() || spells.Any(s => s.Level > 0))
					alert = new ColorText("増援は一回しか呼べません。",new Color(1,0,0));
				else
					alert = new ColorText("増援は一回しか呼べません。");

				return text.EndWith(new ColorTextLine(alert)).EndWith(faction.GetResourceInfoText(costs));
			};

			Kind = ButtonKind.CreateUnit;
			CreateUnitEvent = (building, isAppend) =>
			{
				var faction = building.Faction;
				var spells = faction.GetSpellWithName(typeof(Spell.Common.EikiSkill).FullName);
				if(spells != null && !spells.IsEmpty() && spells.All(_=>_.Level < 1))
				{
					/// hier
					for(int i = 0;i<costs.Length;++i)
						faction.DecResource(i,costs[i]);

					/// lvup eiki skill
					faction.LvUpSpellWithName(typeof(Spell.Common.EikiSkill).FullName);

					/// call mercenary units
					faction.GetUnitParameter("Unit.Common.Mercenary01").Description = u => this.MouseOverTextFunc(faction).Skip(1);
					Utl.PutUnitAroundBuilding(faction.World,faction,typeof(Unit.Common.Mercenary01).FullName,building);
//					Utl.PutUnitAroundBuilding(faction.World,faction,typeof(Unit.Common.Mercenary01).FullName,building);
					faction.GetUnitParameter("Unit.Common.Mercenary02").Description = u => this.MouseOverTextFunc(faction).Skip(1);
					Utl.PutUnitAroundBuilding(faction.World, faction, typeof(Unit.Common.Mercenary02).FullName, building);
//					Utl.PutUnitAroundBuilding(faction.World,faction,typeof(Unit.Common.Mercenary02).FullName,building);
					faction.GetUnitParameter("Unit.Common.Eiki").Description = u => this.MouseOverTextFunc(faction).Skip(1);
					Utl.PutUnitAroundBuilding(faction.World, faction, typeof(Unit.Common.Eiki).FullName, building);
				}
			};
			CanExecute = (faction, _, building) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				if ( !building.UserValues.ContainsKey(Building.General.BaseStructure.FirstBuildingKey))
				{
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("増援は初期拠点からしか呼べません", new Color(1, 0, 0))));
				}
				else if (Enumerable.Range(0, costs.Length).Any(i => faction.GetResource(i) < costs[i]))
				{
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
				}
				else 
				{
					var spells = faction.GetSpellWithName(typeof(Spell.Common.EikiSkill).FullName);
					if(spells == null || spells.IsEmpty() || spells.Any(s => s.Level > 0))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に増援を呼んでいます", new Color(1, 0, 0))));
					}

				}

				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}
	}

	public class StationButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public StationButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._1);
			ButtonImagePath = "LotusCraft/Image/Button/Station";
			ButtonName = "駐留";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText("駐留します")
						)
				}
			);
			Kind = ButtonKind.SelectFriend;
			SelectUnitBuildingButtonEvent = (unit, units, buildings, isAppend) => buildings.Where(b => b.IsStationableObject()).Select(b => b.StayUnit(unit)).TakeWhile(b => !b).ForEach(_ => { });
		}
	}
	public class UnStationButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public UnStationButton(bool simultaneously = true)
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._2);
			ButtonImagePath = "LotusCraft/Image/Button/UnStation";
			ButtonName = "駐留解除";
			ShortcutKeyword = "UnStation";
			MouseOverTextFunc = _ =>
				new[]
				{
					new ColorTextLine
						(
							new ColorText("駐留を解きます")
						)
				};
			Kind = ButtonKind.Normal;
			if(simultaneously)
				NormalButtonEvent = (building, isAppend) => building.ReleaseAllUnit();
			else
				NormalButtonEvent = (building, isAppend) => 
				{
					var ability = building.AbilityManager.GetAbilityWithName(UnitReleaseAsyncAbility.AbilityName) as UnitReleaseAsyncAbility;
					if(null == ability)
						building.AbilityManager.AppendAbility(new UnitReleaseAsyncAbility(building.StayedUnits));
					else
						ability.AddReleaseUnits(building.StayedUnits);
				};
		}
		class UnitReleaseAsyncAbility:thrts.AbilityTemplate
		{
			int counter = 0;
			List<IUnit> releaseUnits;
			const int releaseTime = Def.FPS/3;
			public const string AbilityName = "UnStation";
			public UnitReleaseAsyncAbility(IUnit[] releaseUnits = null)
			{
				this.releaseUnits = new List<IUnit>();
				if(null != releaseUnits)
				{
					foreach (var u in releaseUnits) {
						if(null == u || this.releaseUnits.Contains(u))
							continue;
						this.releaseUnits.Add(u);
					}
				}
				DurationTime = this.releaseUnits.Count * releaseTime;
				counter = 0;
				Name = AbilityName;
			}
			public void AddReleaseUnits (IUnit[] releaseUnits = null)
			{
				if (null == releaseUnits)
					return;

				foreach (var u in releaseUnits) {
					if(null == u || this.releaseUnits.Contains(u))
						continue;
					this.releaseUnits.Add(u);
					DurationTime += releaseTime;
				}
			}
			public override bool ExecBuildingAbility (IBuilding building, AbilityOrder order)
			{
				switch (order)
				{
					case AbilityOrder.OtherExecs:
						if (counter++ % releaseTime == 0)
						{
							var u = releaseUnits.FirstOrDefault();
							if (null != u)
							{
								building.ReleaseSelectedUnit(u.UniqueID);
								releaseUnits.Remove(u);
							}
						}
						break;
					default:
						break;
				}
				return base.ExecBuildingAbility (building, order);
			}
		}
	}
	public class UnStationUnitButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public UnStationUnitButton(bool simultaneously = true)
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._2);
			ButtonImagePath = "LotusCraft/Image/Button/UnStation";
			ButtonName = "駐留解除";
			ShortcutKeyword = "UnStation";
			MouseOverTextFunc = _ =>
				new[]
				{
					new ColorTextLine
						(
							new ColorText("駐留を解きます")
						)
				};
			Kind = ButtonKind.Normal;
			if(simultaneously)
				NormalButtonEvent = (unit, isAppend) => unit.ReleaseAllUnit();
			else
				NormalButtonEvent = (unit, isAppend) => 
			{
				var ability = unit.AbilityManager.GetAbilityWithName(UnitReleaseAsyncAbility.AbilityName) as UnitReleaseAsyncAbility;
				if(null == ability)
					unit.AbilityManager.AppendAbility(new UnitReleaseAsyncAbility(unit.StayedUnits));
				else
					ability.AddReleaseUnits(unit.StayedUnits);
			};
		}
		
		class UnitReleaseAsyncAbility:thrts.AbilityTemplate
		{
			int counter = 0;
			List<IUnit> releaseUnits;
			const int releaseTime = Def.FPS/3;
			public const string AbilityName = "UnStation";
			public UnitReleaseAsyncAbility(IUnit[] releaseUnits = null)
			{
				this.releaseUnits = new List<IUnit>();
				if(null != releaseUnits)
				{
					foreach (var u in releaseUnits) {
						if(null == u || this.releaseUnits.Contains(u))
							continue;
						this.releaseUnits.Add(u);
					}
				}
				DurationTime = this.releaseUnits.Count * releaseTime;
				counter = 0;
				Name = AbilityName;
			}
			public void AddReleaseUnits (IUnit[] releaseUnits = null)
			{
				if (null == releaseUnits)
					return;
				
				foreach (var u in releaseUnits) {
					if(null == u || this.releaseUnits.Contains(u))
						continue;
					this.releaseUnits.Add(u);
					DurationTime += releaseTime;
				}
			}
			public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
			{
				switch (order)
				{
					case AbilityOrder.OtherExecs:
						if (counter++ % releaseTime == 0)
						{
							var u = releaseUnits.FirstOrDefault();
							if (null != u)
							{
								unit.ReleaseSelectedUnit(u.UniqueID);
								releaseUnits.Remove(u);
							}
						}
						break;
					default:
						break;
				}
				return base.ExecUnitAbility (unit, order);
			}
		}
	}
	public class BuildingButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public BuildingButton()
		{
			Position = Tuple.Create(ButtonPosition._0, ButtonPosition._2);
			ButtonImagePath = "LotusCraft/Image/Button/Building";
			ButtonName = "建築";
			MouseOverTextFunc = Faction =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText ("建築する建物を選択します")
						)
				}
			);
			Kind = ButtonKind.Dummy;
		}
	}
	public class CancelBuildingButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public CancelBuildingButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._3);
			ButtonImagePath = "LotusCraft/Image/Button/cancel";
			ButtonName = "キャンセル";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText("キャンセルします")
						)
				}
			);
			Kind = ButtonKind.Dummy;
		}
	}
	public class MoveButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public MoveButton()
		{
			Position = Tuple.Create(ButtonPosition._0, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/MoveState";
			ButtonName = "移動";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText ("指定地点まで移動する。")
						),
					new ColorTextLine
						(
							new ColorText ("途中で敵を発見しても攻撃しない。")
						),
				}
			);
			Kind = ButtonKind.SelectPlace;
			SelectPlaceButtonEvent = (self, pos, isAppend) =>
			{
				var units = new[] { self };
				self.Faction.World.UnitManager.SelectTarget(units, (Vector2D)pos, isAppend);
			};
			CanExecute = (_, targets, __) =>
				{
					var executable = targets.All(u => 0.0f < u.Speed);
					return new ExecutableInfo(executable, executable ? "" : new ColorTextLine(new ColorText("対象ユニットに動けないユニットが含まれています", new Color(1, 0, 0))));
				};

			ShortcutKeyword = "Move";
		}
	}

	public class StopButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public StopButton()
		{
			Position = Tuple.Create(ButtonPosition._1, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/Stop";
			ButtonName = "停止";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText("全ての動作を停止する。")
						),
				}
			);
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) => { unit.Stop(isAppend); };

			ShortcutKeyword = "Stop";
		}
	}

	public class HoldPositionButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public HoldPositionButton()
		{
			Position = Tuple.Create(ButtonPosition._2, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/HoldPositionState";
			ButtonName = "待機";

			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText("現在位置で待機する。")
						),
					new ColorTextLine
						(
							new ColorText("射程内に入った敵ユニットに対して攻撃のみ攻撃し、追撃しない。")
						),
				}
			);
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) => { unit.HoldPosition(isAppend); };

			ShortcutKeyword = "HoldPosition";
		}
	}

	public class PatrolButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public PatrolButton()
		{
			Position = Tuple.Create(ButtonPosition._3, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/Patrol";
			ButtonName = "パトロール";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText ("2点間を往復する。")
						),
				}
			);
			Kind = ButtonKind.SelectObjectOrPlace;
			SelectObjectOrPlaceButtonEvent = (self, u, b, r, p, isAppend) =>
				{
					var units = new int[] { self.UniqueID };

					if (u.Any())
					{						
						self.Faction.World.UnitManager.Patrol(units, u.First(), isAppend);
					}

					if (b.Any())
					{
						self.Faction.World.UnitManager.Patrol(units, b.First(), isAppend);
					}

					if (r.Any())
					{
						self.Faction.World.UnitManager.Patrol(units, r.First(), isAppend);
					}

					if (p.HasValue)
					{
						self.Faction.World.UnitManager.Patrol(units, (Vector2D)p, isAppend);
					}
				};

			ShortcutKeyword = "Patrol";
		}
	}

	public class AttackMoveButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public AttackMoveButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._0);
			ButtonImagePath = "LotusCraft/Image/Button/AttackState";
			ButtonName = "攻撃移動";
			MouseOverTextFunc = _ =>
			(
				new[]
				{
					new ColorTextLine
						(
							new ColorText ("指定地点まで移動する。")
						),
					new ColorTextLine
						(
							new ColorText ("途中で敵を発見した場合、攻撃する。")
						),
				}
			);
			Kind = ButtonKind.SelectObjectOrPlace;
			SelectObjectOrPlaceButtonEvent = (self, u, b, r, p, isAppend) =>
			{
				var units = new int[] { self.UniqueID };

				if (u.Any())
				{
					self.Faction.World.UnitManager.AttackMove(units, u.First(), isAppend);
				}

				if (b.Any())
				{
					self.Faction.World.UnitManager.AttackMove(units, b.First(), isAppend);
				}

				if (r.Any())
				{
					self.Faction.World.UnitManager.AttackMove(units, r.First(), isAppend);
				}

				if (p.HasValue)
				{
					self.Faction.World.UnitManager.AttackMove(units, (Vector2D)p, isAppend);
				}
			};

			ShortcutKeyword = "AttackMove";
		}
	}
	public class DissolveBuildingButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public DissolveBuildingButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._3);
			ButtonImagePath = "LotusCraft/Image/Button/Delete";
			ButtonName = "解体";
			ShortcutKeyword = "DissolveBuilding";
			MouseOverTextFunc = _ =>
				new[]
				{
					new ColorTextLine
						(
							new ColorText("建物を解体し、資源を一部回収する。")
						)
				};

			Kind = ButtonKind.Normal;
			NormalButtonEvent = (building, isAppend) =>
			{
				if (building.IsComplete)
					building.GetIAutoCastWithName("Dissolve").ToggleAutoCast();
				else
					building.CancelProgress();
			};
		}
	}

	public class DissolveHomeBaseButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public DissolveHomeBaseButton()
		{
			Position = Tuple.Create(ButtonPosition._4, ButtonPosition._3);
			ButtonImagePath = "LotusCraft/Image/Button/Delete";
			ButtonName = "解体";
			ShortcutKeyword = "DissolveBuilding";
			MouseOverTextFunc = _ =>
				new[]
				{
					new ColorTextLine
						(
							new ColorText("建物を解体し、資源を回収する。建築予定時のみしか実行できない。")
						)
				};

			Kind = ButtonKind.Normal;
			NormalButtonEvent = (building, isAppend) =>
			{
				if (building.GetProgress != 0.0f) return;
				building.CancelProgress();
			};

			CanExecute = (faction, _, __) =>
				{
					return new ExecutableInfo(__.GetProgress == 0.0f);
				};
		}
	}


	public class CreateBuildingByUnitButton : thrts.ButtonParameter<thrts.IUnit>
	{
		public const string BuildingMaxKey = "building_max";
		public const string BuildingCurrentKey = "building_cur";
		string className = string.Empty;
		public CreateBuildingByUnitButton(BuildingParameter building,
										   thrts.ButtonPosition row,
										   thrts.ButtonPosition column,
										  string[] requires = null,
										  string shortcutKeyword = null)
		{
			Position = Tuple.Create(row, column);
			ButtonImagePath = building.IconPath;

			className = building.GetType().FullName;
			ButtonName = building.Name + "作成";

			if (null != shortcutKeyword)
				ShortcutKeyword = shortcutKeyword;
			else if(null != building.ShortcutKeyword)
				ShortcutKeyword = building.ShortcutKeyword;
			else
				ShortcutKeyword = string.Empty;

			var text = building.ButtonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
					.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => 
			{
				var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
				var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
				if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
				{
					int c = buildingCurrentDic[className];
					int m = buildingMaxDic[className];
					ColorTextLine maxText;
					if(c<m)
						maxText = new ColorTextLine(new ColorText("　建設数 : " + c + "/" + m));
					else
						maxText = new ColorTextLine(new ColorText("　建設数 : " + c + "/" + m,new Color(1,0,0)));

					return text
						.EndWith(maxText)
						.EndWith(faction.GetBuildingCostInfoText(building.Costs,building.Supply,building.MakingTime/Def.FPS))
						.EndWith(faction.GetRequireBuildingsInfoText(building.RequireBuildings));
				}
				else
				{
					return text
						.EndWith(faction.GetBuildingCostInfoText(building.Costs,building.Supply,building.MakingTime/Def.FPS))
						.EndWith(faction.GetRequireBuildingsInfoText(building.RequireBuildings));
				}
			};

			Kind = ButtonKind.CreateBuilding;
			CreateBuildingEvent = (unit, isAppend) => className;
			building.Faction.GetBuildingParameter(className).Description = b => MouseOverTextFunc(b.Faction).Take(MouseOverTextFunc(b.Faction).Count() - 1);

			if (requires != null)
			{
				CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if (Enumerable.Range(0, building.Costs.Length)
						.Any(i => faction.GetResource(i) < building.Costs[i]))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("資源が足りません", new Color(1, 0, 0))));
					}

					if (!faction.IsSatisfyBuildings(requires))
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("指定した建物が建設済みではありません", new Color(1, 0, 0))));

					var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
					var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
					if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
					{
						if(buildingCurrentDic[className ]>= buildingMaxDic[className])
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("建築最大数を超えています", new Color(1, 0, 0))));
						}
					}
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
			else if (building.RequireBuildings.Length > 0)
			{
				CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if (Enumerable.Range(0, building.Costs.Length)
						.Any(i => faction.GetResource(i) < building.Costs[i]))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("資源が足りません", new Color(1, 0, 0))));
					}

					if (!faction.IsSatisfyBuildings(building.RequireBuildings))
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("指定した建物が建設済みではありません", new Color(1, 0, 0))));

					var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
					var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
					if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
					{
						if(buildingCurrentDic[className ]>= buildingMaxDic[className])
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("建築最大数を超えています", new Color(1, 0, 0))));
						}
					}
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
			else
			{
				CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if (Enumerable.Range(0, building.Costs.Length)
						.Any(i => faction.GetResource(i) < building.Costs[i]))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("資源が足りません", new Color(1, 0, 0))));
					}

					var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey] : null) as Dictionary<string, int>;
					var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey] : null) as Dictionary<string, int>;
					if (null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
					{
						if (buildingCurrentDic[className] >= buildingMaxDic[className])
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("建築最大数を超えています", new Color(1, 0, 0))));
						}
					}
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
		}
		public CreateBuildingByUnitButton(string factionName,
										  string building,
										  string buildingName,
										   thrts.ButtonPosition row,
										   thrts.ButtonPosition column,
										  string buttonDescription = null,
										  string shortcutKeyword = null)
		{
			Position = Tuple.Create(row, column);
			ButtonImagePath = "LotusCraft/Image/Button/" + factionName + "/Building/" + building;


			if (null != shortcutKeyword)
				ShortcutKeyword = shortcutKeyword;
			else
				ShortcutKeyword = string.Empty;

			className = "LotusCraft.Building." + factionName + "." + building;
			if (buttonDescription == null)
				buttonDescription = string.Empty;
			ButtonName = buildingName + "作成";

			var text = buttonDescription.Split (new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select (_ => new ColorText (_))
					.Select (_ => new ColorTextLine (_))
					.ToList ();

			MouseOverTextFunc = _ => text;

			Kind = ButtonKind.CreateBuilding;
			CreateBuildingEvent = (unit, isAppend) => className;
		}
	}
	public class CreateBuildingByBuildingButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		public const string BuildingMaxKey = "building_max";
		public const string BuildingCurrentKey = "building_cur";
		string className = string.Empty;
		public CreateBuildingByBuildingButton(BuildingParameter building,
										  thrts.ButtonPosition row,
										  thrts.ButtonPosition column, string[] requires = null,
											  string shortcutKeyword = null)
		{
			Position = Tuple.Create(row, column);
			ButtonImagePath = building.IconPath;

			className = building.GetType().FullName;
			ButtonName = building.Name + "作成";

			if (null != shortcutKeyword)
				ShortcutKeyword = shortcutKeyword;
			else if(null != building.ShortcutKeyword)
				ShortcutKeyword = building.ShortcutKeyword;
			else
				ShortcutKeyword = string.Empty;
			

			var text = building.ButtonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => 
			{
				var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
				var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
				if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
				{
					int c = buildingCurrentDic[className];
					int m = buildingMaxDic[className];
				ColorTextLine maxText;
					if(c<m)
						maxText = new ColorTextLine(new ColorText("　建設数 : " + c + "/" + m));
					else
						maxText = new ColorTextLine(new ColorText("　建設数 : " + c + "/" + m,new Color(1,0,0)));

					return text
						.EndWith(maxText)
							.EndWith(faction.GetBuildingCostInfoText(building.Costs,building.Supply,building.MakingTime/Def.FPS))
							.EndWith(faction.GetRequireBuildingsInfoText(building.RequireBuildings));
				}
				else
				{
					return text
						.EndWith(faction.GetBuildingCostInfoText(building.Costs,building.Supply,building.MakingTime/Def.FPS))
							.EndWith(faction.GetRequireBuildingsInfoText(building.RequireBuildings));
				}
			};

			Kind = ButtonKind.CreateBuilding;
			CreateBuildingEvent = (unit, isAppend) => className;

			if (requires != null)
			{
				CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if (Enumerable.Range(0, building.Costs.Length)
						.Any(i => faction.GetResource(i) < building.Costs[i]))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("資源が足りません", new Color(1, 0, 0))));
					}

					if (!faction.IsSatisfyBuildings(requires))
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("指定した建物が建設済みではありません", new Color(1, 0, 0))));

					var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
					var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
					if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
					{
						if(buildingCurrentDic[className ]>= buildingMaxDic[className])
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("建築最大数を超えています", new Color(1, 0, 0))));
						}
					}
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
			else if (building.RequireBuildings.Length > 0)
			{
				CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if (Enumerable.Range(0, building.Costs.Length)
						.Any(i => faction.GetResource(i) < building.Costs[i]))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("資源が足りません", new Color(1, 0, 0))));
					}

					if (!faction.IsSatisfyBuildings(building.RequireBuildings))
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("指定した建物が建設済みではありません", new Color(1, 0, 0))));

					var buildingMaxDic = (faction.UserValues.ContainsKey(BuildingMaxKey) ? faction.UserValues[BuildingMaxKey]:null) as Dictionary<string,int>;
					var buildingCurrentDic = (faction.UserValues.ContainsKey(BuildingCurrentKey) ? faction.UserValues[BuildingCurrentKey]:null) as Dictionary<string,int>;
					if(null != buildingMaxDic && buildingMaxDic.ContainsKey(className) && null != buildingCurrentDic && buildingCurrentDic.ContainsKey(className))
					{
						if(buildingCurrentDic[className ]>= buildingMaxDic[className])
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("建築最大数を超えています", new Color(1, 0, 0))));
						}
					}
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
		}
	}

	public class BombButton : thrts.ButtonParameter<thrts.IBomb>
	{
		public BombButton (BombParameter bombParam, Action<IBomb, BombButtonArguments, bool> bombEvent, Func<IFaction, BombButtonArguments, ExecutableInfo> canExec = null,string iconName = null)
		{
			Position = Tuple.Create (ButtonPosition._0, ButtonPosition._0);
			ButtonName = bombParam.Name;
			if(iconName == null)
				ButtonImagePath = "LotusCraft/Image/Button/Normal";
			else
				ButtonImagePath = "LotusCraft/Image/Bomb/" + iconName;

			Kind = ButtonKind.Bomb;
			BombButtonEvent = bombEvent;
			if (null != canExec) {
				CanBombExecute = canExec;
			} else {
				CanBombExecute = (faction,args) => {
					var cantExecuteMessage = new List<ColorTextLine> ();
					if (null == faction) {
						cantExecuteMessage.Add (new ColorTextLine (new ColorText ("勢力がnullです", new Color (1, 0, 0))));
					} else if(!args.CreatedOnceForGame) {
						cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインが生産されていません", new Color (1, 0, 0))));
					} else if(null == args.FirstHiroin || !args.FirstHiroin.Exists() || args.FirstHiroin.IsEscaping) {
						cantExecuteMessage.Add (new ColorTextLine (new ColorText ("1stヒロインがマップに存在しません", new Color (1, 0, 0))));
					}
					
					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
			}
			var text = bombParam.Description.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToList();
			MouseOverTextFunc = faction => text.EndWith(faction.GetResourceInfoText(bombParam.Costs));

		}
	}
}
