﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Button
{
	public class CreateUnitByBuildingButton : thrts.ButtonParameter<thrts.IBuilding>
	{
		string className = string.Empty;
		string unitName = string.Empty;
		public CreateUnitByBuildingButton(UnitParameter unit,
			thrts.ButtonPosition row,
			thrts.ButtonPosition column,
			string iconPath = null,
			string shortcutKeyword = null)
		{
			Position = Tuple.Create(row, column);
			if (null == iconPath)
				ButtonImagePath = unit.IconPath;

			className = unit.GetType().FullName;
			unitName = unit.Name;

			// 
			ShortcutKeyword = shortcutKeyword != null ? shortcutKeyword : (null != unit.ShortcutKeyword ? unit.ShortcutKeyword : string.Empty);

			/// 説明に人口条件や時間条件を追記
			string buttonDescription = string.Empty;

			buttonDescription += unit.ButtonDescription;

			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
				.Select(_ => new ColorTextLine(_))
				.ToArray();
			List<ColorText> unitNameText = new List<ColorText>();
			unitNameText.Add(new ColorText(unitName + " ["));
			foreach (var t in unit.UnitType)
			{
				switch (t)
				{
					case TargetObjectType.Clay:
						unitNameText.Add(new ColorText(" 岩", new Color(1, 0, 0)));
						break;
					case TargetObjectType.Water:
						unitNameText.Add(new ColorText(" 水", new Color(0, 191 / 255f, 243 / 255f)));
						break;
					case TargetObjectType.Structure:
						unitNameText.Add(new ColorText(" 鋼", new Color(0.5f, 0.5f, 0.5f)));
						break;
					case TargetObjectType.Hero:
						unitNameText.Add(new ColorText(" 白", new Color(1, 1, 1)));
						break;
				}
			}
			unitNameText.Add(new ColorText(" ]"));
			var unitNameTextLine = new[] { new ColorTextLine(unitNameText) };

			MouseOverTextFunc = faction => unitNameTextLine
					.EndWith(faction.GetAttackBonusText(unit))
					.EndWith(text)
					.EndWith(
						faction.World.UnitManager.DontUseInitCost(faction, unit.GetType().FullName)
							? faction.GetCostInfoText(unit.Costs, unit.PopulationWeight, unit.MakingTime / Def.FPS)
							: faction.GetCostInfoText(unit.InitCosts, unit.PopulationWeight, unit.MakingTime / Def.FPS))
					.EndWith(faction.GetRequireBuildingsInfoText(unit.RequireBuildings))
					.EndWith(faction.GetRequireUnitsInfoText(unit.RequireUnits))
					.EndWith(unit.IsUnique
						? (faction.IsSatisfyUnits(new string[] { className }, false)
								? new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない", new Color(1, 0, 0))) }
								: new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない")) }
							)
						: new ColorTextLine[0])
					.EndWith((unit.IsUnique || unit.IsDefence) && faction.World.IsInPreparatoryPeriod()
							 ? new ColorTextLine[] { new ColorTextLine(new ColorText("停戦中", new Color(1, 0, 0))) }
					: new ColorTextLine[0])
					;

			Kind = ButtonKind.CreateUnit;
			CreateUnitEvent = (building, isAppend) => building.AddCreationOfUnitQueue(unitName, className);
			unit.Faction.GetUnitParameter(className).Description = u => MouseOverTextFunc(unit.Faction).Take(MouseOverTextFunc(unit.Faction).Count() - (u.Parameter.IsUnique ? 2 : 1)).Skip(1);
			CanExecute = (faction, _, __) =>
				{
					var cantExecuteMessage = new List<ColorTextLine>();

					if ((unit.IsUnique || unit.IsDefence) && faction.World.IsInPreparatoryPeriod())
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("停戦中です", new Color(1, 0, 0))));
					}
					if (unit.IsUnique && faction.IsSatisfyUnits(new string[] { className }, false))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に " + unit.Name + " を生産しています", new Color(1, 0, 0))));
					}
					else if (unit.MaximumExists > 0)
					{
						var unitNum = faction.World.UnitManager.GetUnits( u => null != u && u.Exists() && u.Faction == faction && u.Parameter.GetType() == unit.GetType()).Count ();
						var unitInQueueNum = faction.World.BuildingManager.GetBuildings(b => null != b && b.Exists() && b.Faction == faction).Sum( b=>b.GetCountInProductionQueue(unit.GetType ().FullName));
						if(unitNum + unitInQueueNum >= unit.MaximumExists)
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に " + unit.Name + " を " + unit.MaximumExists +" 体生産しています", new Color(1, 0, 0))));
					}
					if (faction.World.UnitManager.DontUseInitCost(faction, unit.GetType().FullName) || unit.InitCosts == null)
					{
						if (Enumerable.Range(0, unit.Costs.Length)
							.Any(i => faction.GetResource(i) < unit.Costs[i]))
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
						}
					}
					else
					{
						if (Enumerable.Range(0, unit.Costs.Length)
							.Any(i => faction.GetResource(i) < unit.InitCosts[i]))
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
						}
					}
					if (unit.RequireBuildings != null && unit.RequireBuildings.Any())
					{
						if (!faction.IsSatisfyBuildings(unit.RequireBuildings))
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提建物が存在しません", new Color(1, 0, 0))));
						}
					}

					return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
				};
		}

		public CreateUnitByBuildingButton(UnitParameter unit,
										  thrts.ButtonPosition row,
										  thrts.ButtonPosition column,
										  int concurrents,
										  string[] requireBuildings,
										  string iconPath = null,
										  string shortcutKeyword = null)
		{
			Position = Tuple.Create(row, column);
			if (null == iconPath)
				ButtonImagePath = unit.IconPath;
			if(concurrents < 1)
				concurrents = 1;
			if(null == requireBuildings)
				requireBuildings = new string[0];

			className = unit.GetType().FullName;
			unitName = unit.Name;
			
			// 
			ShortcutKeyword = shortcutKeyword != null ? shortcutKeyword : (null != unit.ShortcutKeyword ? unit.ShortcutKeyword : string.Empty);
			
			/// 説明に人口条件や時間条件を追記
			string buttonDescription = string.Empty;
			
			buttonDescription += unit.ButtonDescription + " " + concurrents + "体同時生産";
			
			var text = buttonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
					.Select(_ => new ColorTextLine(_))
					.ToArray();
			List<ColorText> unitNameText = new List<ColorText>();
			unitNameText.Add(new ColorText(unitName + " ["));
			foreach (var t in unit.UnitType)
			{
				switch (t)
				{
				case TargetObjectType.Clay:
					unitNameText.Add(new ColorText(" 岩", new Color(1, 0, 0)));
					break;
				case TargetObjectType.Water:
					unitNameText.Add(new ColorText(" 水", new Color(0, 191 / 255f, 243 / 255f)));
					break;
				case TargetObjectType.Structure:
					unitNameText.Add(new ColorText(" 鋼", new Color(0.5f, 0.5f, 0.5f)));
					break;
				case TargetObjectType.Hero:
					unitNameText.Add(new ColorText(" 白", new Color(1, 1, 1)));
					break;
				}
			}
			unitNameText.Add(new ColorText(" ]"));
			var unitNameTextLine = new[] { new ColorTextLine(unitNameText) };
			
			MouseOverTextFunc = faction => unitNameTextLine
				.EndWith(faction.GetAttackBonusText(unit))
					.EndWith(text)
					.EndWith(
						faction.World.UnitManager.DontUseInitCost(faction, unit.GetType().FullName)
						? faction.GetCostInfoText(unit.Costs.Select(_=>_*concurrents).ToArray(), unit.PopulationWeight*concurrents, unit.MakingTime / Def.FPS)
						: faction.GetCostInfoText(unit.InitCosts.Select(_=>_*concurrents).ToArray(), unit.PopulationWeight*concurrents, unit.MakingTime / Def.FPS))
					.EndWith(faction.GetRequireBuildingsInfoText(null != unit.RequireBuildings?unit.RequireBuildings.Concat(requireBuildings).ToArray():requireBuildings))
					.EndWith(faction.GetRequireUnitsInfoText(unit.RequireUnits))
					.EndWith(unit.IsUnique
							 ? (faction.IsSatisfyUnits(new string[] { className }, false)
					   ? new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない", new Color(1, 0, 0))) }
					: new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない")) }
					)
							 : new ColorTextLine[0])
					.EndWith((unit.IsUnique || unit.IsDefence) && faction.World.IsInPreparatoryPeriod()
							 ? new ColorTextLine[] { new ColorTextLine(new ColorText("停戦中", new Color(1, 0, 0))) }
							 : new ColorTextLine[0]);
			
			Kind = ButtonKind.CreateUnit;
			CreateUnitEvent = (building, isAppend) => building.AddCreationOfUnitQueue(unitName, className, concurrents);
			unit.Faction.GetUnitParameter(className).Description = u => MouseOverTextFunc(unit.Faction).Take(MouseOverTextFunc(unit.Faction).Count() - (u.Parameter.IsUnique ? 2 : 1)).Skip(1);
			CanExecute = (faction, _, __) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				
				if ((unit.IsUnique || unit.IsDefence) && faction.World.IsInPreparatoryPeriod())
				{
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("停戦中です", new Color(1, 0, 0))));
				}
				if (unit.IsUnique && faction.IsSatisfyUnits(new string[] { className }, false))
				{
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に " + unit.Name + " を生産しています", new Color(1, 0, 0))));
				}
				else if (unit.MaximumExists > 0)
				{
					var unitNum = faction.World.UnitManager.GetUnits( u => null != u && u.Exists() && u.Faction == faction && u.Parameter.GetType() == unit.GetType()).Count ();
					var unitInQueueNum = faction.World.BuildingManager.GetBuildings(b => null != b && b.Exists() && b.Faction == faction).Sum( b=>b.GetCountInProductionQueue(unit.GetType ().FullName));
					if(unitNum + unitInQueueNum >= unit.MaximumExists)
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に " + unit.Name + " を " + unit.MaximumExists +" 体生産しています", new Color(1, 0, 0))));
				}
				if (faction.World.UnitManager.DontUseInitCost(faction, unit.GetType().FullName) || unit.InitCosts == null)
				{
					if (Enumerable.Range(0, unit.Costs.Length)
						.Any(i => faction.GetResource(i) < unit.Costs[i]*concurrents))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
					}
				}
				else
				{
					if (Enumerable.Range(0, unit.InitCosts.Length)
						.Any(i => faction.GetResource(i) < unit.InitCosts[i]*concurrents))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
					}
				}
				if (unit.RequireBuildings != null && unit.RequireBuildings.Any())
				{
					if (!faction.IsSatisfyBuildings(unit.RequireBuildings.Concat(requireBuildings).ToArray()))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提建物が存在しません", new Color(1, 0, 0))));
					}
				}
				else
				{
					if (!faction.IsSatisfyBuildings(requireBuildings))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提建物が存在しません", new Color(1, 0, 0))));
					}
				}
				
				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}
	}

	public class CreateAkyuButton : LotusCraft.Button.CreateUnitByBuildingButton
	{
		public CreateAkyuButton(thrts.ButtonPosition row, thrts.ButtonPosition column)
			: base(new LotusCraft.Unit.White.Unique.Akyu(), row, column)
		{
		}
	}

}
