﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using LotusCraft.Spell;
using thrts;

namespace LotusCraft.Button
{
	public class TechnologyButton<ButtonHolder> : thrts.ButtonParameter<ButtonHolder> where ButtonHolder : thrts.ITargetObject
	{
		string techClassName = string.Empty;
		public TechnologyParameter<ITargetObject> TechParam;
		public TechnologyButton(TechnologyParameter<ITargetObject> tech,string shortcutKeyword = null)
		{
			TechParam = tech;

			Position = Tuple.Create(TechParam.ButtonPosX, TechParam.ButtonPosY);
			ButtonImagePath = TechParam.IconPath;
			ButtonName = TechParam.Name;

			IsLocal = TechParam.DoesOnlyTarget;

			if(null != shortcutKeyword)
				ShortcutKeyword = shortcutKeyword;
			else if(null != TechParam.ShortcutKeyword)
				ShortcutKeyword= TechParam.ShortcutKeyword;
			else
				ShortcutKeyword = string.Empty;

			var text = TechParam.ButtonDescription.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select(_ => new ColorText(_))
				.Select(_ => new ColorTextLine(_))
				.ToList();

			MouseOverTextFunc = 
				faction => text
					.EndWith(faction.GetCostInfoText(TechParam.Costs,0,TechParam.TechTime/Def.FPS))
					.EndWith(faction.GetRequireBuildingsInfoText(TechParam.RequireBuildings))
					.EndWith(faction.GetRequireUnitsInfoText(TechParam.RequireUnits));

			Kind = ButtonKind.CreateUnit;
			CreateUnitEvent = (target, isAppend) =>
			{
				var techName = target.Faction.AddpendTechnology(target, TechParam);
				if (target is IBuilding && techName != null && techName != string.Empty)
				{
					(target as IBuilding).AddCreationOfTechnology(TechParam.Name, techName);
				}
				return;
			};

			techClassName = TechParam.GetType().FullName;

			CanExecute = (faction, _, holder) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();
				bool ret = true;
				/// コストが足りているか
				bool flag = !Enumerable.Range(0, TechParam.Costs.Length).Any(i => faction.GetResource(i) < TechParam.Costs[i]);
				if (ret && !flag)
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("コストが足りません", new Color(1, 0, 0))));
				ret = ret&&flag;
				/// 研究開始済か
				if(TechParam.DoesOnlyTarget)
					flag = !faction.IsSatisfyTechnologies(holder,new[] { techClassName }, false);
				else
					flag = !faction.IsSatisfyTechnologies(new[] { techClassName }, false);
				if (ret && !flag)
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に研究を開始しています", new Color(1, 0, 0))));
				ret = ret&&flag;
				/// 前提研究は研究済か
				flag = (TechParam.RequireTechnologies == null || faction.IsSatisfyTechnologies(TechParam.RequireTechnologies));
				if (ret && !flag)
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提研究を研究していません", new Color(1, 0, 0))));
				ret = ret&&flag;
				/// 前提建物は存在するか
				flag = (TechParam.RequireBuildings == null || faction.IsSatisfyBuildings(TechParam.RequireBuildings));
				if (ret && !flag)
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提建物が存在しません", new Color(1, 0, 0))));
				ret = ret&&flag;
				/// 前提ユニットは存在するか
				flag = (TechParam.RequireUnits == null || faction.IsSatisfyUnits(TechParam.RequireUnits));
				if (ret && !flag)
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提ユニットが存在しません", new Color(1, 0, 0))));
				ret = ret&&flag;

				return new ExecutableInfo(ret, cantExecuteMessage);
			};
		}
	}
}
