﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft
{
	static class FactionExtention
	{
		public static thrts.ColorTextLine[] GetRequireBuildingsInfoText (this thrts.IFaction faction, string[] requireBuildings)
		{
			if (requireBuildings != null && requireBuildings.Any ()) {
				List<ColorText> reqBuildingTxt = new List<ColorText> (){ new ColorText("　必要建物:")};
				reqBuildingTxt.AddRange (
					requireBuildings
					.Where (_ => faction.GetBuildingParameter (_) != null)
					.Select (bs => faction.IsSatisfyBuildings (new string[]{bs})
						 ? new ColorText ("　"+faction.GetBuildingParameter (bs).Name)
						 : new ColorText ("　"+faction.GetBuildingParameter (bs).Name, new Color (1, 0, 0)) 
						 )
					);
				return new ColorTextLine[]{new ColorTextLine(reqBuildingTxt)};
			}
			else
				return new ColorTextLine[0];
		}

		public static thrts.ColorTextLine[] GetRequireUnitsInfoText (this thrts.IFaction faction, string[] requireUnits)
		{
			if (requireUnits != null && requireUnits.Any ()) {
				List<ColorText> reqUnitTxt = new List<ColorText> (){new ColorText("　必要ユニット:")};
				reqUnitTxt.AddRange (
					requireUnits
					.Where (_ => faction.GetUnitParameter (_) != null)
					.Select (us => faction.IsSatisfyUnits (new string[]{us})
						 ? new ColorText ("　"+faction.GetUnitParameter (us).Name)
						 : new ColorText ("　"+faction.GetUnitParameter (us).Name, new Color (1, 0, 0)) 
						 )
					);
				return new ColorTextLine[]{new ColorTextLine(reqUnitTxt)};
			}
			else
				return new ColorTextLine[0];
		}
		public static thrts.ColorTextLine[] GetAttackBonusText (this thrts.IFaction faction, UnitParameter unitParam)
		{
			if (unitParam.Bullets.IsEmpty ())
				return new ColorTextLine[0];

			var index = 0;
			if (unitParam.SelectBullet != null ) {
				index = unitParam.SelectBullet(null,null,-1);
				if(index < 0 || index >= unitParam.Bullets.Length)
					return new ColorTextLine[0];
			}

			List<ColorText> bonusText = new List<ColorText>();
			var bulletParam = faction.GetBulletParameter(unitParam.Bullets[index]);
			if(null == bulletParam)
				return new ColorTextLine[0];

			int bonusIndex;
			var bonus = bulletParam.DamageBonus;

			if(null == bulletParam.DamageBonus)
				return new ColorTextLine[0];

			bonusIndex = (int)thrts.TargetObjectType.Clay;
			if(bonus.Length > bonusIndex && bonus[bonusIndex] != 1)
			{
				if(bonusText.Count> 0) bonusText.Add(new ColorText("　"));
				bonusText.Add(new ColorText("岩に"+(bonus[bonusIndex] - 1).ToString("+###.%;-###.%"),new Color(1,0,0)));
			}
			bonusIndex = (int)thrts.TargetObjectType.Water;
			if(bonus.Length > bonusIndex && bonus[bonusIndex] != 1)
			{
				if(bonusText.Count> 0) bonusText.Add(new ColorText("　"));
				bonusText.Add(new ColorText("水に"+(bonus[bonusIndex] - 1).ToString("+###.%;-###.%"),new Color (0, 191/255f,243/255f)));
			}
			bonusIndex = (int)thrts.TargetObjectType.Structure;
			if(bonus.Length > bonusIndex && bonus[bonusIndex] != 1)
			{
				if(bonusText.Count> 0) bonusText.Add(new ColorText("　"));
				bonusText.Add(new ColorText("鋼に"+(bonus[bonusIndex] - 1).ToString("+###.%;-###.%"),new Color(0.5f,0.5f,0.5f)));
			}
			bonusIndex = (int)thrts.TargetObjectType.Hero;
			if(bonus.Length > bonusIndex && bonus[bonusIndex] != 1)
			{
				if(bonusText.Count> 0) bonusText.Add(new ColorText("　"));
				bonusText.Add(new ColorText("主に"+(bonus[bonusIndex] - 1).ToString("+###.%;-###.%"),new Color(1,1,1)));
			}

			return new []{new ColorTextLine(bonusText)};
		}
		public static thrts.ColorTextLine GetCostInfoText(this thrts.IFaction faction, int[] cost,int populationWeight,int makeTime)
		{
			return new ColorTextLine
				(
					Enumerable.Range(0, cost.Length)
					.Select(i => faction.GetResource(i))
					.Zip(cost, new List<string>() { "　酒:", "　霊石:" }, Tuple.Create)
					.Where(t => t.Item2 != 0 || t.Item3 ==  "　酒:")
					.SelectMany(t => (
					t.Item1 < t.Item2
					? new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString(), new Color(1, 0, 0)) }
				: new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString()) }
				)
							).Concat(
					faction.Supply - faction.Population < populationWeight
					? new List<ColorText>() { 
					new ColorText("　人口:"), new ColorText(populationWeight.ToString() , new Color(1, 0, 0)),
					new ColorText("　時間:"), new ColorText(makeTime.ToString()),
				}
				: new List<ColorText>() { 
					new ColorText("　人口:"), new ColorText(populationWeight.ToString()), 
					new ColorText("　時間:"), new ColorText(makeTime.ToString()),
				}
				)
					);
		}
		public static thrts.ColorTextLine GetBuildingCostInfoText(this thrts.IFaction faction, int[] cost,int supply,int makeTime)
		{
			return new ColorTextLine
				(
					Enumerable.Range(0, cost.Length)
					.Select(i => faction.GetResource(i))
					.Zip(cost, new List<string>() { "　酒:", "　霊石:" }, Tuple.Create)
					.Where(t => t.Item2 != 0 || t.Item3 ==  "　酒:")
					.SelectMany(t => (
						t.Item1 < t.Item2
							? new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString(), new Color(1, 0, 0)) }
							: new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString()) })
					).Concat(
						supply > 0 
							? new List<ColorText>() { 
								new ColorText("　時間:" + makeTime.ToString()),
								new ColorText("　人口:+" + supply.ToString()), 
								}
							: new List<ColorText>(){
								new ColorText("　時間:" + makeTime.ToString()),
								}
						)
					);
		}
		public static thrts.ColorTextLine GetResourceInfoText(this thrts.IFaction faction, int[] cost)
		{
			return new ColorTextLine
					(
						Enumerable.Range(0, cost.Length)
						.Select(i => faction.GetResource(i))
						.Zip(cost, new List<string>() { "　酒:", "　霊石:" }, Tuple.Create)
					.Where(t => t.Item2 != 0 || t.Item3 ==  "　酒:")
						.SelectMany(t =>
						(t.Item1 < t.Item2
							? new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString(), new Color(1, 0, 0)) }
							: new List<ColorText>() { new ColorText(t.Item3), new ColorText(t.Item2.ToString()) })
						)
					);
		}
		public static thrts.ColorTextLine GetPopulationInfoText(this thrts.IFaction faction, int populationWeight)
		{
			return new ColorTextLine
				(
					faction.Supply - faction.Population < populationWeight
						? new List<ColorText>() { new ColorText("　人口:"), new ColorText(populationWeight.ToString() , new Color(1, 0, 0)) }
						: new List<ColorText>() { new ColorText("　人口:"), new ColorText(populationWeight.ToString()) }
				);
		}
	}
}
