﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft
{
	public class MissionUtls
	{
		/// <summary>
		/// 直線上にユニットを配置する
		/// </summary>
		/// <param name="centerX">中心(x)</param>
		/// <param name="centerY">中心(y)</param>
		/// <param name="d">ユニット間の距離</param>
		/// <param name="theta">隣のユニットの方向</param>
		/// <param name="n">ユニット数</param>
		/// <param name="world">world</param>
		/// <param name="unitName">ユニットクラス名</param>
		/// <param name="pID">プレイヤー番号</param>
		static public void putLine_Units(float centerX, float centerY, float d, float theta, int n, ref IWorld world, int index, string unitName)
		{
			var faction = world.Factions[index];
			float x = n / 2.0f * d * (float)Math.Sin((theta + 180.0 - 45.0) * Math.PI / 180.0);
			float y = n / 2.0f * d * (float)Math.Cos((theta + 180.0 - 45.0) * Math.PI / 180.0);

			for (int i = 0; i < n; i++)
			{
				x += d * (float)Math.Sin((theta - 45.0) * Math.PI / 180.0);
				y += d * (float)Math.Cos((theta - 45.0) * Math.PI / 180.0);
				world.UnitManager.Create(unitName, faction, centerX + x, centerY + y, faction);
			}

		}
		/// <summary>
		/// 円弧状にユニットを並べる
		/// </summary>
		/// <param name="centerX">中心(x)</param>
		/// <param name="centorY">中心(y)</param>
		/// <param name="distance">中心からの距離</param>
		/// <param name="originTheta">弧の開始角度[deg]</param>
		/// <param name="tickTheta">ユニット間の角度[deg]</param>
		/// <param name="n">ユニット数</param>
		/// <param name="world">ワールド</param>
		/// <param name="unitName">ユニットクラス名</param>
		/// <param name="pID">プレイヤー番号</param>
		static public void putArc_Units(float centerX, float centorY, float distance, float originTheta, float tickTheta, int n, ref IWorld world, int index, string unitName)
		{
			var faction = world.Factions[index];
			float x = 0;
			float y = 0;
			for (int i = 0; i < n; i++)
			{
				x = distance * (float)Math.Sin((originTheta + i * tickTheta - 45.0) * Math.PI / 180.0);
				y = distance * (float)Math.Cos((originTheta + i * tickTheta - 45.0) * Math.PI / 180.0);

				world.UnitManager.Create(unitName, faction, centerX + x, centorY + y, faction);
			}

		}
		/// <summary>
		/// 直線上にユニットを配置する
		/// </summary>
		/// <param name="centerX">中心(x)</param>
		/// <param name="centerY">中心(y)</param>
		/// <param name="d">ユニット間の距離</param>
		/// <param name="theta">隣のユニットの方向</param>
		/// <param name="m">ユニット数(thetaに直交する方向)</param>
		/// <param name="n">ユニット数(theta方向)</param>
		/// <param name="world">world</param>
		/// <param name="unitName">ユニットクラス名</param>
		/// <param name="pID">プレイヤー番号</param>
		static public void putBox_Units(float centerX, float centerY, float d, float theta, int m, int n, ref IWorld world, int index, string unitName)
		{
			float x = m / 2.0f * d * (float)Math.Sin((theta + 180.0 + 90.0 - 45.0) * Math.PI / 180.0);
			float y = m / 2.0f * d * (float)Math.Cos((theta + 180.0 + 90.0 - 45.0) * Math.PI / 180.0);
			for (int i = 0; i < m; i++)
			{
				x += d * (float)Math.Sin((theta + 90 - 45.0) * Math.PI / 180.0);
				y += d * (float)Math.Cos((theta + 90 - 45.0) * Math.PI / 180.0);
				putLine_Units(centerX + x, centerY + y, d, theta, n, ref world, index, unitName);
			}

		}
	}
}
