﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Unit.Black;

namespace LotusCraft.Spell.Black
{
	class Hihou : thrts.SpellParameter<thrts.IUnit>
	{

		int[] chargeTime;
		float[] affectRadiuses;
		float[] intervals;

		const int duration = 8 * Def.FPS;
		const int interval = 1;
		public Hihou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "秘宝「斑鳩寺の天球儀」";
			string basePath = "Black/Spell/Hihou/";


			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 25 * 30;
			CoolTimes[2] = 20 * 30;
			CoolTimes[3] = 15 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * 32;
			MaxRanges[2] = 7 * 32;
			MaxRanges[3] = 8 * 32;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 2 * Def.FPS;
			chargeTime[2] = 2 * Def.FPS;
			chargeTime[3] = 2 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;

			intervals = new float[MaxLevel + 1];
			intervals[0] = 9999;
			intervals[1] = 1;
			intervals[2] = 0.8f;
			intervals[3] = 0.6f;

			Description = "周囲に飛んでいる弾を撃ち落とす飛行オブジェクトを生成する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				intervals[1] + "秒に1回、範囲" + affectRadiuses[1] + "にある弾を一発撃ち落とす。",
				intervals[2] + "秒に1回、範囲" + affectRadiuses[2] + "にある弾を一発撃ち落とす。",
				intervals[3] + "秒に1回、範囲" + affectRadiuses[3] + "にある弾を一発撃ち落とす。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName, targetPos + new Vector3D(0,0,20), selfUnit.Faction);
				var scale = 1.5f;
				effect.SetScale(new Vector3D(scale, scale, scale));
				scale = 1.0f * (affectRadius);
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
					effect.Stop();

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				var u = Utl.PutUnitAroundPoint(selfUnit.Faction.World,selfUnit.Faction,typeof(Armillary).FullName, new Vector2D(targetPos.X,targetPos.Y));

				if(null != u)
				{
					/// create spell Effect;
					var objEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HihouEffect).FullName, u.Pos + offset + new Vector3D(0,0,-32-4));
					objEffect.SetScale(new Vector3D(scale*5, scale*5, scale*5));

					var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(CommonBolAreaEffect).FullName,targetPos + offset);
					areaEffect.SetScale(new Vector3D(scale, scale, scale));

					u.Vanished += (unit) => { 
						if(null != objEffect) 
						{
							objEffect.Stop(); 
							objEffect = null;
						}
						if(null != areaEffect) 
						{
							areaEffect.Stop(); 
							areaEffect = null;
						}
					};

					int counter = 0;
					int counter2 = 0;
					u.HoldPosition(false);

					u.Updating += () => 
					{
						if(counter<intervals[level]*Def.FPS)
						{
							counter++;
						}
						else if(++counter2 > 5)
						{
							// 弾の取得が重い可能性があるため、5フレームに1回に制限
							counter2 = 0;
							var bullets = u.Faction.World.GetBulletsFromCircle(new Circle2D(affectRadius*Def.MapTipLengthOnField,new Vector2D(u.Pos.X,u.Pos.Y)));
							foreach(var b in bullets)
							{
								// 飛んでいる敵の弾以外はスルー
								if(null == b || !b.Exists || b.Vector.LengthSquared < 1 || (null != b.Generator && u.Faction.World.IsSameTeam(b.Generator.Faction,u.Faction)))
									continue;
		
								// remove bullet
								b.Vanish();
								counter = 0;
								break;
							}
						}
					};
	
				}

			}
			yield return false;
		}

	}
}
