﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Hoshifuru : thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 16 * Def.FPS;
		const int chargeTime = 2 * Def.FPS;
		const int interval = 1*Def.FPS;

		const float affectSize = 8;

		const int damage = 80;
		const float damageRatio = 0.6f;

		const string totalAbsHpKey = "total_abs_hp_key";
		bool[] isTargeted;
		public Hoshifuru()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "「星降る神霊廟」";
			string basePath = "Black/Spell/Hoshifuru/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180 * Def.FPS;

			Description = "見えている敵兵の周囲に、ランダムで敵へダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				interval/Def.FPS +"秒毎に"+damage+"ダメージ、"+duration/Def.FPS+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();
			selfUnit.ShowCutin ();

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

			// create spell Effect;
			var areaEffect = world.EffectManager.CreateEffect (typeof(HoshifuruSelfEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 5.0f * (affectSize / 2);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (areaEffect, duration + chargeTime, this.Name + "areaEffect")))
				areaEffect.Stop ();
			

			selfFaction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);


			int count = (int)Math.Ceiling (world.Map.Count / affectSize);
			isTargeted = new bool[count * count];
			for (int i = 0; i < isTargeted.Length; ++i)
				isTargeted [i] = false;
			bool[] isSelected = new bool[count * count];
			for (int i = 0; i < isSelected.Length; ++i)
				isSelected [i] = false;

			var rect = new RectF (0, 0, affectSize * Def.MapTipLengthOnField, affectSize * Def.MapTipLengthOnField);

			// check targets
			int m = (int)Math.Ceiling (count * count / (float)chargeTime);
			int num = 0;
			int seenNum = 0;

			// charge
			for (int i = 0; i < chargeTime; ++i) {
				for (int j = 0; j < m; ++j) {
					int index = i * m + j;
					if (index >= count * count)
						break;

					isTargeted [index] = world.GetObjectFromRectangle (rect, rect).OfType<IUnit> ()
						.Any (_ =>
							  null != _
						&& !_.Parameter.IsFarmer
						&& !world.IsSameTeam (_.Faction, selfFaction)
						&& (_.Pos.X >= 0.0f && _.Pos.X < world.Map.Count * Def.MapTipLengthOnField)
						&& (_.Pos.Y >= 0.0f && _.Pos.Y < world.Map.Count * Def.MapTipLengthOnField)
						&& world.Map.BeSeen (selfFaction, (int)(_.Pos.X / Def.MapTipLengthOnField), (int)(_.Pos.Y / Def.MapTipLengthOnField)));

					var ix = (int)(rect.X / Def.MapTipLengthOnField + affectSize / 2);
					var iy = (int)(rect.Y / Def.MapTipLengthOnField + affectSize / 2);

					if (isTargeted [index] || (0 <= ix && ix < world.Map.Count && 0 <= iy && iy < world.Map.Count && world.Map.BeSeen (selfFaction, ix, iy))) {
						isSelected [index] = true;
						seenNum ++;
					}

					if (isTargeted [index]) {
						var targetEffect = world.EffectManager.CreateEffect (typeof(HoshifuruAreaEffect).FullName, world.GetGroundAtPos (new Vector2D (rect.X + rect.Width / 2, rect.Y + rect.Height / 2)));
						var targetEffectScale = 10 * affectSize / 2;
						targetEffect.SetScale (new Vector3D (targetEffectScale, targetEffectScale, targetEffectScale));

						if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (targetEffect, duration + (chargeTime - i), this.Name + "targetEffect" + index)))
							targetEffect.Stop ();

						num++;
					}
					if (index % count == count - 1) {
						rect.X = 0;
						rect.Y += affectSize * Def.MapTipLengthOnField;
					} else {
						rect.X += affectSize * Def.MapTipLengthOnField;
					}
				}
				yield return true;
			}

			int x = 0;
			int y = 0;

			// 落下候補地点が足りなければ補充
			while (num < Math.Min (seenNum,32)) {
				x = world.NextRand(count);
				//  空廻し
				for(int i=0;i<100;++i)
					world.NextRand ();
				y = world.NextRand(count);
				//  空廻し
				for(int i=0;i<100;++i)
					world.NextRand ();
				var index = y * count + x;
				if(!isTargeted[index] && isSelected[index])
				{
					isTargeted[index] = true;
					rect.X = x*affectSize*Def.MapTipLengthOnField;
					rect.Y = y *affectSize*Def.MapTipLengthOnField;

					var targetEffect = world.EffectManager.CreateEffect (typeof(HoshifuruAreaEffect).FullName, world.GetGroundAtPos (new Vector2D (rect.X + rect.Width / 2, rect.Y + rect.Height / 2)));
					var targetEffectScale = 10 * affectSize / 2;
					targetEffect.SetScale (new Vector3D (targetEffectScale, targetEffectScale, targetEffectScale));
					
					if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (targetEffect, duration , this.Name + "targetEffect" + index)))
						targetEffect.Stop ();

					num++;
				}
			}

			bool ret = isTargeted.Any(_ => _);

			// 隕石を降らす
			for (int i = 0; i < duration; ++i)
			{
				if (i % interval == 0 && ret)
				{

					for(int j=0;j<isSelected.Length;++j)
					{
						isSelected[j] = false;
					}
					for (int j = 0; j <= Math.Min (num,4); ++j)
					{
						// 対象をランダムに選択
						do
						{
							x = world.NextRand(count);
							//  空廻し
							for(int k=0;k<100;++k)
								world.NextRand ();
							y = world.NextRand(count);
							//  空廻し
							for(int k=0;k<100;++k)
								world.NextRand ();
						} while(!isTargeted[y * count + x] && !isSelected[y * count + x]);

						isSelected[y * count + x] = true;

						rect.X = x * affectSize * Def.MapTipLengthOnField;
						rect.Y = y * affectSize * Def.MapTipLengthOnField;

						// area effect
						var effect = world.EffectManager.CreateEffect(
							typeof(HoshifuruHitEffect).FullName,
							new Vector3D(rect.X + rect.Width / 2, rect.Y + rect.Height / 2, world.Map.GetHeightOnPoint(rect.X + rect.Width / 2, rect.Y + rect.Height / 2) + 10));
						var areaEffectScale = affectSize * Def.MapTipLengthOnField / 12.0f;
						effect.SetScale(new Vector3D(areaEffectScale, areaEffectScale, areaEffectScale));


						var targets = world.GetObjectFromRectangle(rect, rect);
						foreach (var t in targets)
						{
							if(world.IsSameTeam(t.Faction,selfFaction))
								continue;

							var ix = (int)(t.Pos.X / Def.MapTipLengthOnField);
							var iy = (int)(t.Pos.Y / Def.MapTipLengthOnField);

							if (!(0 <= ix && ix < world.Map.Count)) continue;
							if (!(0 <= iy && iy < world.Map.Count)) continue;

							if (null == t || !t.Exists() || t.UniqueID == selfUnit.UniqueID || !world.Map.BeSeen(selfFaction, ix, iy))
								continue;

							var u = t as IUnit;
							var b = t as IBuilding;

							if (null != u || null != b)
							{
								var hiteffect = world.EffectManager.CreateEffect(typeof(ShinkouHitEffect).FullName, t.Pos + new Vector3D(1.0f, 1.0f, 2.0f));
								hiteffect.SetScale(new Vector3D(8, 8, 8));
								world.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, t.Pos);
							}

							if (null != u)
								u.BeDamaged(selfUnit, damage, AttackBonusSet.None, true);
							if (null != b)
								b.BeDamaged(selfUnit, damage, AttackBonusSet.None, true);
						}
					}
				}
				yield return true;
			}

			yield return false;
		}
	}
}
