﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Black;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Spell.Black
{
	class Senfu:thrts.SpellParameter<thrts.IUnit>
	{
		public static string SpellName = "仙符「日出ずる処の道士」";
		const float affectRadius = 1;
		float[] ViewRange;
		float[] incSpellRange;

		const int interval = 10;
		const int duration = 10*Def.FPS;

		public Senfu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Black/Spell/Senfu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 999;
			CoolTimes[1] = 20*Def.FPS;
			CoolTimes[2] = 20*Def.FPS;
			CoolTimes[3] = 20*Def.FPS;

			ViewRange = new float[MaxLevel + 1];
			ViewRange[0] = 0;
			ViewRange[1] = 20*Def.MapTipLengthOnField;
			ViewRange[2] = 30*Def.MapTipLengthOnField;
			ViewRange[3] = 40*Def.MapTipLengthOnField;


			Description =  duration/Def.FPS +"秒間、周囲の視界を得る。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲+"+ViewRange[1]/Def.MapTipLengthOnField,
				"範囲+"+ViewRange[2]/Def.MapTipLengthOnField,
				"範囲+"+ViewRange[3]/Def.MapTipLengthOnField,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();
			
			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D(2,2,2);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(SenfuEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius * 12;
				if (level == 2) { scale *= 1.5f; }
				else if (level == 3) { scale *= 2; }
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);

				for(int range=(int)selfUnit.ViewRange;range<ViewRange[level];range+=(int)Def.MapTipLengthOnField)
				{
					selfUnit.Faction.GetSite(range,selfUnit.Pos.X,selfUnit.Pos.Y,1);
					yield return true;
				}

				if (! selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),effect,offset,duration,this.Name+"areaEffect",interval)))
					effect.Stop();
			}
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			selfUnit.Faction.GetSite(ViewRange[level],selfUnit.Pos.X,selfUnit.Pos.Y,interval*2);
			return true;
		}

		public float GetIncSpellRange (IUnit unit)
		{
			if(null == unit || !unit.Exists())
				return 0;

			var spell = unit.GetSpellWithName(this.Name);
			if(null == spell)
				return 0;

			return incSpellRange[spell.Level];
		}
	}
}