﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Spell.Black
{
	class Shinkou : thrts.SpellParameter<thrts.IUnit>
	{

		int[] chargeTime;
		float[] affectRadiuses;
		float[] decAttackSpeedRatio;
		int[] damages;

		const int duration = 6 * Def.FPS;
		const int interval = Def.FPS;
		public Shinkou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "神光「逆らう事なきを宗とせよ」";
			string basePath = "Black/Spell/Shinkou/";


			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 25 * 30;
			CoolTimes[2] = 25 * 30;
			CoolTimes[3] = 25 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 10 * Def.MapTipLengthOnField;
			MaxRanges[2] = 15 * Def.MapTipLengthOnField;
			MaxRanges[3] = 20 * Def.MapTipLengthOnField;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 1 * Def.FPS;
			chargeTime[2] = 1 * Def.FPS;
			chargeTime[3] = 1 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 4;

			decAttackSpeedRatio = new float[MaxLevel + 1];
			decAttackSpeedRatio[0] = 0;
			decAttackSpeedRatio[1] = 0.1f;
			decAttackSpeedRatio[2] = 0.2f;
			decAttackSpeedRatio[3] = 0.3f;

			damages = new int[MaxLevel+1];
			damages[0] = 0;
			damages[1] = 14;
			damages[2] = 22;
			damages[3] = 30;


			Description = "指定地点の周囲の、\n見えている敵ユニットにダメージを与え攻撃速度を減少させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"、毎秒" + damages[1] + "ダメージ、攻撃速度-" + decAttackSpeedRatio[1].ToString("###.%"),
				"範囲"+ affectRadiuses[2]+"、毎秒" + damages[2] + "ダメージ、攻撃速度-" + decAttackSpeedRatio[2].ToString("###.%"),
				"範囲"+ affectRadiuses[3]+"、毎秒" + damages[3] + "ダメージ、攻撃速度-" + decAttackSpeedRatio[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			// その位置に視界が無ければ戻る
			if(!selfUnit.Faction.World.Map.BeSeen(selfUnit.Faction,(int)(targetPosition.X/Def.MapTipLengthOnField),(int)(targetPosition.Y/Def.MapTipLengthOnField)))
			{
				yield return false;
				yield break;
			}

			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ShinkouChargeEffect).FullName, targetPos + offset, selfUnit.Faction);
				var scale = 8.0f * (affectRadius);
				effect.SetScale(new Vector3D(scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
					effect.Stop();

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, targetPos);

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				var offset2 = new Vector3D(0.0f, 1.0f, 250.0f);
				var scale2 = 5.0f * (affectRadius);

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ShinkouAreaEffect).FullName, targetPos + offset2, selfUnit.Faction);
				areaEffect.SetScale(new Vector3D(scale2, scale2, scale2));

				var ability = new SpellContinuousAbility((unit) => Spelling(unit, targetPosition, level), areaEffect, offset, duration, this.Name + "areaEffect", interval);
				ability.FollowEffect = false;
				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, targetPos);

			}
			yield return false;
		}
		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			;
		}

		public bool Spelling(IUnit selfUnit, Vector2D targetPosition, int level)
		{
			float affectRadius = affectRadiuses[level];

			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			// effect offset
			Vector3D offset = new Vector3D(0.0f, 1.0f, 1.0f);

			var selfFaction = selfUnit.Faction;

			// search targets
			var targets = selfFaction.World.GetObjectFromCircle(
				new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(targetPos.X, targetPos.Y))).
				OfType<IUnit>().
				Where(_ => _ != null && _.Exists() && !_.Parameter.IsFarmer && !selfFaction.World.IsSameTeam(_.Faction,selfFaction));

			IEffect effect = null;
			foreach (var u in targets)
			{
				var tip = u.Pos/Def.MapTipLengthOnField;
				if(!u.Faction.World.Map.BeSeen(selfFaction,(int)tip.X,(int)tip.Y))
					continue;

				var ability = u.AbilityManager.GetAbilityWithName(ShinkouAbility.AbilityName) as ShinkouAbility;
				if (null == ability)
				{
					if (u != selfUnit)
					{
						if (effect == null)
						{
							effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(DownDPSEffect).FullName, u.Pos + offset);
							effect.SetScale(new Vector3D(12, 12, 12));
						}
					}
					if(!u.AbilityManager.AppendAbility(new ShinkouAbility(interval + 2, decAttackSpeedRatio[level], effect)) && null != effect)
					{
						effect.Stop();
						effect = null;
					}
				}
				else
				{
					ability.DecAttackSpeedRatio = decAttackSpeedRatio[level];
					ability.ExtendLifetime(interval,interval*2);
				}

				if (null != u)
				{
					var faction = selfUnit.Faction;
					var hiteffect = faction.World.EffectManager.CreateEffect(typeof(ShinkouHitEffect).FullName, u.Pos + new Vector3D(1.0f, 1.0f, 2.0f));
					hiteffect.SetScale(new Vector3D(8, 8, 8));
					faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, u.Pos);

					u.BeDamaged(selfUnit,damages[level],AttackBonusSet.None,true);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class ShinkouAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		public float DecAttackSpeedRatio;
		public const string AbilityName = "神光「逆らう事なきを宗とせよ」";
		public ShinkouAbility (int duration,float decMoveRatio, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.DecAttackSpeedRatio = decMoveRatio;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.AttackDelay += (int)(b.AttackDelay*DecAttackSpeedRatio);
					b.AttackRigor += (int)(b.AttackRigor*DecAttackSpeedRatio);
					b.AttackSpeed += (int)(b.AttackSpeed*DecAttackSpeedRatio);
				}
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}



