﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Building.Black;

namespace LotusCraft.Spell.Black
{
	class OkuSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const float affectRadius = 2f;
		const int damage = 12;
		public OkuSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "制御「セルフトカマク」";
			string basePath = "Black/Spell/OkuSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 30*Def.FPS;
			CoolTimes[1] = 30*Def.FPS;

			MaxRanges = new float[2];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6*Def.MapTipLengthOnField;

			Description = "周囲の敵ユニットに一定間隔でダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"4秒毎にダメージ"+(damage)+ "",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D(2,2,1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CommonBolAreaEffect).FullName, selfUnit.Pos);
				var scale = affectRadius;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(effect,offset,1*Def.FPS,this.Name+"areaEffect")))
					effect.Stop();
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName (15,this.Name+"areaEffect",1*Def.FPS);
			}
			
			var selfFaction = selfUnit.Faction;
			
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>()) {
				if(u.UniqueID != selfUnit.UniqueID && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && !u.AbilityManager.ContainsAbility(OkuSkillAbility.AbilityName))
				{
					if(u.AbilityManager.AppendAbility(new OkuSkillAbility()))
					{
						u.BeDamaged(selfUnit,damage,AttackBonusSet.None);
						var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoHitEffect).FullName, u.Pos);
						var scale = new Vector3D(5, 5, 5);
						effect.SetScale(scale);
						selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitShortRangeAttack).FullName, u.Pos);
					}
					else
					{
						;
					}
				}
			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Black
{
	class OkuSkillAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "制御「セルフトカマク」";
		public OkuSkillAbility()
		{
			this.Name = AbilityName;
			this.DurationTime = 4*Def.FPS;
		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return true;
		}
	}
	
}