using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Byoufu:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;

		int[] useToken;
		int[] damages;
		public Byoufu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "猫符「怨霊猫乱歩」";
			string basePath = "Black/Spell/Byoufu/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 8*30;
			CoolTimes[2] = 8*30;
			CoolTimes[3] = 8*30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6*32;
			MaxRanges[2] = 9*32;
			MaxRanges[3] = 12*32;

			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = 1*Def.FPS;
			chargeTime[2] = 1*Def.FPS;
			chargeTime[3] = 1*Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 2.5f;
			affectRadiuses[3] = 3;

			useToken = new int[MaxLevel + 1];
			useToken[0] = 99;
			useToken[1] = 2;
			useToken[2] = 2;
			useToken[3] = 1;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 70;
			damages[2] = 140;
			damages[3] = 210;

			Description = "指定地点へ瞬時に移動し、近くの敵ユニット1体にダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken[1]+"消費"+"、最大距離" + (MaxRanges[1]/Def.MapTipLengthOnField) + "。"+damages[1]+"ダメージ",
				"トークン"+useToken[2]+"消費"+"、最大距離" + (MaxRanges[2]/Def.MapTipLengthOnField) + "。"+damages[2]+"ダメージ",
				"トークン"+useToken[3]+"消費"+"、最大距離" + (MaxRanges[3]/Def.MapTipLengthOnField) + "。"+damages[3]+"ダメージ",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if (null == spell || !spell.CanExec) {
					yield return false;
					yield break;
				}
				var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;
				
				if (token.Count < useToken[level]) {
					yield return false;
					yield break;
				}
				
				token.DecToken( useToken[level] );

				spell.ResetCoolTime ();

				float affectRadius = affectRadiuses [level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(ByoufuChargeEffect).FullName, targetPos + offset);
				var scale = 6.0f * (affectRadius);
				effect.SetScale (new Vector3D (scale, scale, scale));

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Fire).FullName, selfUnit.Pos);
				
				if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (effect, chargeTime [level], this.Name + "chargeEffect")))
					effect.StopRoot();
				
				for (int i=0; i<chargeTime[level]; ++i) {
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;
 
				var selfFaction = selfUnit.Faction;
				var world = selfFaction.World;
				selfUnit.Warp (targetPosition,true);

				/// area effect
				var areaeffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ByoufuAreaEffect).FullName, selfUnit.Pos + offset);
				areaeffect.SetScale(new Vector3D(scale, scale, scale));
				areaeffect.SetPosition(selfUnit.Pos + offset);

				var targets = world.GetObjectFromCircle(new Circle2D(affectRadius*Def.MapTipLengthOnField,targetPosition)).OfType<IUnit>();
				foreach(var u in targets)
				{
					if(null == u || !u.Exists() || world.IsSameTeam(u.Faction,selfFaction))
						continue;

					u.BeDamaged(selfUnit,damages[level],AttackBonusSet.Hero);
					selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitRedTextureEffect).FullName, u.Pos);
					selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, u.Pos);
					break;
				}


			}
			yield return false;
		}
		public void OnFinishPrivilegeMode (IUnit selfUnit,int level)
		{
			;
		}
	}
}