using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.AutoCast;
using LotusCraft.Effect.Black;
using LotusCraft.Token;
using LotusCraft.Unit.Black;

namespace LotusCraft.Spell.Black
{
	class Jusei : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;
		
		int[] maxSummon;
		
		const int useToken = 2;
		
		public Jusei()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "呪精「ゾンビフェアリー」";
			string basePath = "Black/Spell/Jusei/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 8 * 30;
			CoolTimes[2] = 8 * 30;
			CoolTimes[3] = 8 * 30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;
			
			maxSummon = new int[MaxLevel + 1];
			maxSummon[0] = 0;
			maxSummon[1] = 3;
			maxSummon[2] = 5;
			maxSummon[3] = 7;

			Description = "ゾンビフェアリーを召喚する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken+"消費、ゾンビフェアリーを召喚、最大"+maxSummon[1]+"体",
				"トークン"+useToken+"消費、ゾンビフェアリーを召喚、最大"+maxSummon[2]+"体",
				"トークン"+useToken+"消費、ゾンビフェアリーを召喚、最大"+maxSummon[3]+"体",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			if (!selfUnit.UserValues.ContainsKey (CorpseToken.TokenName)) {
				yield return false;
				yield break;
			}
			
			var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;
			
			if (token.Count < useToken) {
				yield return false;
				yield break;
			}

			// check fairy count
			var fairy = selfUnit.Faction.World.UnitManager.GetUnits(_=> _.Faction == selfUnit.Faction && _.Parameter.GetType() == typeof(ZombieFairy));
			if(fairy.Count() >= maxSummon[level])
			{
				yield return false;
				yield break;
			}


			token.DecToken( useToken);
			
			// クールタイム再設定
			spell.ResetCoolTime();
			
			{
				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);
				
				var unit = Utl.PutUnitAroundPoint( selfUnit.Faction.World,selfUnit.Faction, typeof(ZombieFairy).FullName, new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y));
				if(null != unit)
				{
					var size = 5;
					var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(SummonEffect).FullName, unit.Pos + offset);
					effect.SetScale(new Vector3D(size, size, size));
				}

			}
			yield return false;
		}
		
		public int DisableSelectBullet(IUnit unit,ITargetObject target,float distance)
		{
			return -1;
		}
	}
}