﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Youkai : thrts.SpellParameter<thrts.IUnit>
	{
		float affectRadius = 3;

		const int duration = 8*Def.FPS;
		const int interval = Def.FPS/2;
		const int useToken = 2;
		const int extendDuration = 4*Def.FPS;
		public Youkai()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "妖怪「火焔の車輪」";
			string basePath = "Black/Spell/Youkai/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*Def.FPS;


			Description = "周囲のユニットにダメージを与える床を設置する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"効果"+duration/Def.FPS+"秒間、トークン"+useToken+"で"+extendDuration/Def.FPS+"秒延長。\n床は0.5秒毎に10ダメージ(3回,防御力無視)",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			selfUnit.ShowCutin();
			spell.ResetCoolTime ();


			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 2.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(YoukaiEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 12.0f*(affectRadius/4);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;

			// put area bullet
			var bullet = selfUnit.CreateBullet (typeof(YoukaiBullet).FullName, selfUnit.Pos, selfUnit.Pos);

			if (null != bullet) {
				var effect = selfFaction.World.EffectManager.CreateEffect(typeof(YoukaiAreaEffect).FullName, selfUnit.Pos + offset);
				var effectScale = 8 * 2 *affectRadius;
				effect.SetScale (new Vector3D (effectScale, effectScale, effectScale)); 
				effect.SetRotation(new Vector3D( 0, 0, 45 ));
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Fire).FullName, selfUnit.Pos);

				bullet.Vanished += (IBullet obj) => 
				{
					if(effect != null)
					{
						effect.Stop();
						effect = null;
					}
				};
			}

			var ability = selfUnit.AbilityManager.GetAbilityWithName (this.Name + "areaEffect");
			if (null != ability && ability.DurationTime <= 1 * Def.FPS && selfUnit.UserValues.ContainsKey(CorpseToken.TokenName)) {
				// トークンを消費して時間を延長
				var token = selfUnit.UserValues[CorpseToken.TokenName] as CorpseToken;
				if(null != token && token.Count >= useToken)
				{
					token.DecToken(useToken);
					ability.ExtendLifetime(extendDuration);
				}
			}
			return true;
		}
	}
}


namespace LotusCraft.Bullet.Black
{
	public class YoukaiBullet : General.CannonBullet
	{
		public YoukaiBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			HitType = BulletHitType.Multi;
			CollisionType = BulletCollisionType.Square;
			LengthX = 6*Def.MapTipLengthOnField;
			LengthY = 6*Def.MapTipLengthOnField;

			Speed = 0;
			Gravity = 0;
			MaxRange = Def.MapTipLengthOnField;;

			Duration = (int)(1.5f*Def.FPS);
			Again = Def.FPS/2;

			this.Attack = 10;
			IgnoreArmor = true;

			AttackDelay = (int)(0.5f * 30);
			AttackRigor = (int)(0.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor = 0;
			
			OnHit = OnHitEffect;
			
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalYellow);
		}
	}
}
