using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.AutoCast;
using LotusCraft.Token;

namespace LotusCraft.Spell.Black
{
	class Taofu : thrts.SpellParameter<thrts.IUnit>
	{
		const int useToken = 5;

		const int duration = 12*Def.FPS;

		public const string SeigaSummonUnitsKey = "seiga_summon_units";

		public Taofu()
		{
			DefaultLevel = 0;
			RequireLevels[0] = 6;
			MaxLevel = 1;
			Name = "道符「タオ胎動」";
			string basePath = "Black/Spell/Taofu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 12 * Def.FPS;

			Description = "青娥と青娥が召喚したユニットの攻撃力と防御力を上昇させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"トークン"+useToken+"消費、攻撃力+"+TaofuAbility.incAtk+"、防御力+"+TaofuAbility.incArmor,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (!selfUnit.UserValues.ContainsKey (CorpseToken.TokenName)) {
				yield return false;
				yield break;
			}
			
			var token = selfUnit.UserValues [CorpseToken.TokenName] as CorpseToken;
			
			if (token.Count < useToken) {
				yield return false;
				yield break;
			}
			
			token.DecToken (useToken);

			// クールタイム再設定
			spell.ResetCoolTime ();

			// カットイン
			selfUnit.ShowCutin ();
			var selfFaction = selfUnit.Faction;
			{
				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 10.0f);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(TaoSelfEffect).FullName, new Vector3D (selfUnit.Pos.X, selfUnit.Pos.Y, selfUnit.Faction.World.Map.GetHeightOnPoint (selfUnit.Pos.X, selfUnit.Pos.Y)) + offset);
				var scale = 12.0f;
				effect.SetScale (new Vector3D (scale, scale, scale));

				// append ability
				{
					var ability = selfUnit.AbilityManager.GetAbilityWithName (TaofuAbility.AbilityName) as TaofuAbility;
					if (null != ability) {
						ability.DurationTime = duration;
					} else {
						selfUnit.AbilityManager.AppendAbility (new TaofuAbility (duration, effect));
					}
				}
			}
			{
				if(!selfUnit.UserValues.ContainsKey(Taofu.SeigaSummonUnitsKey))
					selfUnit.UserValues.Add(Taofu.SeigaSummonUnitsKey,new List<IUnit>());
				
				List<IUnit> summonUnits = selfUnit.UserValues[Taofu.SeigaSummonUnitsKey] as List<IUnit>;
				foreach(var u in summonUnits)
				{
					if(null != u && u.Exists())
					{
						// effect offset
						Vector3D offset = new Vector3D (0.0f, 0.0f, 10.0f);
						var effect = selfFaction.World.EffectManager.CreateEffect (typeof(TaoSelfEffect).FullName, new Vector3D (u.Pos.X, u.Pos.Y, u.Faction.World.Map.GetHeightOnPoint (u.Pos.X, u.Pos.Y)) + offset);
						var scale = 6.0f;
						effect.SetScale (new Vector3D (scale, scale, scale));
						
						// append ability
						{
							var ability = u.AbilityManager.GetAbilityWithName (TaofuAbility.AbilityName) as TaofuAbility;
							if (null != ability) {
								ability.DurationTime = duration;
							} else {
								u.AbilityManager.AppendAbility (new TaofuAbility (duration, effect));
							}
						}
					}
				}
			}
			yield return false;
		}

		public int DisableSelectBullet(IUnit unit,ITargetObject target,float distance)
		{
			return -1;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class TaofuAbility : FollowEffectAbility
	{
		public const string AbilityName = "道符「タオ胎動」";
		public const int incAtk = 15;
		public const int incArmor = 15;
		public IEffect effect = null;
		public TaofuAbility(int duration,IEffect effect=null)
			:base(effect,new Vector3D(),duration,AbilityName)
		{
			this.effect = effect;
		}
		public TaofuAbility(int duration,Vector3D offset,IEffect effect)
			:base(effect,offset,duration,AbilityName)
		{
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.ConstIncParam:
				unit.Armor += incArmor;
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += incAtk;
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(2, 2, 10));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}