﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Shako : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 3;
		const int interval = 4*Def.FPS;
		int[] incArmors;
		int[] damages;

		public Shako()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "遮光「核熱バイザー」";
			string basePath = "Black/Spell/Shako/";

			incArmors = new int[MaxLevel + 1];
			incArmors[0] = 0;
			incArmors[1] = 2;
			incArmors[2] = 4;
			incArmors[3] = 7;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 3;
			damages[2] = 5;
			damages[3] = 8;

//			Description = "防御力が上昇し、前方に存在するユニットに対し\n敵味方関係無くダメージを与える。";
			Description = "防御力が上昇し、前方に存在する敵ユニットに対しダメージを与える。";

			// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"防御力+" + incArmors[1] + "、"+interval/Def.FPS+"秒毎に前方へ"+(damages[1])+ "ダメージ",
				"防御力+" + incArmors[2] + "、"+interval/Def.FPS+"秒毎に前方へ"+(damages[2])+ "ダメージ",
				"防御力+" + incArmors[3] + "、"+interval/Def.FPS+"秒毎に前方へ"+(damages[3])+ "ダメージ"+"移動速度+0.1",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			var selfFaction = selfUnit.Faction;
			selfUnit.Armor += incArmors[level];
			if (level > 2)
				selfUnit.Speed += 0.1f * UnitParameter.BasisSpeed;

			// 範囲エフェクトの表示
			string areaEffectName = this.Name + "areaEffect";
			if (selfUnit.AbilityManager.ContainsAbility(areaEffectName))
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName(15, areaEffectName,1*Def.FPS);
			}
			else
			{
				var offset = new Vector3D(0, 0, 0);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(UtsuhoKakunetuAreaEffect).FullName, selfUnit.Pos);
				var scale = affectRadius * 6;
				effect.SetScale(new Vector3D(scale, scale, scale));
				effect.SetPosition(selfUnit.Pos + offset);

				var ability = new FollowEffectAbility(effect, offset, 1 * Def.FPS, areaEffectName);
				ability.IsDirectionChanged = true;
				ability.FrontOffset = 32.0f;

				if (!selfUnit.AbilityManager.AppendAbility(ability))
				{
					effect.Stop();
				}
			}

			// 攻撃アビリティ設定
			if (!selfUnit.AbilityManager.ContainsAbility(ShakoSkillAbility.AbilityName))
			{
				// search targets
				var front = selfUnit.GetFrontDirection();

				var targets = selfFaction.World.GetObjectOnRectangle(
					selfUnit.Pos.X + front.X * 32.0f,
					selfUnit.Pos.Y + front.Y * 32.0f,
					200,
					32,
					selfUnit.GetFrontAngle(),
					true,
					false);

				/*
				selfUnit.Faction.World.DrawDebugRectangle(new Vector3D(
					selfUnit.Pos.X + front.X * 32.0f,
					selfUnit.Pos.Y + front.Y * 32.0f,
					selfUnit.Pos.Z + 10.0f),
					200,
					32,
					selfUnit.GetFrontAngle(),
					new Color(1.0f, 1.0f, 1.0f, 1.0f));
				*/

				foreach (var u in targets.OfType<IUnit>())
				{
					if (null != u && u.Exists() && !selfFaction.World.IsSameTeam (u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID)
					{
						{
							var faction = selfUnit.Faction;
							var effect = faction.World.EffectManager.CreateEffect(typeof(UtsuhoKakunetuHitEffect).FullName, u.Pos);
							effect.SetScale(new Vector3D(4, 4, 4));
							faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitShortRangeAttack).FullName, u.Pos);
						}
						u.BeDamaged(selfUnit, damages[level], AttackBonusSet.None);
					}
				}

				selfUnit.AbilityManager.AppendAbility(new ShakoSkillAbility(interval));

			}

			return true;
		}

	}
}

namespace LotusCraft.Ability.Black
{
	class ShakoSkillAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "遮光「核熱バイザー」";
		public ShakoSkillAbility(int duration)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					break;
				default:
					break;
			}

			return true;
		}
	}
}
#endif
