﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;
using LotusCraft.Spell.Black;

namespace LotusCraft.Spell.Black
{
	class Shouki : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;
		
		const int duration = 20*Def.FPS;
		const int interval = 10;

		public const float DecMoveRatio = 0.1f;
		public const float DecAttackRatio = 0.2f;
		
		public Shouki()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "瘴気「原因不明の熱病」";
			string basePath = "Black/Spell/Shouki/";
			
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*Def.FPS;
			
			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 8;
			
			
			Description = "周囲の敵ユニットは毒状態になり、移動速度と攻撃力が大幅に低下する。\nこの効果は感染する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"12秒持続する毒BSを付与、移動速度-"+DecMoveRatio.ToString("###.%")+"、攻撃力-"+DecAttackRatio.ToString("###.%")+"、"+duration/Def.FPS+"秒間",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			selfUnit.ShowCutin();

			float affectRadius = affectRadiuses[level];
			
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = world.EffectManager.CreateEffect (typeof(NetubyouEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 8.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect",interval)))
				areaEffect.Stop();

			for (int i = 0; i < (1 * Def.FPS)/2; ++i)
			{
				yield return true;
			}

			selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Poison).FullName, selfUnit.Pos);
			
			yield return false;
		}

		bool Spelling (IUnit selfUnit,int level)
		{
			float affectRadius = affectRadiuses[level];
			
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			/// search targets
			var targets = world.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>();
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID ) {
					
					if(u.AbilityManager.ContainsAbility(ShoukiAbility.AbilityName))
					{
						u.AbilityManager.ExtendLifetimeWithName(interval,ShoukiAbility.AbilityName,12*Def.FPS);
					}
					else
					{
						/// エフェクトを描画
						var faction = u.Faction;
						Vector3D offset2 = new Vector3D(2.0f, 2.0f, 1.0f);
						// 50dmg/1s => 50dps

						var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(YamameNetubyouTextureEffect).FullName, u.Pos + offset2);
						effect.SetPosition(u.Pos + offset2);
						
						u.AbilityManager.AppendAbility(new ShoukiAbility(12*Def.FPS,effect));
					}

					if(u.AbilityManager.ContainsAbility(PoisonAbility.AbilityName))
					{
						u.AbilityManager.ExtendLifetimeWithName(interval,PoisonAbility.AbilityName,12*Def.FPS);
					}
					else
					{
						IEffect effect = null;
						/// エフェクトを描画
						var faction = u.Faction;
						Vector3D offset2 = new Vector3D(2.0f, 2.0f, 1.0f);
						// 50dmg/1s => 50dps
						
						var effect_ = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(YamameNetubyouTextureEffect).FullName, u.Pos + offset2);
						effect_.SetPosition(u.Pos + offset2);
						u.AbilityManager.AppendAbility(new PoisonAbility(12*Def.FPS,effect_));
					}
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class ShoukiAbility : thrts.AbilityTemplate
	{
		IEffectCommon effect;
		int counter;
		const int interval = 1*Def.FPS;
		const float infectionRadius = 2;
		public const string AbilityName = "瘴気「原因不明の熱病」";
		public ShoukiAbility (int duration = 12*Def.FPS,IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= unit.Speed*Shouki.DecMoveRatio;
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack -= (int)Math.Ceiling(b.Attack*Shouki.DecAttackRatio);

				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				// infection
				if(counter++ > interval)
				{
					counter = 0;
					var world = unit.Faction.World;
					/// search targets
					var targets = world.GetObjectFromCircle (new Circle2D (infectionRadius * Def.MapTipLengthOnField, new Vector2D (unit.Pos.X, unit.Pos.Y))).OfType<IUnit>();
					foreach (var u in targets) {
						if ( null != u && u.Exists() && !u.Parameter.IsUnique && world.IsSameTeam(u.Faction,unit.Faction) ) {
							
							if(!u.AbilityManager.ContainsAbility(ShoukiAbility.AbilityName))
							{
								IEffect shoukiEffect = null;
								u.AbilityManager.AppendAbility(new ShoukiAbility(DurationTime,shoukiEffect));
							}
							
							if(!u.AbilityManager.ContainsAbility(PoisonAbility.AbilityName))
							{
								IEffect poisonEffect = null;
								u.AbilityManager.AppendAbility(new PoisonAbility(DurationTime,poisonEffect));
							}
						}
					}
				}

				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif