﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Ability.Black;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Black
{
	class Reicho:thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		int[] atks;
		float[] radiuses;
		int[] shotCounts;

		public Reicho()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "霊蝶「蝶の羽風生に暫く」";
			string basePath = "Black/Spell/Reicho/";


			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 28*30;
			CoolTimes[3] = 25*30;
			
			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6*32;
			MaxRanges[2] = 7*32;
			MaxRanges[3] = 8*32;

			atks = new int[MaxLevel+1];
			atks[0] = 0;
			atks[1] = 8;
			atks[2] = 10;
			atks[3] = 12;

			shotCounts = new int[MaxLevel+1];
			shotCounts[0] = 0;
			shotCounts[1] = 2;
			shotCounts[2] = 3;
			shotCounts[3] = 4;


			radiuses = new float[MaxLevel +1];
			radiuses[0] = 0;
			radiuses[1] = 4;
			radiuses[2] = 5;
			radiuses[3] = 6;

			Description = "指定地点から放射状に蝶弾が飛び立ちます。\nこの蝶弾は貫通し防御力を無視してダメージを与えます。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力" + atks[1] + "x"+shotCounts[1]+"、防御力無視、範囲" + radiuses[1],
				"攻撃力" + atks[2] + "x"+shotCounts[2]+"、防御力無視、範囲" + radiuses[2],
				"攻撃力" + atks[3] + "x"+shotCounts[3]+"、防御力無視、範囲" + radiuses[3],
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (spell == null || !spell.CanExec) {
				yield return false;
				yield break;
			}

			var world = selfUnit.Faction.World;

			var targetPos = world.GetGroundAtPos (targetPosition);

			var bulletParam = selfUnit.GetModifiedBulletParameter (typeof(ButterflyBullet).FullName);
			if (null == bulletParam) {
				yield return false;
				yield break;
			}

			// check reicho bullets
			if (selfUnit.AbilityManager.ContainsAbility (ReichoAbility.AbilityName)) {
				yield return false;
				yield break;
			}

			// shot bullets
			// duration = shot 2sec + cont. 4sec
			if (selfUnit.AbilityManager.AppendAbility (new ReichoAbility (atks[level],targetPos,radiuses[level]*Def.MapTipLengthOnField,shotCounts[level],6*Def.FPS))) {
				// reset cooltime
				spell.ResetCoolTime ();
			}


			// end
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class ReichoAbility : AbilityTemplate
	{
		public const string AbilityName = "霊蝶「蝶の羽風生に暫く」";
		public int Attack = 0;
		public float Radius = 0;
		public int ShotCount = 0;
		public Vector3D Center;
		int shotCounter = 0;
		int counter;
		const int shotInterval = Def.FPS/2;

		public ReichoAbility(int attack,Vector3D center,float radius,int shotCount,int duration)
		{
			Name = AbilityName;
			DurationTime = duration;
			Attack = attack;
			Center = center;
			Radius = radius;
			ShotCount = shotCount;
			counter = 0;
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				var bulletParam = unit.GetModifiedBulletParameter (typeof(ButterflyBullet).FullName);
				if (null != bulletParam) {
					bulletParam.Attack = Attack;
					bulletParam.MaxRange = Radius;
					if(++counter >= shotInterval)
					{
						counter = 0;
						if(shotCounter++ < ShotCount)
						{
							// 8 way
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (Radius * 1, 0, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (0, Radius * 1, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (-Radius * 1, 0, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (0, -Radius * 1, 1)),Center);
							
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (Radius * 0.7071f, Radius * 0.7071f, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (Radius * 0.7071f, -Radius * 0.7071f, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (-Radius * 0.7071f, Radius * 0.7071f, 1)),Center);
							unit.CreateBullet (typeof(ButterflyBullet).FullName, Center + (new Vector3D (-Radius * 0.7071f, -Radius * 0.7071f, 1)),Center);
						}
					}
				}
				break;
			default:
				break;
			}

			return base.ExecUnitAbility (unit, order);
		}

	}
}

#endif