﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Icicle : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		float[] decMoveSpeed;
		const int duration = 4*Def.FPS;
		public Icicle()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "アイシクルアタック";
			string basePath = "Blue/Spell/Icicle/";
			decMoveSpeed = new float[MaxLevel + 1];
			decMoveSpeed[0] = 0;
			decMoveSpeed[1] = 0.1f;
			decMoveSpeed[2] = 0.3f;
			decMoveSpeed[3] = 0.5f;


			Description = "通常攻撃が命中した相手は、凍結バッドステータスとスローを得る。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"凍結(攻撃速度-10%,4秒間), "+duration/Def.FPS+"秒間の移動速度-"+decMoveSpeed[1].ToString("###%"),
				"凍結(攻撃速度-10%,4秒間), "+duration/Def.FPS+"秒間の移動速度-"+decMoveSpeed[2].ToString("###%"),
				"凍結(攻撃速度-10%,4秒間), "+duration/Def.FPS+"秒間の移動速度-"+decMoveSpeed[3].ToString("###%"),
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D (2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(IcicleEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius * 5;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (effect, offset, 15, this.Name + "areaEffect")))
					effect.Stop ();
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName (1, this.Name + "areaEffect",15);
			}

			foreach (var b in selfUnit.ModifiedBulletParameters) {
				switch(level)
				{
				case 1:
					b.OnHit += OnHit_lv1;
					break;
				case 2:
					b.OnHit += OnHit_lv2;
					break;
				case 3:
					b.OnHit += OnHit_lv3;
					break;
				default:
					break;
				}
			}

			return true;
		}
		
		public void OnHit_lv1 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,1);
		}
		public void OnHit_lv2 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,2);
		}
		public void OnHit_lv3 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,3);
		}
		public void OnHit (IBullet bullet, ITargetObject target, int level)
		{
			var selfUnit = bullet.Generator as IUnit;

			if (null == selfUnit)
				return;

			var u = target as IUnit;

			if (null != u)
			{
				IEffectCommon effect = null;
				Vector3D offset = new Vector3D(0.0f, 0.0f, 4.0f);

				// エフェクトを描画
				{
					var e = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(ChirnoFreezingTextureEffect).FullName, u.Pos + offset);
					effect = e;
				}

				var ability = u.AbilityManager.GetAbilityWithName(IcicleAbility.AbilityName) as IcicleAbility;
				if(null == ability)
				{
				if (u.AbilityManager.AppendAbility(new IcicleAbility(decMoveSpeed[level],duration)))
					;
				}
				else
				{
					ability.DecMoveRatio = decMoveSpeed[level];
					ability.DurationTime = duration;
				}

				if (!u.AbilityManager.AppendAbility(new ChilblainAbility(effect, offset)))
				{
					effect.Stop();
				}

			}
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class IcicleAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "アイシクルアタック";
		public float DecMoveRatio = 0.2f;
		public IcicleAbility (float decMoveRatio,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			DecMoveRatio = decMoveRatio;
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= DecMoveRatio*unit.Speed;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(0, 0, 4));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
	class ChilblainAbility : FollowEffectAbility
	{
		public const float DecAttackSpeedRatio = 0.1f;
		public const string AbilityName ="凍結";
		IEffectCommon effect;
		public ChilblainAbility(IEffectCommon effect, Vector3D offset, int duration = 4*Def.FPS)
			: base(null, offset, duration, AbilityName)
		{
			this.DurationTime = duration;
			this.Name = AbilityName;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.AttackDelay += (int)Math.Ceiling(DecAttackSpeedRatio*b.AttackDelay);
					b.AttackRigor += (int)Math.Ceiling(DecAttackSpeedRatio*b.AttackRigor);
					b.AttackSpeed += (int)Math.Ceiling(DecAttackSpeedRatio*b.AttackSpeed);
				}
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				effect.SetPosition(unit.Pos + new Vector3D(0, 0, 4));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}

		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}

		public override bool OnVanish(IBullet bullet)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(bullet);
		}

		public override bool OnVanish(IBuilding building)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(building);
		}
	}
}
