using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Spell.Blue
{
	class Shimofu : thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;

		int[] damages;

		const int duration = 8 * Def.FPS;

		public Shimofu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "霜符「フロストコラムス」";
			string basePath = "Blue/Spell/Shimofu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30 * 30;
			CoolTimes[2] = 20 * 30;
			CoolTimes[3] = 10 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * 32;
			MaxRanges[2] = 7 * 32;
			MaxRanges[3] = 8 * 32;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 2 * Def.FPS;
			chargeTime[2] = 2 * Def.FPS;
			chargeTime[3] = 2 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 10;
			damages[2] = 15;
			damages[3] = 20;

			Description = "範囲内の敵ユニットに凍結バッドステータスとダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"、凍結(攻撃速度-10%,4秒間)、4秒毎に" + damages[1] + "ダメージx2",
				"範囲"+ affectRadiuses[2]+"、凍結(攻撃速度-10%,4秒間)、4秒毎に" + damages[2] + "ダメージx2",
				"範囲"+ affectRadiuses[3]+"、凍結(攻撃速度-10%,4秒間)、4秒毎に" + damages[3] + "ダメージx2",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName, targetPos + offset, selfUnit.Faction);
				var scale = 1.0f * (affectRadius);
				effect.SetScale(new Vector3D(scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
					effect.Stop();

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ShimofuEffect).FullName, targetPos + offset, selfUnit.Faction);
				scale = affectRadius * Def.MapTipLengthOnField / 10.0f;
				areaEffect.SetScale(new Vector3D(scale, scale, scale));

				var ability = new SpellContinuousAbility((unit) => Spelling(unit, targetPosition, level), areaEffect, offset, duration, this.Name + "areaEffect");
				ability.FollowEffect = false;
				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();

			}
			yield return false;
		}
		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			;
		}

		public bool Spelling(IUnit selfUnit, Vector2D targetPosition, int level)
		{
			float affectRadius = affectRadiuses[level];

			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			// effect offset
			Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(targetPos.X, targetPos.Y))).OfType<IUnit>().Where(_ => _ != null);
			foreach (var u in targets)
			{
				if (null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction, selfFaction) && u.UniqueID != selfUnit.UniqueID)
				{

					IEffectCommon effect = null;
					offset = new Vector3D(2.0f, 2.0f, 4.0f);

					// エフェクトを描画
					{
						var e = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(ChirnoFreezingTextureEffect).FullName, u.Pos + offset);
						effect = e;
					}

					// onHit unit
					if (u.AbilityManager.AppendAbility(new ChilblainAbility(effect, offset)))
					{
						var faction = selfUnit.Faction;
						/*
						var hiteffect = faction.World.EffectManager.CreateEffect(typeof(ChilnohitEffect).FullName, u.Pos + offset, faction);
						hiteffect.SetScale(new Vector3D(8, 8, 8));
						*/
						Unit.Utls.ShowHitEffect(u, Unit.Utls.HitEffectType.NormalBlue);
						faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, u.Pos);
						u.BeDamaged(selfUnit, damages[level], AttackBonusSet.None);
					}
					else
					{
						effect.Stop();
					}
				}
			}

			return true;
		}
	}
}
