﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Yukifu : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		int[] damages;
		float[] decMoveSpeed;
		float[] affectRadiuses;

		const int duration = 8*Def.FPS;
		
		public Yukifu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "雪符「ダイアモンドブリザード」";
			string basePath = "Blue/Spell/Yukifu/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 25*30;
			CoolTimes[3] = 20*30;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 1;
			damages[2] = 3;
			damages[3] = 5;

			decMoveSpeed = new float[MaxLevel + 1];
			decMoveSpeed[0] = 0;
			decMoveSpeed[1] = 0.1f;
			decMoveSpeed[2] = 0.2f;
			decMoveSpeed[3] = 0.3f;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 4;
			affectRadiuses[2] = 5;
			affectRadiuses[3] = 6;


			Description = "周囲の敵ユニットにダメージと移動速度低下を与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1秒毎に"+damages[1]+"ダメージ(凍傷時3倍)、移動速度-"+decMoveSpeed[1].ToString("###%")+"",
				"1秒毎に"+damages[2]+"ダメージ(凍傷時3倍)、移動速度-"+decMoveSpeed[2].ToString("###%")+"",
				"1秒毎に"+damages[3]+"ダメージ(凍傷時3倍)、移動速度-"+decMoveSpeed[3].ToString("###%")+"",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			
			float affectRadius = affectRadiuses[level];
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(YukifuEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = Def.MapTipLengthOnField * (affectRadius) / 8.0f;
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect")))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			float affectRadius = affectRadiuses[level];

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && u.UniqueID != selfUnit.UniqueID ) {

					if(u.AbilityManager.ContainsAbility(YukifuAbility.AbilityName))
						continue;

					var damage = damages[level];
					// onHit unit
					var ability = new YukifuAbility(level);
					if(u.AbilityManager.ContainsAbility(ChilblainAbility.AbilityName))
					{
//						ability.DecMoveRatio = decMoveSpeed[level] * 3;
						damage *= 3;
					}
					if(u.AbilityManager.AppendAbility(ability))
					{
						var faction = selfUnit.Faction;
						/*
						var effect = faction.World.EffectManager.CreateEffect(typeof(ChilnohitEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(8, 8, 8));
						*/
						Unit.Utls.ShowHitEffect(u, Unit.Utls.HitEffectType.NormalBlue);
						faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, u.Pos);
						u.BeDamaged (selfUnit, damage, AttackBonusSet.None);
					}
				}
			}
			
			return true;		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class YukifuAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "雪符「ダイアモンドブリザード」";
		//public float DecMoveRatio = 0.2f;
		public float DecMoveRatio;
		public YukifuAbility (int level, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 1*Def.FPS;
			this.effect = effect;
			switch(level)
				{
				case 1:
					DecMoveRatio = 0.1f;
					break;
				case 2:
					DecMoveRatio = 0.2f;
					break;
				case 3:
					DecMoveRatio = 0.3f;
					break;
				default:
					DecMoveRatio = 0;
					break;
				}
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= DecMoveRatio*unit.Speed;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}
