﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{

	class Shinsou:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 5;
		public Shinsou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "深層「無意識の遺伝子」";
			string basePath = "Blue/Spell/Shinsou/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 60*30;
			CoolTimes[1] = 20*30;
			CoolTimes[2] = 20*30;
			CoolTimes[3] = 20*30;

			Description = "敵AIから発見されず、もし発見されても一定時間で見失います。\nこいしは敵を発見しても自動で攻撃しません。\n敵ヒロインの近くでは透明になれません。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"停止時透明、" + "敵AIに見つからない",
				"停止・移動時透明、" + "敵AIに見つからない",
				"常に透明、" + "敵AIに見つからない",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,0),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool CastAction (thrts.IUnit selfUnit, int level)
		{

			var isTransInFree = level > 0;
			var isTransInMove = level > 1;
			var isTransAlways = level > 2;
			
			var ability = selfUnit.AbilityManager.GetAbilityWithName(ShinsouAbility.AbilityName) as ShinsouAbility;

			// 近くにヒロインが居たら透明になれない
			if(selfUnit.Faction.World.GetObjectFromCircle(new Circle2D(selfUnit.ViewRange,new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y)))
			   .OfType<IUnit>()
			   .Any(_=>!selfUnit.Faction.World.IsSameTeam(_.Faction,selfUnit.Faction) && _.Parameter.IsUnique))
			{
				isTransInFree = false;
				isTransInMove = false;
				isTransAlways = false;
			}
			if (null == ability) {
				var offset = new Vector3D(2,2,1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MuishikiEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = 8;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility(new ShinsouAbility(effect,offset,interval*2,isTransInFree,isTransInMove,isTransAlways)))
					effect.Stop();
			}
			else
			{
				ability.ExtendLifetime(interval,interval*2);
				ability.IsTransparentInFree = isTransInFree;
				ability.IsTransparentInMove = isTransInMove;
				ability.IsTransparentAlways = isTransAlways;
			}

			/// 全てに優先してAIの索敵から除外 ( ability -> autocast の順のため )
			selfUnit.IsPreventedFromFindingEnemy = true;

			return true;
		}

	}
}

namespace LotusCraft.Ability.Blue
{
	class ShinsouAbility : FollowEffectAbility
	{
		int counter = 0;
		public bool IsTransparentInFree = false;
		public bool IsTransparentInMove = false;
		public bool IsTransparentAlways = false;
		const int cutTargettingInterval = 4*Def.FPS;
		public static string AbilityName = "深層「無意識の遺伝子」";
		public ShinsouAbility(IEffect effect,Vector3D offset,int duration,bool isTransInFree = false,bool isTransInMove = false,bool isTransAlways = false)
			:base(effect,offset,duration,AbilityName)
		{
			IsTransparentInFree = isTransAlways;
			IsTransparentInMove = isTransInMove;
			IsTransparentAlways = isTransAlways;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 自動索敵を停止
				unit.DoFindEnemyAutomatically = false;
				// 不可視系の設定は定数増加で行う
				unit.IsSemitransparent = true;
				unit.IsPreventedFromFindingEnemy = true;
				// 
				if(IsTransparentAlways
				   || (IsTransparentInMove && unit.ActualAction == UnitAction.Move)
				   || (IsTransparentInFree && unit.ActualAction == UnitAction.Free))
				{
					unit.IsTransparent = true;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if( cutTargettingInterval < ++counter ){
					unit.IsntFoundByEnemy = true;
					counter = 0;
				}
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				// 自動索敵を再開
				unit.DoFindEnemyAutomatically = true;
				// reset
				unit.IsTransparent = false;
				unit.IsPreventedFromFindingEnemy = false;
			}
			return base.OnVanish (unit);
		}
	}
}

#endif