using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{
	class Dive:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;
		int[] damages;

		const int duration = 1*Def.FPS;
		const int interval = 1;
		public Dive()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "鷹符「イルスタードダイブ」";
			string basePath = "Blue/Spell/Dive/";
			
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 4*30;
			CoolTimes[2] = 4*30;
			CoolTimes[3] = 4*30;
			
			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 4*32;
			MaxRanges[2] = 6*32;
			MaxRanges[3] = 8*32;
			
			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = 1*Def.FPS;
			chargeTime[2] = 1*Def.FPS;
			chargeTime[3] = 1*Def.FPS;
			
			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 1.0f;
			affectRadiuses[2] = 1.2f;
			affectRadiuses[3] = 1.4f;
			
			damages = new int[MaxLevel +1];
			damages[0] = 0;
			damages[1] = 6;
			damages[2] = 9;
			damages[3] = 12;
			
			Description = "指定地点に向かってダイブする。\n空中に居る場合は地上に、地上に居る場合は空中に移動する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"の敵ユニットに"+damages[1]+"ダメージ",
				"範囲"+ affectRadiuses[2]+"の敵ユニットに"+damages[2]+"ダメージ",
				"範囲"+ affectRadiuses[3]+"の敵ユニットに"+damages[3]+"ダメージ",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				null,
				null,
				null,
				null,
				new AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,ChangeBulletSpaceType,true)
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				float affectRadius = affectRadiuses[level];

				var targetPos = (selfUnit.Pos);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);


				// dive to target position
				var diff = targetPos - selfUnit.Pos;
				diff.X /= chargeTime[level];
				diff.Y /= chargeTime[level];

				bool isFlying = selfUnit.FlyingState == FlyingState.Fly || selfUnit.FlyingState == FlyingState.ToTouchGround;

				var world = selfUnit.Faction.World;
				// push away unit
				{
					var pushRadius = Math.Max(affectRadiuses[level] * Def.MapTipLengthOnField, selfUnit.Radius);
					foreach (var u in world.GetObjectFromCircle(new Circle2D(pushRadius, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>())
					{
						if (u == null || !u.Exists() || u.UniqueID == selfUnit.UniqueID || u.FlyingState == FlyingState.Fly)
						{
							continue;
						}

						diff = selfUnit.Pos - u.Pos;
						var distance = diff.X * diff.X + diff.Y * diff.Y;
						if (distance > 1)
						{
							u.TryMove(diff.X * pushRadius / distance, diff.X * pushRadius / distance, true);
						}
						else
						{
							var rot = world.NextRandDouble();
							rot *= 2 * Math.PI;
							u.TryMove(pushRadius * (float)Math.Cos(rot), pushRadius * (float)Math.Sin(rot), true);
						}
					}
				}

				if (!isFlying)
				{
					if (!selfUnit.Fly())
					{
						yield return false;
						yield break;
					}
				}
				else
				{
					if (!selfUnit.TouchGround())
					{
						yield return false;
						yield break;
					}
				}

				spell.ResetCoolTime();

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);

				diff.Normalize();

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// move to target pos
					if (selfUnit.TryMove(diff.X, diff.Y, true))
					{
						yield return true;
					}
					else
					{
						yield return false;
						yield break;
					}
				}



				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(DiveEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
				var scale = 5.0f * (affectRadius);
				areaEffect.SetScale(new Vector3D(scale, scale, scale));

				var ability = new EffectContinuousAbility(areaEffect, duration, this.Name + "areaEffect");
				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();


				// damage 
				/// search targets
				var selfFaction = selfUnit.Faction;
				var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(targetPos.X, targetPos.Y))).OfType<IUnit>().Where(_ => _ != null);
				foreach (var u in targets)
				{
					if (null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction, selfFaction) && u.UniqueID != selfUnit.UniqueID)
					{
						Unit.Utls.ShowHitEffect(u, Unit.Utls.HitEffectType.NormalYellow);
						u.BeDamaged(selfUnit, damages[level], AttackBonusSet.Hero);
						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitZangeki).FullName, u.Pos);
					}
				}
			}
			yield return false;
		}
		
		public bool ChangeBulletSpaceType (IUnit selfUnit)
		{

			var bullet = selfUnit.GetModifiedBulletParameter (typeof(MystiaBullet).FullName);

			if (null != bullet) {
				if(selfUnit.FlyingState == FlyingState.TouchGround || selfUnit.FlyingState == FlyingState.ToFly)
				{
					bullet.SpaceType = BulletSpaceType.Earth;
				}else if(selfUnit.FlyingState == FlyingState.Fly || selfUnit.FlyingState == FlyingState.ToTouchGround)
				{
					bullet.SpaceType = BulletSpaceType.Sky;
				}
			}
			return true;
		}
	}
}
