﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Yamou : thrts.SpellParameter<thrts.IUnit>
	{
		float[] decViewRange;
		float[] decAttackRange;
		float[] affectRadiuses;
		float[] decMoveSpeedRates;

		const int interval = 1;

		public Yamou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "夜盲「夜雀の歌」";
			string basePath = "Blue/Spell/Yamou/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 25*30;
			CoolTimes[3] = 20*30;

			decViewRange = new float[MaxLevel + 1];
			decViewRange[0] = 0;
			decViewRange[1] = 2;
			decViewRange[2] = 2;
			decViewRange[3] = 4;

			decAttackRange = new float[MaxLevel + 1];
			decAttackRange[0] = 0;
			decAttackRange[1] = 0;
			decAttackRange[2] = 1;
			decAttackRange[3] = 1;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 9;
			affectRadiuses[2] = 10;
			affectRadiuses[3] = 11;

			decMoveSpeedRates = new float[MaxLevel + 1];
			decMoveSpeedRates[0] = 0;
			decMoveSpeedRates[1] = 0;
			decMoveSpeedRates[2] = 0;
			decMoveSpeedRates[3] = 0.1f;

			Description = "周囲の敵ユニットの視界を減少させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"視界-"+decViewRange[1],
				"視界-"+decViewRange[2] + "攻撃範囲-" + decAttackRange[2],
				"視界-"+decViewRange[3] + "攻撃範囲-" + decAttackRange[3]+"移動速度-"+decMoveSpeedRates[3].ToString("P1"),
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1", 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2", 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3", 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool CastAction (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			float affectRadius = affectRadiuses [level];

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

			var selfFaction = selfUnit.Faction;

			if (selfUnit.AbilityManager.ContainsAbility (this.Name + "effect")) {
				selfUnit.AbilityManager.ExtendLifetimeWithName (interval, this.Name + "effect",interval*2);
			} else {
				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(YamouAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
				var scale = 5.0f * (affectRadius);
				areaEffect.SetScale (new Vector3D (scale, scale, scale));
				if(!selfUnit.AbilityManager.AppendAbility(new FollowEffectAbility(areaEffect,offset,interval*2,this.Name+"effect")) && null != areaEffect)
					areaEffect.Stop();
			}

			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>();
			foreach (var u in targets) {
				if ( null == u || !u.Exists() || selfFaction.World.IsSameTeam(u.Faction,selfFaction))
					continue;

				{
					var ability = u.AbilityManager.GetAbilityWithName(YamouAbility.AbilityName) as YamouAbility;

					if(null != ability)
					{
						ability.DecViewRange = decViewRange[level]*Def.MapTipLengthOnField;
						ability.DecAttackRange = decAttackRange[level]*Def.MapTipLengthOnField;
						ability.DecMoveSpeedRate = decMoveSpeedRates[level];
						ability.ExtendLifetime(interval,interval*2);
					}
					else
					{
						IEffect effect = null;
						offset = new Vector3D(2.0f, 2.0f, 4.0f);

						// エフェクトを描画
						{
							var e = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(YamouEffect).FullName, u.Pos + offset);
							e.SetScale(new Vector3D(8, 8, 8));
							effect = e;
						}
						if(!u.AbilityManager.AppendAbility(new YamouAbility(decViewRange[level]*Def.MapTipLengthOnField,decAttackRange[level]*Def.MapTipLengthOnField,decMoveSpeedRates[level],interval*2,effect)))
							if(null != effect)
								effect.Stop();
					}
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class YamouAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		public const string AbilityName = "夜盲「夜雀の歌」";
		public float DecViewRange = 0;
		public float DecAttackRange = 0;
		public float DecMoveSpeedRate = 0f;
		public YamouAbility (float decViewRange,float decAtkRange,float decMoveSpeedRate, int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.DecAttackRange = decAtkRange;
			this.DecViewRange = decViewRange;
			this.DecMoveSpeedRate = decMoveSpeedRate;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.ViewRange = Math.Max (unit.ViewRange - DecViewRange,0);
				foreach(var b in unit.ModifiedBulletParameters)
				{
					if(b.MaxRange > Math.Max (1*Def.MapTipLengthOnField , DecAttackRange))
						b.MaxRange = Math.Max(b.MaxRange - DecAttackRange,1*Def.MapTipLengthOnField);
				}
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= Math.Min(unit.Speed,unit.Speed*DecMoveSpeedRate);
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}
