﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Yojaku : thrts.SpellParameter<thrts.IUnit>
	{
		float affectRadius = 12;
		const float maxViewRange = 3;
		const float maxAtkRange = 2;

		const int duration = 18*Def.FPS;

		const int interval = 10;

		const int chargeTime = Def.FPS;
		public Yojaku()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "夜雀「真夜中のコーラスマスター」";
			string basePath = "Blue/Spell/Yojaku/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 180*30;

			Description = "周囲の敵ユニットの視界と射程を大幅に制限する。\nヒロインには効果がない。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"視界最大" + maxAtkRange +"、射程最大" + maxViewRange,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			spell.ResetCoolTime ();
			// charge

			for(int i=0;i<chargeTime;++i)
				yield return true;

			selfUnit.ShowCutin();

			// 効果範囲は視界
			affectRadius = selfUnit.ViewRange/Def.MapTipLengthOnField;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(YojakuAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 5.0f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));


			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,duration,this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			// 効果範囲は視界
			affectRadius = selfUnit.ViewRange/Def.MapTipLengthOnField;

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			// damage 
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null != u && u.Exists() && !selfFaction.World.IsSameTeam(u.Faction,selfFaction) && !u.Parameter.IsUnique) {

					var ability = u.AbilityManager.GetAbilityWithName(YojakuAbility.AbilityName);
					if(null == ability)
					{
						IEffect effect = null;
						offset = new Vector3D(2.0f, 2.0f, 4.0f);
						
						// エフェクトを描画
						{
							var e = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(YojakuEffect).FullName, u.Pos + offset);
							e.SetScale(new Vector3D(8, 8, 8));
							effect = e;
						}
						
						if(u.AbilityManager.AppendAbility(new YojakuAbility(maxViewRange*Def.MapTipLengthOnField,maxAtkRange*Def.MapTipLengthOnField,interval*2,effect)))
						{
						}
					}
					else
					{
						ability.ExtendLifetime(interval,interval*2);
					}
				}
			}
			
			return true;		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class YojakuAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "夜雀「真夜中のコーラスマスター」";
		public float MaxViewRange = 0;
		public float MaxAtkRange = 0;
		public YojakuAbility (float maxViewRange,float maxAtkRange,int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime =  duration;
			this.effect = effect;
			this.MaxViewRange = maxViewRange;
			this.MaxAtkRange = maxAtkRange;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				// 効果に関係無く固定値設定のため最後に行う
				unit.ViewRange = Math.Min (unit.ViewRange,MaxViewRange);
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.MaxRange = Math.Min(b.MaxRange,MaxAtkRange);
					b.MinRange -= Math.Min (b.MinRange,Math.Max (0,b.MaxRange - MaxAtkRange));
				}
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}
