﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;

namespace LotusCraft.Spell.Blue
{
	class Arm : thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 1 * Def.FPS;

		public Arm()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "のびーるアーム";
			Description = "敵ユニット1体をにとりの近くまで引き寄せる。\n引き寄せられたユニットは1秒間行動不能になる。";
			string basePath = "Blue/Spell/Arm/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 30 * Def.FPS;
			CoolTimes[2] = 30 * Def.FPS;
			CoolTimes[3] = 30 * Def.FPS;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 4 * Def.MapTipLengthOnField;
			MaxRanges[2] = 5 * Def.MapTipLengthOnField;
			MaxRanges[3] = 7 * Def.MapTipLengthOnField;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程" + MaxRanges[1]/Def.MapTipLengthOnField ,
				"射程" + MaxRanges[2]/Def.MapTipLengthOnField ,
				"射程" + MaxRanges[3]/Def.MapTipLengthOnField ,
			};

			/// ボタンの位置
			// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "2", PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "3", PrivilegeModeSpelling, 3),
			};

			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, IUnit targetUnit,IBuilding targetBuilding, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
			{
				yield return false;
				yield break;
			}
			
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			if (null == targetUnit || !targetUnit.Exists() || targetUnit.HP < 1) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();

			var world = selfUnit.Faction.World;

			// charge ?

			var tick = selfUnit.Pos - targetUnit.Pos ;
			var length = tick.Length - Def.MapTipLengthOnField;
			tick.Normalize();
			tick *= length;

			{
				var offset = new Vector3D(0, 0, 32);
				var areaEffect = world.EffectManager.CreateEffect(typeof(ArmEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale(new Vector3D(16, 16, 16));
				areaEffect.SetRotation(new Vector3D(0, 0, (float)(Math.Atan2(tick.X, tick.Y) * 180 / Math.PI)));

				targetUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, targetUnit.Pos);

				// エフェクトを生成
				var ability = new FollowEffectAbility(areaEffect, offset, duration, this.Name + "areaEffect");
				ability.StopEffectRootOnVanish = true;

				if (!targetUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();
			}

			// 通りぬけ
			var bsAblity = new ThroughOthersAbility();
			targetUnit.AbilityManager.AppendAbility(bsAblity);
			bsAblity.ExtendLifetime(2);

			// 1秒で手元に移動
			int count = Def.FPS;

			tick *= 1 / (float)count;


			for (int i = 0; i < duration; ++i)
			{
				if (i <= count)
					targetUnit.TryMove(tick.X, tick.Y, true);

				bsAblity.ExtendLifetime(1);
				yield return true;
			}

			bsAblity.ExtendLifetime(1);
			yield return true;

			targetUnit.AbilityManager.AppendAbility(new StunAbility(null,new Vector3D(0,0,0),1*Def.FPS,this.Name+"stun"));

			yield return false;
		}

		class ThroughOthersAbility : AbilityTemplate
		{
			public ThroughOthersAbility()
			{
				Name = "ThroughOthers";
			}

			public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.RateDecParam) {
					unit.Speed = 0;
				}
				if (order == AbilityOrder.ConstDecParam) {
					unit.IsForbiddenAttacking = true;
					unit.IsForbiddenSpelling = true;
				}
				if (order == AbilityOrder.OtherExecs)
				{
					unit.ThroughOthers = true;
				}

				return true;
			}
		}
	}
}
