﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Camouflage : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		float[] affectRadiuses;

		const int interval = 10;
		public Camouflage()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "光学「ハイドロカモフラージュ」";
			string basePath = "Blue/Spell/Camouflage/";

			CoolTimes = new int[MaxLevel+1];

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;


			Description = "にとりと周囲にいるヒロイン以外の味方兵ユニットを透明にする。\n敵軍にヒロインが居れば見え、敵AIには気付かれる。";
			// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲" + affectRadiuses[1] + "",
				"範囲" + affectRadiuses[2] + "、にとりがAIに気付かれない",
				"範囲" + affectRadiuses[3] + "、にとりがAIに気付かれない",
			};
			
			// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>CastAction(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool CastAction (thrts.IUnit selfUnit, int level)
		{
			float affectRadius = affectRadiuses [level];
			
			var ability = selfUnit.AbilityManager.GetAbilityWithName(this.Name + "areaEffect") as FollowEffectAbility;
			if (ability != null) {
				ability.DurationTime = 2*interval;
				var scale = 5.0f * (affectRadius);
				ability.Effect.SetScale (new Vector3D (scale, scale, scale));
			}
			else
			{
				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(CamouflageEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
				var scale = 4.0f * (affectRadius);
				areaEffect.SetScale (new Vector3D (scale, scale, scale));
				areaEffect.IsShownToOnlyAlly = true;
			
				if (!selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (areaEffect, offset, interval *2, this.Name + "areaEffect")))
					areaEffect.Stop ();
			}

			Spelling(selfUnit,level);

			return true;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			float affectRadius = affectRadiuses[level];

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (var t in targets) {
				var u = t as IUnit;
				var b = t as IBuilding;
				if ( null != u && u.Exists() && selfFaction.World.IsSameTeam(u.Faction,selfFaction) ) {
					if(u.Parameter.IsUnique && u.UniqueID != selfUnit.UniqueID)
						continue;

					// onHit unit
					var ability = u.AbilityManager.GetAbilityWithName(CamouflageAbility.AbilityName) as CamouflageAbility;
					if(null != ability)
					{
						ability.ExtendLifetime(interval,interval*2);
						ability.Level = level;
					}
					else
					{
						u.AbilityManager.AppendAbility(new CamouflageAbility(level,interval*2,selfUnit));
					}
				}
				if ( null != b && b.Exists() && !selfFaction.World.IsSameTeam(b.Faction,selfFaction) ) {
					// onHit unit
					var ability = b.AbilityManager.GetAbilityWithName(CamouflageAbility.AbilityName) as CamouflageAbility;
					if(null != ability)
					{
						ability.ExtendLifetime(interval,interval*2);
						ability.Level = level;
					}
					else
					{
						b.AbilityManager.AppendAbility(new CamouflageAbility(level,interval*2,selfUnit));
					}
				}
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class CamouflageAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		IUnit speller;
		public const string AbilityName = "光学「オプティカルカモフラージュ」";
		public int Level = 0;
		public CamouflageAbility (int level,int duration,IUnit speller, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.Level = level;
			this.speller = speller;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 近くにヒロインが居たら透明になれない
				if(unit.Faction.World.GetObjectFromCircle(new Circle2D(16*Def.MapTipLengthOnField,new Vector2D(unit.Pos.X,unit.Pos.Y)))
				   .OfType<IUnit>()
				   .Any(_=>!unit.Faction.World.IsSameTeam(_.Faction,unit.Faction) && _.Parameter.IsUnique && Utl.GetDistance(unit.Pos.X,unit.Pos.Y,_.Pos.X,_.Pos.Y) <= _.ViewRange))
				{
					unit.IsTransparent = false;
					unit.IsSemitransparent = false;
				}
				else
				{
					// 不可視系の設定は定数増加で行う
					unit.IsTransparent = true;
					unit.IsSemitransparent = true;
				}
				if(Level >= 2 && unit.UniqueID == speller.UniqueID)
					unit.IsPreventedFromFindingEnemy = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool ExecBuildingAbility(thrts.IBuilding building, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 不可視系の設定は定数増加で行う
//				building.IsTransparent = true;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(building.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				unit.IsTransparent = false;
				unit.IsSemitransparent = false;
				if (unit.UniqueID == speller.UniqueID)
					unit.IsPreventedFromFindingEnemy = false;
			}
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
		public override bool OnVanish (IBuilding building)
		{
			if (null != building && building.Exists ()) {
//				building.IsTransparent = false;
			}
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (building);
		}
	}
}

#endif