﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Mizufu : thrts.SpellParameter<thrts.IUnit>
	{

		const int damageInterval = 2*Def.FPS;
		const int damage = 50;
		const int affectRadius = 8;
		const int duration = 12*Def.FPS;
		const int chargeTime = Def.FPS;

		public Mizufu()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "水符「河童の幻想大瀑布」";
			string basePath = "Blue/Spell/Mizufu/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*Def.FPS;


			Description = "周囲の敵ユニットを押し流す。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲" + affectRadius + "、" +damageInterval/Def.FPS +  "秒毎に" + damage + "ダメージ、", + duration/Def.FPS + "秒間"
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();
			// charge
			
			for(int i=0;i<chargeTime;++i)
				yield return true;
			
			selfUnit.ShowCutin();
			

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 15.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MizufuEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 5.0f * (affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));

			selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (areaEffect, offset, duration, this.Name + "areaEffect"));
			
			for (int i=0; i<duration; ++i) {
				var selfFaction = selfUnit.Faction;
				// search targets
				var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit> ().Where (_ => _ != null);
				foreach (var u in targets) {
					if (null != u && u.Exists () && !selfFaction.World.IsSameTeam (u.Faction, selfFaction) && u.UniqueID != selfUnit.UniqueID) {
						// 中心から遠ざかる方向に速度0.5で移動
						var tick = (u.Pos - selfUnit.Pos);
						tick.Normalize ();
						tick *= 0.5f * UnitParameter.BasisSpeed;
						u.TryMove (tick.X, tick.Y, false);
						if (i % damageInterval == 0) {
							u.BeDamaged (selfUnit, damage, AttackBonusSet.None, true);
						}
					}
				}


				areaEffect.SetPosition (selfUnit.Pos + offset);
				yield return true;
			}
			yield return false;
		}
	}
}

#endif