﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Effect.Blue;

namespace LotusCraft.Spell.Blue
{
	class Torpedo:thrts.SpellParameter<thrts.IUnit>
	{
		int[] damages;
		const int chargeTime = Def.FPS/2;
		const int rigorTime = 0*Def.FPS;

		public Torpedo()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "漂溺「光り輝く水底のトラウマ」";
			string basePath = "Blue/Spell/Torpedo/";
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;

			MaxRanges= new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6*Def.MapTipLengthOnField;
			MaxRanges[2] = 8*Def.MapTipLengthOnField;
			MaxRanges[3] = 10*Def.MapTipLengthOnField;

			damages = new int[MaxLevel +1];
			damages[0] = 0;
			damages[1] = 20;
			damages[2] = 40;
			damages[3] = 60;

			Description = "指定地点に向け光弾を発射する。光弾がヒットした地点の周囲に\n居るユニットは防御無視ダメージを受け、秒間行動不能になる。\n作業員へはダメージ半分。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程" + MaxRanges[1]/Def.MapTipLengthOnField +"ダメージ" + damages[1] ,
				"射程" + MaxRanges[2]/Def.MapTipLengthOnField +"ダメージ" + damages[2] ,
				"射程" + MaxRanges[3]/Def.MapTipLengthOnField +"ダメージ" + damages[3] ,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit,Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}
				
				spell.ResetCoolTime ();

				// charge 

				for(int i=0;i<chargeTime;++i)
					yield return true;

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				var b = selfUnit.GetModifiedBulletParameter(typeof(TorpedoBullet).FullName);
				if(null != b)
				{
					var bullet = selfUnit.CreateBullet(typeof(TorpedoBullet).FullName,targetPos,selfUnit.Pos);
					bullet.Attack = damages[level];
				}

			
				// 硬直
				for(int i=0;i<rigorTime;++i)
					yield return true;
			}
			yield return false;
		}
	}
}

namespace LotusCraft.Bullet.Blue
{
	public class TorpedoBullet : General.CannonBullet
	{
		public TorpedoBullet()
		{
			Type = BulletType.LongRange;
			this.Attack = 0;

			FarmerDamageRatio = 0.5f;
			Speed = 22.0f;
			Gravity = 0;
			MaxRange = 18.0f * Def.MapTipLengthOnField;

			Radius = 32;
			
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates.Add(3*Def.MapTipLengthOnField, 1.0f);
			IgnoreArmor = true;

			MaxRange = 10 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(2.5f * Def.FPS);
			AttackRigor = (int)(1.5f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit += OnHitEffect;
			OnHitSplash += OnHitSplashEffect;
			OnMiss += OnMissEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_blue";
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			TextureAnimation.Width = 32;
			TextureAnimation.Height = 32;
			TextureAnimation.Center.X = 16;
			TextureAnimation.Center.Y = 16;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 4;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 4;
		}
		void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, target.Pos, scale);

			if (target is IUnit) {
				AddStunAbility( target as IUnit );
				
			}
		}

		int OnHitSplashEffect (IBullet bullet, ITargetObject target, int damage)
		{
			if (target is IUnit) {
				AddStunAbility( target as IUnit );

			}
			return damage;
		}
		
		void OnMissEffect(IBullet bullet)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, bullet.Position, scale);
		}
		
		public static void CreateEffect(IFaction faction, Vector3D pos, float scale)
		{
			var offset = new Vector3D(0, 0, 4);
			var effect = faction.World.EffectManager.CreateEffect(typeof(TorpedoEffect).FullName, pos + offset, faction);
			scale = 8.0f * 5;
			effect.SetScale(new Vector3D(scale, scale, scale));
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, pos);
		}

		public static void AddStunAbility(IUnit u)
		{
			if(u != null && u.Exists())
			{
				u.AbilityManager.AppendAbility(new Ability.StunAbility(null,new Vector3D(0,0,0),1*Def.FPS,"漂溺「光り輝く水底のトラウマ」_stun"));
			}
		}
	}
}

#endif
