﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Ability.Blue;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Blue
{
	class Chifu : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 1;
		static public float[] DurationTime{ get; private set; }
		const string SpellName = "地符「不譲土壌の剣」";
		public Chifu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Blue/Spell/Chifu/";
			IsPassive = true;

			DurationTime = new float[MaxLevel + 1];
			DurationTime[0] = 0;
			DurationTime[1] = 2f;
			DurationTime[2] = 4f;
			DurationTime[3] = 8f;


			Description = "天子がダメージを与えた建物の建築・生産・研究が一定時間停止させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				DurationTime[1] + "秒間、建築・生産・研究が停止",
				DurationTime[2] + "秒間、建築・生産・研究が停止",
				DurationTime[3] + "秒間、建築・生産・研究が停止",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D (2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(ChifuEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius*8.0f;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility (new FollowEffectAbility (effect, offset, 15, this.Name + "areaEffect")))
					effect.Stop ();
			} else {
				selfUnit.AbilityManager.ExtendLifetimeWithName (1, this.Name + "areaEffect",2);
			}

			foreach (var b in selfUnit.ModifiedBulletParameters) {
				switch(level)
				{
				case 1:
					b.OnHit += OnHit_lv1;
					break;
				case 2:
					b.OnHit += OnHit_lv2;
					break;
				case 3:
					b.OnHit += OnHit_lv3;
					break;
				default:
					break;
				}
			}

			return true;
		}
		
		public void OnHit_lv1 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,1);
		}
		public void OnHit_lv2 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,2);
		}
		public void OnHit_lv3 (IBullet bullet, ITargetObject target)
		{
			OnHit (bullet,target,3);
		}
		public void OnHit (IBullet bullet, ITargetObject target, int level)
		{
			var selfUnit = bullet.Generator as IUnit;

			if (null == selfUnit)
				return;

			var b = target as IBuilding;

			if (null != b) {
				var ability = b.AbilityManager.GetAbilityWithName(ChifuAbility.AbilityName);
				if(null == ability)
				{
					if(!b.AbilityManager.AppendAbility(new ChifuAbility((int)(DurationTime[level]*Def.FPS))))
						;
					selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
				}
				else
				{
					ability.DurationTime = (int)(DurationTime[level]*Def.FPS);
				}
			}
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class ChifuAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		const int affectRadius = 2;
		public const string AbilityName = "地符「不譲土壌の剣」";
		public ChifuAbility (int duration, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;

		}
		public override bool ExecBuildingAbility(thrts.IBuilding building, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				building.ProgressStopped = true;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(building.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IBuilding building)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (building);
		}
	}
}

#endif