#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Spell.Blue;

namespace LotusCraft.Spell.Blue
{
	class Jishin : thrts.SpellParameter<thrts.IUnit>
	{
		public const int StructureBonus = 9;
		
		int[] chargeTime;
		float[] affectRadiuses;

		int[] damages;

		int[] delay;

		public Jishin()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "地震「先憂後楽の剣」";
			string basePath = "Blue/Spell/Jishin/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 60 * 30;
			CoolTimes[2] = 45 * 30;
			CoolTimes[3] = 30 * 30;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 1 * Def.FPS;
			chargeTime[2] = 1 * Def.FPS;
			chargeTime[3] = 1 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 6;
			affectRadiuses[2] = 8;
			affectRadiuses[3] = 10;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 10;
			damages[2] = 20;
			damages[3] = 30;

			delay = new int[MaxLevel + 1];
			delay[0] = 9999;
			delay[1] = 4*Def.FPS;
			delay[2] = 4*Def.FPS;
			delay[3] = 4*Def.FPS;


			Description = "少しの間をおいた後、範囲内の敵にダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"、" + damages[1] + "ダメージ×3"+"、建物に"+StructureBonus+"倍",
				"範囲"+ affectRadiuses[2]+"、" + damages[2] + "ダメージ×3"+"、建物に"+StructureBonus+"倍",
				"範囲"+ affectRadiuses[3]+"、" + damages[3] + "ダメージ×3"+"、建物に"+StructureBonus+"倍",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName(Name);
			if (spell == null || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			// クールタイム再設定
			spell.ResetCoolTime();

			{

				float affectRadius = affectRadiuses[level];

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				// チャージエフェクト
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(JishinChargeEffect).FullName, selfUnit.Pos + offset);
				var scale = (affectRadius)*8.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
				{
					effect.Stop();
				}

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

				for (int i = 0; i < chargeTime[level]; ++i)
				{

					yield return true;
				}



				// stay stop
				selfUnit.Speed = 0.0f;

				selfUnit.AbilityManager.AppendAbility(new JishinDelayAbility(delay[level],damages[level],affectRadius,selfUnit.Pos,null));

			}
			yield return false;
		}
	}
}


namespace LotusCraft.Ability.Blue
{
	/// <summary>
	/// Jishin ability.
	/// </summary>
	class JishinDelayAbility : EffectContinuousAbility
	{
		public const string AbilityName = "地震「先憂後楽の剣」";

		public int Damage = 0;
		public float AffectRadius = 4.0f;

		Vector3D targetPos;

		public JishinDelayAbility (int delay, int damage,float radius,Vector3D pos, IEffect effect = null)
			:base(effect,delay,AbilityName)
		{
			this.Damage = damage;
			this.AffectRadius = radius;
			this.targetPos = pos;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (unit != null && unit.HP > 0 && unit.Exists()) {
				var bulletParam = unit.GetModifiedBulletParameter (typeof(JishinBullet).FullName);

				if (null != bulletParam) {
					var faction = unit.Faction;
					var offset = new Vector3D (0, 0, 1);

					var effect = faction.World.EffectManager.CreateEffect (typeof(JishinAreaEffect).FullName, targetPos + offset);
					var scale = (AffectRadius) * 8.0f;
					effect.SetScale (new Vector3D (scale, scale, scale));
			
					if (!unit.AbilityManager.AppendAbility (new EffectContinuousAbility (effect, bulletParam.Duration, this.Name + "areaEffect"))) {
						effect.Stop ();
					}
			
					faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.Jishin).FullName, targetPos);

					var bullet = unit.CreateBullet (typeof(JishinBullet).FullName, targetPos, targetPos);
					bullet.Attack = Damage;
					bullet.Radius = AffectRadius * Def.MapTipLengthOnField;

					Action<IUnit> vanishUnitAction = null;
					vanishUnitAction= (IUnit u) =>
					{
						if(null != effect)
						{
							effect.Stop ();
							effect = null;
						}
						if(null != bullet)
						{
							bullet.Vanish ();
							bullet = null;
						}
						u.Vanished -= vanishUnitAction;
					};
					unit.Vanished += vanishUnitAction;
				}
			}
			return base.OnVanish (unit);
		}
	}
}

namespace LotusCraft.Bullet.Blue
{
	public class JishinBullet : General.HeroBullet
	{
		public JishinBullet ()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			this.Attack = 0;
			Speed = 0;
			Gravity = 0;
			HitType = BulletHitType.Multi;
			Again = 1 * Def.FPS;
			// 4回(0,1,2,3秒)のうち最後のヒットをさせないために10フレームだけ短く設定
			Duration = 3*Def.FPS - 10;
			Radius = 0;

			MaxRange = 4 * 32;
			
			AttackDelay = (int)(0.5f * 30);
			AttackRigor = (int)(0.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor = 0;
			
			OnHit = OnHitEffect;

			DamageBonus[(int)TargetObjectType.Hero] = 1.0f;
			DamageBonus[(int)TargetObjectType.Structure] = Spell.Blue.Jishin.StructureBonus*1.0f;

		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D(-2,-2,8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(HitCloseRedEffect).FullName,target.Pos+offset);
			var scale = new Vector3D(8,8,8);
			effect.SetScale(scale);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitDageki).FullName, target.Pos);
		}
	}
}

#endif