﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Common;
using LotusCraft.Bullet.Common;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Common
{
	class DefenceSkill:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const int interval = 10;

		public DefenceSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "防衛モード";
			string basePath = "Red/Spell/MeirinSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			Description = "動かざること山の如し";
			/// レベル別の能力説明
			//EffectDescriptions = new string[]
			//{
			//	移動不可,
			//};
			EffectDescriptions = new string[]
			{
				"移動不可となる防衛モードになる。また、毎秒HPが2回復するようになる。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			{
				var unitRadius = selfUnit.Parameter.Radius;
				Vector3D offset = new Vector3D (0.0f, 0.0f, unitRadius/2);

				if (selfUnit.AbilityManager.ContainsAbility(DefenceSkillAbility.AbilityName))
				{
					selfUnit.AbilityManager.ExtendLifetimeWithName(interval,DefenceSkillAbility.AbilityName,interval*2);
				}
				else
				{
					//var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(BackLightEffect).FullName, selfUnit.Faction, selfUnit.Pos + offset);
					//var scale = unitRadius/Def.MapTipLengthOnField * 2;
					//effect.SetScale (new Vector3D (scale,scale,scale));
					
					var effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(DefenceAuraTextureEffect).FullName, selfUnit.Pos);
					
					selfUnit.AbilityManager.AppendAbility(new DefenceSkillAbility(effect,offset,interval*2));

				}
			}
			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}
	}
}

namespace LotusCraft.Ability
{
	class DefenceSkillAbility : AbilityTemplate
	{
		public const string AbilityName = "防衛モード";
		public IEffectCommon effect = null;
		int counter = 0;
		int HealDelay = Def.FPS;
		public DefenceSkillAbility (IEffectCommon effect,Vector3D offset,int duration)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.CanStateNow = false;
				//飛行していた場合、ホールドポジションを解除する
				/*
				if(unit.FlyingState == FlyingState.Fly)
				{
					unit.Speed = 1.0f*UnitParameter.BasisSpeed;
				}else{
				*/
				// 移動しようとしていたらホールドポジションに強制変更
				if(!unit.IsHoldPosition || unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);
				// set speed=0;
				unit.Speed = 0.0f;
				
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				//回復
				counter++;
				if (counter > HealDelay && unit.HP < unit.MaxHP)
				{
					counter = 0;
					unit.HP += 2;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
}
