﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Common;
using LotusCraft.Bullet.Common;
using LotusCraft.Ability;
using LotusCraft.Ability.Common;


namespace LotusCraft.Spell.Common
{
	class ShinsokuSkill:thrts.SpellParameter<thrts.IUnit>
	{
		protected const int duration = 2*Def.FPS;
		public const string SpellName =  "突撃";
		public ShinsokuSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = SpellName;

			//string basePath = "Black/Spell/KokoroSkill/";
			//basePath = "Black/Spell/KokoroSkill/Kuruma/";
			string basePath = "Red/Spell/KamiNoKaze/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes [0] = 99 * Def.FPS;
			CoolTimes [1] = 28 * Def.FPS;

			Description = "移動速度と攻撃力が短時間だけ大幅に上昇しする。攻撃すると効果は終了する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"2秒間移動速度が+1.0、攻撃力が1.25倍",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
				
			};
		}
		
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			selfUnit.GetSpellWithName (Name).ResetCoolTime ();
			
			var world = selfUnit.Faction.World;
			
			
			{
				/// show effect
				if (selfUnit.AbilityManager.ContainsAbility (ShinsokuSkillAbility .AbilityName)) {
					selfUnit.AbilityManager.ExtendLifetimeWithName (duration, ShinsokuSkillAbility .AbilityName, duration);
				} else {
					IEffect effect = null;
					Vector3D offset = new Vector3D (2.0f, 2.0f, 4.0f);
					var e = world.EffectManager.CreateEffect (typeof(Effect.Common.FastBombEffect).FullName, selfUnit.Pos + offset);
					e.SetScale (new Vector3D (8, 8, 8));
					effect = e;
					
					var ability = new ShinsokuSkillAbility (effect, offset, duration);
					if (!selfUnit.AbilityManager.AppendAbility (ability))
						effect.Stop ();
				}
			}
			
			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Common
{
	class ShinsokuSkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "突撃";
		const float incSpeedRatio = 0.6f;
		const float incAtkRatio = 0.25f;
		public ShinsokuSkillAbility(IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				//unit.SelectBullet = (u,t,d) => -1;

				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				//unit.ViewRange += 4*Def.MapTipLengthOnField;
				//unit.DamageRatio = 0.75f;
				//unit.Speed += 1.0f*UnitParameter.BasisSpeed;
				unit.Speed += incSpeedRatio*UnitParameter.BasisSpeed;
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += (int)Math.Ceiling(b.Attack*incAtkRatio);
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			bullet.AddOnHitEvent(OnHit);
			return base.ModifyBullet (bullet);
		}
		public override bool OnVanish (IUnit unit)
		{
			var spell = unit.GetSpellWithName(Spell.Common.ShinsokuSkill.SpellName);
			if(null != spell)
				spell.BeDecCooltime = true;
			return base.OnVanish (unit);
		}
		public void OnHit(IBullet bullet,ITargetObject target)
		{
			this.DurationTime = 0;
		}
	}
}