﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class BlackHole : thrts.SpellParameter<thrts.IUnit>
	{

		int[] chargeTime;
		float[] affectRadiuses;
		float[] decMoveRatio;

		const int duration = 8 * Def.FPS;
		const int interval = 1;
		public BlackHole()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "符の参「追儺返しブラックホール」";
			string basePath = "Green/Spell/BlackHole/";


			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 25 * 30;
			CoolTimes[2] = 20 * 30;
			CoolTimes[3] = 15 * 30;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * 32;
			MaxRanges[2] = 7 * 32;
			MaxRanges[3] = 8 * 32;

			chargeTime = new int[MaxLevel + 1];
			chargeTime[0] = 0;
			chargeTime[1] = 2 * Def.FPS;
			chargeTime[2] = 2 * Def.FPS;
			chargeTime[3] = 2 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;

			decMoveRatio = new float[MaxLevel + 1];
			decMoveRatio[0] = 0;
			decMoveRatio[1] = 0.2f;
			decMoveRatio[2] = 0.3f;
			decMoveRatio[3] = 0.4f;

			Description = "指定地点の周囲に居る敵兵ユニットの移動速度を低下させ、\nさらに指定地点へ引き寄せる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"範囲"+ affectRadiuses[1]+"、移動速度-" + decMoveRatio[1].ToString("###.%"),
				"範囲"+ affectRadiuses[2]+"、移動速度-" + decMoveRatio[2].ToString("###.%"),
				"範囲"+ affectRadiuses[3]+"、移動速度-" + decMoveRatio[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName(Name);
				if (null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}

				spell.ResetCoolTime();

				float affectRadius = affectRadiuses[level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName, targetPos + offset, selfUnit.Faction);
				var scale = 1.0f * (affectRadius);
				effect.SetScale(new Vector3D(scale / 3 * 2, scale / 3 * 2, scale / 3 * 2));

				if (!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(effect, chargeTime[level], this.Name + "chargeEffect")))
					effect.Stop();

				for (int i = 0; i < chargeTime[level]; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

				// stay stop
				selfUnit.Speed = 0.0f;

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(BlackHoleEffect).FullName, targetPos + offset, selfUnit.Faction);
				areaEffect.SetScale(new Vector3D(scale * 8, scale * 8, scale * 8));

				var ability = new SpellContinuousAbility((unit) => Spelling(unit, targetPosition, level), areaEffect, offset, duration, this.Name + "areaEffect", interval);
				ability.FollowEffect = false;
				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();
				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin02).FullName, selfUnit.Pos);

			}
			yield return false;
		}
		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			;
		}

		public bool Spelling(IUnit selfUnit, Vector2D targetPosition, int level)
		{
			float affectRadius = affectRadiuses[level];

			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);

			// effect offset
			Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

			var selfFaction = selfUnit.Faction;

			// search targets
			var targets = selfFaction.World.GetObjectFromCircle(
				new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(targetPos.X, targetPos.Y))).
				OfType<IUnit>().
				Where(_ => _ != null && _.Exists() && !selfFaction.World.IsSameTeam(_.Faction, selfFaction) && !_.Parameter.IsFarmer && _.UniqueID != selfUnit.UniqueID);

			foreach (var u in targets)
			{
				var ability = u.AbilityManager.GetAbilityWithName(BlackHoleAbility.AbilityName) as BlackHoleAbility;
				if (null == ability)
				{
					u.AbilityManager.AppendAbility(new BlackHoleAbility(2 * interval, decMoveRatio[level], null));
				}
				else
				{
					ability.DecMoveRatio = decMoveRatio[level];
					ability.ExtendLifetime(interval,interval*2);
				}
				Vector3D diff = targetPos - u.Pos;
				float distance = (float)Math.Sqrt (diff.X * diff.X + diff.Y * diff.Y);
				if (distance > Def.MapTipLengthOnField)
				{
					var front = u.GetFrontDirection();
					diff.X /= distance;
					diff.Y /= distance;

//					u.Warp( new Vector2D(u.Pos.X+1 * interval * diff.X, u.Pos.Y+1 * interval * diff.Y) ,true);
					u.TryMove(1 * interval * diff.X, 1 * interval * diff.Y,true);
					u.ChangeDirection(front);
				}


			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class BlackHoleAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		public float DecMoveRatio;
		public const string AbilityName = "符の参「追儺返しブラックホール」";
		public BlackHoleAbility (int duration,float decMoveRatio, IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
			this.DecMoveRatio = decMoveRatio;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= unit.Speed*DecMoveRatio;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			
			return base.OnVanish (unit);
		}
	}
}

#endif