﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class DollsWar:thrts.SpellParameter<thrts.IUnit>
	{
		DollsWarAbility[] spellAbilities = new []
		{
			new DollsWarAbility(0),
			new DollsWarAbility(1),
			new DollsWarAbility(2),
			new DollsWarAbility(3),
		};
		int autocastInterval = thrts.Def.FPS;
		public static string SpellName = "戦操「ドールズウォー」";
		public DollsWar()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			Description = "周囲にいる人形ユニットを強化する。";
			string basePath = "Green/Spell/DollsWar/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1秒に"+(spellAbilities[1].healHP)+"回復、攻撃力+"+spellAbilities[1].incAtk+"、移動+"+spellAbilities[1].incMove+"、人形の最大数 +2体",
				"1秒に"+(spellAbilities[2].healHP)+"回復、攻撃力+"+spellAbilities[2].incAtk+"、移動+"+spellAbilities[2].incMove+"、人形の最大数 +4体",
				"1秒に"+(spellAbilities[3].healHP)+"回復、攻撃力+"+spellAbilities[3].incAtk+"、移動+"+spellAbilities[3].incMove+"、人形の最大数 +6体",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,autocastInterval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,autocastInterval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,autocastInterval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,autocastInterval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			// set effect position

			/// get farction and world
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			/// get dolls
			var dolls = world.UnitManager.GetUnits (_ => _.Faction == selfFaction && _.Parameter.GetType ().FullName == typeof(Dolls).FullName);
			foreach (var u in dolls) {
				if(u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					// extention ability time
					u.AbilityManager.ExtendLifetimeWithName(autocastInterval,spellAbilities[level].Name,spellAbilities[level].DurationTime);
				}
				else
				{
					// add effect
					//var effect  = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(WarEffect).FullName, u.Pos + new thrts.Vector3D(1,1,1));
					//effect.SetScale(new thrts.Vector3D(8,8,8));

					IEffectCommon effect = null;
					effect = selfUnit.Faction.World.EffectManager.CreateTextureEffect(typeof(WarTextureEffect).FullName, u.Pos);
					
					// add dollswar ability
					if(!u.AbilityManager.AppendAbility(new DollsWarAbility(level,effect)))
						effect.Stop();
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class DollsWarAbility : AbilityTemplate
	{
		public int incAtk = 0;
		public float incMove = 0;
		public int healHP = 0;
		public int interval = Def.FPS;
		int counter;
		public IEffectCommon effect;

		public DollsWarAbility(int level,IEffectCommon effect = null)
		{
			this.Name = "戦操「ドールズウォー」 lv"+level.ToString();
			this.DurationTime = 1*Def.FPS;
			counter = 0;
			this.effect = effect;
			
			switch (level)
			{
			case 1:
				incAtk = 4;
				incMove = 0.1f;
				healHP = 1;
				break;
			case 2:
				incAtk = 6;
				incMove = 0.2f;
				healHP = 2;
				break;
			case 3:
				incAtk = 8;
				incMove = 0.3f;
				healHP = 2;
				break;
			default:
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.Speed += incMove*UnitParameter.BasisSpeed;
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += incAtk;
				
				if(++counter > interval)
				{
					counter = 0;
					unit.HP += healHP;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			
			return base.ModifyBullet(bullet);
		}
		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
	}
	
}
