﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Building.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Unit.Green;


namespace LotusCraft.Spell.Green
{
	class Kaika:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 15;
		const int flowerDuration = 15*Def.FPS;
		int[] maxNum = new int[4];
		public Kaika()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "花符「幻想郷の開花」";
			string basePath = "Green/Spell/Kaika/";

			maxNum = new int[MaxLevel+1];
			maxNum[0] = 0;
			maxNum[1] = 3;
			maxNum[2] = 5;
			maxNum[3] = 7;

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 99*Def.FPS;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;
			
			Description += "周囲の花畑から敵ユニットを攻撃する向日葵を出現させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"出現数 " + (maxNum[1] - maxNum[0]),
				"出現数 " + (maxNum[2] - maxNum[0]),
				"出現数 " + (maxNum[3] - maxNum[0]),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// スキマアビリティ取得のためのオートキャストを設定
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists()) {
				yield return false;
				yield break;
			}

			var spell = selfUnit.GetSpellWithName(this.Name);

			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			var ret = Spelling(selfUnit,spell.Level);
			if(ret)
				spell.ResetCoolTime();

			yield return ret;
		}

		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if(level < 0 || MaxLevel < level)
				return false;

			// get doll count
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var pos = new Vector2D(0,0);

			int num = 0;
			for(int r = 1 ; r < 4 && num < maxNum[level]; ++r)
			{
				for(int j = 0; j < r*maxNum[level];++j)
				{
					pos.X = selfUnit.Pos.X + r * (float)Math.Cos( 2*Math.PI * j/(r*maxNum[level]) ) * 1.5f * Def.MapTipLengthOnField;
					pos.Y = selfUnit.Pos.Y + r * (float)Math.Sin( 2*Math.PI * j/(r*maxNum[level]) ) * 1.5f * Def.MapTipLengthOnField;
					var tip = selfFaction.World.Map.GetMapTipIndex(pos.X,pos.Y);

					if(selfFaction.World.Map.IsSpecialArea(tip.X,tip.Y))
					{
						var unit = selfUnit.Faction.World.UnitManager.Create(typeof(FlowerUnit).FullName,selfFaction,pos.X,pos.Y,selfFaction);
						if(null != unit)
						{
							num++;
							unit.AbilityManager.AppendAbility(new Ability.SuicideAbility(null,flowerDuration,this.Name+"suicide"));

							// 中に何かあったら外に出す
							var targets = world.GetObjectFromCircle (new Circle2D(unit.Radius + Def.MapTipLengthOnField/4,pos)).OfType<IUnit> ();
							
							foreach (var tu in targets) {
								if(selfFaction.World.IsSameTeam(tu.Faction,selfFaction))
									continue;

								tu.Warp(new Vector2D(tu.Pos.X,tu.Pos.Y),true);
							}

							if(num >= maxNum[level])
								break;
						}
					}
				}

			}
			

			return num != 0;
		}
	}
}

namespace LotusCraft.Unit.Green
{
	public class FlowerUnit : thrts.UnitParameter
	{
		public FlowerUnit ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Green/SunFlower";
			#endregion

			#region UnitParams
			Name = "向日葵";
			ButtonDescription += "花畑に踏み入る不届きもをを攻撃する向日葵。一定時間経つと枯れる。\n";
			HP = 120;
			Armor = 0;
			Radius = (float)Utls.UnitSize.Small;
			Speed = 0.0f * BasisSpeed;
			Exp = 0;
			
			IsSummon = true;
			CanState = false;

			PopulationWeight = 0;
			
			ViewRange = 8*Def.MapTipLengthOnField;
			
			MakingTime = 0 * 30;
			
			// lowest priority
			SelectPriority = 999;
			
			/// mine,gas,sup,?
			Costs = new [] {0,0,0,0};
			#endregion
			
			#region Animation
			this.SetFlowerAnimation("LotusCraft/Image/Unit/Green/SunFlower");
			Radius = 0;
			#endregion
			
			#region BuildableObjcets
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(FlowerBullet).FullName, };
			#endregion
			
		}
		public override void OnLoad (IFaction faction)
		{
			
		}
		
		public override void OnVanish (IUnit unit)
		{
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
		public override void OnAppear (IUnit unit)
		{
			unit.HoldPosition(false);
			unit.Updating += () => 
			{
				var selfTip = unit.Faction.World.Map.GetMapTipIndex(unit.Pos.X,unit.Pos.Y);
				if(!unit.Faction.World.Map.IsSpecialArea(selfTip.X,selfTip.Y))
					unit.HP = 0;
				if(unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);
				
			};
			
			base.OnAppear (unit);
		}
	}
}

namespace LotusCraft.Bullet.Green
{
	public class FlowerBullet : General.LongRangeBullet
	{
		public FlowerBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			DoesHitOnlyTarget = true;
			this.Attack = 10 ;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;

			MinRange = 3*Def.MapTipLengthOnField;
			MaxRange = 7*Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(0.0f * 30);
			AttackSpeed  = (int)(0.8f * 30);
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			TextureAnimation.Offset.X = 128 * 3 + 16 * 3;
			TextureAnimation.Offset.Y = 128 * 0 + 16 * 0;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}