﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Effect.Green;
using LotusCraft.AutoCast;
using LotusCraft.Token;

namespace LotusCraft.Spell.Green
{
	class KurumiSkill : thrts.SpellParameter<thrts.IUnit>
	{
		int[] bounds;
		const int interval = 10;

		public KurumiSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "くるみスキル";
			string basePath = "Green/Spell/KurumiSkill/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 10 * 30;

			bounds = new int[MaxLevel + 1];
			bounds[0] = 0;
			bounds[1] = 2 ;

			Description =  "弾が多段ヒットする";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"多段" + bounds[1] + "回",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1", 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (IUnit selfUnit, int level)
		{
//			var selfFaction = selfUnit.Faction;
//			var world = selfFaction.World;

			var ability = selfUnit.AbilityManager.GetAbilityWithName(KurumiSkillAbility.AbilityName) as KurumiSkillAbility;
			if (null != ability) {
				ability.ExtendLifetime (interval,interval*2);
				ability.BounsCount = bounds[level];
			}
			else {
				IEffect effect = null;
				if(selfUnit.AbilityManager.AppendAbility(new KurumiSkillAbility(bounds[level],interval*2,effect)))
				{
					;
				}
			}
			 

			return true;
		}
	}
}


namespace LotusCraft.Ability.Green
{
	class KurumiSkillAbility : FollowEffectAbility
	{
		public const string AbilityName = "くるみスキル";
		public int BounsCount;
		public KurumiSkillAbility(int bounsCount,int duration,IEffect effect = null)
			:base()
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.Effect = effect;
			this.BounsCount = bounsCount;
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			var u = bullet.Generator as IUnit;

			if (null != u) {
				var bulletParam = u.GetModifiedBulletParameter (typeof(KurumiBullet).FullName);
				if (null != bulletParam && bullet.Parameter == bulletParam) {
					switch (BounsCount) {
					case 0:
						break;
					case 1:
						bullet.AddOnHitEvent ((b,t) => KurumiBounsBullet.Bouns (b, t, typeof(KurumiBounsBullet).FullName));
						break;
					case 2:
						bullet.AddOnHitEvent ((b,t) => KurumiBounsBullet.Bouns (b, t, typeof(KurumiBounsBullet1).FullName));
						break;
					case 3:
						bullet.AddOnHitEvent ((b,t) => KurumiBounsBullet.Bouns (b, t, typeof(KurumiBounsBullet2).FullName));
						break;
					default:
						bullet.AddOnHitEvent ((b,t) => KurumiBounsBullet.Bouns (b, t, typeof(KurumiBounsBullet3).FullName));
						break;
					}
				}
			}
			return base.ModifyBullet (bullet);
		}
	
	}
}

namespace LotusCraft.Bullet.Green
{
	public class KurumiBounsBullet:KurumiBullet
	{
		const float bousRatio = 1.0f; //  damage ratio
		const int bounsNum = 1;
		public KurumiBounsBullet ()
		{
			DoesHitOnlyTarget = true;
		}
		public static void Bouns (IBullet bullet, ITargetObject target, string bulletKind)
		{
			if(null == target)
				return;
			var u = bullet.Generator as IUnit;
			if (u == null)
				return;

			var bulletParam = u.GetModifiedBulletParameter(bulletKind);
			if(null == bulletParam)
				return;

			var world = u.Faction.World;

			float offsetR = 0;
			{
				var tb = target as IBuilding;
				var tu = target as IUnit;
				if (null != tb) {
					offsetR = tb.Parameter.Size * 0.707f;
				}
				if (null != tu) {
					offsetR = tu.Radius;
				}
				
				// shift
				offsetR += 3;
			}

			var targets = world.GetObjectFromCircle (new Circle2D (bulletParam.MaxRange + offsetR, new Vector2D (target.Pos.X, target.Pos.Y)))
				.Where (_ => null != _ && _ != target && !world.IsSameTeam (u.Faction, _.Faction))
					.OrderByDescending (_ => (_.Pos - target.Pos).LengthSquared);



			int cnt = 0;
			foreach (var t in targets.OfType<IUnit>()) {
				if (cnt >= bounsNum)
					break;

				IBullet b = null;
				if (null != t && t.Exists()) {
					var d = (t.Pos - target.Pos);
					d.Normalize ();
					
					var initPos = target.Pos + d * offsetR;
					while(t.IsHit(initPos.X,initPos.Y,1))
						initPos += d;
					
					b = u.CreateBullet (bulletKind, t, initPos);
				}
				if (b != null)
				{
					b.Attack = (int)Math.Ceiling(bullet.Attack * bousRatio);
					cnt++;
				}
				
			}
			foreach (var t in targets.OfType<IBuilding>()) {
				if (cnt >= bounsNum)
					break;

				IBullet b = null;
				if (null != t && t.Exists()) {
					var d = (t.Pos - target.Pos);
					d.Normalize ();
					
					var initPos = target.Pos + d * offsetR;
					while(t.IsHit(initPos.X,initPos.Y,1))
						initPos += d;
					
					b = u.CreateBullet (bulletKind, t, initPos);
				}
				if (b != null)
				{
					b.Attack = (int)Math.Ceiling(bullet.Attack * bousRatio);
					cnt++;
				}
				
			}
			// 対象が居なければランダムに跳ね返り
			// くるみの場合は無し
			/*
			for(int i=0;i< bounsNum - cnt;++i)
			{
				var th = world.NextRandDouble () * Math.PI * 2;
				var tgt = new Vector3D ((float)Math.Sin (th), (float)Math.Cos (th), 0);
				var initPos = target.Pos + tgt * offsetR;
				b = u.CreateBullet (bulletKind, tgt * (bulletParam.MaxRange) + initPos, initPos);

				if (b != null)
					b.Attack = (int)Math.Ceiling (bullet.Attack * bousRatio);
			}
			*/
		}
	}
	/// <summary>
	/// Bouns 1 times.
	/// </summary>
	public class KurumiBounsBullet1:KurumiBounsBullet
	{
		public KurumiBounsBullet1 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(KurumiBounsBullet).FullName);
		}
	}
	/// <summary>
	/// Bouns 2 times.
	/// </summary>
	public class KurumiBounsBullet2:KurumiBounsBullet
	{
		public KurumiBounsBullet2 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(KurumiBounsBullet1).FullName);
		}
	}
	/// <summary>
	/// Bouns 3 times.
	/// </summary>
	public class KurumiBounsBullet3:KurumiBounsBullet
	{
		public KurumiBounsBullet3 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(KurumiBounsBullet2).FullName);
		}
	}
	/// <summary>
	/// Bouns 4 times.
	/// </summary>
	public class KurumiBounsBullet4:KurumiBounsBullet
	{
		public KurumiBounsBullet4 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(KurumiBounsBullet3).FullName);
		}
	}
}

#endif