﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;
using LotusCraft.Spell.Green;

namespace LotusCraft.Spell.Green
{
	class Michibiku : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 4;
		const int interval = 5;
		public const string MichibikiUnitCountKey = "michibiku_unit_count";

		int[] maxUnitCount;
		float[] durationTime;
		public Michibiku()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "動物を導く程度の能力";
			string basePath = "Green/Spell/Michibiki/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*30;
			CoolTimes[2] = 30*30;
			CoolTimes[3] = 30*30;
			
			maxUnitCount = new int[MaxLevel+1];
			maxUnitCount[0] = 0;
			maxUnitCount[1] = 3;
			maxUnitCount[2] = 5;
			maxUnitCount[3] = 7 ;
			durationTime = new float[MaxLevel+1];
			durationTime[0] = 0;
			durationTime[1] = 8;
			durationTime[2] = 10;
			durationTime[3] = 12;

			Description = "範囲内の中立ユニットや敵ユニットを一定時間味方にする。\n効果範囲から出ると効果が解ける。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"効果"+durationTime[1]+"秒、中立のみ、最大"+maxUnitCount[1]+"体",
				"効果"+durationTime[2]+"秒、中立と召喚ユニット、最大"+maxUnitCount[2]+"体",
				"効果"+durationTime[3]+"秒、中立と召喚ユニットと下級ユニット、最大"+maxUnitCount[3]+"体",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			if (!selfUnit.AbilityManager.ContainsAbility (this.Name + "areaEffect")) {
				var offset = new Vector3D (2, 2, 1);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MichibikuAreaEffect).FullName, selfUnit.Pos);
				var scale = affectRadius * 10;
				effect.SetScale (new Vector3D (scale, scale, scale));
				effect.SetPosition (selfUnit.Pos + offset);
				if (! selfUnit.AbilityManager.AppendAbility (new SpellContinuousAbility ((unit)=>Spelling(unit,level),effect, offset, (int)(durationTime[level]*Def.FPS), this.Name + "areaEffect",interval)))
					effect.Stop ();
				else
					spell.ResetCoolTime ();
				
			}
			

			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;


			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit> ();
			foreach (var u in targets) {
				if (null == u || !u.Exists () || u.Parameter.IsUnique || u.Parameter.IsFarmer)
					continue;
				if(!(u.Faction.Parameter.GetType() == typeof(Faction.Neutral) || u.IsSummon) && (u.Parameter is LotusCraft.Unit.General.Special))
					continue;
				if(u.Parameter.GetType() == typeof(Unit.Common.Eiki) || u.Parameter.GetType() == typeof(Unit.Common.Mercenary01) || u.Parameter.GetType() == typeof(Unit.Common.Mercenary02))
					continue;

				if(level < 3 && !(u.Faction.Parameter.GetType() == typeof(Faction.Neutral) || u.IsSummon))
					continue;

				if(level < 2 && !(u.Faction.Parameter.GetType() == typeof(Faction.Neutral)))
					continue;



				var ability = u.AbilityManager.GetAbilityWithName (MichibikuAbility.AbilityName) as MichibikuAbility;
				
				if (!selfFaction.World.IsSameTeam (u.Faction, selfFaction)) {
					if (null == ability) {
						// 選択最大数以上なら追加しない
						if(selfUnit.UserValues.ContainsKey(MichibikiUnitCountKey))
						{
							var michibikuUnitCount = (int)selfUnit.UserValues[MichibikiUnitCountKey];
							if(michibikuUnitCount >= maxUnitCount[level])
								continue;
						}
						var offset = new Vector3D (2, 2, 1);
						var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(MichibikuEffect).FullName, u.Pos);
						var scale = 1 * 5;
						effect.SetScale (new Vector3D (scale, scale, scale));
						effect.SetPosition (u.Pos + offset);
						if (! u.AbilityManager.AppendAbility (new MichibikuAbility (selfUnit, u.Faction, effect, offset, interval * 2)))
							effect.Stop ();
						else
						{
							// 
							u.ChangeFaction(selfFaction);

							if(!selfUnit.UserValues.ContainsKey(MichibikiUnitCountKey))
								selfUnit.UserValues.Add(MichibikiUnitCountKey,0);

							var michibikuUnitCount = (int)selfUnit.UserValues[MichibikiUnitCountKey];
							selfUnit.UserValues[MichibikiUnitCountKey] = michibikuUnitCount+1;
						}
					}
					else if(ability.Speller.UniqueID == selfUnit.UniqueID)
					{
						ability.ExtendLifetime (interval,interval*2);
					}
				} else if(null != ability && ability.Speller.UniqueID == selfUnit.UniqueID) {
					ability.ExtendLifetime (interval,interval*2);
				}
			}
			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class MichibikuAbility : FollowEffectAbility
	{
		public const string AbilityName = "動物を導く程度の能力";
		public IFaction OrigFaction = null;
		public IUnit Speller = null;
		public MichibikuAbility (IUnit speller,IFaction origFaction,IEffect effect, Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			OrigFaction = origFaction;
			Speller = speller;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != OrigFaction && null != unit && unit.Exists ())
				unit.ChangeFaction (OrigFaction);
			if (null != Speller && Speller.Exists () && Speller.UserValues.ContainsKey(Michibiku.MichibikiUnitCountKey)) {
				var mitibikuUnitCount = (int)Speller.UserValues[Michibiku.MichibikiUnitCountKey];
				Speller.UserValues[Michibiku.MichibikiUnitCountKey] = mitibikuUnitCount - 1;

			}
			return base.OnVanish (unit);
		}
	}
}

#endif