﻿#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Unit.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class MissingPower:thrts.SpellParameter<thrts.IUnit>
	{
		public static string SpellName = "鬼符「ミッシングパワー」";
		const int interval = 10;
		int[] incAtk;
		int[] incArmor;
		public MissingPower()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			string basePath = "Green/Spell/MissingPower/";

			IsPassive= true;

			incAtk = new int[MaxLevel + 1];
			incAtk[0] = 0;
			incAtk[1] = 3;
			incAtk[2] = 5;
			incAtk[3] = 8;

			incArmor = new int[MaxLevel + 1];
			incArmor[0] = 0;
			incArmor[1] = 3;
			incArmor[2] = 5;
			incArmor[3] = 8;

			Description = " 伊吹萃香1体を対象とする。それはゲーム終了時まで以下の修正を受ける。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"防御力+" + incArmor[1] + "/攻撃力+" + incAtk[1] ,
				"防御力+" + incArmor[2] + "/攻撃力+" + incAtk[2] ,
				"防御力+" + incArmor[3] + "/攻撃力+" + incAtk[3] ,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var ability = selfUnit.AbilityManager.GetAbilityWithName (MissingPowerAbility.AbilityName) as MissingPowerAbility;
			if (null != ability)
			{
				ability.IncAtk = incAtk[level];
				ability.incArmor = incArmor[level];
				ability.ExtendLifetime(interval, Def.FPS);
			}
			else
			{
				selfUnit.AbilityManager.AppendAbility(new MissingPowerAbility(incAtk[level], incArmor[level], selfUnit));
			}

			return true;
		}
	}
}
namespace LotusCraft.Ability.Green
{
	class MissingPowerAbility : thrts.AbilityTemplate
	{
		public int IncAtk = 0;
		public int incArmor = 0;
		public const string AbilityName = "鬼符「ミッシングパワー」";

		IEffect effect = null;
		IUnit unit = null;

		public MissingPowerAbility(int incAtk,int incArmor, IUnit unit)
		{
			this.Name = AbilityName;
			this.DurationTime = 1*Def.FPS;
			this.IncAtk = incAtk;
			this.incArmor = incArmor;

			this.unit = unit;
			this.effect = unit.Faction.World.EffectManager.CreateEffect(typeof(SuikaMPEffect).FullName, unit.Pos + new Vector3D(0, 0, 4));
			this.effect.SetScale(new Vector3D(6, 6, 6));
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case AbilityOrder.ConstIncParam:
					unit.Armor += incArmor;
					
					var b = unit.GetModifiedBulletParameter(typeof(SuikaBullet).FullName);
					if(null != b)
					{
						b.Attack += IncAtk;
					}
					break;
				case AbilityOrder.OtherExecs:
					effect.SetPosition(unit.Pos + new Vector3D(0, 0, 4));
					break;
				default:
					break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}

		public override bool OnVanish(IUnit unit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
	}
}

#endif