﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Unit.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class MissingPurplePower:thrts.SpellParameter<thrts.IUnit>
	{
		Dictionary<float,float>[] splashs;
		int[] incArmors;
		float[] incAtkRatios;
		int[] durations;
		public MissingPurplePower()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "鬼神「ミッシングパープルパワー」";
			string basePath = "Green/Spell/MissingPurplePower/";
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*30;

			splashs = new Dictionary<float, float>[]
			{
				new Dictionary<float, float>(),
				new Dictionary<float, float>(){{1.0f*Def.MapTipLengthOnField,1.0f},{1.2f*Def.MapTipLengthOnField,0.8f},{1.8f*Def.MapTipLengthOnField,0.5f},{2.5f*Def.MapTipLengthOnField,0.25f},},
			};

			incArmors = new int[MaxLevel+1];
			incArmors[0] = 0;
			incArmors[1] = 6;
			durations = new int[MaxLevel +1];
			durations[0] = 0*Def.FPS;
			durations[1] = 12*Def.FPS;

			incAtkRatios = new float[MaxLevel+1];
			incAtkRatios[0] = 0;
			incAtkRatios[1] = 0.5f;

			Description = "自身を大幅に強化する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力+"+incAtkRatios[1].ToString("###.%")+"、防御力+"+incArmors[1]+"、範囲攻撃2.5追加、"+durations[1]/Def.FPS+"秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (selfUnit == null || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime ();
			selfUnit.ShowCutin ();

			if (selfUnit.AbilityManager.ContainsAbility (MissingPurplePowerAbility.AbilityName))
				selfUnit.AbilityManager.ExtendLifetimeWithName (durations [level], MissingPurplePowerAbility.AbilityName,durations[level]);
			else {
				// effect
				
				IEffectCommon effect = null;

				if(!selfUnit.AbilityManager.AppendAbility(new MissingPurplePowerAbility(durations[level],incArmors[level],incAtkRatios[level],splashs[level], selfUnit)) && null != effect)
				{
					effect.Stop ();
					effect = null;
				}
			}
			yield return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class MissingPurplePowerAbility : thrts.AbilityTemplate
	{
		public float IncAtkRatio = 0;
		public int incArmor = 0;
		public Dictionary<float, float> splash = null;
		public const string AbilityName = "鬼神「ミッシングパープルパワー」";

		IEffect effect = null;
		IUnit unit = null;

		public MissingPurplePowerAbility(int duration, int incArmor, float incAtkRatio, Dictionary<float, float> splash, IUnit unit)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.IncAtkRatio = incAtkRatio;
			this.incArmor = incArmor;
			this.splash =splash;

			this.unit = unit;
			this.effect = unit.Faction.World.EffectManager.CreateEffect(typeof(SuikaMPEffect).FullName, unit.Pos + new Vector3D(0, 0, 4));
			this.effect.SetScale(new Vector3D(12, 12, 12));
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			BulletParameter b = null;
			switch (order)
			{
			case AbilityOrder.ConstIncParam:
				unit.Armor += incArmor;
				
				b = unit.GetModifiedBulletParameter(typeof(SuikaBullet).FullName);
				if(null != b)
				{
					if(null != splash)
					{
						b.HasSplash = true;
						b.SplashDamageRates = splash;
					}
				}

				unit.Parameter.Animation.Scale.X = 2.0f;
				unit.Parameter.Animation.Scale.Y = 2.0f;
				unit.Parameter.AnimationFrame.Scale.X = 2.0f;
				unit.Parameter.AnimationFrame.Scale.Y = 2.0f;

				unit.Parameter.AnimationWalk.Scale.X = 2.0f;
				unit.Parameter.AnimationWalk.Scale.Y = 2.0f;
				unit.Parameter.AnimationWalkFrame.Scale.X = 2.0f;
				unit.Parameter.AnimationWalkFrame.Scale.Y = 2.0f;

				break;
			case AbilityOrder.RateIncParam:
				b = unit.GetModifiedBulletParameter(typeof(SuikaBullet).FullName);
				if(null != b)
				{
					b.Attack += (int)Math.Ceiling(IncAtkRatio * b.Attack);
				}
				
				break;
			case AbilityOrder.OtherExecs:
				effect.SetPosition(unit.Pos + new Vector3D(0, 0, 4));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}

		public override bool OnVanish(IUnit unit)
		{
			unit.Parameter.Animation.Scale.X = 1.0f;
			unit.Parameter.Animation.Scale.Y = 1.0f;
			unit.Parameter.AnimationFrame.Scale.X = 1.0f;
			unit.Parameter.AnimationFrame.Scale.Y = 1.0f;

			unit.Parameter.AnimationWalk.Scale.X = 1.0f;
			unit.Parameter.AnimationWalk.Scale.Y = 1.0f;
			unit.Parameter.AnimationWalkFrame.Scale.X = 1.0f;
			unit.Parameter.AnimationWalkFrame.Scale.Y = 1.0f;

			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}

			return base.OnVanish(unit);
		}
	}
}



#endif