﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Green;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Spell.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Unit.Green;

namespace LotusCraft.Spell.Green
{
	class Parade : thrts.SpellParameter<thrts.IUnit>
	{
		public static string SpellName = "人形「レミングスパレード」";
		const int interval = 10;
		const int duration = 18 * Def.FPS;
		public Parade()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = SpellName;
			string basePath = "Green/Spell/Parade/";

			CoolTimes = new int[2];
			CoolTimes[0] = int.MaxValue;
			CoolTimes[1] = 180 * Def.FPS;

			Description = "人形の生産時間が減少し、人形の攻撃力等が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				duration/Def.FPS+"秒間 人形の攻撃力+30%、移動速度+30%、被ダメージ+30%。\n人形の最大数 +6体。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>Spelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> Spelling(thrts.IUnit selfUnit, int level)
		{
			var spell = selfUnit.GetSpellWithName(this.Name);
			if (spell == null || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			var selfEffectOffset = new Vector3D(0.0f, 0.0f, 4.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ParadeSelfEffect).FullName, selfUnit.Pos);
			var scale = 3.5f * 8;
			effect.SetScale(new Vector3D(scale, scale, scale));
			effect.SetPosition(selfUnit.Pos + selfEffectOffset);

			if (!selfUnit.AbilityManager.ContainsAbility(ParadeAliceAbility.AbilityName))
			{
				if (!selfUnit.AbilityManager.AppendAbility(new ParadeAliceAbility(duration, effect, selfEffectOffset)))
					effect.Stop();
				else
				{
					spell.ResetCoolTime();
					// カットイン
					selfUnit.ShowCutin();
				}
			}

			yield return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class ParadeAliceAbility : FollowEffectAbility
	{
		public static string AbilityName = "人形「レミングスパレード」";
		public ParadeAliceAbility(int duration,IEffect effect, Vector3D offset)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.Effect = effect;
			this.Offset = offset;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.OtherExecs:
				/// get farction and world
				var selfFaction = unit.Faction;
				var world = selfFaction.World;
				
				/// get doll count
				var dolls = world.UnitManager.GetUnits (_ => _.Faction == selfFaction && _.Parameter.GetType ().FullName == typeof(Dolls).FullName);
				foreach (var u in dolls) {
					if(!u.AbilityManager.ContainsAbility(ParadeDollAbility.AbilityName))
					{
						//var effect = world.EffectManager.CreateEffect (typeof(ParadeDollEffect).FullName,u.Pos);
						//var scale = 3.0f * 5;
						//effect.SetScale(new Vector3D(scale,scale,scale));
						//effect.SetPosition (u.Pos + new Vector3D(1.0f, 1.0f, 10.0f));
						IEffectCommon effect = null;
						{
							effect = unit.Faction.World.EffectManager.CreateTextureEffect(typeof(ParadeDollTextureEffect).FullName, u.Pos);
						}

						if( !u.AbilityManager.AppendAbility(new ParadeDollAbility(effect)))
							effect.Stop();

					}
					else
					{
						u.AbilityManager.ExtendLifetimeWithName(1,ParadeDollAbility.AbilityName,5);
					}
				}

				this.Effect.SetPosition(unit.Pos + Offset);
				
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
	
	class ParadeDollAbility : AbilityTemplate
	{
		public float incAtkRate = 0.3f;
		public float incSpeedRate = 0.3f;
		public float incDamageRate = 0.3f;
		public static string 	AbilityName = "人形「レミングスパレード」";

		int damage = 100;
		
		float d100R = 1*Def.MapTipLengthOnField;
		float d50R = 2*Def.MapTipLengthOnField;
		float d25R = 2.5f*Def.MapTipLengthOnField;
		IEffectCommon effect = null;

		public ParadeDollAbility(IEffectCommon effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 5;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case AbilityOrder.RateIncParam:
				unit.DamageRatio += incDamageRate;
				unit.Speed += unit.Speed*incSpeedRate;
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.Attack += (int)Math.Ceiling(b.Attack*incAtkRate);
				}
				break;
			case AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new Vector3D(2, 2, 1));
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool OnAdd (IUnit unit)
		{
			unit.AddOnVanishEvent(OnVanishHolder);
			return base.OnAdd (unit);
		}
		public void OnVanishHolder (IUnit unit)
		{
			/*
			// suicide bomb
			{
				// search target
				var selfFaction = unit.Faction;
				var world = selfFaction.World;
				var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(d25R, new Vector2D(unit.Pos.X, unit.Pos.Y))).Where(_ => _ != null);
				foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction)))
				{
					if(Utl.GetDistance(unit.Pos.X,unit.Pos.Y,u.Pos.X,u.Pos.Y) <= d100R)
					{
						u.BeDamaged(unit,damage,AttackBonusSet.Explosive);
					}
					else if(Utl.GetDistance(unit.Pos.X,unit.Pos.Y,u.Pos.X,u.Pos.Y) <= d50R)
					{
						u.BeDamaged(unit,(int)Math.Ceiling(damage*0.5f),AttackBonusSet.Explosive);
					}
					else
					{
						u.BeDamaged(unit,(int)Math.Ceiling(damage*0.25f),AttackBonusSet.Explosive);
					}

				}
				var offset = new Vector3D(-2,-2,8);
				var hiteffect = world.EffectManager.CreateEffect(typeof(ParadeHitEffect).FullName,unit.Pos+offset);
				unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, unit.Pos);
				var scale = new Vector3D(16,16,16);
				hiteffect.SetScale(scale);

			}
			*/
			return;
		}

		public override bool OnVanish(IUnit unit)
		{
			unit.RemoveOnVanishEvent(OnVanishHolder);
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
	}
	
}



