﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Spell.Green
{
	class Poteto : thrts.SpellParameter<thrts.IUnit>
	{
		int[] healHPs;
		const int useAlchol = 5;
		public Poteto()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "「スイートポテトルーム」";
			string basePath = "Green/Spell/Poteto/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 6 * Def.FPS;
			CoolTimes[2] = 6 * Def.FPS;
			CoolTimes[3] = 6 * Def.FPS;


			MinRanges = new float[MaxLevel + 1];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 5 * Def.MapTipLengthOnField;
			MaxRanges[2] = 6 * Def.MapTipLengthOnField;
			MaxRanges[3] = 7 * Def.MapTipLengthOnField;

			healHPs = new int[MaxLevel + 1];
			healHPs[0] = 0;
			healHPs[1] = 30;
			healHPs[2] = 50;
			healHPs[3] = 70;

			Description = "酒を消費して、周囲のユニットのHPを回復させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"酒"+useAlchol+"消費、HP" + healHPs[1] +  "回復",
				"酒"+useAlchol+"消費、HP" + healHPs[2] +  "回復",
				"酒"+useAlchol+"消費、HP" + healHPs[3] +  "回復",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>AutoCastAction(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>AutoCastAction(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,15,(unit)=>AutoCastAction(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1", 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2", 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3", 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, IUnit targetUnit, IBuilding targetBuilding, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			if (null == targetUnit || !targetUnit.Exists ()) {
				yield return false;
				yield break;
			}

			if (Spelling (selfUnit, targetUnit, level)) {
				// reset cooltime
				spell.ResetCoolTime ();
			}

			yield return false;
		}
		public bool Spelling (IUnit selfUnit,IUnit u, int level)
		{
			if(null == selfUnit || !selfUnit.Exists())
				return false;
			if(null == u || !u.Exists() || u.HP == u.MaxHP)
				return false;

			if(selfUnit.Faction.GetResource(0) < useAlchol)
				return false;

			selfUnit.Faction.DecResource(0,useAlchol);
			u.HP += healHPs[level];
			if (u.HP > u.MaxHP)
				u.HP = u.MaxHP;
			
			/// エフェクトを描画
			Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
			// 50dmg/1s => 50dps
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(PotetoEffect).FullName, u.Pos + offset, selfUnit.Faction);
			effect.SetScale(new Vector3D(6, 6, 6));
			if (!u.AbilityManager.AppendAbility(new FollowEffectAbility(effect, new Vector3D(0, 0, 1), 2 * Def.FPS, this.Name + "HealEffect")))
				effect.Stop();
			return true;
		}

		public bool AutoCastAction(IUnit selfUnit,int level)
		{
			if(null == selfUnit || !selfUnit.Exists())
				return false;

			var spell = selfUnit.GetSpellWithName(this.Name);

			if(null == spell || !spell.CanExec)
				return false;


			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			if(selfFaction.GetResource(0) < useAlchol)
				return false;

			// search target
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(MaxRanges[level], new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where(_ => _ != null).OrderBy(__=>__.HP/ __.MaxHP);
			foreach (IUnit u in targets)
			{
				if(null == u || !u.Exists() 
				   || u.UniqueID == selfUnit.UniqueID
				   || !world.IsSameTeam(selfFaction, u.Faction) 
				   || u.HP + healHPs[level]/4 > u.MaxHP)
					continue;

				if(Spelling(selfUnit,u,level))
				{
					spell.ResetCoolTime();
					break;
				}
			}


			return true;
		}
	}
}
