﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Shugyou : thrts.SpellParameter<thrts.IUnit>
	{
		const int affectRadius = 15;
		float[] expBonusRatio;
		float[] incMove;
		const int interval = 10;
		public Shugyou()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "仙人の修行";
			string basePath = "Green/Spell/Shugyo/";

			expBonusRatio = new float[MaxLevel + 1];
			expBonusRatio[0] = 0;
			expBonusRatio[1] = 0.20f;
			expBonusRatio[2] = 0.40f;
			expBonusRatio[3] = 0.60f;

			incMove = new float[MaxLevel+1];
			incMove[0] = 0;
			incMove[1] = 0.1f;
			incMove[2] = 0.2f;
			incMove[3] = 0.3f;

			Description = "自分と周囲の味方ヒロインが取得する経験値と移動速度が増加する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"入手経験値+"+expBonusRatio[1].ToString("###.%")+"、移動速度+"+incMove[1],
				"入手経験値+"+expBonusRatio[2].ToString("###.%")+"、移動速度+"+incMove[2],
				"入手経験値+"+expBonusRatio[3].ToString("###.%")+"、移動速度+"+incMove[3],
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;


			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit> ();
			foreach (var u in targets) {
				if (null != u && u.Exists () && u.Parameter.IsUnique && selfFaction.World.IsSameTeam (u.Faction, selfFaction)) {
					var ability = u.AbilityManager.GetAbilityWithName(ShugyouAbility.AbilityName) as ShugyouAbility;

					if (null == ability) {
						var offset = new Vector3D (2, 2, 1);
						var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(ShugyouEffect).FullName, u.Pos);
						var scale = 1 * 8;
						effect.SetScale (new Vector3D (scale, scale, scale));
						effect.SetPosition (u.Pos + offset);
						if (! u.AbilityManager.AppendAbility (new ShugyouAbility (expBonusRatio[level],incMove[level], effect, offset, interval * 2)))
							effect.Stop ();
					} else {
						ability.ExtendLifetime(interval,interval*2);
						ability.ExpBonus = expBonusRatio[level];
						ability.IncMove = incMove[level];
					}
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class ShugyouAbility : FollowEffectAbility
	{
		public const string AbilityName = "仙人の修行";
		public float ExpBonus = 0;
		public float IncMove = 0;
		public ShugyouAbility (float expBonus,float incMove,IEffect effect, Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			ExpBonus = expBonus;
			IncMove = incMove;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.ExpBonusRatio += ExpBonus;
				unit.Speed += IncMove*UnitParameter.BasisSpeed;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}

#endif