﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Green;
using LotusCraft.Bullet.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Green
{
	class Solo : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		float[] durationTime;

		const int interval = 10;

		public Solo()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "騒符「リリカ・ソロライブ」";
			string basePath = "Green/Spell/Solo/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 7;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 12;

			
			Description = "周囲にいる敵の移動速度が低下し、味方の攻撃力が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				durationTime[1] + "秒間、敵の移動速度-" + SoloEnemyAbility.DecMoveRatio.ToString("###.%") + "、味方の攻撃力+"+SoloFriendAbility.IncAtkRatio.ToString("###.%") ,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();

			selfUnit.ShowCutin();

			float affectRadius = affectRadiuses[level];
			
			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(SoloAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 3.5f*(affectRadius);
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,(int)(durationTime[level]*Def.FPS),this.Name + "areaEffect",interval)))
				areaEffect.Stop();
			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;
			
			float affectRadius = affectRadiuses[level];
			
			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).OfType<IUnit>().Where (_ => _ != null);
			foreach (var u in targets) {
				if ( null == u || !u.Exists() || u.UniqueID == selfUnit.UniqueID )
					continue;

				if ( selfFaction.World.IsSameTeam(u.Faction,selfFaction) ) {
					
					if(u.AbilityManager.ContainsAbility(SoloFriendAbility.AbilityName))
					{
						var ability = u.AbilityManager.GetAbilityWithName(SoloFriendAbility.AbilityName);
						
						
						if(null != ability)
						{
							ability.ExtendLifetime (interval,interval*2);
						}
					}
					else
					{
						/// エフェクトを描画
						Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						var effect = selfFaction.World.EffectManager.CreateEffect(typeof(SoloEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(8, 8, 8));

//						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);

						var ability = new SoloFriendAbility(effect,offset,interval*2);
						if(!u.AbilityManager.AppendAbility(ability))
						{
							if (effect != null)
							{
								effect.Stop();
								effect = null;
							}
						}
					}
				}
				else
				{
					if(u.AbilityManager.ContainsAbility(SoloEnemyAbility.AbilityName))
					{
						var ability = u.AbilityManager.GetAbilityWithName(SoloEnemyAbility.AbilityName);
						
						
						if(null != ability)
						{
							ability.ExtendLifetime (interval,interval*2);
						}
					}
					else
					{
						/// エフェクトを描画
						Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						var effect = selfFaction.World.EffectManager.CreateEffect(typeof(SoloDebuffEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(8, 8, 8));
						
//						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);
						
						var ability = new SoloEnemyAbility(effect, offset, interval*2);
						if(!u.AbilityManager.AppendAbility(ability))
						{
							if (effect != null)
							{
								effect.Stop();
								effect = null;
							}
						}
					}
				}
			
			}
			
			return true;
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class SoloFriendAbility : FollowEffectAbility
	{
		public const string AbilityName = "騒符「リリカ・ソロライブ」";
		public const float IncAtkRatio = 0.4f;
		public SoloFriendAbility (IEffect effect, Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
					b.Attack += (int)Math.Ceiling(b.Attack*IncAtkRatio);
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
	class SoloEnemyAbility : FollowEffectAbility
	{
		public const string AbilityName = "騒符「リリカ・ソロライブ」";
		public const float DecMoveRatio = 0.2f;
		public SoloEnemyAbility (IEffect effect, Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed -= unit.Speed*DecMoveRatio;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}

#endif