﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Green;
using LotusCraft.Ability.Green;
using LotusCraft.Building.Green;


namespace LotusCraft.Spell.Green
{
	class Syunka:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 15;
		int[] maxNum = new int[4];
		public Syunka()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "幻想春花";
			string basePath = "Green/Spell/Shunka/";

			maxNum = new int[MaxLevel+1];
			maxNum[0] = 0;
			maxNum[1] = 6;
			maxNum[2] = 12;
			maxNum[3] = 24;

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 10*Def.FPS;
			CoolTimes[1] = 10*Def.FPS;
			CoolTimes[2] = 10*Def.FPS;
			CoolTimes[3] = 10*Def.FPS;
			
			MaxRanges = new float[MaxLevel+1];
			MaxRanges[0] = 1*Def.MapTipLengthOnField;
			MaxRanges[1] = 2*Def.MapTipLengthOnField;
			MaxRanges[2] = 2*Def.MapTipLengthOnField;
			MaxRanges[3] = 3*Def.MapTipLengthOnField;
			

			Description += "花畑の上に、新たな花畑の中心となる種を植える。\n種は敵AIに発見されず、敵ヒロインからしか見えない。\n";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"最大数 " + (maxNum[1] - maxNum[0]),
				"最大数 " + (maxNum[2] - maxNum[0]),
				"最大数 " + (maxNum[3] - maxNum[0]),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// スキマアビリティ取得のためのオートキャストを設定
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, ISpell spell)
		{
			if (null == spell) {
				yield return false;
				yield break;
			}

			var ret = Spelling(selfUnit,targetPosition,spell.Level);
			if(ret)
				spell.ResetCoolTime();

			yield return ret;
		}

		public bool Spelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if(level < 0 || MaxLevel < level)
				return false;


			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			// 花畑の上かチェック
			var tip = world.Map.GetMapTipIndex(targetPosition.X,targetPosition.Y);
			if(!world.Map.IsSpecialArea(tip.X,tip.Y))
				return false;

			// 種の最大数をチェック
			var seeds = world.UnitManager.GetUnits (_ => _ != null && _.Exists () && _.Faction == selfFaction && _.Parameter.GetType () == typeof(Unit.Green.SeedUnit));
			if (seeds.Count () >= maxNum[level]) {
				return false;
			}

			var unit = world.UnitManager.Create(typeof(Unit.Green.SeedUnit).FullName,selfFaction,targetPosition.X,targetPosition.Y,selfFaction);

			return unit != null && unit.Exists();
		}
	}
}

namespace LotusCraft.Unit.Green
{
	public class SeedUnit : thrts.UnitParameter
	{
		public SeedUnit ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Green/Seed";
#endregion

			#region UnitParams
			Name = "種";
			ButtonDescription += "花畑の中心になる種です。AIに発見さず、ヒロイン以外は見る事も出来ません。\n";
			HP = 100;
			Armor = 4;
			Radius = 4 * 16;
			Speed = 0.0f * BasisSpeed;
			Exp = 0;
			
			PopulationWeight = 0;

			ViewRange = 4*Def.MapTipLengthOnField;
			
			MakingTime = 0 * 30;
			
			// lowest priority
			SelectPriority = 999;
			
			/// mine,gas,sup,?
			Costs = new [] {0,0,0,0};
#endregion
			
			#region Animation
			this.SetFlowerAnimation("LotusCraft/Image/Unit/Green/YuukaBall");
			Radius = 0;
#endregion
			
			#region BuildableObjcets
#endregion
			
			#region Bullets
#endregion
			
		}
		public override void OnLoad (IFaction faction)
		{

		}

		public override void OnAppear (IUnit unit)
		{
			unit.AbilityManager.AppendAbility(new Ability.Green.PropagateGardenAbility(10,1,5*Def.FPS,2));
			unit.AbilityManager.AppendAbility(new SeedAbility());
			unit.HoldPosition(false);

			base.OnAppear (unit);
		}
	}
}

namespace LotusCraft.Ability.Green
{
	class SeedAbility : thrts.AbilityTemplate
	{
		public static string AbilityName = "種";
		public SeedAbility()
		{
			Name = AbilityName;
		}
		
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				if(unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);

				// 自動索敵を停止
				unit.DoFindEnemyAutomatically = false;
				// 不可視系の設定は定数増加で行う
				unit.IsSemitransparent = true;

				// 近くにヒロインが居なければ不可視かつAIに発見されない
				if(unit.Faction.World.GetObjectFromCircle(new Circle2D(16*Def.MapTipLengthOnField,new Vector2D(unit.Pos.X,unit.Pos.Y)))
						.OfType<IUnit>()
						.Any(_=>!unit.Faction.World.IsSameTeam(_.Faction,unit.Faction) && _.Parameter.IsUnique))
				{
					unit.IsTransparent = false;
					unit.IsPreventedFromFindingEnemy = false;
				}
				else
				{
					unit.IsTransparent = true;
					unit.IsPreventedFromFindingEnemy = true;
				}

				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);
				break;
			default:
				break;
			}
			
			return true;
		}

		public override void DecDurationTime ()
		{
			;
		}
	}
}