﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Green;
using LotusCraft.Effect.Green;
using LotusCraft.Unit.Green;

namespace LotusCraft.Spell.Green
{
	class TripWire:thrts.SpellParameter<thrts.IUnit>
	{
		public static string SpellName = "注力「トリップワイヤー」";
		public static string TripTimerAbilityName = SpellName + "tripTimer";
		int[] damages;
		const int duration = 6*Def.FPS;
		const int interval = 5;
		const int tripInterval = Def.FPS/2;
		public TripWire()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			Description = "アリスと人形とをつなぐ糸上の敵ユニットへダメージを与える。";
			string basePath = "Green/Spell/TripWire/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 20*Def.FPS;
			CoolTimes[2] = 20*Def.FPS;
			CoolTimes[3] = 20*Def.FPS;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 6;
			damages[2] = 9;
			damages[3] = 12;
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				damages[1] + "ダメージ, "+duration/(float)Def.FPS+"秒間、0.5秒で回復。人形の最大数 +1体",
				damages[2] + "ダメージ, "+duration/(float)Def.FPS+"秒間、0.5秒で回復。人形の最大数 +2体",
				damages[3] + "ダメージ, "+duration/(float)Def.FPS+"秒間、0.5秒で回復。人形の最大数 +3体",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			/// reset cooltime
			spell.ResetCoolTime ();

			IEffect selfEffect = null;
			Vector3D offset = new Vector3D();
			selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling (unit,level),selfEffect,offset,duration,this.Name + "selfEffect",interval));

			// get dolls
			var dolls = selfUnit.Faction.World.UnitManager.GetUnits (_ => _.Faction == selfUnit.Faction && _.Parameter.GetType () == typeof(Dolls));

			// add tripwire effect
			foreach (var d in dolls)
			{
				var dolleffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(WireEffect).FullName, d.Pos);
				var scale = 3.5f * 3;
				dolleffect.SetScale(new Vector3D(scale, scale, scale));
				dolleffect.SetPosition(selfUnit.Pos + new Vector3D(1.0f, 1.0f, 5.0f));
				if(!d.AbilityManager.AppendAbility(new FollowEffectAbility(dolleffect,new Vector3D(0,1,5), duration, this.Name + "DollEffect")))
					dolleffect.Stop();
			}

			yield return false;
		}

		public bool Spelling (IUnit selfUnit, int level)
		{
			// get farction and world
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			
			// get dolls
			var dolls = world.UnitManager.GetUnits (_ => _.Faction == selfFaction && _.Parameter.GetType () == typeof(Dolls));

			// add tripwire effect
			foreach (var d in dolls) {
				if(null == d || !d.Exists())
					continue;

				IEffect wireEffect = null;


				if(!d.AbilityManager.ContainsAbility(TripTimerAbilityName))
				{
					var targets = world.GetObjectOnLine(new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y),new Vector2D(d.Pos.X,d.Pos.Y),true,false).OfType<IUnit>();
					foreach(var u in targets)
					{
						if(null == u || !u.Exists() || world.IsSameTeam(u.Faction,selfFaction) || u.AbilityManager.ContainsAbility(this.Name + "tripRigor"))
							continue;

						var offset = new Vector3D(-2, -2, 8);
						Unit.Utls.ShowHitEffect(u, Unit.Utls.HitEffectType.NormalGreen);
						
						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.BiriBiri).FullName, selfUnit.Pos);

						u.BeDamaged(selfUnit,damages[level],AttackBonusSet.None);
						// 多段ヒット防止
						u.AbilityManager.AppendAbility(new EffectContinuousAbility(null,tripInterval,this.Name + "tirpRigor"));
					}
					if(!d.AbilityManager.AppendAbility(new EffectContinuousAbility(null,tripInterval,TripTimerAbilityName)))
						; 
				}

			}

			return true;
		}
	}
}
