﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;

namespace LotusCraft.Spell.Red
{
	class Gyakuhu : thrts.SpellParameter<thrts.IUnit>
	{
		GyakuhuAbility[] spellAbilities = new[]
		{
			new GyakuhuAbility(0),
			new GyakuhuAbility(1),
		};
		int spellDuration = 4*Def.FPS;
		public Gyakuhu()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			CoolTimes = new int[4];
			CoolTimes[0] = 240*30;
			CoolTimes[1] = 240*30;
			Name = "逆風「人間禁制の道」";
			Description = "すべての敵の移動速度を減少させる。";
			string basePath = "Red/Spell/Gyakuhu/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-60%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(IUnit selfUnit, int level)
		{
			// reset cooltime
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			// show cutin
			selfUnit.ShowCutin();
			

			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 5;
					break;
				default:
					affectRadius = 0;
					break;
			}
			/// get call Effect;
			var offset = new Vector3D(2, 2, 1);
			IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(GyakuhuAreaEffect).FullName, selfUnit.Pos + offset);
			effect.SetScale(new Vector3D(2 * affectRadius * 2, 2 * affectRadius * 2, 2 * affectRadius * 2));
			/// add spell continuous ability
			bool flag = selfUnit.AbilityManager.AppendAbility(new Ability.SpellContinuousAbility((unit) => Spelling(unit, level), effect, offset, 45 * 30, this.Name + "AreaEffect", 15));
			if (!flag)
				effect.Stop();

			for (int i = 0; i < (1 * Def.FPS)/2; ++i)
			{
				yield return null;
			}
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, selfUnit.Pos);


			yield return true;
		}
		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 5;
					break;
				
				default:
					affectRadius = 0;
					
					break;
			}

			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !world.IsSameTeam(selfFaction, _.Faction)))
			{
				if (!u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					IEffect effect = null;
					if (u != selfUnit)
					{
						/*
						/// エフェクトを描画
						var faction = u.Faction;
						Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
						effect = world.EffectManager.CreateEffect(typeof(KibakujinEffect).FullName, selfFaction, u.Pos + offset);
						effect.SetScale(new Vector3D(12, 12, 12));
						 * */
					}

					if (u.AbilityManager.AppendAbility(new GyakuhuAbility(level, 15*2, effect))){
					
					}
					else
						effect.Stop();
				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(15,spellAbilities[level].Name ,15*2);
				}
			}

			return true;
		}
	}
}


namespace LotusCraft.Ability.Red
{
	class GyakuhuAbility : thrts.AbilityTemplate
	{
		public float SpeedRate{ get; private set; }
		IEffect effect;
		public GyakuhuAbility (int level,int duration =10,IEffect effect=null)
		{
			this.Name = "逆風「人間禁制の道」";
			this.DurationTime = duration;
			this.effect = effect;
			
			switch (level)
			{
			case 1:
				SpeedRate = 0.4f;
				break;
			default:
				SpeedRate = 1.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				///移動速度をspeedRate倍
				unit.Speed *= SpeedRate;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	
}

#endif