﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class Hujin:thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="LotusCraft.Spell.Red.NijuKekkai"/> class.
		/// </summary>
		HujinAbility[] spellAbilities = new []
		{
			new HujinAbility(0),
			new HujinAbility(1),
			new HujinAbility(2),
			new HujinAbility(3),
		};
		const int interval = Def.FPS;
		int[] damages;
		int[] durations;
		public Hujin()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			CoolTimes = new int[4];
			CoolTimes[0] = 45*Def.FPS;
			CoolTimes[1] = 45*Def.FPS;
			CoolTimes[2] = 45*Def.FPS;
			CoolTimes[3] = 45*Def.FPS;

			damages = new int[MaxLevel+1];
			damages[0] = 0;
			damages[1] = 6;
			damages[2] = 8;
			damages[3] = 15;

			durations = new int[MaxLevel+1];
			durations[0] = 0;
			durations[1] = 6*Def.FPS;
			durations[2] = 10*Def.FPS;
			durations[3] = 12*Def.FPS;


			Name = "疾風「風神少女」";
			Description = "自身の移動速度を上昇させる。\n周囲の敵兵ユニットは1秒毎にダメージを受ける。";
			string basePath = "Red/Spell/Hujin/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度+40%、範囲3に毎秒"+damages[1]+"ダメージ、6秒間",
				"移動速度+60%、範囲3に毎秒"+damages[2]+"ダメージ、10秒間",
				"移動速度+80%、範囲3に毎秒"+damages[3]+"ダメージ、12秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1", PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2", PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3", PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling (IUnit selfUnit, int level)
		{
			// reset cooltime
			selfUnit.GetSpellWithName (Name).ResetCoolTime ();

			/// get call Effect;

			Vector3D offset = new Vector3D (2.0f, 2.0f, 1.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(HujinSelfEffect).FullName, selfUnit.Pos + offset);
			var scale = 5.0f;
			effect.SetScale (new Vector3D (scale, scale, scale));

			selfUnit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.Wind).FullName, selfUnit.Pos);

			/// add spell continuous ability

			var ability = new Ability.SpellContinuousAbility ((unit) => Spelling (unit, level), effect, offset, durations[level], this.Name + "SelfEffect",interval);
			ability.FollowEffect = false;

			bool flag = selfUnit.AbilityManager.AppendAbility (ability);

			if (!flag) {
				effect.Stop ();
				effect = null;
			}

			/// エフェクトを描画
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(HujinAreaEffect).FullName, new Vector3D (selfUnit.Pos.X + offset.X, selfUnit.Pos.Y + offset.Y, selfUnit.Faction.World.Map.GetHeightOnPoint (selfUnit.Pos.X, selfUnit.Pos.Y) + offset.Z));
			float effectScale = 1.7f * 5;
			areaEffect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			
			if (!selfUnit.AbilityManager.AppendAbility (new HujinAbility (level, interval*2, effect))) {
				areaEffect.Stop();
				areaEffect = null;
			}

			yield return true;
		}

		public bool Spelling (IUnit selfUnit, int level)
		{
			float affectRadius = 3;
			int damage = damages[level];

			if (!selfUnit.AbilityManager.ContainsAbility(spellAbilities[level].Name))
			{
				var selfFaction = selfUnit.Faction;
				var world = selfFaction.World;
				/// エフェクトを描画
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
				var effect = world.EffectManager.CreateEffect(typeof(HujinAreaEffect).FullName, new Vector3D(selfUnit.Pos.X + offset.X, selfUnit.Pos.Y + offset.Y, world.Map.GetHeightOnPoint(selfUnit.Pos.X, selfUnit.Pos.Y) + offset.Z));
				float effectScale = 1.7f * 5;
				effect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
				
				if(!selfUnit.AbilityManager.AppendAbility(new HujinAbility(level,interval*2,effect)))
				{
					effect.Stop();
					effect = null;
				}
			}
			else
			{
				/// 既に掛かっていたら時間を延長
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval, spellAbilities[level].Name,durations[level]);
			}

//			if (selfUnit.ActualAction == UnitAction.Privilege)
			{
				var selfFaction = selfUnit.Faction;
				var world = selfFaction.World;
				var targets = world.GetObjectFromCircle(new Circle2D(affectRadius * Def.MapTipLengthOnField / 2, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);

				foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !world.IsSameTeam(selfFaction, _.Faction)))
				{
					if (!u.AbilityManager.ContainsAbility(this.Name))
					{
							u.BeDamaged(selfUnit, (int)(damage), AttackBonusSet.Hero);

					}
					else
					{
						u.BeDamaged(selfUnit, (int)(damage), AttackBonusSet.Hero);
					}
				}
			}


			return true;
		}
	}
}

namespace LotusCraft.Ability.Red
{
	class HujinAbility : thrts.AbilityTemplate
	{
		public float SpeedRate{ get; private set; }
		IEffect effect;
		public HujinAbility (int level,int duration = 4*Def.FPS,IEffect effect=null)
		{
			this.Name = "疾風「風神少女」 lv"+level.ToString();
			this.effect = effect;
			this.DurationTime = duration;

			switch (level)
			{
			case 1:
				SpeedRate = 1.4f;
				break;
			case 2:
				SpeedRate = 1.6f;
				break;
			case 3:
				SpeedRate = 1.8f;
				break;
			default:
				SpeedRate = 1.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				///移動速度をspeedRate倍
				unit.Speed *= SpeedRate;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	
}

#endif