﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;

namespace LotusCraft.Spell.Red
{
	class KouyouSenpu : thrts.SpellParameter<thrts.IUnit>
	{
		KouyouSenpuAbility[] spellAbilities = new []
		{
			new KouyouSenpuAbility(0),
			new KouyouSenpuAbility(1),
			new KouyouSenpuAbility(2),
			new KouyouSenpuAbility(3),
		};
		float spellDuration = 4.0f;
		float[] affectRadiuses = new float[0];

		int stunTime = 4*Def.FPS;

		public KouyouSenpu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			CoolTimes = new int[4];
			CoolTimes[0] = 20*Def.FPS;
			CoolTimes[1] = 25*Def.FPS;
			CoolTimes[2] = 25*Def.FPS;
			CoolTimes[3] = 20*Def.FPS;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 2*Def.MapTipLengthOnField;
			MaxRanges[2] = 3*Def.MapTipLengthOnField;
			MaxRanges[3] = 4*Def.MapTipLengthOnField ;


			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 1;
			affectRadiuses[2] = 2;
			affectRadiuses[3] = 2;

			Name = "旋符「紅葉扇風」";
			Description = "指定位置近くの敵ユニットを空中に持ち上げる。\nヒロイン及び作業員には効果無し。";
			string basePath = "Red/Spell/KouyouSenpu/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程"+MaxRanges[1]/Def.MapTipLengthOnField+"、範囲"+affectRadiuses[1]+"、4秒間",
				"射程"+MaxRanges[2]/Def.MapTipLengthOnField+"、範囲"+affectRadiuses[2]+"、4秒間",
				"射程"+MaxRanges[3]/Def.MapTipLengthOnField+"、範囲"+affectRadiuses[3]+"、4秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(IUnit selfUnit, Vector2D targetPosition, int level)
		{
			// reset cooltime
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			float affectRadius = affectRadiuses[level];

			spellDuration = 3.0f;

			// get call Effect;
			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);
			var offset = new Vector3D(2,2,1);
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KouyouSenpuEffect).FullName, targetPos+offset);
			areaEffect.SetScale (new Vector3D (6 *affectRadius*2, 6*affectRadius*2, 6f*affectRadius*2));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);

			/// add effect continuous ability
			bool flag = selfUnit.AbilityManager.AppendAbility(new Ability.EffectContinuousAbility( areaEffect, (int)spellDuration * Def.FPS, this.Name + "AreaEffect"));
			if (!flag)
			{
				areaEffect.Stop();
				areaEffect = null;
			}

			Spelling(selfUnit, targetPosition, level);

			yield return true;
		}
		public bool Spelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			float affectRadius = affectRadiuses[level];

			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius*32, targetPosition)).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!world.IsSameTeam(selfFaction,_.Faction)))
			{
				if(u == null || !u.Exists() || u.Parameter.IsFarmer || u.Parameter.IsUnique)
					continue;

				var offset2 = new Vector3D(1, 1, 8);
				var effect = world.EffectManager.CreateEffect(typeof(KouanHitEffect).FullName, u.Pos + offset2);
				float effectScale = 1.7f * 2;
				effect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

				// add ability or extend lifetime
				var ability = u.AbilityManager.GetAbilityWithName(spellAbilities[level].Name);
				if (null == ability)
				{
					/// エフェクトを描画
					Vector3D offset = new Vector3D(2.0f, 2.0f, 3.0f);

					var stunEffect = world.EffectManager.CreateEffect(typeof(StanEffect).FullName, u.Pos + offset);
					float stunEffectScale = 1.7f * 4;
					stunEffect.SetScale(new Vector3D(stunEffectScale, stunEffectScale, stunEffectScale));

					if (!u.AbilityManager.AppendAbility(new KouyouSenpuAbility(level, stunTime, stunEffect)))
					{
						stunEffect.Stop();
						effect.Stop();
					}

					if (!u.Exists())
					{
						effect.Stop();
						stunEffect.Stop();
					}
				}
				else
				{
					ability.ExtendLifetime(stunTime,stunTime);
				}

			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.Red
{
	class KouyouSenpuAbility : thrts.AbilityTemplate
	{
		IEffect effect;
		bool isHitFlying = false;
		const string AbilityName = "旋符「紅葉扇風」";
		public KouyouSenpuAbility (int level,int duration = 4*Def.FPS,IEffect effect=null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.effect = effect;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				///移動速度をspeedRate倍
				unit.Speed = 0.0f;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				unit.IsForbiddenAttacking = true;
				unit.ThroughOthers = true;
				
				if(DurationTime < 1*Def.FPS && isHitFlying && unit.FlyingState == FlyingState.Fly)
				{
					// 真下を空ける
					{
						var world = unit.Faction.World;
						var targets = world.GetObjectFromCircle(new Circle2D(unit.Radius, new Vector2D(unit.Pos.X,unit.Pos.Y))).OfType<IUnit>().Where(_ => _ != null);
						foreach( var u in targets)
						{
							var diff = u.Pos - unit.Pos;
							var len = diff.Length;
							diff.Normalize();
							var move =diff*Math.Abs(unit.Radius-len); 
							u.TryMove(move.X,move.Y,true);
						}
					}
					// 着地出来なかったら効果を延長
					if(!unit.TouchGround())
					{
						this.ExtendLifetime(1,Def.FPS);
					}
				}
				
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(1,1,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnAdd (IUnit unit)
		{
			if (unit.FlyingState == FlyingState.TouchGround) {
				isHitFlying = true;
				unit.Fly ();
			} else {
				isHitFlying = false;
			}
			return base.OnAdd (unit);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			if (unit != null)
			{
				unit.IsForbiddenAttacking = false;
				unit.ThroughOthers = false;
				if(isHitFlying && unit.FlyingState == FlyingState.Fly)
					unit.TouchGround();
			}
			return base.OnVanish (unit);
		}
	}
	
}

#endif