﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Red
{
	class KanakoSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 10;
		Dictionary<float,float> splash;
		public KanakoSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			IsPassive = true;
			Name = "御柱「メテオリックオンバシラ」";
			string basePath = "Red/Spell/KanakoSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4*30;
			CoolTimes[1] = 4*30;

			splash = new Dictionary<float, float>(){{0.5f*Def.MapTipLengthOnField,0.8f},{0.8f*Def.MapTipLengthOnField,0.5f},{1.3f*Def.MapTipLengthOnField,0.25f},};

			Description = "攻撃にスプラッシュが付加され、射程が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程+1, スプラッシュ追加",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,(unit)=>Spelling(unit,1),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			if (!selfUnit.Exists ())
				return false;
			
			if (selfUnit.AbilityManager.ContainsAbility (KanakoSkillAbility.AbilityName)) {
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval,KanakoSkillAbility.AbilityName,interval*2);
			}
			else
			{
				selfUnit.AbilityManager.AppendAbility(new KanakoSkillAbility(interval*2,splash));
			}
			
			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			yield return false;
		}

		class KanakoSkillAbility : thrts.AbilityTemplate
		{
			Dictionary<float,float> splash;
			public const string AbilityName = "御柱「メテオリックオンバシラ」";
			public KanakoSkillAbility(int duration,Dictionary<float,float> splash)
			{
				DurationTime = duration;
				Name = AbilityName;
				this.splash = splash;
			}
			public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
			{
				switch (order)
				{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					var bulletParam = unit.GetModifiedBulletParameter(typeof(KanakoBullet).FullName);
					if(null != bulletParam)
					{
						bulletParam.HasSplash = true;
						bulletParam.SplashDamageRates = splash;
						bulletParam.MaxRange += 1*Def.MapTipLengthOnField;
					}
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					break;
				default:
					break;
				}
				
				return base.ExecUnitAbility(unit,order);
			}

		}
	}
}
