using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;

namespace LotusCraft.Spell.Red
{
	class MeirinSkill : thrts.SpellParameter<thrts.IUnit>
	{

		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const float modeChangeTime = 3.0f;
		const float modeDisChangeTime = 1.0f;
		const int interval = 10;
		public MeirinSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "紅魔館の門番";
			string basePath = "Red/Spell/MeirinSkill/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4 * Def.FPS;
			CoolTimes[1] = 4 * Def.FPS;

			//			Description = 
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"迎撃モードに移行し、移動能力を失い範囲攻撃を得る。\n探知を持つユニットにしか発見されない(ヒロイン等)",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
			thrts.AbilityTemplate abiity = null;
			selfUnit.AbilityManager.AppendAbility(abiity);
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;



			// クールタイムをリセット
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			int modeCTime = (int)(modeChangeTime * Def.FPS);
			int modeDisCTime = (int)(modeDisChangeTime * Def.FPS);

			var orgFaction = selfUnit.Parameter.Faction; // for effect

			if (!selfUnit.AbilityManager.ContainsAbility(InterceptionModeAbility.AbilityName))
			{

				// 変形開始エフェクト
				{
					var change = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(MeirinMobChangeEffect).FullName, selfUnit.Pos + new Vector3D(0, 0, 1), orgFaction);
					change.SetScale(new Vector3D(16.0f, 16.0f, 16.0f));
				}
				selfUnit.HoldPosition(false);
				for (int i = 0; i < modeCTime; ++i)
				{
					// ユニット動作停止
					selfUnit.Speed = 0.0f;

					if (i == 0 || i % (Def.FPS * 2) == 0)
					{
						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Alarm01).FullName, selfUnit.Pos);
					}
					yield return true;
				}

				// Create Area Effect And Add Ability
				CreateAreaEffect(selfUnit, level);
			}
			else
			{
				var ability = selfUnit.AbilityManager.GetAbilityWithName(InterceptionModeAbility.AbilityName) as InterceptionModeAbility;
				if (null != ability)
					ability.NowPrepareing = true;

				for (int i = 0; i < modeDisCTime; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}
				// Remove Interception mode ability
				selfUnit.AbilityManager.RemoveAbility(InterceptionModeAbility.AbilityName, true);

			}

			yield return false;
		}

		private static void CreateAreaEffect(thrts.IUnit selfUnit, int level)
		{
			var orgFaction = selfUnit.Parameter.Faction; // for effect
			// 範囲表示
			if (!selfUnit.AbilityManager.ContainsAbility(Ability.Red.InterceptionModeAbility.AbilityName))
			{
				var affectRadius = selfUnit.Parameter.Faction.GetBulletParameter(typeof(MeirinMobBigBullet).FullName).MaxRange;

				// まだ範囲が表示されていない場合
				var effect = selfUnit.Faction.World.EffectManager.CreateModelEffect(typeof(MeirinMobAreaModelEffect).FullName, selfUnit.Pos, orgFaction);
				var scale = affectRadius / 100.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));

				effect.IsShownToOnlyAlly = true;

				// アビリティとして追加(ユニット死亡時、エフェクトが自動破棄される)
				if (!selfUnit.AbilityManager.AppendAbility(new InterceptionModeAbility(level, effect)))
				{
					effect.Stop();
				}
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval, Ability.Red.InterceptionModeAbility.AbilityName, 15);
			}
		}
	}
}

namespace LotusCraft.Ability.Red
{
	class InterceptionModeAbility : thrts.AbilityTemplate
	{
		IModelEffect effect;
		Vector3D rotation;
		
		public const string AbilityName = "紅魔館の門番";

		public bool NowPrepareing = false;
		public int ShootTimeCounter = 0;

		public InterceptionModeAbility(int level, IModelEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 1; // inf
			this.effect = effect;
			this.NowPrepareing = false;
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.CanStateNow = false;
				// 移動しようとしていたらホールドポジションに強制変更
				if(!unit.IsHoldPosition || unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);
				
				// set speed=0;
				unit.Speed = 0.0f;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				// 不可視系の設定は定数増加で行う
				// 近くにDetectorが居たら透明になれない
				if(!NowPrepareing && unit.Faction.World.GetObjectFromCircle(new Circle2D(16*Def.MapTipLengthOnField,new Vector2D(unit.Pos.X,unit.Pos.Y)))
				   .OfType<IUnit>()
				   .Any(_=>!unit.Faction.World.IsSameTeam(_.Faction,unit.Faction) && _.IsDetector && Utl.InCircle(unit.Pos,_.Pos,_.ViewRange) ) )
				{
					unit.IsTransparent = false;
					unit.IsSemitransparent = true;
					unit.IsPreventedFromFindingEnemy = false;
				}
				else
				{
					unit.IsTransparent = true;
					unit.IsSemitransparent = true;
					unit.IsPreventedFromFindingEnemy = true;
				}

				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				unit.SelectBullet = SelectIntercepterBullet;
				ShootTimeCounter++;
				var bullet = unit.GetModifiedBulletParameter(typeof(MeirinMobBigBullet).FullName);
				if(null != bullet)
				{
					 unit.IsMPBarShown = true;

					if(ShootTimeCounter > bullet.AttackSpeed)
						ShootTimeCounter = bullet.AttackSpeed;

					unit.MPBarMax = bullet.AttackSpeed;
					unit.MPBarCurrent = bullet.AttackSpeed - ShootTimeCounter;
				}
				if (effect != null)
				{
					rotation.Z += 1.0f;
					effect.SetRotation(rotation);
					effect.SetPosition(unit.Pos + new Vector3D(0, 0, 1));
				}
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				unit.CanStateNow = true;
				unit.IsMPBarShown = false;
				;
			}
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			
			return base.OnVanish(unit);
		}
		// inf. ability
		public override void DecDurationTime ()
		{
			;
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			ShootTimeCounter = 0;
			return base.ModifyBullet (bullet);
		}
		static int SelectIntercepterBullet(IUnit unit,ITargetObject target,float distance)
		{
			/// return aja bullet
			return 0;
		}
	}
	
}
