﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Red
{
	class Hinotori:thrts.SpellParameter<thrts.IUnit>
	{
		float spellDuration = 6.0f;
		float damage = 20;
		int interval = 30;

		IEffect areaEffect = null;
		IEffect areaEffect2 = null;
		IEffect effect = null;
		HinotoriAbility[] spellAbilities = new[]
		{
			new HinotoriAbility(0),
			new HinotoriAbility(1),
			new HinotoriAbility(2),
			new HinotoriAbility(3),
		};
		public Hinotori()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "不死「火の鳥　-鳳翼天翔-」";
			Description = "一定時間移動速度が上昇し、近くの敵ユニットにダメージを与える";
			string basePath = "Red/Spell/Hinotori/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度:+"+(spellAbilities[1].SpeedRate - 1).ToString("###.%")+", 3ダメージ/秒, "+spellAbilities[1].DurationTime/Def.FPS+"秒",
				"移動速度:+"+(spellAbilities[2].SpeedRate - 1).ToString("###.%")+", 5ダメージ/秒, "+spellAbilities[2].DurationTime/Def.FPS+"秒",
				"移動速度:+"+(spellAbilities[3].SpeedRate - 1).ToString("###.%")+", 7ダメージ/秒, "+spellAbilities[3].DurationTime/Def.FPS+"秒",
			};

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30 * Def.FPS;
			CoolTimes[2] = 30 * Def.FPS;
			CoolTimes[3] = 30 * Def.FPS;

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			float affectRadius = 0;


			switch (level)
			{
				case 1:
					affectRadius = 1.5f;
					spellDuration = spellAbilities[1].DurationTime/Def.FPS;
					interval = Def.FPS;
					damage = 3;
					break;
				case 2:
					affectRadius = 1.8f;
					spellDuration = spellAbilities[2].DurationTime/Def.FPS;
					interval = Def.FPS;
					damage = 5;
					break;
				case 3:
					affectRadius = 2.0f;
					spellDuration = spellAbilities[3].DurationTime/Def.FPS;
					interval = Def.FPS;
					damage = 7;
					break;
				default:
					affectRadius = 0;
					break;
			}
			/// get call Effect;
			var offset = new Vector3D(1, 2, 1);
			var size = 8 * affectRadius;
			areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HinotoriAreaEffect).FullName, selfUnit.Pos + offset);
			areaEffect.SetRotation(new Vector3D(0f, 0f, 225f));
			areaEffect.SetScale(new Vector3D(size, size, size));

			bool flag = selfUnit.AbilityManager.AppendAbility(new Ability.SpellContinuousAbility((unit) => Spelling(unit, level), areaEffect, offset, (int)(spellDuration * Def.FPS), this.Name + "AreaEffect", interval));

			if (!flag)
			{
				areaEffect.Stop();
				areaEffect = null;
				// don't search enemy
				selfUnit.DoFindEnemyAutomatically = true;
			}

			if (!selfUnit.AbilityManager.ContainsAbility(this.Name + "SelfEffect"))
			{
				var offset2 = new Vector3D(0, 0, 0);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HinotoriSelfEffect).FullName, selfUnit.Pos);
				effect.SetScale(new Vector3D(size + 2, size + 2, size + 2));
				effect.SetPosition(selfUnit.Pos + offset);

				var ability = new FollowEffectAbility(effect, offset, (int)(spellDuration * Def.FPS), this.Name + "SelfEffect");
				ability.IsDirectionChanged = true;
				ability.FrontOffset = 0.0f;

				if (!selfUnit.AbilityManager.AppendAbility(ability))
				{
					effect.Stop();
				}
			}
			
			//自身の能力
			if (selfUnit.AbilityManager.AppendAbility(new HinotoriAbility(level)))
			{
			}

			yield return true;
		}
		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			float affectRadius = 0;
			switch (level)
			{
				case 1:
					damage = 3;
					affectRadius = 1.5f;
					break;
				case 2:
					damage = 5;
					affectRadius = 1.8f;
					break;
				case 3:
					damage = 7;
					affectRadius = 2.0f;
					break;
				default:
					affectRadius = 0;
					damage = 0;
					break;
			}

			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius * 32, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !world.IsSameTeam(selfFaction, _.Faction)))
			{
				effect = world.EffectManager.CreateEffect(typeof(HitSplashRedEffect).FullName, selfFaction, u.Pos);
				var scale = new Vector3D(8, 8, 8);
				effect.SetScale(scale);

				var hitAbility = new Ability.EffectContinuousAbility(effect, 20, this.Name + "HitEffect");
				var flag = u.AbilityManager.AppendAbility(hitAbility);
				if (!flag)
					effect.Stop();

				u.BeDamaged(selfUnit, (int)damage, AttackBonusSet.Hero);
				selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, u.Pos);

				if (!u.Exists())
				{
					effect.Stop();
				}
			}

			return true;
		}

		public void OnVanishUnit(thrts.IUnit selfUnit)
		{
			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}

			if (areaEffect2 != null)
			{
				areaEffect2.Stop();
				areaEffect2 = null;
			}
		}
	}
}

namespace LotusCraft.Ability.Red
{
	class HinotoriAbility : thrts.AbilityTemplate
	{
		public float SpeedRate{ get; private set; }
		IEffect effect;
		public HinotoriAbility(int level, IEffect effect = null)
		{
			this.Name = "不死「火の鳥　-鳳翼天翔-」 lv" + level.ToString();
			this.DurationTime = 6*30;
			this.effect = effect;
			
			switch (level)
			{
			case 1:
				SpeedRate = 1.2f;
				this.DurationTime = 8 * Def.FPS;
				break;
			case 2:
				SpeedRate = 1.4f;
				this.DurationTime = 10 * Def.FPS;
				break;
			case 3:
				SpeedRate = 1.6f;
				this.DurationTime = 12* Def.FPS;
				break;
			default:
				SpeedRate = 1.0f;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				///移動速度をspeedRate倍
				unit.Speed *= SpeedRate;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				if(effect != null)
					effect.SetPosition(unit.Pos+new Vector3D(2,2,1));
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (effect != null) {
				effect.Stop ();
				effect = null;
			}
			return base.OnVanish (unit);
		}
	}
	
}
