﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Ability.Red;
using LotusCraft.Ability;

namespace LotusCraft.Spell.Red
{
	class RoyalFlare : thrts.SpellParameter<thrts.IUnit>
	{
		const float affectRadius = 12;

		const float durationTime = 12;

		const int damage = 75;

		const int interval = Def.FPS/2;
		const int chargeTime = 4*Def.FPS;

		public RoyalFlare()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "日符「ロイヤルフレア」";
			string basePath = "Red/Spell/RoyalFlare/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 180*Def.FPS;


			Description = "範囲内に居るユニット・建物にダメージを与える。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"0.5秒毎に"+damage+"ダメージ、" + durationTime+ "秒間" ,
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// reset cooltime
			var spell = selfUnit.GetSpellWithName (Name);
			if(null == spell || !spell.CanExec)
			{
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime ();

			// show cutin
			selfUnit.ShowCutin();

			// effect offset
			Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);
			
			/// create spell Effect;
			var chargeEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(RoyalSelfEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = (affectRadius) * 16;
			chargeEffect.SetScale (new Vector3D (scale, scale, scale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

			if(!selfUnit.AbilityManager.AppendAbility(new EffectContinuousAbility(chargeEffect, chargeTime,this.Name + "chargeEffect")))
				chargeEffect.Stop();
			
			for(int i = 0;i<chargeTime;++i)
				yield return true;
			
			/// create spell Effect;
			var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(RoyalAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			scale = affectRadius * 2;
			areaEffect.SetScale (new Vector3D (scale, scale, scale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom12).FullName, selfUnit.Pos);
			
			if(!selfUnit.AbilityManager.AppendAbility(new SpellContinuousAbility((unit)=>Spelling(unit,level),areaEffect,offset,(int)(durationTime*Def.FPS),this.Name + "areaEffect",interval)))
				areaEffect.Stop();

			for(int i=0;i<durationTime*Def.FPS;++i)
				yield return true;

			yield return false;
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ())
				return false;

			var selfFaction = selfUnit.Faction;
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, new Vector2D (selfUnit.Pos.X, selfUnit.Pos.Y))).Where (_ => _ != null);
			foreach (var t in targets) {
				if(selfFaction.World.IsSameTeam(t.Faction,selfFaction))
					continue;
				var u = t as IUnit;
				var b = t as IBuilding;
				if ( null != u && u.Exists() && u.UniqueID != selfUnit.UniqueID ) {
					u.BeDamaged(selfUnit,damage,AttackBonusSet.None);
					selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, u.Pos);
				}

				if( null != b && b.Exists())
				{
					b.BeDamaged(selfUnit,damage,AttackBonusSet.None);
				}
			}
			
			return true;
		}
	}
}

#endif