#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Bullet.Red;

namespace LotusCraft.Spell.Red
{
	class ScarletDevil : thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime = new int[4];
		IEffect areaEffect = null;
		float affectRange = 7;
		int spellDuration = 8;
		int damage = 150;

		public ScarletDevil()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "紅魔「スカーレットデビル」";
			Description = "範囲内の敵に継続してダメージを与える。";
			string basePath = "Red/Spell/Scarlet/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 120 * 30;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"0.5秒毎に"+damage/2+"ダメージ、" + spellDuration+ "秒間" ,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			var unitRadius = selfUnit.Parameter.Radius;

			var offset = new Vector3D(0, 0, 1);
			areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HujinAreaEffect).FullName, selfUnit.Pos + offset);
			var effectScale = 3.2f * affectRange;
			areaEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);

			selfUnit.DoFindEnemyAutomatically = false;


			for (int i = 0; i < 20; ++i) yield return true;

			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}

			// show cutin
			selfUnit.ShowCutin();

			areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ScarletAreaEffect).FullName, selfUnit.Pos + offset);
			effectScale = 3.5f * affectRange;
			areaEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
			var areaAbility = new Ability.SpellContinuousAbility((unit) => Spelling(unit, level), areaEffect, offset, (int)(spellDuration * Def.FPS), this.Name + "Areadamage", 15);
			areaAbility.StopEffectRootOnVanish = true;
			bool flag = selfUnit.AbilityManager.AppendAbility(areaAbility);

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, selfUnit.Pos);
			

			for (int i = 0; i < spellDuration * Def.FPS; ++i) { yield return true; }

			selfUnit.DoFindEnemyAutomatically = true;

			if (areaEffect != null)
			{
				areaEffect.StopRoot();
				areaEffect = null;
			}

			yield return true;
		}

		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRange * 32, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !world.IsSameTeam(selfFaction, _.Faction)))
			{
				if (!u.AbilityManager.ContainsAbility(this.Name + "HitEffect"))
				{
					/// エフェクトを描画
					var faction = u.Faction;
					Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);
					var effect = world.EffectManager.CreateEffect(typeof(HitSplashRedEffect).FullName, u.Pos);
					var scale = new Vector3D(8, 8, 8);
					effect.SetScale(scale);

					var hitAbility = new Ability.EffectContinuousAbility(effect, 5, this.Name + "HitEffect");
					var flag = u.AbilityManager.AppendAbility(hitAbility);
					if (!flag)
						effect.Stop();
					else
					{
						u.BeDamaged(selfUnit, damage / 2, AttackBonusSet.None);
						selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, u.Pos);
						selfFaction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitOther).FullName, u.Pos);
					}

				}
			}
			return true;
		}

		public void OnVanishUnit(thrts.IUnit selfUnit)
		{
			if (areaEffect != null)
			{
				areaEffect.Stop();
				areaEffect = null;
			}
		}
	}
}

#endif