﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Effect.Red;

namespace LotusCraft.Spell.Red
{
	class Servant:thrts.SpellParameter<thrts.IUnit>
	{
		int[] bulletCounts;
		const int damage = 10;
		const float sightRadius = 4*Def.FPS;
		const string servantBulletsKey =  "flyer_bullets";
		const string servantEffectsKey =  "flyer_effects";
		public Servant()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "サーヴァントフライヤー";
			string basePath = "Red/Spell/Servant/";
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 16*30;
			CoolTimes[2] = 16*30;
			CoolTimes[3] = 16*30;

			MaxRanges= new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 30*Def.MapTipLengthOnField;
			MaxRanges[2] = 30*Def.MapTipLengthOnField;
			MaxRanges[3] = 30*Def.MapTipLengthOnField;

			bulletCounts = new int[MaxLevel + 1];
			bulletCounts[0] = 0;
			bulletCounts[1] = 1;
			bulletCounts[2] = 3;
			bulletCounts[3] = 5;

			Description = "指定地点に向け、視界を持ち敵を捜す蝙蝠弾を発射する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃力" + damage + "、"+bulletCounts[1]+"匹",
				"攻撃力" + damage + "、"+bulletCounts[2]+"匹",
				"攻撃力" + damage + "、"+bulletCounts[3]+"匹",
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit,Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if(null == spell || !spell.CanExec)
				{
					yield return false;
					yield break;
				}
				
				spell.ResetCoolTime ();


				var rel = new Vector2D(targetPosition.X - selfUnit.Pos.X,targetPosition.Y - selfUnit.Pos.Y);
				var d = Utl.GetDistance(targetPosition.X,targetPosition.Y,selfUnit.Pos.X,selfUnit.Pos.Y);
				rel.X *= MaxRanges[level]/d;
				rel.X += selfUnit.Pos.X;
				rel.Y *= MaxRanges[level]/d;
				rel.Y += selfUnit.Pos.Y;
				var targetPos = selfUnit.Faction.World.GetGroundAtPos(rel);
				var initOffest = new Vector3D(0,0,64);

				if(!selfUnit.UserValues.ContainsKey(servantBulletsKey))
				{
					selfUnit.UserValues.Add(servantBulletsKey,new IBullet[bulletCounts.Max()]);
				}
				
				IBullet[] bullets = selfUnit.UserValues[servantBulletsKey] as IBullet[];

				if(!selfUnit.UserValues.ContainsKey(servantEffectsKey))
				{
					selfUnit.UserValues.Add(servantEffectsKey,new IEffect[bulletCounts.Max()]);
				}
				
				IEffect[] effects = selfUnit.UserValues[servantEffectsKey] as IEffect[];

				for(int i=0;i<bulletCounts[level];++i)
				{
					if(null != bullets[i])
					{
						bullets[i].Vanish();
						bullets[i] = null;
						effects[i].Stop();
						effects[i] = null;
					}
				}

				if(null != selfUnit.GetModifiedBulletParameter(typeof(ServantBullet).FullName))
				{
					var r = Def.MapTipLengthOnField;
					for(int i=0;i<bulletCounts[level];++i)
					{
						var th = 2*Math.PI*i/bulletCounts[level];
						var offset = new Vector3D(r*(float)Math.Cos(th),r*(float)Math.Sin(th),32);
						bullets[i] = selfUnit.CreateBullet(typeof(ServantBullet).FullName,targetPos+offset,selfUnit.Pos+offset);
						effects[i] = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ServantBulletEffect).FullName,bullets[i].Position);
						effects[i].SetScale(new Vector3D(16,16,16));
					}
				}

				Action<IUnit> vanished = null;
				vanished = (u) =>
					{
						u.Vanished -= vanished;

						IBullet[] bullets_ = selfUnit.UserValues[servantBulletsKey] as IBullet[];
						IEffect[] effects_ = selfUnit.UserValues[servantEffectsKey] as IEffect[];

						for (int i = 0; i < bullets_.Length; i++)
						{
							if (bullets_[i] != null)
							{
								bullets_[i].Vanish();
								bullets_[i] = null;
							}
						}
						for (int i = 0; i < effects_.Length; i++)
						{
							if (effects_[i] != null)
							{
								effects_[i].Stop();
								effects_[i] = null;
							}
						}
					};

				selfUnit.Vanished += vanished;

			}
			yield return false;
		}

		public override void OnUpdate (IUnit holder)
		{
			if (holder.UserValues.ContainsKey (servantBulletsKey)) 
			{
				IBullet[] bullets = holder.UserValues[servantBulletsKey] as IBullet[];
				IEffect[] effects = holder.UserValues[servantEffectsKey] as IEffect[];
				for(int i=0;i<bullets.Length;++i)
				{
					var b = bullets[i];
					var e = effects[i];
					if(null == b)
					{
						if(null != e)
						{
							e.Stop();
							e = null;
						}
						continue;
					}

					if(b.Exists)
					{
						var pos = b.Position;
						e.SetPosition(pos);
						var direction = b.Vector;
						e.SetRotation(new Vector3D(0,0,(float)Math.Atan2(direction.X,direction.Y)));
						b.Generator.Faction.GetSite(sightRadius,pos.X,pos.Y,2);
					}
					else
					{
						b.Vanish();
						b = null;

						if(null != e)
						{
							e.Stop();
							e = null;
						}
					}
				}
			}
		}
	}
}

namespace LotusCraft.Bullet.Red
{
	public class ServantBullet : General.HeroBullet
	{
		public ServantBullet()
		{
			Type = BulletType.LongRange;
			this.Attack = 10;
			Speed = 3.0f * BasisSpeed;
//			Speed = 1;
			Gravity = 0;
			MaxRange = 30.0f * Def.MapTipLengthOnField;

			IsSplashMissHit = true;
			HasSplash = true;
			
			AttackDelay = (int)(1.0f * Def.FPS);
			AttackRigor = (int)(3.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor = 0;
			
			OnHit = OnHitEffect;
			
		}
		
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
		}
	}
}

#endif