#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Effect.Red;
using LotusCraft.AutoCast;
using LotusCraft.Token;

namespace LotusCraft.Spell.Red
{
	class Ginfu : thrts.SpellParameter<thrts.IUnit>
	{
		int[] bounds;
		const int interval = 10;

		public Ginfu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "銀符「シルバーバウンド」";
			string basePath = "Red/Spell/Ginfu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 10 * 30;
			CoolTimes[2] = 10 * 30;
			CoolTimes[3] = 10 * 30;

			bounds = new int[MaxLevel + 1];
			bounds[0] = 0;
			bounds[1] = 1;
			bounds[2] = 2;
			bounds[3] = 4;

			Description =  "弾がヒットしたとき、ランダムに跳ね返る。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"バウンド" + bounds[1] + "回",
				"バウンド" + bounds[2] + "回",
				"バウンド" + bounds[3] + "回",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,1,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,1),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,2),true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>Spelling(unit,3),true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1", 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2", 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3", 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (IUnit selfUnit, int level)
		{
//			var selfFaction = selfUnit.Faction;
//			var world = selfFaction.World;

			var ability = selfUnit.AbilityManager.GetAbilityWithName(GinfuAbility.AbilityName) as GinfuAbility;
			if (null != ability) {
				ability.ExtendLifetime (interval,interval*2);
				ability.Level = level;
			}
			else {
				IEffect effect = null;
				if(selfUnit.AbilityManager.AppendAbility(new GinfuAbility(level,interval*2,effect)))
					;
			}
			 

			return true;
		}
	}
}


namespace LotusCraft.Ability.Red
{
	class GinfuAbility : FollowEffectAbility
	{
		public const string AbilityName = "銀符「シルバーバウンド」";
		public int Level;
		public GinfuAbility(int level,int duration,IEffect effect = null)
			:base()
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.Effect = effect;
			this.Level = level;
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			var u = bullet.Generator as IUnit;

			if (null != u) {
				var bulletParam = u.GetModifiedBulletParameter (typeof(SakuyaBullet).FullName);
				if (null != bulletParam && bullet.Parameter == bulletParam) {
					switch (Level) {
					case 1:
						bullet.AddOnHitEvent ((b,t) => BounsBullet.Bouns (b, t, typeof(BounsBullet).FullName));
						break;
					case 2:
						bullet.AddOnHitEvent ((b,t) => BounsBullet.Bouns (b, t, typeof(BounsBullet1).FullName));
						break;
					case 3:
						bullet.AddOnHitEvent ((b,t) => BounsBullet.Bouns (b, t, typeof(BounsBullet2).FullName));
						break;
					default:
						break;
					}
				}
			}
			return base.ModifyBullet (bullet);
		}
	
	}
}

namespace LotusCraft.Bullet.Red
{
	public class BounsBullet:SakuyaBullet
	{
		const float bousRatio = 0.4f;
		const int bounsNum = 1;
		public BounsBullet ()
		{
			DoesHitOnlyTarget = true;
		}
		public static void Bouns (IBullet bullet, ITargetObject target, string bulletKind)
		{
			if(null == target)
				return;
			var u = bullet.Generator as IUnit;
			if (u == null)
				return;

			var bulletParam = u.GetModifiedBulletParameter(bulletKind);
			if(null == bulletParam)
				return;

			var world = u.Faction.World;

			float offsetR = 0;
			{
				var tb = target as IBuilding;
				var tu = target as IUnit;
				if (null != tb) {
					offsetR = tb.Parameter.Size * 0.707f;
				}
				if (null != tu) {
					offsetR = tu.Radius;
				}
				
				// shift
				offsetR += 3;
			}

			var targets = world.GetObjectFromCircle (new Circle2D (bulletParam.MaxRange + offsetR, new Vector2D (target.Pos.X, target.Pos.Y)))
				.Where (_ => null != _ && _ != target && !world.IsSameTeam (u.Faction, _.Faction))
					.OrderByDescending (_ => (_.Pos - target.Pos).LengthSquared);



			IBullet b = null;
			int cnt = 0;
			foreach (var t in targets) {
				if (null != t && t.Exists()) {
					var d = (t.Pos - target.Pos);
					d.Normalize ();

					var initPos = target.Pos + d * offsetR;
					while(t.IsHit(initPos.X,initPos.Y,1))
						initPos += d;

					b = u.CreateBullet (bulletKind, t, initPos);
				}
				if (b != null)
				{
					b.Attack = (int)Math.Ceiling(bullet.Attack * bousRatio);
					cnt++;
				}

				if (cnt >= bounsNum)
					break;
			}

			for(int i=0;i< bounsNum - cnt;++i)
			{
				var th = world.NextRandDouble () * Math.PI * 2;
				var tgt = new Vector3D ((float)Math.Sin (th), (float)Math.Cos (th), 0);
				var initPos = target.Pos + tgt * offsetR;
				b = u.CreateBullet (bulletKind, tgt * (bulletParam.MaxRange) + initPos, initPos);

				if (b != null)
					b.Attack = (int)Math.Ceiling (bullet.Attack * bousRatio);
			}
		}
	}
	/// <summary>
	/// Bouns 1 times.
	/// </summary>
	public class BounsBullet1:BounsBullet
	{
		public BounsBullet1 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(BounsBullet).FullName);
		}
	}
	/// <summary>
	/// Bouns 2 times.
	/// </summary>
	public class BounsBullet2:BounsBullet
	{
		public BounsBullet2 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(BounsBullet1).FullName);
		}
	}
	/// <summary>
	/// Bouns 3 times.
	/// </summary>
	public class BounsBullet3:BounsBullet
	{
		public BounsBullet3 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(BounsBullet2).FullName);
		}
	}
	/// <summary>
	/// Bouns 4 times.
	/// </summary>
	public class BounsBullet4:BounsBullet
	{
		public BounsBullet4 ()
		{
			OnHit += (b,t) => Bouns(b,t,typeof(BounsBullet3).FullName);
		}
	}
}

#endif