#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Effect.Red;
using LotusCraft.AutoCast;
using LotusCraft.Token;

namespace LotusCraft.Spell.Red
{
	class Tokifu : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		float[] durationTime;

		float[] decDamageRatio;

		const int interval = 10;
		const int duration = 8*Def.FPS;

		public Tokifu()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "時符「咲夜特製ストップウォッチ」";
			string basePath = "Red/Spell/Tokifu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 45 * 30;
			CoolTimes[2] = 45 * 30;
			CoolTimes[3] = 45 * 30;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;
			affectRadiuses[2] = 4;
			affectRadiuses[3] = 5;

			durationTime = new float[MaxLevel + 1];
			durationTime[0] = 0;
			durationTime[1] = 2;
			durationTime[2] = 4;
			durationTime[3] = 6;

			decDamageRatio = new float[MaxLevel + 1];
			decDamageRatio[0] = 0.0f;
			decDamageRatio[1] = 0.8f;
			decDamageRatio[2] = 0.5f;
			decDamageRatio[3] = 0.2f;

			Description =  "触れた敵ユニットを行動不能にする時計を作りだします。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"全行動不可、被ダメージ" + decDamageRatio[1].ToString("###.%") + "減",
				"全行動不可、被ダメージ" + decDamageRatio[2].ToString("###.%") + "減",
				"全行動不可、被ダメージ" + decDamageRatio[3].ToString("###.%") + "減",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit | !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();
			var offset = new Vector3D(0,0,8);
			var ability = new TokifuControllAbility(selfUnit.Faction.World,decDamageRatio[level],new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y),null,offset,duration );
			if(!selfUnit.AbilityManager.AppendAbility(ability) && null != ability)
				ability.OnVanish(selfUnit);

			yield return false;
		}

	}
}

namespace LotusCraft.Ability.Red
{
	class TokifuControllAbility : FollowEffectAbility
	{
		public const string AbilityName = "時符「咲夜特製ストップウォッチ」"+"_ctrl";
		public Vector2D[] Positions = new Vector2D[6];
		public IEffect[] Effects = new IEffect[6];
		public IWorld world;
		public float DecDamageRatio;
		public static readonly Vector2D[] directions = new Vector2D[]
		{
			new Vector2D(0.0f,1.0f),
			new Vector2D(0.866f,0.5f),
			new Vector2D(0.866f,-0.5f),
			new Vector2D(0.0f,-1.0f),
			new Vector2D(-0.866f,-0.5f),
			new Vector2D(-0.866f,0.5f),
		};
		const float speed = 3.75f;
		const float radius = 2f;
		const int interval = 5;

		int counter = 0;
		Vector3D offset;
		public TokifuControllAbility(IWorld world,float decDamageRatio,Vector2D center,IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			Name = AbilityName;
			DurationTime = duration;
			this.world = world;
			this.DecDamageRatio = decDamageRatio;
			this.offset = offset;
			for(int i=0;i<directions.Length;++i)
			{
				Positions[i] = (new Vector2D(directions[i].X + center.X,directions[i].Y + center.Y));
				Effects[i] = world.EffectManager.CreateEffect(typeof(SakuyaStopWatchEffect).FullName, world.GetGroundAtPos(Positions[i]) + offset);
				var scale = radius*8;
				Effects[i].SetScale(new Vector3D(scale,scale,scale));
			}
			DecDamageRatio = 0;
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order) {
			case AbilityOrder.OtherExecs:
				for(int i=0;i<Effects.Length;++i)
				{
					if(null != Effects[i])
					{
						Positions[i].X += speed*directions[i].X;
						Positions[i].Y += speed*directions[i].Y;
						Effects[i].SetPosition(world.GetGroundAtPos(Positions[i]) + offset);
						if(counter++ > interval)
						{
							counter = 0;
							// search targets
							var targets = world.GetObjectFromCircle(new Circle2D(radius*Def.MapTipLengthOnField,Positions[i])).OfType<IUnit>();
							foreach(var u in targets)
							{
								if( null == u || !u.Exists() || world.IsSameTeam(unit.Faction,u.Faction))
									continue;
	
								var ability = u.AbilityManager.GetAbilityWithName(TokifuAbility.AbilityName) as TokifuAbility;
								if(null == ability)
								{
									u.AbilityManager.AppendAbility(new TokifuAbility(DecDamageRatio,interval * 2));
								}
								else
								{
									ability.ExtendLifetime (interval,interval*2);
									ability.DecDamageRatio = DecDamageRatio;
								}
							}
							targets = null;
						}
					}
				}
				break;
			default:
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool OnVanish (IUnit unit)
		{
			for(var i=0;i<Effects.Length;++i)
			{
				if(null != Effects[i])
				{
					Effects[i].Stop();
					Effects[i] = null;
				}
			}
			return base.OnVanish (unit);
		}
	}
	class TokifuAbility : AbilityTemplate
	{
		public const string AbilityName = "時符「咲夜特製ストップウォッチ」";
		public float DecDamageRatio = 0.3f;
		public TokifuAbility(float decDamageRatio,int duration)
		{
			Name = AbilityName;
			DurationTime = duration;
			DecDamageRatio = decDamageRatio;
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order) {
			case AbilityOrder.ConstDecParam:
				unit.IsForbiddenAttacking = true;
				unit.IsForbiddenSpelling = true;
				unit.IsSemitransparent = true;
				unit.DamageRatio -= DecDamageRatio;
				unit.Speed = 0;
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool OnVanish (IUnit unit)
		{
			unit.IsForbiddenAttacking = false;
			unit.IsForbiddenSpelling = false;

			return base.OnVanish (unit);
		}
	}
 }

#endif