#if !_TRIAL
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.Red;
using LotusCraft.Bullet.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Token;

namespace LotusCraft.Spell.Red
{
	class Trick:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime;
		float[] affectRadiuses;
		int[] damages;
		int[] maxTargetNum;

		public Trick()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "手品「クロースアップ殺人鬼」";
			string basePath = "Red/Spell/Trick/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 24*Def.FPS;
			CoolTimes[3] = 18*Def.FPS;

			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 5*Def.MapTipLengthOnField;
			MaxRanges[2] = 7*Def.MapTipLengthOnField;
			MaxRanges[3] = 9*Def.MapTipLengthOnField;

			chargeTime = new int[MaxLevel+1];
			chargeTime[0] = 0;
			chargeTime[1] = Def.FPS/2;
			chargeTime[2] = Def.FPS/2;
			chargeTime[3] = Def.FPS/2;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 2;
			affectRadiuses[2] = 3;
			affectRadiuses[3] = 4;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 4;
			damages[2] = 8;
			damages[3] = 12;

			maxTargetNum = new int[MaxLevel + 1];
			maxTargetNum[0] = 0;
			maxTargetNum[1] = 3;
			maxTargetNum[2] = 5;
			maxTargetNum[3] = 7;

			Description = "目標へ瞬間移動し、近の敵にナイフを投げる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"射程" + (MaxRanges[1]/Def.MapTipLengthOnField) + "、範囲"+ affectRadiuses[1] + "、" + damages[1]+" ダメージ"+"、最大"+maxTargetNum[1]+"体",
				"射程" + (MaxRanges[2]/Def.MapTipLengthOnField) + "、範囲"+ affectRadiuses[2] + "、" + damages[2]+" ダメージ"+"、最大"+maxTargetNum[2]+"体",
				"射程" + (MaxRanges[3]/Def.MapTipLengthOnField) + "、範囲"+ affectRadiuses[3] + "、" + damages[3]+" ダメージ"+"、最大"+maxTargetNum[3]+"体",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			{
				// reset cooltime
				var spell = selfUnit.GetSpellWithName (Name);
				if (null == spell || !spell.CanExec) {
					yield return false;
					yield break;
				}

				spell.ResetCoolTime ();

				float affectRadius = affectRadiuses [level];

				var targetPos = selfUnit.Faction.World.GetGroundAtPos (targetPosition);

				// effect offset
				Vector3D offset = new Vector3D (0.0f, 0.0f, 1.0f);

				/// change effect
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect (typeof(SakuyaStopWatchEffect).FullName, targetPos + offset);
				var scale = (affectRadius) * Def.MapTipLengthOnField / 3.0f;
				effect.SetScale (new Vector3D (scale, scale, scale));
				
				if (!selfUnit.AbilityManager.AppendAbility (new EffectContinuousAbility (effect, chargeTime [level], this.Name + "chargeEffect")))
					effect.Stop ();
				
				for (int i=0; i<chargeTime[level]; ++i) {
					yield return true;
				}

				var selfFaction = selfUnit.Faction;
				var world = selfFaction.World;
				selfUnit.Warp (targetPosition,true);

				var targets = world.GetObjectFromCircle(new Circle2D(affectRadius*Def.MapTipLengthOnField,targetPosition));
				int counter = 0;
				foreach(var t in targets)
				{
					if(null == t || !t.Exists() || world.IsSameTeam(t.Faction,selfFaction))
						continue;

					var b = selfUnit.CreateBullet(typeof(SakuyaBullet).FullName,t,selfUnit.Pos);
					if(null != b)
					{
						b.Attack = damages[level];
						counter++;
					}
					if(counter >= maxTargetNum[level])
						break;
				}


			}
			yield return false;
		}
	}
}

#endif