﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Building.White;
using LotusCraft.Unit.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class Hieda:thrts.SpellParameter<thrts.IUnit>
	{
		Dictionary<TechnologyParameter<ITargetObject>,int> techOrigTime; 
		Dictionary<UnitParameter,int> unitOrigTime; 
		float[] decRatio;
		float[] decUnitMakeTimeRatio;
		public Hieda()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "幻想郷縁起";
			Description = "研究に必要な時間が減少する。";
			string basePath = "White/Spell/Hieda/";

			decRatio = new float[MaxLevel+1];
			decRatio[0] = 0;
			decRatio[1] = 0.10f;
			decRatio[2] = 0.20f;
			decRatio[3] = 0.50f;
			decUnitMakeTimeRatio = new float[MaxLevel + 1];
			decUnitMakeTimeRatio[0] = 0;
			decUnitMakeTimeRatio[1] = 0.10f;
			decUnitMakeTimeRatio[2] = 0.20f;
			decUnitMakeTimeRatio[3] = 0.30f;


			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"下級ユニットの生産時間"+( 1- decUnitMakeTimeRatio[1]).ToString("###.%")+"、研究所での研究時間"+( 1- decRatio[1]).ToString("###.%"),
				"下級ユニットの生産時間"+( 1- decUnitMakeTimeRatio[2]).ToString("###.%")+"、研究所での研究時間"+( 1- decRatio[2]).ToString("###.%"),
				"下級ユニットの生産時間"+( 1- decUnitMakeTimeRatio[3]).ToString("###.%")+"、研究所での研究時間"+( 1- decRatio[3]).ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,Def.FPS,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,Def.FPS,this.AutoCastAction_lv1,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,Def.FPS,this.AutoCastAction_lv2,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,Def.FPS,this.AutoCastAction_lv3,true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
			IsPassive = true;

			techOrigTime = new Dictionary<TechnologyParameter<ITargetObject>, int>();
			unitOrigTime = new Dictionary<UnitParameter, int>();
		}
		public bool Spelling(thrts.IUnit selfUnit, int level)
		{
			return true;
		}
		public bool AutoCastAction_lv1(thrts.IUnit unit)
		{
			Spelling(unit, 1);
			return true;
		}
		public bool AutoCastAction_lv2(thrts.IUnit unit)
		{
			Spelling(unit, 2);
			return true;
		}
		public bool AutoCastAction_lv3(thrts.IUnit unit)
		{
			Spelling(unit, 3);
			return true;
		}
		public override void OnLvUp (IFaction faction, int lv)
		{
//			if(faction == null)
//				return;

			/// エフェクトを描画
			//Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
			//var effect = faction.World.EffectManager.CreateEffect(typeof(KiokuEffect).FullName, selfUnit.Pos);
			//var scale = new Vector3D(12, 12, 12);
			//effect.SetScale(scale);
			//
			//var hitAbility = new Ability.EffectContinuousAbility(effect, 10, this.Name + "Effect");
			//var flag = selfUnit.AbilityManager.AppendAbility(hitAbility);
			//if (!flag)
			//	effect.Stop();

			// seach buildings
			var bs = faction.GetBuildingParameter (typeof(BlackSmith).FullName);
			if (bs == null)
				return;

			// get techs
			var techs = bs != null ? bs.TechnologyParams : null;
			if (techs == null)
				return;
			
			// get techs
			var low02 = faction.GetUnitParameter (typeof(Low02).FullName);
			var low03 = faction.GetUnitParameter (typeof(Low03).FullName);
			if (null == low02 || null == low03)
				return;
			
			foreach (var te in techs) {
				if (!techOrigTime.ContainsKey (te)) {
					int time = te.TechTime;
					techOrigTime.Add (te, time);
				}

				/// dec time
				te.TechTime = (int)(techOrigTime[te] * ( 1- decRatio[lv]));
			}

			if (!unitOrigTime.ContainsKey (low02)) {
				int time = low02.MakingTime;
				unitOrigTime.Add(low02,time);
			}
			low02.MakingTime = (int)(unitOrigTime[low02] * ( 1- decUnitMakeTimeRatio[lv]));

			if (!unitOrigTime.ContainsKey (low03)) {
				int time = low03.MakingTime;
				unitOrigTime.Add(low03,time);
			}
			low03.MakingTime = (int)(unitOrigTime[low03] * ( 1- decUnitMakeTimeRatio[lv]));

			faction.ReloadParameters();
			return ;
		}
	}
}
