﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Spell.White;

namespace LotusCraft.Spell.White
{
	class AirMakimono:thrts.SpellParameter<thrts.IUnit>
	{
		int[] durations;
		float[] incMoveRatio;
		float[] attackBonus;
		public const string SpellName =  "「聖尼公のエア巻物」";
		public AirMakimono()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = SpellName;
			Description = "攻撃力、移動速度が上昇する。攻撃すると効果が終了する。";
			string basePath = "White/Spell/AirMakimono/";

			CoolTimes = new int[4];
			CoolTimes[0] = 0;
			CoolTimes[1] = 5 * 30;
			CoolTimes[2] = 4 * 30;
			CoolTimes[3] = 4 * 30;

			incMoveRatio = new float[MaxLevel+1];
			incMoveRatio[0] = 0;
			incMoveRatio[1] = 0.2f;
			incMoveRatio[2] = 0.6f;
			incMoveRatio[3] = 1.0f;

			attackBonus = new float[MaxLevel+1];
			attackBonus[0] = 0;
			attackBonus[1] = 0.4f;
			attackBonus[2] = 0.8f;
			attackBonus[3] = 1.2f;

			durations = new int[MaxLevel+1];
			durations[0] = 0;
			durations[1] = 10*Def.FPS;
			durations[2] = 14*Def.FPS;
			durations[3] = 18*Def.FPS;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				durations[1]/Def.FPS+"秒間、移動速度+"+incMoveRatio[1].ToString("###.%")+"、攻撃力+"+attackBonus[1].ToString("###.%"),
				durations[2]/Def.FPS+"秒間、移動速度+"+incMoveRatio[2].ToString("###.%")+"、攻撃力+"+attackBonus[2].ToString("###.%"),
				durations[3]/Def.FPS+"秒間、移動速度+"+incMoveRatio[3].ToString("###.%")+"、攻撃力+"+attackBonus[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャスト
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists () || selfUnit.AbilityManager.ContainsAbility(AirMakimonoAbility.AbilityName)) {
				yield return false;
				yield break;
			}
			var spell = selfUnit.GetSpellWithName (this.Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			
			spell.ResetCoolTime();
			spell.BeDecCooltime = false;
			
			{
				Vector3D offset = new Vector3D(0,0,-10);
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(AirmakimonoEffect).FullName, selfUnit.Pos + offset);
				var scale = 3 * 8;
				effect.SetScale(new Vector3D(scale, scale, scale));
				if (!selfUnit.AbilityManager.AppendAbility(new AirMakimonoSelfAbility(spell,incMoveRatio[level],attackBonus[level], effect,offset,durations[level])))
					effect.Stop();
			}
			
		}

	}
}

namespace LotusCraft.Ability.White
{
	class AirMakimonoSelfAbility : FollowEffectAbility
	{
		public static string AbilityName = "「聖尼公のエア巻物」";
		public float IncMoveRatio;
		public float AttackBonus;
		ISpell spell;
		public AirMakimonoSelfAbility(ISpell spell,float incMoveRatio,float incAtkRatio,IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			IncMoveRatio = incMoveRatio;
			AttackBonus = incAtkRatio;
			this.spell = spell;
		}

		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch (order) {
			case AbilityOrder.RateIncParam:
				unit.Armor += 2;
				unit.Speed *= (IncMoveRatio+1.0f);
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.Attack += (int)Math.Ceiling (b.Attack*AttackBonus);
				}
				break;
			case AbilityOrder.OtherExecs:
				if(null != spell)
					spell.BeDecCooltime = false;
				break;
			default:
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool ModifyBullet (IBullet bullet)
		{
			bullet.AddOnHitEvent(OnHit);
			return base.ModifyBullet (bullet);
		}
		public override bool OnVanish (IUnit unit)
		{
			var spell = unit.GetSpellWithName(AirMakimono.SpellName);
			if(null != spell)
				spell.BeDecCooltime = true;
			return base.OnVanish (unit);
		}
		public void OnHit(IBullet bullet,ITargetObject target)
		{
			this.DurationTime = 0;
		}
	}
}

#endif