﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Ability.White;
using LotusCraft.Ability;

namespace LotusCraft.Spell.White
{
	class Chojin:thrts.SpellParameter<thrts.IUnit>
	{
		int[] chargeTime = new int[2];

		ChojinAbility[] spellAbilities = new[]
		{
			new ChojinAbility(0),
			new ChojinAbility(1),
		};

		IEffect effect = null;
		public Chojin()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "超人「聖白蓮」";
			Description = "攻撃速度と移動速度が大幅に上昇する。";
			string basePath = "White/Spell/Chojin/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃速度+"+(spellAbilities[1].AtkSpeedRatio - 1.0f).ToString("###.%")+"/移動速度"+spellAbilities[1].IncSpeedRatio
			};

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 80 * 30;


			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			selfUnit.ShowCutin();
			
			Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);
			effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(ChojinEffect).FullName,selfUnit.Pos + offset);
			var scale = 17.0f;
			effect.SetScale(new Vector3D(scale, scale, scale));

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);

			bool flag = selfUnit.AbilityManager.AppendAbility(new ChojinAbility(level, effect));

			if (!flag)
			{
				effect.Stop();
				effect = null;
			}

			for (int i = 0; i < (1 * Def.FPS) / 3 * 2; ++i)
			{
				yield return null;
			}

			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Shuiin01).FullName, selfUnit.Pos);
			
			yield return true;
		}

		public void OnVanishUnit(IUnit selfUnit)
		{
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
		}
		
	}
}

namespace LotusCraft.Ability.White
{
	class ChojinAbility : thrts.AbilityTemplate
	{
		public float IncSpeedRatio{ get; private set; }
		public float AtkSpeedRatio{ get; private set; }
		thrts.IEffect effect;
		public ChojinAbility(int level, thrts.IEffect effect = null)
		{
			this.Name = "超人「聖白蓮」";
			this.DurationTime = 10*30;
			this.effect = effect;
			switch (level)
			{
			case 1:
				IncSpeedRatio = 1.0f;
				AtkSpeedRatio = 2.5f;
				break;
			default:
				IncSpeedRatio= 0;
				break;
			}
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order) {
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				///移動速度をspeedRate倍
				unit.Speed += unit.Speed*IncSpeedRatio;

				///攻撃速度をAtkpeedRate倍
				var bullet = unit.GetModifiedBulletParameter(typeof(HijiriBullet).FullName);
				if(null != bullet)
				{
					bullet.AttackDelay  = (int)Math.Ceiling(bullet.AttackDelay /AtkSpeedRatio);
					bullet.AttackRigor = (int)Math.Ceiling(bullet.AttackRigor /AtkSpeedRatio);
					bullet.AttackSpeed = (int)Math.Floor(bullet.AttackSpeed /AtkSpeedRatio);
				}
				break;
			case thrts.AbilityOrder.OtherExecs:
				if (effect != null)
					effect.SetPosition(unit.Pos + new thrts.Vector3D(2, 2, 1));
				break;
			default: 
				break;
			}
			
			return true;
		}
		public override bool OnVanish(thrts.IUnit unit)
		{
			if (effect != null)
			{
				effect.StopRoot();
				effect = null;
			}
			return base.OnVanish(unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			
			return base.ModifyBullet(bullet);
		}
	}
	
}

#endif