﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class StarSword : thrts.SpellParameter<thrts.IUnit>
	{
		StarSwordAbility[] spellAbilities = new[]
		{
			new StarSwordAbility(0),
			new StarSwordAbility(1),
			new StarSwordAbility(2),
			new StarSwordAbility(3),
		};
		
		int[] durations;
		public StarSword()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "「スターソードの護法」";
			string basePath = "White/Spell/StarSword/";

			CoolTimes = new int[4];
			CoolTimes[0] = 0;
			CoolTimes[1] = 20 * 30;
			CoolTimes[2] = 20 * 30;
			CoolTimes[3] = 20 * 30;


			MinRanges = new float[4];
			MinRanges[0] = 0;
			MinRanges[1] = 0;
			MinRanges[2] = 0;
			MinRanges[3] = 0;
			MaxRanges = new float[4];
			MaxRanges[0] = 0;
			MaxRanges[1] = 10 * 32;
			MaxRanges[2] = 10 * 32;
			MaxRanges[3] = 10 * 32;

			durations = new int[MaxLevel + 1];
			durations[0] = 0;
			durations[1] = 4 * Def.FPS;
			durations[2] = 6 * Def.FPS;
			durations[3] = 8 * Def.FPS;


			Description = "指定地点の周囲のユニットのスペルを一定時間使用不可にする。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"効果時間:4s",
				"効果時間:6s",
				"効果時間:8s"
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, OnFinishPrivilegeMode, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling, OnFinishPrivilegeMode, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling, OnFinishPrivilegeMode, 3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
				// reset cooltime
				selfUnit.GetSpellWithName(Name).ResetCoolTime();
				int affectRange = 0;
				switch (level)
				{
					case 1:
						affectRange = 2;
						break;
					case 2:
						affectRange = 3;
						break;
					case 3:
						affectRange = 4;
						break;
					default:
						break;
				}

				var targetPos = selfUnit.Pos;

				// don't search enemy
				selfUnit.DoFindEnemyAutomatically = false;


				// effect offset
				Vector3D offset = new Vector3D(2.0f, 2.0f, 1.0f);


				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Burst).FullName, targetPos);

				/// add spell continuous ability
				IEffect selfEff = null;
				var ability = new Ability.SpellContinuousAbility((unit) => Spelling(unit, targetPos, level, affectRange), selfEff, offset, 1 * 30, this.Name, 15);

				/// create spell Effect;
				var areaEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(StarSwordAreaEffect).FullName, targetPos + offset);
				var effectScale = 1.7f * affectRange * 3;
				areaEffect.SetScale(new Vector3D(effectScale, effectScale, effectScale));

				var areaAbility = new Ability.EffectContinuousAbility(areaEffect, 5*Def.FPS, this.Name + "AreaEffect");
				
				var flag = selfUnit.AbilityManager.AppendAbility(ability);
				flag = selfUnit.AbilityManager.AppendAbility(areaAbility) && flag;
				if (!flag)
				{
					areaEffect.Stop();
					areaEffect = null;
					// don't search enemy
					selfUnit.DoFindEnemyAutomatically = true;
				}

			yield return false;
		}
		public void OnFinishPrivilegeMode(IUnit selfUnit, int level)
		{
			
		}
		public bool Spelling(IUnit selfUnit, Vector3D targetPos, int level, int affectRange = 0)
		{
			var duration = durations[level];

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRange * 32, new Vector2D(targetPos.X, targetPos.Y))).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_ => !selfFaction.World.IsSameTeam(selfFaction, _.Faction)))
			{
				if (!u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					/// エフェクトを描画
					var faction = u.Faction;
					Vector3D offset = new Vector3D(2.0f, 2.0f, 10.0f);
					// 50dmg/1s => 50dps
					var effect = world.EffectManager.CreateEffect(typeof(StarSwordEffect).FullName, u.Pos + offset);
					float effectScale = 1.7f * 5;
					effect.SetScale(new Vector3D(effectScale, effectScale, effectScale));
					if (!u.AbilityManager.AppendAbility(new StarSwordAbility(level, effect)))
					{
						effect.Stop();
					}
					else
					{
						u.IsForbiddenSpelling = true;
					}
				}
				else
				{
					u.IsForbiddenSpelling = true;
				}
			}

			return true;
		}
		
		public void OnVanishUnit(IUnit selfUnit)
		{
		}
	}
}

#endif