﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.White;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
namespace LotusCraft.Spell.White
{
	class HourainoKusuri : thrts.SpellParameter<thrts.IUnit>
	{
		const float affectRange = 5 * 32;
		const int duration = 12*Def.FPS;
		const int interval = 10;
		public HourainoKusuri()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "禁薬「蓬莱の薬」";
			Description = "効果中、周囲に居るヒロイン以外の味方ユニットは\n死亡時に復活するようになる。ただし、効果終了時に死亡する。";
			string basePath = "White/Spell/HourainoKusuri/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 180*Def.FPS;
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				duration/(float)Def.FPS + "秒間",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, "White/Spell/HourainoKusuri/LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}

			spell.ResetCoolTime();
			// charge effect

			// show cutin
			selfUnit.ShowCutin();


			/// エフェクトの描画
			Vector3D offset = new Vector3D(0.0f, 0.0f, 4.0f);
			var areaeffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HouraiAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 15.0f;
			areaeffect.SetScale(new Vector3D(scale, scale, scale));

			var areaAbility = new SpellContinuousAbility(Spelling, areaeffect, offset, duration, this.Name + "areaEffect", interval);
			areaAbility.StopEffectRootOnVanish = true;
			if (!selfUnit.AbilityManager.AppendAbility(areaAbility))
				areaeffect.Stop(); 

			yield return true;
		}
		public bool Spelling(IUnit selfUnit)
		{
			if(null == selfUnit || !selfUnit.Exists())
				return false;

			Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);
			var selfFaction = selfUnit.Faction;
			var target = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRange,new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y))).OfType<IUnit>();
			foreach (thrts.IUnit u in target)
			{
				/// ヒロインと敵には掛からない
				if (null == u || !u.Exists() || u.Parameter.IsUnique || !selfFaction.World.IsSameTeam(u.Faction,selfFaction))
					continue;

				var ability = u.AbilityManager.GetAbilityWithName(HourainoKusuriAbility.AbilityName);
				if(null == ability)
				{
					var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(HouraiEffect).FullName, u.Pos + offset, selfUnit.Faction);
					var scale2 = 5.0f;
					effect.SetScale(new Vector3D(scale2, scale2, scale2));
					u.AbilityManager.AppendAbility(new Ability.White.HourainoKusuriAbility(effect,offset,interval*2));
				}
				else
				{
					ability.ExtendLifetime(interval,interval*2);
				}
			}
			return true;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class HourainoKusuriAbility : FollowEffectAbility
	{
		public const string AbilityName =  "禁薬「蓬莱の薬」";
		bool isRevived = false;
		public HourainoKusuriAbility(IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
		}
		public override bool OnAdd (IUnit unit)
		{
			isRevived = false;
			unit.Vanishing += Vanishing;
			return base.OnAdd (unit);
		}
		public override bool OnVanish(thrts.IUnit unit)
		{
			unit.Vanishing -= Vanishing;
			// 蓬莱の薬で復活していたら死亡
			if(isRevived)
				unit.HP = 0;
			return base.OnVanish(unit);
		}
		public void Vanishing(ITargetObject obj)
		{
			var u = obj as IUnit;

			if(u == null)
				return;

			// HPフルで復活
			u.HP = u.MaxHP;
			isRevived = true;
		}
	}
}

#endif