﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft;
using LotusCraft.Effect.White;
using LotusCraft.Ability;
using LotusCraft.Ability.White;

namespace LotusCraft.Spell.White
{
	public class Kochomumaru : SpellParameter<IUnit>
	{
		float[] incSpeedRate;
		float[] incDpsRate;
		const float decHpRatio = 0.2f;
		const int duration = 15 * Def.FPS;
		public Kochomumaru()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "胡蝶夢丸ナイトメア";
			Description = "周囲の味方兵ユニットの攻撃力と攻撃速度を増加させる。\n民兵等召喚ユニットには効果無し。";
			string basePath = "White/Spell/Kochomumaru/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 15 * Def.FPS;
			CoolTimes[2] = 15 * Def.FPS;
			CoolTimes[3] = 15 * Def.FPS;

			incSpeedRate = new float[MaxLevel + 1];
			incSpeedRate[0] = 0;
			incSpeedRate[1] = 0.15f;
			incSpeedRate[2] = 0.30f;
			incSpeedRate[3] = 0.50f;
			incDpsRate = new float[MaxLevel + 1];
			incDpsRate[0] = 0;
			incDpsRate[1] = 0.2f;
			incDpsRate[2] = 0.4f;
			incDpsRate[3] = 0.6f;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"HP"+decHpRatio.ToString("###.%")+"消費、移動速度+"+incSpeedRate[1].ToString("###.%")+"、攻撃速度+"+incDpsRate[1].ToString("###.%"),
				"HP"+decHpRatio.ToString("###.%")+"消費、移動速度+"+incSpeedRate[2].ToString("###.%")+"、攻撃速度+"+incDpsRate[2].ToString("###.%"),
				"HP"+decHpRatio.ToString("###.%")+"消費、移動速度+"+incSpeedRate[3].ToString("###.%")+"、攻撃速度+"+incDpsRate[3].ToString("###.%"),
			};

			/// ボタンの位置
			ButtonPosX = ButtonPosition._0;
			ButtonPosY = ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new AutoCastParameter<IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,60,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,CoolTimes[1] + Def.FPS,this.AutoCastAction_lv1),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,CoolTimes[2] + Def.FPS,this.AutoCastAction_lv2),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,CoolTimes[3] + Def.FPS,this.AutoCastAction_lv3),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivirageModeSpelling(unit,1)),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",(unit)=>PrivirageModeSpelling(unit,2)),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",(unit)=>PrivirageModeSpelling(unit,3)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public IEnumerable<object> PrivirageModeSpelling(IUnit selfUnit, int level)
		{

			if (!Spelling(selfUnit, level))
			{
				yield return false;
				yield break;
			}
			yield return false;
		}
		public bool Spelling(IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
				return false;

			var spell = selfUnit.GetSpellWithName(Name);
			if (null == spell || !spell.CanExec)
				return false;

			spell.ResetCoolTime();

			float affectRange = 0;
			switch (level)
			{
				case 1:
					affectRange = 5 * 32;
					break;
				case 2:
					affectRange = 6 * 32;
					break;
				case 3:
					affectRange = 6 * 32;
					break;
				default:
					affectRange = 0;
					break;
			}

			/// エフェクトの描画
			Vector3D offset = new Vector3D(0.0f, 0.0f, 4.0f);
			var areaeffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NightmareAreaEffect).FullName, selfUnit.Pos + offset, selfUnit.Faction);
			var scale = 15.0f;
			areaeffect.SetScale(new Vector3D(scale, scale, scale));

			var effectAbility = new EffectContinuousAbility(areaeffect, 2 * Def.FPS, "NightmareAreaEffect");
			effectAbility.StopEffectRootOnVanish = true;
			if (!selfUnit.AbilityManager.AppendAbility(effectAbility))
				areaeffect.Stop();

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var target = world.GetObjectFromRectangle(selfUnit.Pos.X, selfUnit.Pos.Y, affectRange, affectRange, selfUnit.Pos.X, selfUnit.Pos.Y, affectRange, affectRange);
			offset = new Vector3D(0.0f, 0.0f, 2.0f);
			foreach (IUnit u in target.OfType<IUnit>())
			{
				if (null == u || !u.Exists() || u.Parameter.IsUnique || u.Parameter.IsFarmer || u.IsSummon || !world.IsSameTeam(u.Faction, selfFaction))
					continue;

				var ability = u.AbilityManager.GetAbilityWithName(KochomumaruAbility.AbilityName) as KochomumaruAbility;
				if (null == ability)
				{
					int decHP = (int)Math.Ceiling(u.MaxHP*decHpRatio);
					if (u.HP > decHP)
					{
						/// エフェクトを描画
						var faction = u.Faction;
						var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NightmareEffect).FullName, u.Pos + offset, selfUnit.Faction);
						var scale2 = 16.0f;
						effect.SetScale(new Vector3D(scale2, scale2, scale2));

						u.HP -= decHP;
						var targetAbility = new Ability.White.KochomumaruAbility(1 + incSpeedRate[level], 1 + incDpsRate[level], effect, offset, duration);
						u.AbilityManager.AppendAbility(targetAbility);
					}
				}
				else
				{
					ability.SpeedRate = 1 + incSpeedRate[level];
					ability.DpsRate = 1 + incDpsRate[level];
				}

			}

			return true;
		}
		public bool AutoCastAction_lv1(IUnit unit)
		{
			return Spelling(unit, 1);
		}
		public bool AutoCastAction_lv2(IUnit unit)
		{
			return Spelling(unit, 2);
		}
		public bool AutoCastAction_lv3(IUnit unit)
		{
			return Spelling(unit, 3);
		}
	}
}

namespace LotusCraft.Ability.White
{
	class KochomumaruAbility : FollowEffectAbility
	{
		public float SpeedRate;
		public float DpsRate;
		public const string AbilityName = "胡蝶夢丸ナイトメア";

		public KochomumaruAbility(float speedRate,float dpsRate, thrts.IEffect effect,Vector3D offset,int duration)
			:base(effect,offset,duration,AbilityName)
		{
			this.SpeedRate =speedRate;
			this.DpsRate = dpsRate;
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				///移動速度をspeedRate倍
				unit.Speed *= SpeedRate;
				/// DPSをdpsRate倍
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.AttackDelay = (int)Math.Round(b.AttackDelay / DpsRate);
					b.AttackRigor = (int)Math.Round(b.AttackRigor / DpsRate);
					b.AttackSpeed = (int)Math.Floor(b.AttackSpeed / DpsRate);
				}
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			
			return base.ModifyBullet(bullet);
		}
	}
	
}

#endif