﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Spell.White
{
	class OmoikaneDevice : thrts.SpellParameter<thrts.IUnit>
	{
		static string buildingBaseName = "LotusCraft.Building.White.Omoikane";

		public OmoikaneDevice()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			string basePath = "White/Spell/OmoikaneDevice/";
			Name = "操神「オモイカネディバイス」";
			Description = "周囲ユニットのHPを徐々に回復させるオブジェクトを設置する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1% HP/s 回復、30秒",
				"2% HP/s 回復、30秒",
				"3% HP/s 回復、30秒",
			};
			
			/// クールタイムの設定
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 99*Def.FPS;
			CoolTimes[1] = 40*Def.FPS;
			CoolTimes[2] = 40*Def.FPS;
			CoolTimes[3] = 40*Def.FPS;

			MaxRanges = new float[MaxLevel+1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 4*Def.MapTipLengthOnField;
			MaxRanges[2] = 6*Def.MapTipLengthOnField;
			MaxRanges[3] = 8*Def.MapTipLengthOnField;

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath+"LvUp");
		}

		/// <summary>
		/// 建物の生成
		/// </summary>
		/// <param name="selfUnit"></param>
		/// <param name="level"></param>
		/// <returns></returns>
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, float x, float y, int level)
		{


			var spell = selfUnit.GetSpellWithName (this.Name);

			if (spell == null) {
				yield return false;
				yield break;
			}

			if(!spell.CanExec){
				yield return false;
				yield break;
			}

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var targetPositon = new thrts.Vector2D (x, y);

			/// 選択建物名を選択
			string buildingName = string.Empty;
			switch (level) {
			case 1:
				buildingName = buildingBaseName + "_lv1";
				break;
			case 2:
				buildingName = buildingBaseName + "_lv2";
				break;
			case 3:
				buildingName = buildingBaseName + "_lv3";
				break;
			default:
				break;
			}

			var buildingParam = selfFaction.GetBuildingParameter (buildingName);
			if (null == buildingParam) {
				yield return false;
				yield break;
			}

			// 建物の内接円の中に何かあったら失敗
			if (world.GetObjectFromCircle (new Circle2D (buildingParam.Size * 0.707f, targetPositon)).Any ()) {
				yield return false;
				yield break;
			}


			// ヒーリングオブジェクトの配置
			var building = world.BuildingManager.Create (buildingName, selfFaction, (int)(x / thrts.Def.MapTipLengthOnField), (int)(y / thrts.Def.MapTipLengthOnField), 100.0f, selfFaction);
			if (building != null) {
				spell.ResetCoolTime ();

				// 建物の中に何かあったら外に出す
				var width = building.Size * Def.MapTipLengthOnField;
				var height = building.Size * Def.MapTipLengthOnField;
				var targets = world.GetObjectFromRectangle (new RectF(), new RectF (building.Pos.X-width/2, building.Pos.Y-height/2,width,height)).OfType<IUnit> ();

				foreach (var tu in targets) {
					tu.Warp(new Vector2D(tu.Pos.X,tu.Pos.Y),true);
				}

			}
			// エフェクトの描画


			yield return false;
		}
	}
}

#endif