﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.White;
using LotusCraft.Building.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class Nyoki:thrts.SpellParameter<thrts.IUnit>
	{
		static string unitBaseName = "LotusCraft.Unit.White.NyokiObj";
		static string unitBaseName2 = "LotusCraft.Unit.White.NyokiObj2";

		public Nyoki()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "鉄拳「入道にょき」";
			Description = "通り抜け不可・移動不可の雲山の拳を出現させる。";
			string basePath = "White/Spell/Nyoki/";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"横長",
				"さらに横長",
				"さらに横長",
			};

			/// クールタイムの設定
			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 25 * Def.FPS;
			CoolTimes[1] = 25 * Def.FPS;
			CoolTimes[2] = 25 * Def.FPS;
			CoolTimes[3] = 25 * Def.FPS;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 8 * Def.MapTipLengthOnField;
			MaxRanges[2] = 8 * Def.MapTipLengthOnField;
			MaxRanges[3] = 9 * Def.MapTipLengthOnField;

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		/// <summary>
		/// 建物の生成
		/// </summary>
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit,float x,float y, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime();

			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;

			var targetPos = world.GetGroundAtPos(new thrts.Vector2D(x, y));

			int length = 0;
			switch (level)
			{
				case 1:
					length = 2;
					break;
				case 2:
					length = 4;
					break;
				case 3:
					length = 7;
					break;
				default:
					break;
			}

			float offset = (float)LotusCraft.Unit.Utls.UnitSize.Large * 2 + 1;

			//sr0注意
			float sy = y - selfUnit.Pos.Y;
			float sx = x - selfUnit.Pos.X;
			float sr = (float)Math.Sqrt(sy * sy + sx * sx);
			if (sr != 0)
			{
				sy = sy * offset / sr;
				sx = sx * offset / sr;
			}
			else
			{
				sy = selfUnit.Pos.Y;
				sx = selfUnit.Pos.X;
			}

			float ty = -(x - selfUnit.Pos.X);
			float tx = y - selfUnit.Pos.Y;
			float tr = (float)Math.Sqrt(ty * ty + tx * tx);
			if (tr != 0)
			{
				ty = ty * offset / tr;
				tx = tx * offset / tr;
			}
			else
			{
				ty = selfUnit.Pos.Y;
				tx = selfUnit.Pos.X;
			}

			for (int i = 0; i <= length; i++)
			{
				for (int j = 0; j <= 1; j++)
				{
					Vector3D effectoffset = new Vector3D(0.0f, 0.0f, 1.0f);
					
					var newu = world.UnitManager.Create(unitBaseName, selfFaction, x + i * tx - j * sx, y + i * ty - j * sy, selfFaction);

					if (newu == null) { newu = world.UnitManager.Create(unitBaseName2, selfFaction, x + i * tx - j * sx, y + i * ty - j * sy, selfFaction);}

					if (newu != null)
					{
						newu.IsPreventedFromFindingEnemy = true;
						newu.HoldPosition(false);
						/// create Effect;
						
						var objEffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NyokiEffect).FullName, newu.Pos + effectoffset);
						objEffect.SetScale(new Vector3D(8, 8, 8));
						newu.Vanished += (unit) =>
						{
							if (null != objEffect)
							{
								objEffect.StopRoot();
								objEffect = null;
							}
						};
					}

					if (i != 0)
					{
						var newu2 = world.UnitManager.Create(unitBaseName, selfFaction, x - i * tx - j * sx, y - i * ty - j * sy, selfFaction);

						if (newu2 == null) { newu2 = world.UnitManager.Create(unitBaseName2, selfFaction, x - i * tx - j * sx, y - i * ty - j * sy, selfFaction); }

						if (newu2 != null) {
							newu2.HoldPosition(false);
							
							var objEffect2 = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(NyokiEffect).FullName, newu2.Pos + effectoffset);
							objEffect2.SetScale(new Vector3D(12, 12, 12));
							newu2.Vanished += (unit) =>
							{
								if (null != objEffect2)
								{
									objEffect2.StopRoot();
									objEffect2 = null;
								}
							};
	
							newu2.IsPreventedFromFindingEnemy = true; 
						}
					}
				}
			}

			yield return false;
		}
	}
}

#endif