﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;

namespace LotusCraft.Spell.White
{
	class KaguyaSkill : thrts.SpellParameter<thrts.IUnit>
	{
		/// <summary>
		/// The mode change time[s].
		/// </summary>
		const float modeChangeTime = 4.0f;
		const int interval = 10;
		public KaguyaSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "新難題「金閣寺の一枚天井」";
			string basePath = "White/Spell/Kaguya/";

			CoolTimes = new int[2];
			CoolTimes[0] = 4 * Def.FPS;
			CoolTimes[1] = 4 * Def.FPS;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"砲撃状態に移行し、移動能力を失い射程と範囲ダメージを得る。",
				"通常状態に戻り、移動能力を得る。",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				null,
				null,
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,(unit)=>AutoCastAction(unit,1)),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",(unit)=>PrivilegeModeSpelling(unit,1)),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}

		public void Spelling(thrts.IUnit selfUnit, int level)
		{
			/// AirMakimono ability
			thrts.AbilityTemplate abiity = null;
			selfUnit.AbilityManager.AppendAbility(abiity);
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;

			

			// クールタイムをリセット
			selfUnit.GetSpellWithName(Name).ResetCoolTime();

			int modeCTime = (int)(modeChangeTime * Def.FPS);

			if (!selfUnit.GetIAutoCastWithName(this.Name).IsEnable)
			{
				// 変形開始エフェクト
				{
					var change = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KaguyaMobChangeEffect).FullName, selfUnit.Pos + new Vector3D(0, 0, 1));
					change.SetScale(new Vector3D(16.0f, 16.0f, 16.0f));
				}
				selfUnit.HoldPosition(false);
				for (int i = 0; i < modeCTime; ++i)
				{
					// ユニット動作停止
					selfUnit.Speed = 0.0f;

					if (i == 0 || i % (Def.FPS * 2) == 0)
					{
						selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Alarm01).FullName, selfUnit.Pos);
					}
					yield return true;
				}

				CreateAreaEffect(selfUnit, level);

				// toggle autocast
				selfUnit.GetIAutoCastWithName(this.Name).ToggleAutoCast();
			}
			else
			{

				/// toggle autocast
				selfUnit.GetIAutoCastWithName(this.Name).ToggleAutoCast();

				/// remove ability
				if (selfUnit.AbilityManager.ContainsAbility(Ability.White.AjaModeAbility.AbilityName))
				{
					selfUnit.AbilityManager.RemoveAbility( Ability.White.AjaModeAbility.AbilityName);
				}

				for (int i = 0; i < modeCTime; ++i)
				{
					// stay stop
					selfUnit.Speed = 0.0f;
					yield return true;
				}

			}


			yield return false;
		}

		private static void CreateAreaEffect(thrts.IUnit selfUnit, int level)
		{
			// 範囲表示
			if (!selfUnit.AbilityManager.ContainsAbility(Ability.White.AjaModeAbility.AbilityName))
			{
				var affectRadius = selfUnit.Parameter.Faction.GetBulletParameter(typeof(KaguyaMobBigBullet).FullName).MaxRange;
				
				// まだ範囲が表示されていない場合
				var effect = selfUnit.Faction.World.EffectManager.CreateModelEffect(typeof(KaguyaMobAreaModelEffect).FullName, selfUnit.Pos, selfUnit.Faction);
				var scale = affectRadius / 100.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));

				/*
				var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KaguyaMobAreaEffect).FullName, selfUnit.Pos + new Vector3D(0, 0, 1),selfUnit.Faction);
				var scale = affectRadius / 5.0f;
				effect.SetScale(new Vector3D(scale, scale, scale));
				*/

				effect.IsShownToOnlyAlly = true;

				// アビリティとして追加(ユニット死亡時、エフェクトが自動破棄される)
				if (!selfUnit.AbilityManager.AppendAbility(new Ability.White.AjaModeAbility(level, effect)))
				{
					effect.Stop();
				}
			}
			else
			{
				selfUnit.AbilityManager.ExtendLifetimeWithName(interval,Ability.White.AjaModeAbility.AbilityName,15);
			}
		}

		public bool AutoCastAction(thrts.IUnit unit, int level)
		{
			if (unit.GetIAutoCastWithName(this.Name).IsEnable)
			{
				CreateAreaEffect(unit, level);
			}
			else
			{
				// remove ability
				if (unit.AbilityManager.ContainsAbility(Ability.White.AjaModeAbility.AbilityName))
				{
					unit.AbilityManager.RemoveAbility( Ability.White.AjaModeAbility.AbilityName);
				}
			}

			return true;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class AjaModeAbility : thrts.AbilityTemplate
	{
		IModelEffect effect;
		Vector3D rotation;

		public static int AjaModeDecArmor = 0;
		public const string AbilityName = "新難題「金閣寺の一枚天井」";
		public AjaModeAbility(int level, IModelEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = 15;
			this.effect = effect;
			
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.CanStateNow = false;
				// 移動しようとしていたらホールドポジションに強制変更
				if(!unit.IsHoldPosition || unit.ActualAction == UnitAction.Move)
					unit.HoldPosition(false);

				// set speed=0;
				unit.Speed = 0.0f;
				// remove armor
				unit.Armor -= AjaModeDecArmor;
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				unit.SelectBullet = SelectAjaBullet;
				if (effect != null)
				{
					rotation.Z += 1.0f;
					effect.SetRotation(rotation);
					effect.SetPosition(unit.Pos + new Vector3D(0, 0, 1));
				}
				break;
			default:
				break;
			}
			
			return true;
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists ()) {
				unit.CanStateNow = true;
				;
			}
			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			
			return base.OnVanish(unit);
		}
		
		static int SelectAjaBullet(IUnit unit,ITargetObject target,float distance)
		{
			/// return aja bullet
			return 1;
		}
	}
	
}
