using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.AutoCast;
using LotusCraft.Effect.White;
using LotusCraft.Token;
using LotusCraft.Unit.White;

namespace LotusCraft.Spell.White
{
	class KosuzuSkill : thrts.SpellParameter<thrts.IUnit>
	{
		float[] affectRadiuses;

		int[] maxSummon;

		const int useHP = 2;
		const int decRes0 = 25; // sake
		const int decRes1 = 25; // isi

		public KosuzuSkill()
		{
			DefaultLevel = 1;
			MaxLevel = 1;
			Name = "判読眼のビブロフィリア";
			string basePath = "White/Spell/Kosuzu/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 110 * Def.FPS;

			affectRadiuses = new float[MaxLevel + 1];
			affectRadiuses[0] = 0;
			affectRadiuses[1] = 3;

			maxSummon = new int[MaxLevel + 1];
			maxSummon[0] = 0;
			maxSummon[1] = 1;

			Description = "魔導書を召喚する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"酒"+decRes0+"、石"+decRes1+"、"+"HP"+useHP+"を消費して魔導書を召喚",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling, 1),
			};
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{

			if (null == selfUnit)
			{
				yield return false;
				yield break;
			}
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName(Name);
			if (spell == null || !spell.CanExec)
			{
				yield return false;
				yield break;
			}

			// check HP
			if (selfUnit.HP < useHP)
			{
				yield return false;
				yield break;
			}

			// check pop
			if (null != selfUnit.Faction)
			{
				var faction = selfUnit.Parameter.Faction;
				var unitParam = faction.GetUnitParameter(typeof(Unit.Common.TheBook).FullName);
				if (null != unitParam && faction.Supply - faction.Population < unitParam.PopulationWeight)
				{
					yield return false;
					yield break;
				}
			}
			// check Resource
			if (selfUnit.Faction.GetResource(0) < decRes0 || selfUnit.Faction.GetResource(1) < decRes1)
			{
				yield return false;
				yield break;
			}
			// クールタイム再設定
			spell.ResetCoolTime();

			{
				// dec HP
				selfUnit.HP -= useHP;
				// dec Resource
				selfUnit.Faction.DecResource(0,decRes0); //sake
				selfUnit.Faction.DecResource(1,decRes1); // isi

				// effect offset
				Vector3D offset = new Vector3D(0.0f, 0.0f, 1.0f);

				var unit = Utl.PutUnitAroundPoint(
					selfUnit.Faction.World, 
					selfUnit.Parameter.Faction, 
					typeof(Unit.Common.TheBook).FullName,
					new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y),
					selfUnit.Faction);

				if (null != unit)
				{
					var size = 5;
					var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(SummonEffect).FullName, unit.Pos + offset);
					effect.SetScale(new Vector3D(size, size, size));
					unit.AbilityManager.AppendAbility(new SuicideAbility(null, 150 * Def.FPS, "summonSuiside"));
				}

			}
			yield return false;
		}

		public override bool CanExec(IUnit unit)
		{
			bool ret = true;
			if (null != unit)
			{
				var faction = unit.Faction;
				if (null != faction)
				{
					var unitParam = faction.GetUnitParameter(typeof(Unit.Common.TheBook).FullName);
					if (null != unitParam)
					{
						// check supply
						ret = faction.Supply - faction.Population >= unitParam.PopulationWeight;
						// check resource
						ret = ret && (faction.GetResource(0)>=decRes0);
						ret = ret && (faction.GetResource(1)>=decRes1);
					}
				}
			}
			return base.CanExec(unit) && ret;
		}
	}
}