﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;

namespace LotusCraft.Spell.White
{
	class BlazingStar : thrts.SpellParameter<thrts.IUnit>
	{
		int[] damages;
		float[] speeds;
		const int duration = 2 * Def.FPS;

		public BlazingStar()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "彗星「ブレイジングスター」";
			Description = "指定地点に向けて突進し敵にダメージを与える。";
			string basePath = "White/Spell/BlazingStar/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 30 * Def.FPS;
			CoolTimes[2] = 30 * Def.FPS;
			CoolTimes[3] = 30 * Def.FPS;

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = 6 * Def.FPS;
			MaxRanges[2] = 8 * Def.FPS;
			MaxRanges[3] = 10 * Def.FPS;

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 3;
			damages[2] = 8;
			damages[3] = 20;

			speeds = new float[MaxLevel + 1];
			speeds[0] = 0;
			speeds[1] = 3 * UnitParameter.BasisSpeed;
			speeds[2] = 4 * UnitParameter.BasisSpeed;
			speeds[3] = 5 * UnitParameter.BasisSpeed;
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"0.2秒毎に" + damages [1] + "ダメージ" + "、効果2秒",
				"0.2秒毎に" + damages [2] + "ダメージ" + "、効果2秒",
				"0.2秒毎に" + damages [3] + "ダメージ" + "、効果2秒",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._2;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = thrts.ButtonPosition._2;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "2", PrivilegeModeSpelling, 2),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "3", PrivilegeModeSpelling, 3),
			};

			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if (null == selfUnit || !selfUnit.Exists())
			{
				yield return false;
				yield break;
			}

			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime();

			var world = selfUnit.Faction.World;

			// charge ?

			var tick = world.GetGroundAtPos(targetPosition) - selfUnit.Pos;
			tick.Normalize();
			tick *= 2 * MaxRanges[level];
			{
				var offset = new Vector3D(0, 0, 32);
				var areaEffect = world.EffectManager.CreateEffect(typeof(MarisaBlazingAreaEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale(new Vector3D(16, 16, 16));
				areaEffect.SetRotation(new Vector3D(0, 0, (float)(Math.Atan2(tick.X, tick.Y) * 180 / Math.PI)));

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);

				// エフェクトを生成
				var ability = new FollowEffectAbility(areaEffect, offset, duration, this.Name + "areaEffect");
				ability.StopEffectRootOnVanish = true;

				if (!selfUnit.AbilityManager.AppendAbility(ability))
					areaEffect.Stop();
			}

			// 通りぬけ
			var bsAblity = new BlazingStarAbility();
			selfUnit.AbilityManager.AppendAbility(bsAblity);
			bsAblity.ExtendLifetime(2);

			// 移動速度3で飛行
			var count = (int)Math.Ceiling(tick.Length / (speeds[level]));
			if (count > duration)
				count = duration;

			tick *= 1 / (float)count;


			for (int i = 0; i < duration; ++i)
			{
				if (i <= count)
					selfUnit.TryMove(tick.X, tick.Y, true);

				if (i % (Def.FPS / 5) == 0)
				{
					var radius = 2 * Def.MapTipLengthOnField;
					// be damage
					var targets = world.GetObjectFromCircle(new Circle2D(radius, new Vector2D(selfUnit.Pos.X, selfUnit.Pos.Y)));
					foreach (var t in targets)
					{
						if (null == t || !t.Exists() || world.IsSameTeam(t.Faction, selfUnit.Faction))
							continue;

						var u = t as IUnit;
						var b = t as IBuilding;

						if (null != u || null != b)
						{
							// show hit effect
							Unit.Utls.ShowHitEffect(t, Unit.Utls.HitEffectType.NormalBlue);
							world.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitZangeki).FullName, t.Pos);
						}

						if (null != u)
						{
							u.BeDamaged(selfUnit, damages[level], AttackBonusSet.Hero);
						}
						if (null != b)
						{
							b.BeDamaged(selfUnit, damages[level], AttackBonusSet.Hero);
						}
					}
				}

				bsAblity.ExtendLifetime(1);
				yield return true;
			}

			bsAblity.ExtendLifetime(1);
			yield return true;
		}

		class BlazingStarAbility : AbilityTemplate
		{
			public BlazingStarAbility()
			{
				Name = "BlazingStar";
			}

			public override bool ExecUnitAbility(IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.OtherExecs)
				{
					unit.ThroughOthers = true;
				}

				return true;
			}
		}
	}
}
