﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;

namespace LotusCraft.Spell.White
{
	class MasterSpark:thrts.SpellParameter<thrts.IUnit>
	{
		const int damage = 200;
		const int chargeTime = 2*Def.FPS;

		const int duration = 8*Def.FPS + 5;
		const int interval = 1*Def.FPS;

		readonly Vector2D sparkSize = new Vector2D(6*Def.MapTipLengthOnField,18*Def.MapTipLengthOnField);

		public MasterSpark()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "恋符「マスタースパーク」";
			string basePath = "White/Spell/MasterSpark/";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0;
			MaxRanges[1] = sparkSize.Y * 0.7f;

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 180*Def.FPS;

			Description = "広範囲にダメージを与えるレーザーを発射する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1秒毎に" + damage + "ダメージx"+duration/Def.FPS,
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, Vector2D targetPosition, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime ();

			// look at target
			selfUnit.ChangeDirection(targetPosition);

			// show cutin
			selfUnit.ShowCutin ();

			for (var i = 0; i < (1 * Def.FPS) / 3; i++) {
				yield return null;
			}

			// charge
			var chargeeffect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.Common.ChargeEffect).FullName, selfUnit.Pos + new Vector3D(0,0,20));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

			for (int i=0; i<chargeTime; ++i) {
				yield return true;
			}

			chargeeffect.Stop();

			var world = selfUnit.Faction.World;
			// create area effect
			var rot = (float)(Math.Atan2 (targetPosition.X - selfUnit.Pos.X, targetPosition.Y - selfUnit.Pos.Y));
			Vector3D offset = new Vector3D (24*(float)Math.Sin (rot),24*(float)Math.Cos (rot),32);
			rot *= 180 / (float)Math.PI;
			var areaEffect = world.EffectManager.CreateEffect (typeof(MarisaMasterSparkAreaEffect).FullName, selfUnit.Pos + offset);
			areaEffect.SetScale (new Vector3D (Def.MapTipLengthOnField, Def.MapTipLengthOnField, Def.MapTipLengthOnField));
			areaEffect.SetRotation (new Vector3D (0, 0, rot));

			var diff = world.GetGroundAtPos(targetPosition) - selfUnit.Pos;
			diff.Z=0;
			diff.Normalize();
			var center = new Vector2D(selfUnit.Pos.X + diff.X*sparkSize.Y/2.0f,selfUnit.Pos.Y + diff.Y*sparkSize.Y/2.0f);
			var ability = new SpellContinuousAbility ((unit) => Spelling (unit, center, rot), areaEffect, offset, duration, this.Name + "areaEffect", interval);
			ability.StopEffectRootOnVanish = true;

			if (!selfUnit.AbilityManager.AppendAbility (ability)) {
				areaEffect.Stop();
			}
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Maspa).FullName, selfUnit.Pos);

			for (int i=0; i<duration; ++i) {
				yield return true;
			}

			yield return false;
		}
		public bool Spelling (IUnit selfUnit, Vector2D center, float rot)
		{
			// search targets;
			var world = selfUnit.Faction.World;

			// 後まで少し範囲を広げる
			var offsetY = 2*Def.MapTipLengthOnField;
			var targets = world.GetObjectOnRectangle (center.X, center.Y + offsetY/2.0f, sparkSize.X, sparkSize.Y + offsetY, rot, true, true);
			/*
			selfUnit.Faction.World.DrawDebugRectangle(new Vector3D(center.X, center.Y - offsetY/2.0f,selfUnit.Pos.Z + 10.0f),
													  sparkSize.X, sparkSize.Y + offsetY, rot,
													  new Color(1.0f, 1.0f, 1.0f, 1.0f));
			//*/
			foreach (var t in targets) {
				if(null == t || !t.Exists() || world.IsSameTeam(t.Faction,selfUnit.Faction))
					continue;

				var u = t as IUnit;
				var b = t as IBuilding;

				if( null != u || null != b)
				{
					// show hit effect
					Unit.Utls.ShowHitEffect(t, Unit.Utls.HitEffectType.NormalOrange);
					world.SoundManager.PlaySoundEffect (typeof(SoundEffect.HitOther).FullName,t.Pos);
				}

				if(null != u)
				{
					u.BeDamaged(selfUnit,damage,AttackBonusSet.Hero);
				}
				if(null != b)
				{
					b.BeDamaged(selfUnit,damage,AttackBonusSet.Hero);
				}
			}

			return true;
		}
	}
}