﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;
using LotusCraft.Spell.White;

// 1203 damage adjasted

namespace LotusCraft.Spell.White
{
	class OrreysSun:thrts.SpellParameter<thrts.IUnit>
	{
		int[] damages;
		int[] bitCount;

		const int interval = 15;
		public OrreysSun()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "儀符「オーレリーズサン」";
			IsPassive = true;
			Description = "ビットが周囲を廻り、通常攻撃に連動してレーザーを発射する。\n作業員へのダメージ-50%";
			string basePath = "White/Spell/OrreysSun/";

			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 5;
			damages[2] = 6;
			damages[3] = 7;

			bitCount = new int[MaxLevel + 1];
			bitCount[0] = 0;
			bitCount[1] = 4;
			bitCount[2] = 5;
			bitCount[3] = 6;


			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"1ビットにつき"+damages[1]+"ダメージ、ビット×"+bitCount[1],
				"1ビットにつき"+damages[2]+"ダメージ、ビット×"+bitCount[2],
				"1ビットにつき"+damages[3]+"ダメージ、ビット×"+bitCount[3],
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = new thrts.AutoCastParameter<thrts.IUnit>[]
			{
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,null),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,this.AutoCastAction_lv1,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,this.AutoCastAction_lv2,true),
				new LotusCraft.AutoCast.SpellAutoCast<thrts.IUnit>(this,interval,this.AutoCastAction_lv3,true),
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public bool Spelling (thrts.IUnit selfUnit, int level)
		{
			var ability = selfUnit.AbilityManager.GetAbilityWithName (OrrerysSunAbility.AbilityName) as OrrerysSunAbility;

			if (null == ability) {
				selfUnit.AbilityManager.AppendAbility (new OrrerysSunAbility (selfUnit, damages[level], bitCount [level], interval*2));
			} else {
				ability.ExtendLifetime(interval,interval*2);
				ability.BitCount = bitCount[level];
				ability.Damage = damages[level];
			}

			return true;
		}
		public bool AutoCastAction_lv1(thrts.IUnit unit)
		{
			Spelling(unit, 1);
			return true;
		}
		public bool AutoCastAction_lv2(thrts.IUnit unit)
		{
			Spelling(unit, 2);
			return true;
		}
		public bool AutoCastAction_lv3(thrts.IUnit unit)
		{
			Spelling(unit, 3);
			return true;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class OrrerysSunAbility : thrts.AbilityTemplate
	{
		thrts.IEffect effect;
		IUnit speller;
		float theta;
		float alpha;

		int counter;

		int stopBitCounts;

		public int BitCount = 0;
		public int Damage = 0;
		public IEffect[] BitEffects = new IEffect[9];

		public const float Radius = 1.8f * Def.MapTipLengthOnField;

		public readonly static string[] BitEffectName = new string[]
		{
			typeof(MarisaOrreriesSunRed).FullName,
			typeof(MarisaOrreriesSunGreen).FullName,
			typeof(MarisaOrreriesSunBlue).FullName,
			typeof(MarisaOrreriesSunYellow).FullName,
			typeof(MarisaOrreriesSunPurple).FullName,
			typeof(MarisaOrreriesSunOrange).FullName,
		};

		public const string AbilityName = "儀符「オーレリーズサン」";

		public OrrerysSunAbility(IUnit speller, int damage, int bitCount, int duration, thrts.IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = duration;
			this.DurationTime = 8 * Def.FPS;
			this.speller = speller;
			this.effect = effect;

			this.BitCount = bitCount;
			this.Damage = damage;

			theta = (float)(2 * Math.PI * speller.Faction.World.NextRandDouble());
			// 角速度 4秒に1周 = 2PI/120 = PI/60
			alpha = (float)(Math.PI / 60.0f);

			counter = 0;
			stopBitCounts = 0;
		}

		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case thrts.AbilityOrder.OtherExecs:

					if (null != BitEffects)
					{

						var dtheta = 2 * Math.PI / (float)BitCount;
						for (int i = 0; i < Math.Min(BitEffects.Length, BitCount); ++i)
						{
							var relPos = new Vector3D(Radius * (float)Math.Sin(theta + dtheta * i), Radius * (float)Math.Cos(theta + dtheta * i), 32);
							if (null == BitEffects[i])
							{
								BitEffects[i] = speller.Faction.World.EffectManager.CreateEffect(BitEffectName[i], relPos + speller.Pos);
								var scale = 2;
								BitEffects[i].SetScale(new Vector3D(scale, scale, scale));
							}
							else
							{
								// move ball effect
								BitEffects[i].SetPosition(relPos + speller.Pos);
							}
						}

						if (counter > 0)
						{
							counter--;
						}

						if (counter <= stopBitCounts)
						{
							theta += alpha;
						}
					}

				{
					var bulletParam = unit.GetModifiedBulletParameter(typeof(OrrerysBullet).FullName);
					if(null != bulletParam)
						bulletParam.Attack = Damage;
				}

					if (effect != null)
						effect.SetPosition(unit.Pos + new thrts.Vector3D(2, 2, 1));
					break;
				default:
					break;
			}

			return true;
		}

		public override bool OnVanish(thrts.IUnit unit)
		{
			for (int i = 0; i < BitEffects.Length; ++i)
			{
				if (BitEffects[i] != null)
				{
					BitEffects[i].Stop();
					BitEffects[i] = null;
				}
			}

			if (effect != null)
			{
				effect.Stop();
				effect = null;
			}
			return base.OnVanish(unit);
		}
		public override bool ModifyBullet(thrts.IBullet bullet)
		{
			var u = bullet.Generator as IUnit;
			var bulletParam = u.GetModifiedBulletParameter(typeof(MarisaBullet).FullName);
			var orrerysBulletParam = u.GetModifiedBulletParameter(typeof(OrrerysBullet).FullName);
			if (counter <= 0 && null != bullet.Target && null != bulletParam && bullet.Parameter == bulletParam)
			{
				if (null != orrerysBulletParam)
				{
					bool ret = false;
					var dtheta = 2 * Math.PI / (float)BitCount;
					for (int i = 0; i < Math.Min(BitEffects.Length, BitCount); ++i)
					{
						var relPos = new Vector3D(Radius * (float)Math.Sin(theta + dtheta * i), Radius * (float)Math.Cos(theta + dtheta * i), 32);
						var bitPos = u.Pos + relPos;
						var diff = bullet.Target.Pos - bitPos;
						if (bullet.Target.IsHit(bitPos.X, bitPos.Y, orrerysBulletParam.MaxRange))
						{
							ret = true;
							var center = (diff * orrerysBulletParam.LengthX) / (2 * diff.Length);
							var orreryBullet = u.CreateBullet(typeof(OrrerysBullet).FullName, bullet.Target, u.Faction.World.GetGroundAtPos(new Vector2D(bitPos.X + center.X, bitPos.Y + center.Y)));
							if (null != orreryBullet)
							{
								orreryBullet.Rotation = (float)(Math.Atan2(diff.Y, diff.X) * (-180 / Math.PI));

								IEffect orreryEffect = null;

								orreryBullet.Updating += (IBullet obj) =>
								{
									// follow bits
									obj.SetPosition(u.Faction.World.GetGroundAtPos(new Vector2D(u.Pos.X + relPos.X + center.X, u.Pos.Y + relPos.Y + center.Y)));

									// デバッグ
									/*
									{
										float dis = (obj.Parameter.LengthX / 2) - obj.Parameter.RotateCenterX;
										var disx = (float)(Math.Cos(-1*(obj.Rotation)*Math.PI/180.0) ) * dis;
										var disy = (float)Math.Sin(-1*(obj.Rotation)*Math.PI/180.0 ) * dis;
										u.Faction.World.DrawDebugRectangle(new Vector3D(obj.Position.X + disx, obj.Position.Y + disy, obj.Position.Z),
																			  obj.Parameter.LengthX, obj.Parameter.LengthY, obj.Rotation,
																			  new Color(1.0f, 1.0f, 1.0f, 1.0f));
										u.Faction.World.DrawDebugCircle(new Circle2D(obj.Parameter.LengthY/2.0f,new Vector2D(obj.Position.X,obj.Position.Y)),obj.Position.Z+2,new Color(1.0f, 1.0f, 1.0f, 1.0f));
									}
									//*/

									if (orreryEffect == null)
									{
										// 攻撃エフェクト生成
										orreryEffect = u.Faction.World.EffectManager.CreateEffect(typeof(MarisaOrreriesSunArea).FullName, u.Pos + relPos);
										orreryEffect.SetScale(new Vector3D(8, 8, 8));
										var r = Math.Sqrt(diff.X * diff.X + diff.Y * diff.Y);
										orreryEffect.SetRotation(new Vector3D(0, (float)(Math.Atan2(r, diff.Z) * (-180 / Math.PI)), obj.Rotation));
									}
									else
									{
										orreryEffect.SetPosition(u.Pos + relPos);
									}
								};

								orreryBullet.Vanished += (IBullet obj) =>
								{
									if (orreryEffect != null)
									{
										orreryEffect.Stop();
										//										orreryEffect = null;
									}
								};
							}

						}
					}

					if (ret)
					{
						counter = orrerysBulletParam.AttackSpeed;
						stopBitCounts = Math.Max(counter - orrerysBulletParam.Duration, 0);
					}
				}
			}
			//			bullet.Vanish();
			return base.ModifyBullet(bullet);
		}
	}

}

