﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;
using LotusCraft.Bullet.White;

// 1203 damage adjasted

namespace LotusCraft.Spell.White
{
	class StarDust:thrts.SpellParameter<thrts.IUnit>
	{
		int[] damages;
		int[] durations;
		const int interval = Def.FPS/2;
		public StarDust()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			Name = "魔符「スターダストレヴァリエ」";
			Description = "周囲に居るユニットにダメージを与える。\n作業員へのダメージ-50%。また使用時は受けるダメージが25%減少する";
			string basePath = "White/Spell/StarDust/";

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes[0] = 9999;
			CoolTimes[1] = 30*Def.FPS;
			CoolTimes[2] = 30*Def.FPS;
			CoolTimes[3] = 30*Def.FPS;
			
			damages = new int[MaxLevel + 1];
			damages[0] = 0;
			damages[1] = 3;
			damages[2] = 5;
			damages[3] = 7;

			durations = new int[MaxLevel +1];
			durations[0] = 0;
			durations[1] = 6*Def.FPS;
			durations[2] = 8*Def.FPS;
			durations[3] = 10*Def.FPS;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"0.5秒毎に" + damages[1] + "ダメージ"+"、効果"+durations[1]/(float)Def.FPS+"秒",
				"0.5秒毎に" + damages[2] + "ダメージ"+"、効果"+durations[2]/(float)Def.FPS+"秒",
				"0.5秒毎に" + damages[3] + "ダメージ"+"、効果"+durations[3]/(float)Def.FPS+"秒",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			if (null == selfUnit || !selfUnit.Exists ()) {
				yield return false;
				yield break;
			}
			
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			spell.ResetCoolTime ();
			
			var world = selfUnit.Faction.World;
			
			// charge ?

			{
				var offset = new Vector3D (0, 0, 32);
				var areaEffect = world.EffectManager.CreateEffect (typeof(MarisaStarAreaEffect).FullName, selfUnit.Pos + offset);
				areaEffect.SetScale (new Vector3D (16, 16, 16));

				selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Jet).FullName, selfUnit.Pos);
				
				// エフェクトを生成
				var ability = new StarDustAbility (damages[level],areaEffect, offset, durations[level]);
				if (!selfUnit.AbilityManager.AppendAbility (ability))
					areaEffect.Stop ();
			}

			yield return true;
		}
	}
}

namespace LotusCraft.Ability.White
{
	class StarDustAbility : FollowEffectAbility
	{
		public int Damage;
		int counter = 0;
		const int interval = Def.FPS / 2;
		public const string AbilityName = "魔符「スターダストレヴァリエ」";
		public StarDustAbility(int damage, IEffect effect, Vector3D offset, int duration)
			: base(effect, offset, duration, AbilityName)
		{
			Damage = damage;
			counter = 0;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
				case thrts.AbilityOrder.ConstDecParam:
					//				unit.SelectBullet = DisableBullet;
					break;
				case thrts.AbilityOrder.RateDecParam:
					break;
				case thrts.AbilityOrder.ConstIncParam:
					unit.DamageRatio = unit.DamageRatio - 0.25f;
					break;
				case thrts.AbilityOrder.RateIncParam:
					break;
				case AbilityOrder.OtherExecs:
					if (counter++ > interval)
					{
						counter = 0;
						var world = unit.Faction.World;
						var radius = 2 * Def.MapTipLengthOnField;
						// be damage
						var targets = world.GetObjectFromCircle(new Circle2D(radius, new Vector2D(unit.Pos.X, unit.Pos.Y)));
						foreach (var t in targets)
						{
							if (null == t || !t.Exists() || world.IsSameTeam(t.Faction, unit.Faction))
								continue;

							var u = t as IUnit;
							var b = t as IBuilding;

							if (null != u || null != b)
							{
								// show hit effect
								Unit.Utls.ShowHitEffect(t, Unit.Utls.HitEffectType.NormalOrange);
								world.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitShortRangeAttack).FullName, t.Pos);
							}

							if (null != u)
							{
								if(u.Parameter.IsFarmer)
									u.BeDamaged(unit, (int)Math.Ceiling(Damage*0.5f), AttackBonusSet.Hero);
								else
									u.BeDamaged(unit, Damage, AttackBonusSet.Hero);

							}
							if (null != b)
							{
								b.BeDamaged(unit, Damage, AttackBonusSet.Hero);
							}
						}
					}
					break;
				default:
					break;
			}

			return base.ExecUnitAbility(unit, order);
		}
		public static int DisableBullet(IUnit selfUnit, ITargetObject target, float distance)
		{
			return -1;
		}
	}

}
