﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;
using LotusCraft.Effect.White;

namespace LotusCraft.Spell.White
{
	class Kibakujin : thrts.SpellParameter<thrts.IUnit>
	{
		int duration = 4*Def.FPS;
		const int interval = 10;

		float[] speedRatio;
		public Kibakujin()
		{
			DefaultLevel = 0;
			MaxLevel = 3;
			CoolTimes = new int[4];
			CoolTimes[0] = 12*30;
			CoolTimes[1] = 12*30;
			CoolTimes[2] = 12*30;
			CoolTimes[3] = 12*30;
			Name = "神技「八方鬼縛陣」";
			Description = "敵ユニットの移動速度を低下させる陣を設置する。";
			string basePath = "White/Spell/Kibakujin/";

			MaxRanges = new float[MaxLevel + 1];
			MaxRanges[0] = 0*Def.MapTipLengthOnField;
			MaxRanges[1] = 4*Def.MapTipLengthOnField;
			MaxRanges[2] = 5*Def.MapTipLengthOnField;
			MaxRanges[3] = 5*Def.MapTipLengthOnField;

			speedRatio = new float[MaxLevel + 1];
			speedRatio[0] = 1;
			speedRatio[1] = 0.75f;
			speedRatio[2] = 0.6f;
			speedRatio[3] = 0.4f;

			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-" + (Math.Round(100 - speedRatio[1]*100)).ToString() + "%",
				"移動速度-" + (Math.Round(100 - speedRatio[2]*100)).ToString() + "%",
				"移動速度-" + (Math.Round(100 - speedRatio[3]*100)).ToString() + "%",
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._1;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;

			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "2",PrivilegeModeSpelling,2),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "3",PrivilegeModeSpelling,3),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(IUnit selfUnit,Vector2D targetPosition, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			float affectRadius = 0;
			switch (level)
			{
				case 1:
					affectRadius = 3;
					break;
				case 2:
					affectRadius = 4;
					break;
				case 3:
					affectRadius = 5;
					break;
				default:
					affectRadius = 0;
					break;
			}
			var targetPos = selfUnit.Faction.World.GetGroundAtPos(targetPosition);
			/// get call Effect;
			var offset = new Vector3D(0,0,2);
			IEffect effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(KibakujinAreaEffect).FullName,targetPos+offset);
			effect.SetScale(new Vector3D(2.7f * affectRadius * 2, 3f * affectRadius * 2, 2.7f * affectRadius * 2));
			/// add spell continuous ability
			var ability = new Ability.SpellContinuousAbility((unit)=>Spelling(unit,targetPosition,affectRadius,level),effect,offset,duration,this.Name+"AreaEffect",interval);
			ability.FollowEffect = false;
			bool flag = selfUnit.AbilityManager.AppendAbility(ability);
			if(!flag)
				effect.Stop();
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge1).FullName, selfUnit.Pos);

			yield return true;
		}
		public bool Spelling(thrts.IUnit selfUnit,Vector2D center,float radius, int level)
		{
			float affectRadius = radius;
			/// 地点(x,y)を中心にユニットを選択
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromCircle(new Circle2D(affectRadius*Def.MapTipLengthOnField, center)).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!world.IsSameTeam(selfFaction,_.Faction)))
			{
				var ability = u.AbilityManager.GetAbilityWithName(KibakujinAbility.AbilityName) as KibakujinAbility;

				if(null == ability)
				{
					IEffect effect = null;
					if(u != selfUnit)
					{
						/// エフェクトを描画
						Vector3D offset = new Vector3D(2.0f,2.0f,1.0f);
						effect = world.EffectManager.CreateEffect(typeof(KibakujinEffect).FullName, u.Pos + offset);
						effect.SetScale(new Vector3D(15,15,15));
					}
					if(u.AbilityManager.AppendAbility(new KibakujinAbility(speedRatio[level],interval*2,effect)))
					{
//						u.BeDamaged(selfUnit,damage,AttackBonusSet.None);
						;
					}
					else
					{
						effect.Stop();
						effect = null;
					}
				}
				else
				{
					ability.ExtendLifetime(interval,interval*2);
					ability.SpeedRate = speedRatio[level];
				}
			}

			return true;
		}
	}
}
