﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.White;

namespace LotusCraft.Spell.White
{
	class MusoTensei:thrts.SpellParameter<thrts.IUnit>
	{
		MusoTenseiAbility[] spellAbilities = new []
		{
			new MusoTenseiAbility(0),
			new MusoTenseiAbility(1),
		};
		int[] chargeTime = new int[2];
		const int interval = 10;
		public MusoTensei()
		{
			DefaultLevel = 0;
			MaxLevel = 1;
			RequireLevels[0] = 6;
			Name = "大結界「博麗弾幕結界」";
			string basePath = "White/Spell/MusoTensei/";

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 0;
			CoolTimes[1] = 120*30;

			chargeTime[0] = 0;
			chargeTime[1] = 2*30;

			Description = "近くにいる味方ユニットの防御力と攻撃力と射程と移動速度を上昇させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"防御+"+spellAbilities[1].IncArmor.ToString()+",攻撃+"+((int)((spellAbilities[1].AtkRate-1.0f)*100)).ToString()+"%,射程+1,移動速度+10%"
			};

			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._3;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャスト無し
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		public IEnumerable<object> PrivilegeModeSpelling(thrts.IUnit selfUnit, int level)
		{
			// スペルが現在使用可能か判断
			var spell = selfUnit.GetSpellWithName (Name);
			if (null == spell || !spell.CanExec) {
				yield return false;
				yield break;
			}
			// reset cooltime
			spell.ResetCoolTime();

			int affectRange = 0;
			switch (level)
			{
				case 1:
					affectRange = 9;
					break;
				default:
					break;
			}

			// chage effect
			Vector3D offset = new Vector3D(0.0f,0.0f,1.0f);
			var effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.White.DaiKekkaiChargeEffect).FullName, selfUnit.Pos + offset);
			float effectScale = 2 *affectRange*1.5f;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			selfUnit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Charge2).FullName, selfUnit.Pos);

			for(int i=0;i<chargeTime[level];++i)
				yield return true;

			effect.Stop();

			selfUnit.ShowCutin();

			// create call Effect;
			effect = selfUnit.Faction.World.EffectManager.CreateEffect(typeof(Effect.White.DaiKekkaiShotEffect).FullName, selfUnit.Pos + offset);
			effectScale = 2 *affectRange*1.5f;
			effect.SetScale (new Vector3D (effectScale, effectScale, effectScale));
			// add spell continuous ability
			var areaAbility = new Ability.SpellContinuousAbility((unit) => Spelling(unit, level), effect, offset, 20 * 30, this.Name + "AreaEffect", interval);
			areaAbility.StopEffectRootOnVanish = true;
			bool flag = selfUnit.AbilityManager.AppendAbility(areaAbility);
			if(!flag)
				effect.Stop();

			yield return true;
		}

		public bool Spelling (IUnit selfUnit, int level)
		{
			if(level < 0 || level>MaxLevel)
				return false;

			int affectRange = 0;
			switch (level)
			{
				case 1:
					affectRange = 9;
					break;
				default:
					break;
			}
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			var targets = selfFaction.World.GetObjectFromRectangle(selfUnit.Pos.X, selfUnit.Pos.Y, affectRange * 32, affectRange * 32, selfUnit.Pos.X, selfUnit.Pos.Y, affectRange * 32, affectRange * 32).Where(_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>selfFaction.World.IsSameTeam(selfFaction,_.Faction)))
			{

				if(!u.AbilityManager.ContainsAbility(spellAbilities[level].Name))
				{
					/// エフェクトを描画
					Vector3D offset = new Vector3D(2.0f,2.0f,1.0f);
					var effect = world.EffectManager.CreateEffect(typeof(Effect.White.DaiKekkaiEffect).FullName, u.Pos + offset);
					effect.SetScale(new Vector3D(12,12,12));
					if(!u.AbilityManager.AppendAbility(new MusoTenseiAbility(level,effect)))
						effect.Stop();
				}
				else
				{
					u.AbilityManager.ExtendLifetimeWithName(interval,spellAbilities[level].Name,spellAbilities[level].DurationTime);
				}
			}

			return true;
		}
	}
}
