﻿#if !_TRIAL

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class Story06 : thrts.Mission
	{
		static readonly int[] FirstResoure = new List<int>{ 500, 200, 200 }.ToArray();
		static readonly float[] HPRate = new List<float>{ 0.5f, 1, 1.2f }.ToArray();

		int[] Reimu = null;
		int[] Yukari = null;
		IBuilding whiteBase;
		IBuilding blueBase;
		Dictionary<string, Area> areas = new Dictionary<string, Area>();

		public Story06()
			: base()
		{
			Name = "ストーリー06";
			Description = "白3話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		public override string[] GetSelectableFactions()
		{
			return new string[]{
				typeof( Faction.White ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]{
				typeof( Faction.White ).FullName,
				typeof( Faction.Blue ).FullName
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[]{
				"LotusCraft/Story/Story06.map"
			};
		}

		public override void OnInitialize( IWorld world )
		{
			world.Factions[1].PlayerName = "青勢力";

			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;

			// 本拠の位置を初期視点にする
			var basePos = world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( LotusCraft.Building.White.HomeBase ) ).First().Pos;
			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D( (int)basePos.X, (int)basePos.Y );

			// ボタン仕様禁止
			var home = world.BuildingManager.GetBuildings(_ => _.Faction == world.Factions[0] && _.Parameter.GetType() == typeof(LotusCraft.Building.White.HomeBase)).First();
			var ui = world.StoryUI;
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._1);
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._1);
			ui.HideButton(home, ButtonPosition._0, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._2);


			SetDifficulty( world );

			base.OnInitialize( world );
		}

		public override IEnumerable Update( IWorld world )
		{
			var ui = world.StoryUI;
			whiteBase = GetBuildings<Building.White.HomeBase>( world ).First();
			blueBase = GetBuildings<Building.Blue.HomeBase>( world ).First();
			foreach( var item in world.AllPurposeArea )
			{
				areas[item.Name] = item;
			}

			Reimu = LoadTachies( ui, "LotusCraft/Image/Tachie/Reimu_N", "LotusCraft/Image/Tachie/Reimu_S", "LotusCraft/Image/Tachie/Reimu_W" );
			Yukari = LoadTachies( ui, "LotusCraft/Image/Tachie/Yukari_N", "LotusCraft/Image/Tachie/Yukari_S", "LotusCraft/Image/Tachie/Yukari_W" );

			world.ChangeBGM( BGMList.Boushuen_Story2 );

			foreach( var item in Talk戦闘前( world ) )
			{
				yield return null;
			}

			world.ChangeBGM( BGMList.Battle_Hakurei );

			ui.ShowConditionText( "目的：紫の拠点を破壊する。" );

			var yukariUnit = world.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Blue.Unique.Yukari ) ).First();
			yukariUnit.AddExperience( 15000 );
			bool helped = false;

			foreach( var item in Tactics( world ) )
			{
				if( !helped && yukariUnit.HP < yukariUnit.MaxHP * 0.5f )
				{
					var units = PutEnemy<Unit.Blue.Low02>( world, areas["増援"], 20 );
					world.UnitManager.AttackMove( units.Select( _ => _.UniqueID ), blueBase, false );
					helped = true;
				}

				if( !blueBase.Exists() )
				{
					foreach( var item2 in Talk勝利( world ) )
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();
					world.StoryUtilities.SetDataToSaveData( GetSavedataName( world ), "Clear" );
					world.StoryUtilities.SetDataToSaveData("LotusCraft.White.03", "Clear");

					break;
				}
				else if( !whiteBase.Exists() )
				{
					foreach( var item2 in Talk敗北( world ) )
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();

					break;
				}
				yield return null;
			}

			yield return null;
		}

		private void SetDifficulty( IWorld world )
		{
			int difficulty = world.StoryUtilities.GetDifficulty();

			// 初期資源を配布
			world.Factions[0].IncResource( 0, FirstResoure[difficulty] );
			world.Factions[0].IncResource( 1, FirstResoure[difficulty] );

			var blueTypes = new []{
				typeof( Unit.Blue.Low02 ),
				typeof( Unit.Blue.Low03 ),
				typeof( Unit.Blue.Iku ),
				typeof( Unit.Blue.Sunny ),
				typeof( Unit.Blue.Lunar ),
				typeof( Unit.Blue.Star ),
				typeof( Unit.Blue.Unique.Yukari ),
			};
			blueTypes.Select( _ => world.Factions[1].GetUnitParameter( _.FullName ) )
				.ForEach( _ => _.HP = (int)( _.HP * HPRate[difficulty] ) );
		}

		private string GetSavedataName( IWorld world )
		{
			switch( world.StoryUtilities.GetDifficulty() )
			{
			case 0: return "LotusCraft.White.03.Easy";
			case 1: return "LotusCraft.White.03.Normal";
			case 2: return "LotusCraft.White.03.Hard";
			default: throw new InvalidOperationException( "不正な難易度が設定されています。" );
			}
		}

		#region Story
		private IEnumerable Talk戦闘前( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【紫】\n本当の目的ですって？　おかしなことをおっしゃいますこと" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【霊夢】\n誤魔化さないでってば\nおかしなのはあんたのほうでしょ" );
			ui.ShowText( "【霊夢】\n急にこんな遊びを考案して\n何かくだらないことを考えてるに違いないんだから" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nんー、みんなと遊びたかったじゃ、ダメかしらね" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nダメに決まってるでしょ" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nでも、実害が出ているわけじゃないわよね？" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nだから、あんたが何かよからぬ企みをしているんじゃないかって言ってるの" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n単なるガス抜きよ、ガス抜き。最初に言ったとおり。" );
			ui.ShowText( "【紫】\nそうだ、月に行こうって言ってるのと全然変わらないことなの" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n本当に？" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n私が嘘をついたことなんてあったかしら" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nあんたみたいに胡散臭いが歩いているようなのは、疑られて当然なのよ" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそうねえ、確かにそうかもしれないわね。" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nこんなに大々的にいろんな妖怪たちを集めて、妖怪たちが変な動きをしているのを、\n博麗の巫女が御阿礼の乙女を連れて悪さをしないように見張ってる。" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n大まかなところはその辺でしょうね" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nそうね。人里ではそんなところだったわ" );
			ui.ShowText( "【紫】\nそうよ。実際に何かあればあなたが止めればいいじゃない\nそれが博麗の巫女の仕事でしょう？" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nだから事前に何を企んでいるのか、首謀者に聞いてるんじゃない" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそんなことを言われても困っちゃうわ。本当にそれだけなんだもの。" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n疑われたりすること自体はよくあるものだけど、存在しない証拠を出すなんてことはできないわ\n悪魔の証明よ。そんなの" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n吐かないつもりなのね" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n戦う理由なんてそれで十分。疑念だけで戦に駆り立てられてしまう\nいえ、疑念なんてものも必要ないのかも" );
			ui.ShowText( "【紫】\n戦いたいから、理由を後付けする\n戦いなんてそんなものね" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nごちゃごちゃ言うのを聞く道理はないわ。さあ行くわよ" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
			{
				yield return null;
			}
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		private IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【紫】\nやられてしまいましたわ" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【霊夢】\nじゃあ吐いてもらうわよ" );
			ui.ShowText( "【紫】\n裏の目的なんてないわ。本当に単なるガス抜きのためなのよ。" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n霊夢、苛立つ気持ちはわかるけど" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n苛立つ？　私が？" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそう、苛立ってる" );
			ui.ShowText( "【紫】\nこんなにたくさんの妖怪たちが集まって小競り合いをしているのに、巫女としての力が必要とされない。\nそのことに苛立ってる" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n……" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n気を抜けばいいのよ\nあなただって、今回に限れば登場人物の一人なんだって" );
			ui.ShowText( "【紫】\n私も同じ\nお祭りなんだから、楽しんじゃえばいいのよ" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n納得いかないわ" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nまあ、私だってこれがどこまで広まるかなんてわからないけれど、\nみんなが肩を寄せ合って、窮屈に過ごしてるこの幻想郷じゃ大層な事件なんて起きないでしょ" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n管理者がそんな立場でいいの？" );
			ui.ShowText( "【紫】\n信用してるもの。博麗の巫女を、ね？" );
			ui.ShowImage( Reimu[2], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n信用されても、正直困るわ\nだって、私にはわからないもの" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nあんたたちが何考えてるのかとか、面倒事を起こす理由とかも\nそれなのに、私だけが一方的に信用されてるなんて言われても" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそうかもしれないわね" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\n実際にそうなの。不安なのよ\n隠してることがあるんじゃないかとか、本当に、単なる遊びなのかとか" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nスペルカードルールもそうね。遊びと物事の決着手段が融合してる、比較的新しい決め事" );
			ui.ShowText("【紫】\n今回はそんな大層な物じゃないわ。お寺と聖人様の懇親会とか、吸血鬼と月人の顔合わせとか");
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n月のお姫様にはそでにされちゃったみたいだけどね" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nじゃあ、難しく考えなくてもいいのね" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nええ。信用してって言っても、信じてもらえないかもしれないけど" );
			ui.ShowText( "【霊夢】\n信じるわよ" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nえ？" );
			ui.ShowImage( Reimu[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nあんたが私を信用してるって言うんだから、私も同じぐらい信用してあげる" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそう。ありがとう" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【霊夢】\nなんかあったら怒るから" );
			ui.ShowImage( -1, StoryUIImagePosition.Left, false, false );
			var talkEnd = ui.ShowImage( -1, StoryUIImagePosition.Right, false, false );
			while( !ui.IsCompleted( talkEnd ) )
			{
				yield return null;
			}
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		private IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【紫】\n試合終了ね" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n霊夢が怒ると怖いから、勝ち逃げいたしますわ" );
			ui.ShowImage( Reimu[1], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【霊夢】\nあ、待ちなさいよ・・！" );
			ui.ShowImage( -1, StoryUIImagePosition.Left, false, false );
			var talkEnd = ui.ShowImage( -1, StoryUIImagePosition.Right, false, false );
			while( !ui.IsCompleted( talkEnd ) )
			{
				yield return null;
			}
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		private IEnumerable Tactics( IWorld world )
		{
			const int spanTime = 1320;

			Action<IEnumerable<IUnit>> goToWhiteBase = u =>
				world.UnitManager.AttackMove( u.Select( _ => _.UniqueID ), whiteBase, false );

			for( int i = 0; i < 6300; i++ )
				yield return null;

			while( true )
			{
				goToWhiteBase( PutEnemy<Unit.Blue.Low02>( world, areas["召喚位置１"], 4 ) );

				for( int i = 0; i < spanTime; i++ )
					yield return null;

				goToWhiteBase( PutEnemy<Unit.Blue.Lunar>( world, areas["召喚位置２"], 2 ) );

				for( int i = 0; i < spanTime; i++ )
					yield return null;

				goToWhiteBase( PutEnemy<Unit.Blue.Iku>( world, areas["召喚位置３"], 2 ) );

				for( int i = 0; i < spanTime; i++ )
					yield return null;
			}
		}

		int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W )
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}

		IEnumerable<IUnit> PutEnemy<TUnit>( IWorld world, Area area, int num ) where TUnit : UnitParameter
		{
			var rect = area as AreaRectangle;
			if( rect == null )
				return new IUnit[0];

			List<IUnit> units = new List<IUnit>();
			var type = typeof( TUnit );

			var sx = Math.Min( rect.Start.X, rect.End.X );
			var sy = Math.Min( rect.Start.Y, rect.End.Y );
			var ex = Math.Max( rect.Start.X, rect.End.X );
			var ey = Math.Max( rect.Start.Y, rect.End.Y );

			for( var y = sy; y < ey; y++ )
			{
				for( var x = sx; x < ex; x++ )
				{
					if( units.Count >= num )
						goto LoopEnd;

					var u = world.UnitManager.Create(
						type.FullName,
						world.Factions[1],
						( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
						( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
						world.Factions[1] );

					if( u != null )
						units.Add( u );
				}
			}
		LoopEnd:

			return units;
		}
		#endregion
	}
}

#endif