﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Story
{
	public class StoryBlack02 : thrts.Mission
	{
		public override string[] GetSelectableFactions()
		{
			return new string[] {
				typeof( Faction.Black ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]	{
				typeof( Faction.Green ).FullName,
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryBlack02.map",
			};
		}

		public StoryBlack02()
			: base()
		{
			Name = "ストーリー黒02";
			Description = "黒2話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		int[] Remilia = null;
		int[] Sanae = null;
		int[] Mokou = null;
		int[] Miko = null;
		int[] Yuka = null;
		int[] Suika = null;
		int[] Lyrica = null;

		int[] Utsuho = null;
		int[] Rin = null;
		int[] Yukari = null;
		int[] Minoriko = null;
		int[] Kasen = null;
		int[] Yuyuko = null;

		int Back1 = 0;
		Area[] areas = null;
		IBuilding[] greenHomes = null;
		IBuilding blackHome = null;
		
		const int enemySpan = 1800;

		/// <summary>
		/// スペルを使用されたかどうか
		/// </summary>
		bool executedMikoSpall = false;
		bool executedSuikaSpall = false;
		bool executedLylicaSpall = false;

		public override void OnInitialize(thrts.IWorld world)
		{
			// 最初から負けないように
			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;
			
			// 初期資源を配布
			world.Factions[0].IncResource(0, 200);
			world.Factions[0].IncResource(1, 200);

			// 本拠の位置を初期視点にする
			var home = world.BuildingManager.GetBuildings(_ => _.Parameter.GetType() == typeof(LotusCraft.Building.Black.HomeBase)).First();
			var basePos = home.Pos;

			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D((int)basePos.X, (int)basePos.Y);

			world.Factions[1].PlayerName = "緑勢力";
			
			base.OnInitialize(world);
		}

		public override System.Collections.IEnumerable Update(IWorld world)
		{
			var ui = world.StoryUI;

			blackHome = GetBuildings<Building.Black.HomeBase>(world).First();
			greenHomes = GetBuildings<Building.Green.HomeBase>(world).ToArray();

			Remilia = LoadTachies(ui, "LotusCraft/Image/Tachie/Remilia_N", "LotusCraft/Image/Tachie/Remilia_S", "LotusCraft/Image/Tachie/Remilia_W");
			Mokou = LoadTachies(ui, "LotusCraft/Image/Tachie/Mokou_N", "LotusCraft/Image/Tachie/Mokou_S", "LotusCraft/Image/Tachie/Mokou_W");
			Sanae = LoadTachies(ui, "LotusCraft/Image/Tachie/Sanae_N", "LotusCraft/Image/Tachie/Sanae_S", "LotusCraft/Image/Tachie/Sanae_W");
			Miko = LoadTachies(ui, "LotusCraft/Image/Tachie/Miko_N", "LotusCraft/Image/Tachie/Miko_S", "LotusCraft/Image/Tachie/Miko_W");
			
			Yuka = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuka_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_W") };
			Suika = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Suika_N"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_S"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_W") };
			Lyrica = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_N"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_S"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_W") };

			Utsuho = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Utsuho_N"), ui.LoadImage("LotusCraft/Image/Tachie/Utsuho_S"), ui.LoadImage("LotusCraft/Image/Tachie/Utsuho_W") };
			Rin = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Rin_N"), ui.LoadImage("LotusCraft/Image/Tachie/Rin_S"), ui.LoadImage("LotusCraft/Image/Tachie/Rin_W") };

			Yukari = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yukari_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yukari_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yukari_W") };
			Minoriko = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_N"), ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_S"), ui.LoadImage("LotusCraft/Image/Tachie/Minoriko_W") };
			Kasen = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Kasen_N"), ui.LoadImage("LotusCraft/Image/Tachie/Kasen_S"), ui.LoadImage("LotusCraft/Image/Tachie/Kasen_W") };

			Yuyuko = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuyuko_W") };

			Back1 = ui.LoadImage("LotusCraft/Image/Back/Black");

			int experience = 0;
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				experience = 3000;
			}
			else if (world.StoryUtilities.GetDifficulty() == 1)
			{
				experience = 5000;
			}
			else if (world.StoryUtilities.GetDifficulty() > 1)
			{
				experience = 7000;
			}


			var lyricaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Lyrica)).First();
			lyricaUnit.AddExperience(experience);

			var kasenUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Kasen)).First();
			kasenUnit.AddExperience(experience);

			var minorikoUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Minoriko)).First();
			minorikoUnit.AddExperience(experience);

			world.ChangeBGM(BGMList.Boushuen_Story1);

			//*
			foreach (var item in Talk会話冒頭(world))
			{
				yield return null;
			}
			//*/

			world.ChangeBGM(BGMList.Battle_YoukaiNoYama);

			ui.ShowConditionText("目的：敵拠点の撃破");

			int c = 0;

			while (true)
			{
				// 勝利条件
				if (!greenHomes.Any(_=>_.Exists()))
				{
					world.ChangeBGM(BGMList.Boushuen_Win);

					foreach (var item2 in Talk勝利(world))
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();

					world.StoryUtilities.SetDataToSaveData("LotusCraft.Black.02", "Clear");

					if (world.StoryUtilities.GetDifficulty() == 0)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Black.02.Easy", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 1)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Black.02.Normal", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 2)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Black.02.Hard", "Clear");
					}
					break;
				}

				// 敗北条件
				if (!blackHome.Exists())
				{
					foreach (var item2 in Talk敗北(world))
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();
					break;
				}

				/*
				// スペル発動
				if (mikoUnit.Exists() && !executedMikoSpall)
				{
					TryCallMikoSpell(world, mikoUnit);
				}

				if (kasenUnit.Exists() && !executedSuikaSpall)
				{
					TryCallSuikaSpell(world, kasenUnit);
				}
				if (lyricaUnit.Exists() && !executedLylicaSpall)
				{
					TryCallLylicaSpell(world, lyricaUnit);
				}
				*/

				// ざこ
				if (c % enemySpan == 0)
				{
					if (c >= enemySpan * 5)
					{
						if (c == enemySpan * 5)
						{
							foreach (var green in greenHomes.Where(_ => _.Exists()))
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l2.UniqueID);
								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Komachi).FullName, green);
								uniqueIDs.Add(l4.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, blackHome, false);
							}
						}
						else if (c == enemySpan * 10)
						{
							foreach (var green in greenHomes.Where(_ => _.Exists()))
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l2.UniqueID);
								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l4.UniqueID);
								var l5 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Komachi).FullName, green);
								uniqueIDs.Add(l5.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, blackHome, false);
							}
						}
						else
						{
							foreach (var green in greenHomes.Where(_ => _.Exists()))
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l2.UniqueID);
								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l4.UniqueID);
								var l5 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l5.UniqueID);
								var l6 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, green);
								uniqueIDs.Add(l6.UniqueID);
								
								var l7 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Komachi).FullName, green);
								uniqueIDs.Add(l7.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, blackHome, false);
							}
						}
					}
				}

				c++;
				yield return null;
			}

			yield return base.Update(world);
		}

		private void TryCallMikoSpell(IWorld world, IUnit mikoUnit)
		{
			if (executedMikoSpall) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - mikoUnit.Pos.X) * (u.Pos.X - mikoUnit.Pos.X) + (u.Pos.Y - mikoUnit.Pos.Y) * (u.Pos.Y - mikoUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[2].GetUnitSpellButtonParameter(mikoUnit.Parameter.GetType().FullName, new SpellButtonKey("神光「逆らう事なきを宗とせよ」", 1));
					spell.SelectPlaceButtonEvent(mikoUnit, new Vector2D(u.Pos.X, u.Pos.Y), false);
					executedMikoSpall = true;
				}
			}
		}

		private void TryCallSuikaSpell(IWorld world, IUnit suikaUnit)
		{
			if (executedSuikaSpall) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - suikaUnit.Pos.X) * (u.Pos.X - suikaUnit.Pos.X) + (u.Pos.Y - suikaUnit.Pos.Y) * (u.Pos.Y - suikaUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[1].GetUnitSpellButtonParameter(suikaUnit.Parameter.GetType().FullName, new SpellButtonKey("鬼神「ミッシングパープルパワー」", 1));
					spell.NormalButtonEvent(suikaUnit, false);
					executedSuikaSpall = true;
				}
			}
		}

		private void TryCallLylicaSpell(IWorld world, IUnit lyUnit)
		{
			if (executedLylicaSpall || !lyUnit.Exists()) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - lyUnit.Pos.X) * (u.Pos.X - lyUnit.Pos.X) + (u.Pos.Y - lyUnit.Pos.Y) * (u.Pos.Y - lyUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[1].GetUnitSpellButtonParameter(lyUnit.Parameter.GetType().FullName, new SpellButtonKey("騒符「リリカ・ソロライブ」", 1));
					spell.NormalButtonEvent(lyUnit, false);
					executedLylicaSpall = true;
				}
			}
		}


		#region Story
		private System.Collections.IEnumerable Talk会話冒頭(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk1
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Back1, StoryUIImagePosition.Back, true, false, true);
			ui.ShowText("～仙界～");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\n頼もしい仲間も見つけたことですし、これで喧嘩を売ることができますね");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【お空】\n喧嘩？　誰かと喧嘩するの？");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nええ。実際に喧嘩するわけじゃないんですけどね");
			ui.ShowImage(Utsuho[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お空】\n？？？");
			ui.ShowImage(Rin[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お燐】\nまぁまぁ、とにかくあたいたちはついていけば間違いないのさ");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お空】\nそうなんだ。じゃあついてこう");
			ui.ShowImage(Rin[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【お燐】\nいっつも思うけど、お空は幸せそうだね");
			ui.ShowImage(Utsuho[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お空】\nうん。たぶんそうだよ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【青娥】\nでも太子様。当てはあるのですか？");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\n博麗の巫女とはもう既に会いましたし");
			ui.ShowImage(Remilia[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【神子】\n順当に行くなら紅魔館か");
			ui.ShowImage(Yukari[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【神子】\n主催者か");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【神子】\nもしくは在野の勢力ということになるでしょうね");
			ui.ShowImage(Yuyuko[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽々子】\n紫はどこに居るかわからないし、後回しにしましょ。どうせ呼べば出てくるわ");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nそんなものですか");
			ui.ShowImage(Miko[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nしかしそこまで私も幻想郷に詳しいわけではないので\nどこに行けばというのがさっぱりわからないのですよ");
			ui.ShowImage(Utsuho[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【お空】\nうーん……。私も山ぐらいしか行かないし");
			ui.ShowImage(Rin[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お燐】\nあたいも神社ぐらいかなぁ");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お空】\nあ、でも山には天狗とか河童とか、あとは仙人も居るとか");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\n仙人？　それは少し興味がありますね\n今回と関係があるかはわかりませんが、赴いてみる価値はありそうです");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Back, true, false, true);
			ui.ShowText("～妖怪の山～");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nお空さんお空さん。その方はどこらへんに住んでらっしゃるのでしょうか");
			ui.ShowImage(Utsuho[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【お空】\nうーん");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お空】\nわかんない");
			ui.ShowImage(Miko[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nそれは困りました\n天狗は縄張り意識が強い種族と聞いていますから、下手に入れば不興を買ってしまう");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【穣子】\nやや！　そこの団体さん。さては訳あり！？");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nはい？");
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【穣子】\nあいや、そうでしょうそうでしょう。隠さなくとも結構です");
			ui.ShowImage(Miko[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nああいや、別に出番を提供するために来たわけではなくて");
			ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【穣子】\nこれでもレギュラーメンバーなのに！");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nレギュラー。ということは、誰かと組んでいらっしゃるとb");
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【穣子】\nそーよ！　まさに妖怪の山代表と言っても過言ではないから！");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nなるほど。それは失礼しました。\n私は豊郷耳神子と申しまして、幻想郷の方々に挨拶をして回っているのです。");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\n良ければ、紹介していただけないでしょうか");
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【穣子】\nふふふ。待ってて！");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("～一〇分後～");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【穣子】\nお待たせ！");
			ui.ShowImage(Lyrica[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【リリカ】\n出番があると聞いてやってきたよ！");
			ui.ShowImage(Kasen[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【華扇】\n引率でやってきました……");
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【青娥】\nあら？　いつぞやの仙人様じゃないですか");
			ui.ShowImage(Kasen[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【華扇】\nあ、その時はどうも");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\n山に住んでいらっしゃる仙人様というのはあなたでしたか");
			ui.ShowText("【華扇】\nあなたも……");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nええ。しがない尸解仙をやっています");
			ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【華扇】\nなるほど。共に研鑽……というのもおかしな話ですが、お会いできて嬉しいです");
			ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【穣子】\nちょっと！　私を置いて和気藹々とされても！");
			ui.ShowImage(Lyrica[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【リリカ】\nどうせ私はぽっと出だから……");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\n友好を深めるのはまた別の機会を持つとして、あなた方も今回の催しに参加していらっしゃると");
			ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【華扇】\nええ、旧知の仲の者に参加してくれと頼まれて……。この二人も同じように引っ張られたクチみたいですけど");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【神子】\nなるほど。私はむしろ引っ張る側なものでして");
			ui.ShowImage(Kasen[0], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【華扇】\nあ、そうなんですか……");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Rin[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【お燐】\n(凄い、この二人。生真面目過ぎて会話が続かない……)");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お空】\nとりあえず、戦おうそうしよう！　さっきしたみたいにしたらいいんでしょ？");
			ui.ShowImage(Rin[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【お燐】\n(ナイスお空！　この馴れ合いの空気を上手く流した！)");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nその通りです。会話よりも戦ってみることで、お互いのことがよくわかるやもしれません");
			ui.ShowImage(Kasen[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【華扇】\nあまり暴れすぎて、天狗の方々から注意されないようにしましょう");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("", false);



			var talkEnd = ui.ShowText("");
			while (!ui.IsCompleted(talkEnd))
				yield return null;

			ui.Hide();
			ui.ResumeGame();
			#endregion


			yield return null;
		}



		private System.Collections.IEnumerable Talk勝利(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nいい勝負でした");
			ui.ShowImage(Lyrica[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【リリカ】\nえ、私の出番これでおしまい？　え、嘘でしょ？");
			ui.ShowImage(Utsuho[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お空】\n楽しかったー！");
			ui.ShowImage(Kasen[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【華扇】\nええ、またこれからも会う機会はあると思いますので、その時はよろしくお願いします");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nこちらこそ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Yuyuko[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽々子】\n真面目ねえ");
			ui.ShowImage(Rin[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【お燐】\nですよねえ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【青娥】\nそれが太子様の良いところですので");
			ui.ShowText("", false);


			var talkEnd0 = ui.ShowText("", false);
			while (!ui.IsCompleted(talkEnd0))
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北(IWorld world)
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}
		#endregion

		private int[] LoadTachies(IStoryUI ui, string path_N, string path_S, string path_W)
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		/// <summary>
		/// ユニットをエリア上に生成する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <param name="area"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> PutUnitsOnArea(IWorld world, IFaction faction, Type paramType, Area area)
		{
			var ar = area as AreaRectangle;
			if (ar == null) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min(ar.Start.X, ar.End.X);
			var sy = Math.Min(ar.Start.Y, ar.End.Y);
			var ex = Math.Max(ar.Start.X, ar.End.X);
			var ey = Math.Max(ar.Start.Y, ar.End.Y);

			for (var y = sy; y <= ey; y++)
			{
				for (var x = sx; x <= ex; x++)
				{
					var u =
						world.UnitManager.Create(paramType.FullName, faction, (x + 0.5f) * thrts.Def.MapTipLengthOnField, (y + 0.5f) * thrts.Def.MapTipLengthOnField, faction);
					if (u != null) units.Add(u);
				}
			}

			return units;
		}

		public IEnumerable<IUnit> PutEnemy(IWorld world, Type unitType, Area area, int num, IBuilding target)
		{
			var rect = area as AreaRectangle;
			if (rect == null) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min(rect.Start.X, rect.End.X);
			var sy = Math.Min(rect.Start.Y, rect.End.Y);
			var ex = Math.Max(rect.Start.X, rect.End.X);
			var ey = Math.Max(rect.Start.Y, rect.End.Y);

			for (var y = sy; y < ey; y++)
			{
				for (var x = sx; x < ex; x++)
				{
					if (units.Count >= num)
						goto LoopEnd;

					var u = world.UnitManager.Create(
						unitType.FullName,
						world.Factions[1],
						(x + 0.5f) * thrts.Def.MapTipLengthOnField,
						(y + 0.5f) * thrts.Def.MapTipLengthOnField,
						world.Factions[1]);
					if (u != null)
						units.Add(u);
				}
			}
		LoopEnd:

			if (target != null)
				world.UnitManager.SelectTarget(units, new Vector2D(target.Pos.X, target.Pos.Y), false);

			return units;
		}

		bool IsUnitOnArea(AreaRectangle region, IUnit unit)
		{
			var startx = Math.Min(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var starty = Math.Min(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			var endx = Math.Max(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var endy = Math.Max(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			if (unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y)
			{
				return true;
			}
			return false;
		}

		IEnumerable<IBuilding> GetBuildings<T>(IWorld world) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings(_ => _.Parameter.GetType() == typeof(T));
		}
	}
}

#endif