﻿#if !_TRIAL

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryBlue01 : thrts.Mission
	{
		#region フィールド
		static readonly int[] YukariFirstExp = new List<int> { 4000, 1500, 0 }.ToArray();
		static readonly int[] TenshiFirstExp = new List<int> { 0, 0, 0 }.ToArray();
		static readonly int Exp = 50;
		static readonly int EnemyAmountInArea = 2;
		static readonly int TimeLimit = Def.FPS * 60 * 5;
		static readonly int CreateEnemySpan = Def.FPS * 60 * 2;
		static readonly float[] TenshiSpeed = new List<float> { 0.4f, 0.7f, 1.0f }.ToArray();
		static readonly int TenshiMoveSpan = Def.FPS * 5;

		int[] Yukari;
		int[] Tenshi;
		int Back;

		int difficulty;
		int restTime = TimeLimit;
		int yukariScore
		{
			get { return ( yukariUnit.GetExperiencePoint() - YukariFirstExp[difficulty] ) / Exp; }
		}
		int tenshiScore
		{
			get { return ( tenshiUnit.GetExperiencePoint() - TenshiFirstExp[difficulty] ) / Exp; }
		}

		IUnit yukariUnit;
		IUnit tenshiUnit;
		IFaction yukariFaction;
		IFaction tenshiFaction;
		IFaction neutralFaction;
		UnitParameter[] neutralUnitParameters;
		Dictionary<string, Area> areas = new Dictionary<string, Area>();
		#endregion

		public StoryBlue01()
			: base()
		{
			Name = "青1話";
			Description = "青1話専用マップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		public override string[] GetEnableFactions()
		{
			return new string[]
			{
				typeof( Faction.Blue ).FullName,
				typeof( Faction.Neutral ).FullName,
			};
		}
		public override string[] GetSelectableFactions()
		{
			return new string[]
			{
				typeof( Faction.Blue ).FullName,
			};
		}
		public override string[] GetSelectableMaps()
		{
			return new string[]
			{
				"LotusCraft/Story/StoryBlue01.map"
			};
		}

		public override void OnInitialize( IWorld world )
		{
			yukariFaction = world.Factions[0];
			tenshiFaction = world.Factions[1];
			neutralFaction = world.Factions[2];

			yukariFaction.DoesNotLose = true;
			tenshiFaction.DoesNotLose = true;
			neutralFaction.DoesNotLose = true;

			foreach( var item in world.AllPurposeArea )
			{
				areas[item.Name] = item;
			}

			yukariUnit = world.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Blue.Unique.Yukari ) ).First();
			tenshiUnit = world.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Blue.Unique.Tenshi ) ).First();
			yukariFaction.Parameter.DefaultScreenPosition = To2D( yukariUnit.Pos ).ToPoint2D();

			var neutralTypes = new[]{
				typeof( Unit.Neutral.Low01 ), typeof( Unit.Neutral.Low02 ),
				typeof( Unit.Neutral.Low01 ), typeof( Unit.Neutral.Low02 ),
				// typeof( Unit.Neutral.Low11 ),
				typeof( Unit.Neutral.Low12 ),
				// typeof( Unit.Neutral.High01 ),
				// typeof( Unit.Neutral.High02 )
			};
			neutralUnitParameters =
				neutralTypes.Select( _ => neutralFaction.GetUnitParameter( _.FullName ) )
				.ToArray();
			neutralUnitParameters.ForEach( _ => _.Exp = Exp );

			Yukari = LoadTachies( world.StoryUI, "LotusCraft/Image/Tachie/Yukari_N", "LotusCraft/Image/Tachie/Yukari_S", "LotusCraft/Image/Tachie/Yukari_W" );
			Tenshi = LoadTachies( world.StoryUI, "LotusCraft/Image/Tachie/Tenshi_N", "LotusCraft/Image/Tachie/Tenshi_S", "LotusCraft/Image/Tachie/Tenshi_W" );
			Back = world.StoryUI.LoadImage( "LotusCraft/Image/Back/Black" );

			yukariUnit.Vanishing += _ => _.HP = _.MaxHP;
			tenshiUnit.Vanishing += _ => _.HP = _.MaxHP;
			yukariUnit.Updating += () => yukariUnit.IsntFoundByEnemy = true;
			tenshiUnit.Updating += () => tenshiUnit.IsntFoundByEnemy = true;

			SetDifficulty( world );
		}

		public override IEnumerable Update( IWorld world )
		{
			var ui = world.StoryUI;
			world.ChangeBGM( BGMList.Boushuen_Story1 );

			foreach( var item in Talk冒頭会話( world ) )
				yield return null;

			var enemyAI = EnemyAI( world );
			var tenshiAI = TenshiAI( world );

			world.ChangeBGM( BGMList.Battle_Tenkai );

			while( true )
			{
				var str = new StringBuilder();
				str.AppendLine( "目的：天子より多く敵を倒す。" );
				str.AppendFormat( "残り時間　{0:00}:{1:00}\n", restTime / Def.FPS / 60, restTime / Def.FPS % 60 );
				str.AppendFormat( "紫:{0:##0}体\n", yukariScore );
				str.AppendFormat( "天子:{0:##0}体", tenshiScore );
				ui.ShowConditionText( str.ToString() );

				enemyAI.MoveNext();
				tenshiAI.MoveNext();

				if( restTime == 0 )
				{
					if( yukariScore > tenshiScore )
					{
						foreach( var item in Talk勝利( world ) )
							yield return null;

						world.StoryUtilities.SetDataToSaveData("LotusCraft.Blue.01", "Clear");
						world.StoryUtilities.SetDataToSaveData(GetClearDataName(world), "Clear");
					}
					else if( yukariScore < tenshiScore )
					{
						foreach( var item in Talk敗北( world ) )
							yield return null;
					}
					else if( yukariScore == tenshiScore )
					{
						foreach( var item in Talk引き分け( world ) )
							yield return null;
					}
					break;
				}

				--restTime;
				yield return null;
			}

			yield return null;
		}
		private void SetDifficulty( IWorld world )
		{
			difficulty = world.StoryUtilities.GetDifficulty();
			yukariUnit.AddExperience( YukariFirstExp[difficulty] );
			tenshiUnit.AddExperience( TenshiFirstExp[difficulty] );

			var p = tenshiFaction.GetUnitParameter( typeof( Unit.Blue.Unique.Tenshi ).FullName );
			p.Speed = TenshiSpeed[difficulty] * UnitParameter.BasisSpeed;
		}
		private string GetClearDataName( IWorld world )
		{
			switch( world.StoryUtilities.GetDifficulty() )
			{
			case 0: return "LotusCraft.Blue.01.Easy";
			case 1: return "LotusCraft.Blue.01.Normal";
			case 2: return "LotusCraft.Blue.01.Hard";
			default: throw new InvalidOperationException( "不正な難易度が設定されています。" );
			}
		}

		#region Story
		private IEnumerator EnemyAI( IWorld world )
		{
			var enemyAreas = new[]{
				areas["敵位置１"], areas["敵位置２"], areas["敵位置３"],
				areas["敵位置４"], areas["敵位置５"], areas["敵位置６"], areas["敵位置７"],
			};

			for( int i = 0; ; i++ )
			{
				if( i % CreateEnemySpan == 0 )
				{
					enemyAreas.ForEach( _ => PutUnit( world, neutralFaction, _, EnemyAmountInArea ) );
				}
				yield return null;
			}
		}
		private IEnumerator TenshiAI( IWorld world )
		{
			var targets = new[]{
				areas["天子目標１"],
				areas["天子目標２"],
				areas["天子目標３"],
			};

			int count = 0;
			while( true )
			{
				for( int i = 0; i < TenshiMoveSpan; i++ )
				{
					if( tenshiUnit.GetTarget() != null )
						goto LOOP_END;
					yield return null;
				}

				world.UnitManager.AttackMove(
					tenshiUnit.UniqueID.ToSequence(),
					GetAreaCenter( targets[count % targets.Length] ) * Def.MapTipLengthOnField,
					true );
				++count;

			LOOP_END: yield return null;
			}
		}

		private IEnumerable Talk冒頭会話( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk1

			ui.PauseGame();
			ui.Show();
			ui.ShowText( "～天界～" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【紫】\n鬼が本気で暴れれば、幻想郷は容易く滅茶苦茶になってしまう" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそれは単なる一例で、肩身を寄せ合って、気を遣っているうちに自分の存在意義すらわからなくなり、消えてしまう。" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n幻想郷は楽園と呼べる場所なのかしら" );
			ui.ShowText( "【紫】\n余りにもここは狭すぎる。あまりにもここは脆すぎる" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【天子】\n大妖怪様がなぜこんなところにいらっしゃるのですか？　お帰りはあちらですが" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nあら、あなたにそのような権限が与えられていたとは初耳だけれど" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n小鬼もそうだけど、あんたみたいなのがうろついてるとピリピリするのよね\n用がないのなら立ち去ってもらえないかしら" );
			ui.ShowText( "【紫】\n用ならありますわ" );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n何？　天界までわざわざ出向くんですもの、何か大層な理由があるとお見受けするけれど" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n実は新しい遊びを考えていますの" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n新しい遊び？　結局遊びなんじゃない" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそ。遊び。退屈に喘ぐ者たちを、少しでも掬うためのね" );
			ui.ShowText( "【天子】\n暇が人を殺すのは同意するけれど" );
			ui.ShowText( "【紫】\nそれで、お誘いに来ましたの" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n結論だけを話してちょうだい" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nあなた、私と一緒に、幻想郷を荒らしてちょうだい" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n荒らすって、何が目的で？" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそれは後々。" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそれに、あなただって端っこから見ているだけじゃなくって、たまには参加したいんでしょう？" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n……" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n私はその場を提供する。私はあなたの力を借りる。ほんのそれだけのことよ" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\nま、好きにしても構わないってことだと理解したわ" );
			ui.ShowText( "【紫】\nええ、その通り" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\nでもその前に、貴方の力試しぐらいはさせてもらっても構わないわよね？" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nもちろん" );
			ui.ShowText( "", false );
			ui.ShowImage( Back, StoryUIImagePosition.Front, true );
			ui.ShowImage( -1, StoryUIImagePosition.Right, false );
			var talkEnd1 = ui.ShowImage( -1, StoryUIImagePosition.Left, false );
			while( !ui.IsCompleted( talkEnd1 ) )
				yield return null;

			var enemyAreas = new[]{
				areas["敵位置１"], areas["敵位置２"], areas["敵位置３"],
				areas["敵位置４"], areas["敵位置５"], areas["敵位置６"], areas["敵位置７"],
			};
			enemyAreas.ForEach( _ => PutUnit( world, neutralFaction, _, EnemyAmountInArea ) );

			ui.ShowImage( -1, StoryUIImagePosition.Front, true );

			ui.ResumeGame();
			ui.Hide();

			#endregion

			#region Takl2

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【天子】\nそれでルールは？" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【紫】\nこのあたりに私の式をばら撒きました" );
			ui.ShowText( "【紫】\n兎狩りとはいきませんが、狩り勝負と洒落こみません？" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\nあぁ、なんか変な気配がやけにいるけれど、あれのこと" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\n私の式神ですがもちろんイカサマのたぐいはいたしませぬ" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\n別に構わないわ" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【天子】\nそんなことで後でいちゃもんつけてくると思われてる事の方が不快だけど" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【紫】\nそれは失礼\nでは始めましょうか？多くを狩った者の勝ち、簡単でしょう" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true );
			var talkEnd2 = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd2 ) )
				yield return null;

			ui.ResumeGame();
			ui.Hide();
			#endregion
		}

		private IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;

			var yukariWord = string.Format( "【紫】\n私が倒したのは{0}体、あなたは{1}体", yukariScore, tenshiScore );

			#region Talk

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, true );
			//ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, true );
			ui.ShowText( "【紫】\nはい、試合終了" );
			ui.ShowText( yukariWord );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Right, true );
			var win = ui.ShowImage( Yukari[1], StoryUIImagePosition.Left, false );

			while (!ui.IsCompleted(win))
				yield return null;

			world.ChangeBGM(BGMList.Boushuen_Win);

			ui.ShowText( "【紫】\n私の勝ちですわ" );
			ui.ShowImage( Back, StoryUIImagePosition.Front, true );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true );

			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Right, true, false );
			ui.ShowText( "【天子】\n言っておくけど、私はあなたが気に食わないわ" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Left, true, false );
			ui.ShowText( "【紫】\nあら、私もよ" );
			ui.ShowText( "【天子】\n途中で背中から刺されても文句は言わせないから" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【紫】\nそれぐらいでいいのよ\n本当に刺されちゃったら困るけど、忠実な部下なんてものを求めてはいないから" );
			ui.ShowText( "【天子】\nやっぱりあなたって気に食わないわ" );
			ui.ShowText( "【紫】\nどうも？" );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【天子】\n少しも褒めてないわ" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【紫】\n褒められたついでに、ちょっとお願いことをしたいの。\nこのルールを広める手伝いをしてもらえない？" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【天子】\n一人じゃ手が足りないってことよね？" );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【紫】\nそういうこと。ごめんなさいね。いきなりつまらないことを頼んで" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【天子】\nちゃんと一言言ってくれるんだったら大丈夫よ。そこまで狭量じゃないわ" );
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【紫】\nそれじゃ、一度ここで解散ということで\n一応通信機も渡しておくから。以前地底に行ったときの再利用を" );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Right, false, false );
			ui.ShowText( "【天子】\nなんだかんだでマメなのね……" );
			ui.ShowImage( Yukari[1], StoryUIImagePosition.Left, false, false );
			ui.ShowText( "【紫】\nじゃなきゃこんなことできないわよ" );


			var talkEnd = ui.ShowText( "", false );

			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.ResumeGame();
			ui.Hide();

			#endregion

			yukariFaction.Win();
		}

		private IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;

			var tenshiWord = string.Format( "【天子】\nあなたは{0}体、私は{1}体倒したわ", yukariScore, tenshiScore );

			#region Talk

			ui.PauseGame();
			ui.Show();
			//ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, false );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, true );
			ui.ShowText( "【天子】\n試合終了かしら？" );
			ui.ShowText( tenshiWord );
			ui.ShowImage( Yukari[2], StoryUIImagePosition.Left, true );
			var lose = ui.ShowImage( Tenshi[1], StoryUIImagePosition.Right, false );
			
			while (!ui.IsCompleted(lose))
				yield return null;

			world.ChangeBGM(BGMList.Boushuen_Lose);
			ui.ShowText( "【天子】\n私の勝利ね" );
			ui.ShowImage( Back, StoryUIImagePosition.Front, true );
			ui.ShowImage( -1, StoryUIImagePosition.Left, false );
			ui.ShowImage( -1, StoryUIImagePosition.Right, false );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true );
			var talkEnd = ui.ShowText( "", false );

			// 会話が続く

			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.ResumeGame();
			ui.Hide();

			#endregion

			foreach( var item in Talk敗北_後半( world ) )
				yield return null;

			yield return null;
		}

		private IEnumerable Talk引き分け( IWorld world )
		{
			var ui = world.StoryUI;

			var yukariWord = string.Format( "【紫】\n私が倒したのは{0}体、あなたは{1}体", yukariScore, tenshiScore );

			#region Talk

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yukari[0], StoryUIImagePosition.Left, true );
			// ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, true );
			ui.ShowText( "【紫】\n試合終了ね" );
			ui.ShowText( yukariWord );
			var lose = ui.ShowImage( Tenshi[1], StoryUIImagePosition.Right, true );

			while (!ui.IsCompleted(lose))
				yield return null;

			world.ChangeBGM(BGMList.Boushuen_Lose);
			ui.ShowText( "【天子】\n引き分けね？\nこういう時は挑戦者の負けって相場が決まっているのよ" );
			ui.ShowImage( Back, StoryUIImagePosition.Front, true );
			ui.ShowImage( -1, StoryUIImagePosition.Left, false );
			ui.ShowImage( -1, StoryUIImagePosition.Right, false );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true );
			var talkEnd = ui.ShowText( "", false );

			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.ResumeGame();
			ui.Hide();

			#endregion

			foreach( var item in Talk敗北_後半( world ) )
				yield return null;

			yield return null;
		}

		private IEnumerable Talk敗北_後半( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Front, true, false );
			ui.ShowText( "【天子】\n妖怪の賢者様がこの程度？\n手を抜いて煽てて乗るほど私は安くないわよ" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Front, false, false );
			ui.ShowText( "【天子】\nちゃんと勝てたら、手伝ってあげるわ" );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			tenshiFaction.Win();

			yield return null;
		}
		#endregion

		#region Utility

		Vector2D To2D( Vector3D vector )
		{
			return new Vector2D( vector.X, vector.Y );
		}
		int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W )
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		/// <summary>
		/// フィールドのどこかにユニットを生成する。
		/// </summary>
		/// <param name="world">World</param>
		/// <param name="parameter">生成するユニットのパラメータ</param>
		/// <param name="num">生成する数</param>
		/// <returns>生成されたユニットの配列</returns>
		IUnit[] PutEnemies( IWorld world, int num )
		{
			List<IUnit> units = new List<IUnit>();

			// num*20回の試行でタイムアウト
			for( int i = 0; i < num * 20; i++ )
			{
				var parameter = neutralUnitParameters[world.NextRand( 0, neutralUnitParameters.Length )];
				var pos = GetSomeWhere( world );

				var unit = world.UnitManager.Create(
					parameter.GetType().FullName,
					neutralFaction,
					pos.X,
					pos.Y,
					neutralFaction );

				if( unit != null )
					units.Add( unit );

				if( units.Count >= num )
					break;
			}

			return units.ToArray();
		}

		protected IEnumerable<IUnit> PutUnit( IWorld world, IFaction faction, Area area, int num )
		{
			var rect = area as AreaRectangle;
			if( rect == null )
				return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			while( units.Count < num )
			{
				var parameter = neutralUnitParameters[world.NextRand( 0, neutralUnitParameters.Length )];
				var x = rect.Start.X + (float)world.NextRandDouble() * ( rect.End.X - rect.Start.X );
				var y = rect.Start.Y + (float)world.NextRandDouble() * ( rect.End.Y - rect.Start.Y );

				var u = world.UnitManager.Create(
					parameter,
					x * thrts.Def.MapTipLengthOnField,
					y * thrts.Def.MapTipLengthOnField,
					faction );

				if( u != null )
					units.Add( u );
			}
			return units;
		}

		/// <summary>
		/// フィールド上の位置からランダムな点を得る。
		/// </summary>
		/// <param name="world">乱数を提供するWorld。</param>
		/// <returns></returns>
		Vector2D GetSomeWhere( IWorld world )
		{
			var x = (float)world.NextRandDouble() * 64 * Def.MapTipLengthOnField;
			var y = (float)world.NextRandDouble() * 64 * Def.MapTipLengthOnField;
			return new Vector2D( x, y );
		}

		Vector2D GetAreaCenter( Area area )
		{
			var rect = area as AreaRectangle;
			return ( rect.Start.ToVector2D() + rect.End.ToVector2D() ) / 2;
		}

		#endregion
	}
}


#endif