﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryGreen01 : thrts.Mission
	{
		public override string[] GetSelectableFactions()
		{
			return new string[] {
				typeof( Faction.Green ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]	{
				typeof( Faction.Green ).FullName
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryGreen01.map",
			};
		}

		public StoryGreen01()
			: base()
		{
			Name = "ストーリー緑01";
			Description = "緑1話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		int[] Alice = null;
		int[] Suika = null;

		Area[] areas = null;
		IBuilding ownHome = null;
		IBuilding enemyHome = null;
		IBuilding enemyLow = null;
		IBuilding enemyHyoutan = null;

		int enemyspan = 1400;

		public override void OnInitialize( thrts.IWorld world )
		{
			world.Factions[1].PlayerName = "緑勢力";

			// 最初から負けないように
			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;
			
			// 初期資源を配布
			world.Factions[0].IncResource( 0, 200 );
			world.Factions[0].IncResource( 1, 200 );

			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Green.Low02).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Green.Low03).FullName).HP /= 2;
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				enemyspan = 1200;
			}

			// 本拠の位置を初期視点にする
			var basePos = world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( LotusCraft.Building.Green.HomeBase ) && _.Faction == world.Factions[0] ).First().Pos;
			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition
				= new Point2D( (int)basePos.X, (int)basePos.Y );

			// ボタン仕様禁止
			var home = world.BuildingManager.GetBuildings(_ => _.Faction == world.Factions[0] && _.Parameter.GetType() == typeof(LotusCraft.Building.Green.HomeBase)).First();
			var ui = world.StoryUI;
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._3);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._2, ButtonPosition._2);
			ui.HideButton(home, ButtonPosition._0, ButtonPosition._3);
			ui.HideButton(home, ButtonPosition._1, ButtonPosition._3);


			base.OnInitialize( world );
		}

		public override System.Collections.IEnumerable Update( IWorld world )
		{
			var ui = world.StoryUI;
			
			enemyHome = GetBuildings<Building.Green.HomeBase>(world).Where(_ => _.Faction == world.Factions[1]).First();
			enemyLow = GetBuildings<Building.Green.Barrack>(world).Where(_ => _.Faction == world.Factions[1]).First();
			enemyHyoutan = GetBuildings<Building.Green.Hyoutan>(world).Where(_ => _.Faction == world.Factions[1]).First();
			
			ownHome = GetBuildings<Building.Green.HomeBase>( world ).Where(_=>_.Faction == world.Factions[0]).First();

			Alice = LoadTachies(ui, "LotusCraft/Image/Tachie/Alice_N", "LotusCraft/Image/Tachie/Alice_S", "LotusCraft/Image/Tachie/Alice_W", "LotusCraft/Image/Tachie/Alice_A");
			Suika = LoadTachies( ui, "LotusCraft/Image/Tachie/Suika_N", "LotusCraft/Image/Tachie/Suika_S", "LotusCraft/Image/Tachie/Suika_W" );
			
			var suikaUnit = world.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( Unit.Green.Unique.Suika ) ).First();
			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				suikaUnit.AddExperience(2000);
			}
			else if (world.StoryUtilities.GetDifficulty() == 1)
			{
				suikaUnit.AddExperience(4000);
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				suikaUnit.AddExperience(6000);
			}

			world.ChangeBGM(BGMList.Boushuen_Story1);

			//*
			foreach( var item in Talk会話冒頭( world ) )
			{
				yield return null;
			}
			//*/

			world.ChangeBGM(BGMList.Battle_MahouNoMori);

			for (var c = 0; c < 100; c++) {
				yield return null;
			}

			foreach (var item in Talk戦闘開始(world))
			{
				yield return null;
			}

			ui.ShowConditionText("目的：萃香の本拠を撃破する");

			int i = 0;
			while(true)
			{
				if(!enemyHome.Exists())
				{
					foreach( var item2 in Talk勝利( world ) )
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();

					world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.01", "Clear");
					if (world.StoryUtilities.GetDifficulty() == 0)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.01.Easy", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 1)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.01.Normal", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 2)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.01.Hard", "Clear");
					}
					world.ChangeBGM(BGMList.Boushuen_Win);

					break;
				}
				if( !ownHome.Exists() )
				{
					foreach( var item2 in Talk敗北( world ) )
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();
					world.ChangeBGM(BGMList.Boushuen_Lose);

					break;
				}

				
				// ざこ
				if (i % enemyspan == 0 && i > enemyspan * 5)
				{
					if (i < enemyspan * 8)
					{
						if (enemyHyoutan.Exists() && enemyLow.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, enemyLow);
							uniqueIDs.Add(l1.UniqueID);
							var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, enemyLow);
							uniqueIDs.Add(l2.UniqueID);
							world.UnitManager.AttackMove(uniqueIDs, ownHome, false);
						}
						if (enemyHome.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var h1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, enemyHome);
							uniqueIDs.Add(h1.UniqueID);
							world.UnitManager.AttackMove(uniqueIDs, ownHome, false);
						}
					}
					else
					{
						if (enemyHyoutan.Exists() && enemyLow.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							if (world.StoryUtilities.GetDifficulty() == 2)
							{
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.SuikaSmall).FullName, enemyLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.SuikaSmall).FullName, enemyLow);
								uniqueIDs.Add(l2.UniqueID);
							}
							else
							{
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, enemyLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, enemyLow);
								uniqueIDs.Add(l2.UniqueID);
							}
							world.UnitManager.AttackMove(uniqueIDs, ownHome, false);

						}

						if (enemyLow.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							if (world.StoryUtilities.GetDifficulty() == 2)
							{
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.SuikaSmall).FullName, enemyLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.SuikaSmall).FullName, enemyLow);
								uniqueIDs.Add(l2.UniqueID);
							}
							else
							{
								var h1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low02).FullName, enemyLow);
								uniqueIDs.Add(h1.UniqueID);
								var h2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low02).FullName, enemyLow);
								uniqueIDs.Add(h2.UniqueID);
							}
							world.UnitManager.AttackMove(uniqueIDs, ownHome, false);
						}

						if (enemyHome.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var h1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low03).FullName, enemyHome);
							uniqueIDs.Add(h1.UniqueID);
							var h2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low03).FullName, enemyHome);
							uniqueIDs.Add(h2.UniqueID);
							world.UnitManager.AttackMove(uniqueIDs, ownHome, false);
						}

					}
				}

				i++;
				yield return null;
			}



			yield return base.Update( world );
		}

		#region Story
		private System.Collections.IEnumerable Talk会話冒頭( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk1

			ui.PauseGame();
			ui.Show();
			ui.ShowText("魔法の森～アリス・マーガトロイドの家～");
			ui.ShowText("（コン、コン、コン）");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Left, true, false);
			ui.ShowText("【アリス】\nどなた様？");
			ui.ShowImage(-1, StoryUIImagePosition.Left, true);
			ui.ShowImage(Suika[0], StoryUIImagePosition.Front, true, false);
			ui.ShowText("【萃香】\nよっ");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true);
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, true, false);
			ui.ShowText("【アリス】\nお引き取りください");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, true, false);
			ui.ShowText("【萃香】\nえー、話ぐらい聞いてくれたっていいじゃないか");
			ui.ShowText("【アリス】\n急に頭痛と腹痛と腰痛と腱鞘炎と眼精疲労が襲ってきたから誰とも会えなくなったの");
			ui.ShowImage(Suika[0], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n月のものか");
			ui.ShowText("【アリス】\nうわっ");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nそれに私は鬼だから、嘘は嫌いなんだが？");
			ui.ShowText("【アリス】\nはいはい\n普段と全く違う来客って時点で、ロクでもない話だっていうことが大体わかるのよ");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n実は紫の奴が勝負事を開こうって誘いをしてきて、それが人数がいないと駄目立って言うんで数合わせに");
			ui.ShowText("【アリス】\nいや少しは聞いてよ！");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n嫌よ嫌よも好きのうちって言うし、ちょっとぐらい強引に引っ張ったほうがいいってあいつが……");
			ui.ShowImage(Alice[3], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\nあいつって誰よ。魔理沙？");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nいや、幽香");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\nあいつも絡んでるなんて絶対お断りだわ");
			ui.ShowText("【萃香】\nプリズムリバーだっけ？　幽霊楽団の連中も引っ張られてくるから");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n何、死ぬわけじゃないんだからどーんと一回ぐらい死んでみようや");
			ui.ShowImage(Alice[3], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\n死んでるわよね！？　その言い方だと一回死ぬことになるよね！？");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n大丈夫大丈夫。半分ぐらい死んでもなんとかなるって\n私の古い知り合いにも腕をちょん切られても生きてる奴が居るから");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\nそれでまぁ、できればお断りしたいのだけど、やっぱり無理かしら？");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nまぁいっか。あんたが居なくても何も問題なく回るだろうし");
			ui.ShowText("【アリス】\n…ッ！？");
			ui.ShowText("【萃香】\n本人がそこまで嫌がってるんだったら私も無理にとは言わないさ。");
			ui.ShowText("【萃香】\nちなみに霊夢や魔理沙たちはどうやら生臭坊主と組むみたいだねぇ");
			ui.ShowText("【アリス】\nなっ……");
			ui.ShowText("【萃香】\nいつもいつも自分が頭数に入っていると思わないほうがいいんじゃないか？\nとくに霊夢は大抵の相手に対して平等なんだからね");
			ui.ShowImage(Alice[3], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\n……気が変わったわ。誘いに乗ってあげようじゃない");
			ui.ShowText("【萃香】\nそらきた");
			ui.ShowImage(Alice[1], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\nルールを教えなさい。十秒で覚えて貴方の鼻を明かせてあげるんだから");
			ui.ShowImage(Suika[0], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nはん。思った以上に直情的だね。");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nでも悪くはない。ここらで一つ、実戦練習といこうじゃないか");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true);
			ui.ShowText("【萃香】\nルールは単純、敵を倒し、蹂躙し、攻め滅ぼした方が勝ち\nさぁ、始めようか");


			var talkEnd = ui.ShowText( "", false );

			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk戦闘開始(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk1.5

			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Alice[0], StoryUIImagePosition.Front, true, false);
			ui.ShowText("【アリス】\nなるほど、兵の扱い方はだいたいわかったわ。");
			ui.ShowText("【アリス】\n私に与えられた兵の特徴としては、\"本拠と下級魔法陣から作業者含め全ての兵種を生産可能\"、\"しかし作業者と毛玉兵以外は解禁用の建物が必要\"、\"作業者は建物を建築させると消滅する\"");
			ui.ShowText("【アリス】\nそして個々の兵の能力は低めで、かつ私の使用できるスペルが人形召喚がメインで上級兵のスキルもサポート向きってことを考えると、\"質より量で勝負\"と言えるわね");
			ui.ShowImage(Alice[1], StoryUIImagePosition.Front, false, false);
			ui.ShowText("【アリス】\nじゃあ、さっさと片付けてしまいましょう？");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true);

			var talkEnd = ui.ShowText("", false);

			while (!ui.IsCompleted(talkEnd))
				yield return null;

			ui.Hide();
			ui.ResumeGame();

			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Suika[0], StoryUIImagePosition.Right, true, false);
			ui.ShowText("【萃香】\n案外やるじゃないか");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Left, true, false);
			ui.ShowText("【アリス】\nふん\n手を抜いてたことぐらいわかるんだから、そんな見え透いたお世辞は不要よ");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n何、これからしばらくの間同じ釜の飯を食うんだ\nつっけんどんにならないで仲良くやっていこうじゃないか");
			ui.ShowImage(Alice[2], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\n仲良くしようという気概が欠片も感じられなかったけど");
			ui.ShowText("【萃香】\n過去は過去、今は今さ");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\n小難しいことばかり考えていると魔法使いみたいになっちまうよ\nあいつらは屁理屈ばかりで好かないね");
			ui.ShowText("【アリス】\n私も魔法使いなんだけど");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Right, false, false);
			ui.ShowText("【萃香】\nまだ性根までは染まりきっていないと見える");
			ui.ShowText("【萃香】\nさ、あっちもスカウトを終えてる頃さ。行こうか、太陽の畑に");
			ui.ShowText("【アリス】\nはぁ……\n勢いで受けちゃったけど、ちょっと憂鬱だわね……");
			ui.ShowImage(Alice[0], StoryUIImagePosition.Left, false, false);
			ui.ShowText("【アリス】\nちょっと留守の張り紙をしておくからそこで待ってなさいな");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true);
			
			var talkEnd0 = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd0 ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Suika[1], StoryUIImagePosition.Front, true, false);
			ui.ShowText("【萃香】\nおや、こんなものかい？"); 
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}
		#endregion

		private int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W , string path_A = "")
		{
			return new int[4]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W ),
				ui.LoadImage( path_A )
			};
		}

		/// <summary>
		/// ユニットをエリア上に生成する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <param name="area"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> PutUnitsOnArea( IWorld world, IFaction faction, Type paramType, Area area )
		{
			var ar = area as AreaRectangle;
			if( ar == null ) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min( ar.Start.X, ar.End.X );
			var sy = Math.Min( ar.Start.Y, ar.End.Y );
			var ex = Math.Max( ar.Start.X, ar.End.X );
			var ey = Math.Max( ar.Start.Y, ar.End.Y );

			for( var y = sy; y <= ey; y++ )
			{
				for( var x = sx; x <= ex; x++ )
				{
					var u =
						world.UnitManager.Create( paramType.FullName, faction, ( x + 0.5f ) * thrts.Def.MapTipLengthOnField, ( y + 0.5f ) * thrts.Def.MapTipLengthOnField, faction );
					if( u != null ) units.Add( u );
				}
			}

			return units;
		}

		public IEnumerable<IUnit> PutEnemy( IWorld world, Type unitType, Area area, int num, IBuilding target )
		{
			var rect = area as AreaRectangle;
			if( rect == null ) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min( rect.Start.X, rect.End.X );
			var sy = Math.Min( rect.Start.Y, rect.End.Y );
			var ex = Math.Max( rect.Start.X, rect.End.X );
			var ey = Math.Max( rect.Start.Y, rect.End.Y );

			for( var y = sy; y < ey; y++ )
			{
				for( var x = sx; x < ex; x++ )
				{
					if( units.Count >= num )
						goto LoopEnd;

					var u = world.UnitManager.Create(
						unitType.FullName,
						world.Factions[1],
						( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
						( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
						world.Factions[1] );
					if( u != null )
						units.Add( u );
				}
			}
		LoopEnd:

			if( target != null )
				world.UnitManager.SelectTarget( units, new Vector2D( target.Pos.X, target.Pos.Y ), false );

			return units;
		}

		bool IsUnitOnArea( AreaRectangle region, IUnit unit )
		{
			var startx = Math.Min( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
			var starty = Math.Min( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
			var endx = Math.Max( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
			var endy = Math.Max( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
			if( unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y )
			{
				return true;
			}
			return false;
		}

		IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}
	}
}

#endif