﻿#if !_TRIAL

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryGreen03 : thrts.Mission
	{
		#region フィールド
		static readonly int[] FirstResoure = new List<int>{ 500, 200, 200 }.ToArray();
		static readonly float[] HPRate = new List<float>{ 0.5f, 1, 1.2f }.ToArray();

		IFaction greenFaction;
		IFaction redFaction;
		IFaction blueFaction;
		IBuilding greenHome;
		IBuilding redHome;
		IBuilding blueHome;
		Dictionary<string, Area> areas = new Dictionary<string, Area>();

		int[] Alice;
		int[] Suika;
		int[] Yuka;
		int[] Remilia;
		int[] Patchouli;
		int[] Tenshi;
		int[] Koishi;
		#endregion

		public StoryGreen03()
			: base()
		{
			Name = "緑3話";
			Description = "緑3話専用マップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		public override string[] GetEnableFactions()
		{
			return new[]{
				typeof( Faction.Green ).FullName,
				typeof( Faction.Blue ).FullName,
				typeof( Faction.Red ).FullName,
			};
		}
		public override string[] GetSelectableFactions()
		{
			return new[]{
				typeof(Faction.Green).FullName
			};
		}
		public override string[] GetSelectableMaps()
		{
			return new[]{
				"LotusCraft/Story/StoryGreen03.map"
			};
		}

		public override void OnInitialize( thrts.IWorld world )
		{
			world.Factions[1].PlayerName = "赤勢力";
			world.Factions[2].PlayerName = "青勢力";

			greenFaction = world.Factions[0];
			redFaction = world.Factions[1];
			blueFaction = world.Factions[2];
			world.Factions.ForEach( _ => _.DoesNotLose = true );

			greenHome = GetBuildings<Building.Green.HomeBase>( world ).First();
			redHome = GetBuildings<Building.Red.HomeBase>( world ).First();
			blueHome = GetBuildings<Building.Blue.HomeBase>( world ).First();
			greenFaction.Parameter.DefaultScreenPosition = To2D( greenHome.Pos ).ToPoint2D();

			world.AllPurposeArea.ForEach( _ => areas[_.Name] = _ );

			Alice = LoadTachies( world, "LotusCraft/Image/Tachie/", "Alice_N", "Alice_S", "Alice_W", "Alice_A" );
			Suika = LoadTachies( world, "LotusCraft/Image/Tachie/", "Suika_N", "Suika_S", "Suika_W" );
			Yuka = LoadTachies( world, "LotusCraft/Image/Tachie/", "Yuka_N", "Yuka_S", "Yuka_W" );
			Remilia = LoadTachies( world, "LotusCraft/Image/Tachie/", "Remilia_N", "Remilia_S", "Remilia_W" );
			Patchouli = LoadTachies( world, "LotusCraft/Image/Tachie/", "Patchouli_N", "Patchouli_S", "Patchouli_W" );
			Tenshi = LoadTachies( world, "LotusCraft/Image/Tachie/", "Tenshi_N", "Tenshi_S", "Tenshi_W" );
			Koishi = LoadTachies( world, "LotusCraft/Image/Tachie/", "Koishi_N", "Koishi_S", "Koishi_W" );

			SetDifficulty( world );
		}
		public override IEnumerable Update( thrts.IWorld world )
		{
			var red = new RedFactionTactics( this, world );
			var blue = new BlueFactionTactics( this, world );

			world.ChangeBGM( BGMList.Boushuen_Story2 );
			foreach( var item in Talk冒頭( world ) )
				yield return null;

			world.ChangeBGM( BGMList.Battle_Tenkai );
			foreach( var item in Talk戦闘開始( world ) )
				yield return null;

			

			world.StoryUI.ShowConditionText( "目的：自分の拠点を守り、赤勢力と青勢力の拠点を破壊する。" );

			while( true )
			{
				red.Update();
				blue.Update();

				if( !greenHome.Exists() )
				{
					foreach( var item in Talk敗北( world ) )
						yield return null;

					world.ChangeBGM( BGMList.Boushuen_Lose );
					greenFaction.DoesNotLose = false;
					greenFaction.Lose();
					break;
				}
				else if( !redHome.Exists() && !blueHome.Exists() )
				{
					foreach( var item in Talk勝利( world ) )
						yield return null;

					world.ChangeBGM( BGMList.Boushuen_Win );
					redFaction.DoesNotLose = false;
					blueFaction.DoesNotLose = false;
					redFaction.Lose();
					blueFaction.Lose();
					world.StoryUtilities.SetDataToSaveData( GetClearDataName( world ), "Clear" );
					world.StoryUtilities.SetDataToSaveData("LotusCraft.Green.03", "Clear");
					break;
				}

				yield return null;
			}

			yield return null;
		}
		private void SetDifficulty( IWorld world )
		{
			int difficulty = world.StoryUtilities.GetDifficulty();

			greenFaction.IncResource( 0, FirstResoure[difficulty] );
			greenFaction.IncResource( 1, FirstResoure[difficulty] );

			var redTypes = new[]{
				typeof( Unit.Red.Low01 ), typeof( Unit.Red.Low02 ),
				typeof( Unit.Red.Suwako ), typeof( Unit.Red.Flandre ),
				typeof( Unit.Red.Unique.Remilia ), typeof( Unit.Red.Unique.Patchouli )
			};
			var blueTypes = new[]{
				typeof( Unit.Blue.Low02 ), typeof( Unit.Blue.Low02 ),
				typeof( Unit.Blue.Sunny ), typeof( Unit.Blue.Lunar ), typeof( Unit.Blue.Star ),
				typeof( Unit.Blue.Unique.Tenshi ), typeof( Unit.Blue.Unique.Koishi )
			};
			redTypes.Select( _ => redFaction.GetUnitParameter( _.FullName ) )
				.Concat( blueTypes.Select( _ => blueFaction.GetUnitParameter( _.FullName ) ) )
				.ForEach( _ => _.HP = (int)( _.HP * HPRate[difficulty] ) );
		}
		private string GetClearDataName( IWorld world )
		{
			switch( world.StoryUtilities.GetDifficulty() )
			{
			case 0: return "LotusCraft.Green.03.Easy";
			case 1: return "LotusCraft.Green.03.Normal";
			case 2: return "LotusCraft.Green.03.Hard";
			default: throw new InvalidOperationException( "不正な難易度が設定されています。" );
			}
		}

		IEnumerable Talk冒頭( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk1
			ui.PauseGame();
			ui.Show();
			ui.ShowText( "～天界～" );
			ui.ShowImage( Yuka[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【アリス】\n結局幽香にとって、これに積極的に参加する理由はなんなの？　" );
			ui.ShowText( "【アリス】\n今では殆ど積極的に絡んでくることなんてなかったじゃない\nなのに今回はそうじゃない" );
			ui.ShowText( "【幽香】\n別にどうだっていいのよ。この戦い自体は" );
			ui.ShowText( "【アリス】\n？" );
			ui.ShowText( "【幽香】\n弾幕ゴッコより興味が出た。それで十分じゃない？" );
			ui.ShowImage( Suika[1], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【萃香】\nイベントごとには酒がついてくると昔から決まってるんだ。祭りは頭を空っぽにして楽しむものさ" );
			ui.ShowImage( Alice[3], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【アリス】\nそれにしても、あっちから指定してきたのに遅いわね" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Remilia[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【レミリア】\nなぜ八雲紫に呼ばれて来たのに、違うのがいるんだか" );
			ui.ShowText( "【レミリア】\n全く。わざわざ紅魔館から出てきたっていうのに" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, false, false, true );
			ui.ShowImage( Yuka[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【幽香】\nあら？\nどうして紅魔館のちびっこ様がいらっしゃるのかしら" );
			ui.ShowImage( Remilia[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【レミリア】\n口の聞き方に気をつけるんだな、お花大好きのファンシーさん" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Alice[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【アリス】\nねえパチュリー、あの二人って仲悪いの？" );
			ui.ShowImage( Patchouli[1], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【パチュリー】\n同族嫌悪でしょ\n殆ど面識ないはずだし" );
			ui.ShowImage( Patchouli[2], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【パチュリー】\nそれにしてもなんであなたたちがここに？\n八雲紫に天界に来るように言われたのだけど" );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【アリス】\nあなたたちも？\n同時に呼んだのかしら" );
			ui.ShowText( "【アリス】\nでも、何のために？" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【天子】\n集まっているようね" );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【天子】\n私に踏み台になるべくして集められた雑魚たちが" );
			ui.ShowImage( Koishi[1], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【こいし】\nわーい、悪役みたい\n死体を標本にして地霊殿のホールに飾ってあげる" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【天子】\nえ、そこまでやるの？" );
			ui.ShowImage( Koishi[0], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【こいし】\nお姉ちゃんへのプレゼントにするの" );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【天子】\nえ、お姉ちゃんそれで喜ぶの？" );
			ui.ShowImage( Koishi[1], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【こいし】\nやってみなきゃわからない" );
			ui.ShowImage( Tenshi[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【天子】\nとりあえず殺すのはなしの方向で" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【アリス】\n……要するに三つ巴でやれってことなのかしら？" );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【パチュリー】\n大方、そういうことでしょうね\n私は今回の提案を、単なる幻想郷の交流イベントであると割り切っているし" );
			ui.ShowImage( Alice[0], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【アリス】\nパチュリーもそう思う？" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Patchouli[1], StoryUIImagePosition.Front, true, false, true );
			ui.ShowText( "【パチュリー】\nスペルカードルールの提案も含めて、こういう試みは幾度となくされてきた。" );
			ui.ShowText( "【パチュリー】\nもちろん私は幻想郷では新参に類する身だけど、\n八雲紫が幻想郷の安定に心砕いているのは理解しているつもりよ" );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Front, false, false, true );
			ui.ShowText( "【パチュリー】\nこの狭い世界では、誰しもが隣人なんだもの" );
			ui.ShowImage( Patchouli[1], StoryUIImagePosition.Front, false, false, true );
			ui.ShowText( "【パチュリー】\n隣人の生活全てを知る必要はないけれど、挨拶ぐらいはできるようにしておかないと、\nいざ、本当に何かが起きたとき、手に負えなくなる" );
			ui.ShowText( "【パチュリー】\n調整機構である博麗の巫女も勿論、万能ではないから" );
			ui.ShowImage( Patchouli[2], StoryUIImagePosition.Front, false, false, true );
			ui.ShowText( "【パチュリー】\nなんせ、単独で幻想郷をめちゃくちゃに出来るのが、この場だけでも何名か居るというのがその証拠よ\nそいつらが無差別に暴れだしたら、博麗の巫女が何をするとか、関係ないもの" );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Front, false, false, true );
			ui.ShowText( "【パチュリー】\nだから、私たちはお互いを知らなければいけない。隣人が何を考えているのか" );
			ui.ShowText( "【パチュリー】\n見知らぬ脅威の対象ではなく、見知った仲であることが、安定へと繋がる" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
			ui.ShowImage( Alice[0], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【アリス】\nよく喋るのね" );
			ui.ShowImage( Patchouli[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【パチュリー】\n魔理沙は大雑把すぎるし、あなたは自己完結しすぎなの。" );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【パチュリー】\nレミィだって、あれはあれで楽しんでいるのよ、たぶん" );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【アリス】\n煽られて、一触即発状態だけど" );
			ui.ShowImage( Patchouli[2], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【パチュリー】\n……レミィは幻想郷から放逐したほうが良いのかしらね" );
			ui.ShowImage( Alice[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【アリス】\nさっきと言ってることが変わったわね" );
			ui.ShowImage( Patchouli[1], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【パチュリー】\nどっちにしろ、やりあわなければいけないことに変わりはないんだから、" );
			ui.ShowImage( Patchouli[0], StoryUIImagePosition.Right, false, false, true );
			ui.ShowText( "【パチュリー】\nそろそろ始まるでしょ。また、今度ね" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;

			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}
		IEnumerable Talk戦闘開始( IWorld world )
		{
			var ui = world.StoryUI;
			greenFaction.GetSite( 500, blueHome.Pos.X, blueHome.Pos.Y, 60 );
			greenFaction.GetSite( 500, redHome.Pos.X, redHome.Pos.Y, 60 );
			for( int i = 0; i < 30; i++ )
				yield return null;

			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Suika[0], StoryUIImagePosition.Right, true, false, true );
			var div0 = ui.ShowText( "【萃香】\nアリス、幽香がこっちに来いってさ。どうやら三つ巴で勝者を決めるんだと" );
			while( !ui.IsCompleted( div0 ) )
				yield return null;

			ui.ShowImage( Alice[0], StoryUIImagePosition.Left, true, false, true );
			var div1 = ui.ShowText( "【アリス】\n本人は姿を現さないつもりかしら？" );
			greenFaction.MoveScreenPosition( To2D( blueHome.Pos ) );
			while( !ui.IsCompleted( div1 ) )
				yield return null;

			ui.ShowImage( Suika[2], StoryUIImagePosition.Right, false, false, true );
			var div2 = ui.ShowText( "【萃香】\n紫は紫で、何か用事があるんだろう、きっと" );
			greenFaction.MoveScreenPosition( To2D( redHome.Pos ) );
			while( !ui.IsCompleted( div2 ) )
				yield return null;

			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();

			greenFaction.GetSite( 500, greenHome.Pos.X, greenHome.Pos.Y, 30 );
			greenFaction.MoveScreenPosition( To2D( greenHome.Pos ) );

			yield return null;
		}
		IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;
			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Yuka[0], StoryUIImagePosition.Front, true, false, true );
			ui.ShowText( "【幽香】\n大勝大勝" );
			ui.ShowImage( -1, StoryUIImagePosition.Front, true, false, true );
			ui.ShowImage( Tenshi[1], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【天子】\nぐわー、やーらーれーたー" );
			ui.ShowImage( Koishi[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【こいし】\nお姉ちゃんに死体を贈れなくなっちゃったよ……" );
			ui.ShowImage( Tenshi[2], StoryUIImagePosition.Left, false, false, true );
			ui.ShowText( "【天子】\nそれはむしろ良いことでしょうに" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			ui.ShowImage( Remilia[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【レミリア】\nふん、三つ巴で日の下で実力が測れるものではないわよ" );
			ui.ShowImage( Patchouli[2], StoryUIImagePosition.Left, true, false, true );
			ui.ShowText( "【パチュリー】\n……贔屓目に見ても負け犬の遠吠えにしか聞こえないわよ、レミィ" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}
		IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;
			ui.PauseGame();
			ui.Show();
			ui.ShowImage( Alice[2], StoryUIImagePosition.Front, true, false, true );
			ui.ShowText( "【アリス】\n惨敗惨敗" );
			ui.ShowImage( Yuka[2], StoryUIImagePosition.Right, true, false, true );
			ui.ShowText( "【幽香】\nあー、憂鬱だわ。やっぱり弾幕ごっこのほうがいいかしらね" );
			ui.ShowImage( -1, StoryUIImagePosition.Right, true, false, false );
			ui.ShowImage( -1, StoryUIImagePosition.Left, true, false, true );
			var talkEnd = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}

		int[] LoadTachies( IWorld world, string directory, string file_N, string file_S, string file_W, string file_A = null )
		{
			var ui = world.StoryUI;
			var list = new List<int>();

			if( !directory.EndsWith( "/" ) )
				directory += "/";

			list.Add( ui.LoadImage( directory + file_N ) );
			list.Add( ui.LoadImage( directory + file_S ) );
			list.Add( ui.LoadImage( directory + file_W ) );
			if( file_A != null )
				list.Add( ui.LoadImage( directory + file_A ) );
			return list.ToArray();
		}
		static IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}
		static Vector2D To2D( Vector3D vector )
		{
			return new Vector2D( vector.X, vector.Y );
		}

		#region Tactics
		abstract class FactionTactics
		{
			protected StoryGreen03 Owner;
			protected IWorld World;
			protected IFaction Faction;
			protected IBuilding Home;
			protected IEnumerator Fiber;

			protected FactionTactics( StoryGreen03 owner, IWorld world, IFaction faction, IBuilding home )
			{
				this.Owner = owner;
				this.World = world;
				this.Faction = faction;
				this.Home = home;
			}
			public void Update()
			{
				if( Home.Exists() && Fiber != null )
				{
					Fiber.MoveNext();
				}
			}

			#region Utilities
			protected IEnumerable<IUnit> PutUnit<T>( IFaction faction, Vector2D pos, int num )
			{
				List<IUnit> units = new List<IUnit>();

				// 一定回数で生成を断念
				for( int i = 0; i < 30; i++ )
				{
					var u = World.UnitManager.Create(
						typeof( T ).FullName,
						faction,
						( pos.X + 0.4f * i ) * thrts.Def.MapTipLengthOnField,
						( pos.Y + 0.4f * i ) * thrts.Def.MapTipLengthOnField,
						faction );
					if( u != null )
						units.Add( u );
					if( units.Count == num )
						break;
				}

				return units;
			}
			/// <summary>
			/// 
			/// </summary>
			/// <typeparam name="T"></typeparam>
			/// <param name="num"></param>
			/// <param name="factory">ユニットを生産する建物。指定したもののうち一つでも存在していれば成功します。</param>
			/// <returns></returns>
			protected IEnumerable<IUnit> PutUnit<T>( int num, params IBuilding[] factory ) where T : UnitParameter
			{
				foreach( var item in factory )
				{
					if( item.Exists() )
					{
						return PutUnit<T>( Faction, To2D( item.Pos ) / Def.MapTipLengthOnField, num );
					}
				}
				return Enumerable.Empty<IUnit>();
			}
			protected void AttackMove( IEnumerable<IUnit> units, ITargetObject target )
			{
				World.UnitManager.AttackMove( units.Select( _ => _.UniqueID ), target, true );
			}
			protected void AttackMove( IEnumerable<IUnit> units, Area area )
			{
				World.UnitManager.AttackMove(
					units.Select( _ => _.UniqueID ),
					GetRandomPos( area ) * Def.MapTipLengthOnField,
					true );
			}

			protected IEnumerable<IUnit> GetUnits<T>() where T : UnitParameter
			{
				return World.UnitManager.GetUnits( _ => _.Parameter.GetType() == typeof( T ) );
			}
			protected IEnumerable<IBuilding> GetBuildings<T>() where T : BuildingParameter
			{
				return World.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
			}
			protected Vector2D GetRandomPos( Area area )
			{
				var rect = area as AreaRectangle;
				var minX = Math.Min( rect.Start.X, rect.End.X );
				var maxX = Math.Max( rect.Start.X, rect.End.X );
				var minY = Math.Min( rect.Start.Y, rect.End.Y );
				var maxY = Math.Max( rect.Start.Y, rect.End.Y );
				return new Vector2D(
					(float)( minX + World.NextRandDouble() * ( maxX - minX ) ),
					(float)( minY + World.NextRandDouble() * ( maxY - minY ) ) );
			}
			protected static Vector2D To2D( Vector3D vector )
			{
				return new Vector2D( vector.X, vector.Y );
			}
			#endregion
		}
		class RedFactionTactics : FactionTactics
		{
			enum UnitKind
			{
				Low1, Low2, Suwako, Flandre
			}

			const int preparingTime = Def.FPS * 60 * 5;
			const int createSpan = (int)( Def.FPS * 60 * 1.3f );

			IBuilding barrack;
			IBuilding koumakan;
			IBuilding frog;
			Area[] guardLines;
			bool isTimeToRush
			{
				get
				{
					return Owner.blueHome.HP < Owner.blueHome.MaxHP - 600
						|| Home.HP < Home.MaxHP - 100
						|| !barrack.Exists() || !koumakan.Exists() || !frog.Exists();
				}
			}
			IBuilding blueOrGreenHome
			{
				get { return Owner.blueHome.Exists() ? Owner.blueHome : Owner.greenHome; }
			}

			public RedFactionTactics( StoryGreen03 owner, IWorld world )
				: base( owner, world, owner.redFaction, owner.redHome )
			{
				koumakan = GetBuildings<Building.Red.StairsOfKoumakan>().First();
				frog = GetBuildings<Building.Red.FrogSculpture>().First();
				barrack = GetBuildings<Building.Red.Barrack>().First();
				guardLines = new[]{
					Owner.areas["赤勢力防衛ライン１"],
					Owner.areas["赤勢力防衛ライン２"],
				};
				Fiber = PreparingState().GetEnumerator();
			}

			IEnumerable<IUnit> PutUnit( UnitKind kind, int num )
			{
				switch( kind )
				{
				case UnitKind.Low1: return PutUnit<Unit.Red.Low01>( num, barrack );
				case UnitKind.Low2: return PutUnit<Unit.Red.Low02>( num, barrack );
				case UnitKind.Suwako: return PutUnit<Unit.Red.Suwako>( num, frog );
				case UnitKind.Flandre: return PutUnit<Unit.Red.Flandre>( num, koumakan );
				default: return Enumerable.Empty<IUnit>();
				}
			}
			void Attack( IEnumerable<IUnit> units )
			{
				AttackMove( units, Owner.areas["赤勢力経由点"] );
				AttackMove( units, Owner.greenHome );
			}

			IEnumerable PreparingState()
			{
				List<IUnit> attacker = new List<IUnit>();

				GetUnits<Unit.Red.Unique.Remilia>().First().AddExperience( 3000 );
				GetUnits<Unit.Red.Unique.Patchouli>().First().AddExperience( 3000 );

				AttackMove( PutUnit( UnitKind.Low2, 3 ), guardLines[0] );
				AttackMove( PutUnit( UnitKind.Low2, 3 ), guardLines[1] );

				for( int i = 0; i < preparingTime; i++ )
				{
					if( i % createSpan == 0 )
					{
						attacker.AddRange( PutUnit( UnitKind.Low2, 1 ) );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[1] );
					}
					yield return null;
				}
				AttackMove( attacker, Owner.greenHome );
				Fiber = FightingState().GetEnumerator();
			}
			IEnumerable FightingState()
			{
				for( int i = 0; ; i++ )
				{
					for( int j = 0; j < createSpan; j++ )
						yield return null;

					if( isTimeToRush )
					{
						Fiber = RushState().GetEnumerator();
						break;
					}

					var units = new List<IUnit>();
					switch( i )
					{
					case 0:
					case 1:
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[1] );
						units.AddRange( PutUnit( UnitKind.Low1, 2 ) );
						break;
					case 2:
						units.AddRange( GetUnits<Unit.Red.Unique.Remilia>() );
						units.AddRange( PutUnit( UnitKind.Low1, 4 ) );
						break;
					case 3:
					case 4:
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[1] );
						units.AddRange( PutUnit( UnitKind.Low1, 3 ) );
						units.AddRange( PutUnit( UnitKind.Suwako, 1 ) );
						break;
					case 5:
					case 6:
						AttackMove( PutUnit( UnitKind.Low1, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low1, 1 ), guardLines[1] );
						units.AddRange( PutUnit( UnitKind.Flandre, 2 ) );
						units.AddRange( PutUnit( UnitKind.Suwako, 1 ) );
						break;
					default:
						units.AddRange( PutUnit( UnitKind.Flandre, 2 ) );
						units.AddRange( PutUnit( UnitKind.Suwako, 2 ) );
						break;
					}
					Attack( units );
				}
			}
			IEnumerable RushState()
			{
				{
					var remilia = GetUnits<Unit.Red.Unique.Remilia>().FirstOrDefault();
					var patchouli = GetUnits<Unit.Red.Unique.Patchouli>().FirstOrDefault();
					if( remilia != null )
						AttackMove( remilia.ToSequence(), Owner.greenHome );
					if( patchouli != null )
						AttackMove( patchouli.ToSequence(), blueOrGreenHome );
				}
				for( int i = 0; ; i++ )
				{
					if( i % createSpan == 0 )
					{
						AttackMove( PutUnit( UnitKind.Low2, 2 ), Owner.greenHome );
						AttackMove( PutUnit( UnitKind.Suwako, 2 ), Owner.greenHome );
						AttackMove( PutUnit( UnitKind.Flandre, 2 ), Owner.greenHome );
						AttackMove( PutUnit( UnitKind.Low2, 2 ), blueOrGreenHome );
						AttackMove( PutUnit( UnitKind.Suwako, 2 ), blueOrGreenHome );
						AttackMove( PutUnit( UnitKind.Flandre, 2 ), blueOrGreenHome );
					}
					yield return null;
				}
			}
		}
		class BlueFactionTactics : FactionTactics
		{
			enum UnitKind
			{
				Low2, Low3, Lunar, Sunny, Star
			}

			const int preparingTime = (int)( Def.FPS * 60 * 8 );
			const int createSpan = (int)( Def.FPS * 60 * 1.8f );

			IBuilding barrack;
			IBuilding[] tree;
			IBuilding redOrGreenHome
			{
				get { return Owner.redHome.Exists() ? Owner.redHome : Owner.greenHome; }
			}
			Area[] guardLines;
			bool isTimeToRush
			{
				get
				{
					return Owner.redHome.HP < Owner.redHome.MaxHP - 600
						|| Home.HP < Home.MaxHP - 100
						|| !barrack.Exists()
						|| tree.Any( _ => !_.Exists() );
				}
			}

			public BlueFactionTactics( StoryGreen03 owner, IWorld world )
				: base( owner, world, owner.blueFaction, owner.blueHome )
			{
				barrack = GetBuildings<Building.Blue.Barrack>().First();
				tree = GetBuildings<Building.Blue.BigTree>().ToArray();
				guardLines = new[]{
					Owner.areas["青勢力防衛ライン１"],
					Owner.areas["青勢力防衛ライン２"],
				};
				Fiber = PreparingState().GetEnumerator();
			}

			IEnumerable<IUnit> PutUnit( UnitKind kind, int num )
			{
				switch( kind )
				{
				case UnitKind.Low2: return PutUnit<Unit.Blue.Low02>( num, barrack );
				case UnitKind.Low3: return PutUnit<Unit.Blue.Low03>( num, barrack );
				case UnitKind.Sunny: return PutUnit<Unit.Blue.Sunny>( num, tree );
				case UnitKind.Lunar: return PutUnit<Unit.Blue.Lunar>( num, tree );
				case UnitKind.Star: return PutUnit<Unit.Blue.Star>( num, tree );
				default: return Enumerable.Empty<IUnit>();
				}
			}
			IEnumerable<IUnit> PutAndMove( UnitKind kind, int num, ITargetObject target, bool wandering = true )
			{
				var units = PutUnit( kind, num );
				if( wandering )
				{
					AttackMove( units, Owner.areas["青勢力経由点"] );
				}
				AttackMove( units, target );
				return units;
			}
			IEnumerable<IUnit> PutAndMove( UnitKind kind, int num, Area area )
			{
				var units = PutUnit( kind, num );
				AttackMove( units, area );
				return units;
			}

			IEnumerable PreparingState()
			{
				List<IUnit> attacker1 = new List<IUnit>();
				List<IUnit> attacker2 = new List<IUnit>();

				GetUnits<Unit.Blue.Unique.Tenshi>().First().AddExperience( 3000 );
				GetUnits<Unit.Blue.Unique.Koishi>().First().AddExperience( 3000 );

				AttackMove( PutUnit( UnitKind.Low2, 3 ), guardLines[0] );
				AttackMove( PutUnit( UnitKind.Low2, 3 ), guardLines[1] );

				for( int i = 0; i < preparingTime; i++ )
				{
					if( i % createSpan == 0 )
					{
						attacker1.AddRange( PutUnit( UnitKind.Low2, 1 ) );
						attacker2.AddRange( PutUnit( UnitKind.Low2, 1 ) );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[1] );
					}
					yield return null;
				}
				AttackMove( attacker1, Owner.greenHome );
				AttackMove( attacker2, Owner.redHome );
				Fiber = FightingState().GetEnumerator();
			}
			IEnumerable FightingState()
			{
				for( int i = 0; ; i++ )
				{
					for( int j = 0; j < createSpan; j++ )
						yield return null;

					if( isTimeToRush )
					{
						Fiber = RushState().GetEnumerator();
						break;
					}

					switch( i )
					{
					case 0:
					case 1:
						PutAndMove( UnitKind.Low2, 2, Owner.greenHome );
						PutAndMove( UnitKind.Low2, 2, redOrGreenHome );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Low2, 1 ), guardLines[1] );
						break;
					case 2:
					case 3:
						PutAndMove( UnitKind.Low2, 1, Owner.greenHome );
						PutAndMove( UnitKind.Low2, 1, redOrGreenHome );
						PutAndMove( UnitKind.Sunny, 1, Owner.greenHome );
						PutAndMove( UnitKind.Sunny, 1, redOrGreenHome );
						AttackMove( PutUnit( UnitKind.Lunar, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Star, 1 ), guardLines[1] );
						break;
					case 4:
						PutAndMove( UnitKind.Sunny, 3, Owner.greenHome );
						AttackMove( GetUnits<Unit.Blue.Unique.Tenshi>(), Owner.greenHome );
						PutAndMove( UnitKind.Sunny, 3, redOrGreenHome );
						AttackMove( GetUnits<Unit.Blue.Unique.Koishi>(), redOrGreenHome );
						break;
					case 5:
					case 6:
						PutAndMove( UnitKind.Sunny, 2, Owner.greenHome );
						PutAndMove( UnitKind.Sunny, 2, redOrGreenHome );
						AttackMove( PutUnit( UnitKind.Lunar, 1 ), guardLines[0] );
						AttackMove( PutUnit( UnitKind.Star, 1 ), guardLines[1] );
						break;
					default:
						PutAndMove( UnitKind.Sunny, 3, Owner.greenHome );
						PutAndMove( UnitKind.Sunny, 3, redOrGreenHome );
						break;
					}
				}
			}
			IEnumerable RushState()
			{
				{
					var tenshi = GetUnits<Unit.Blue.Unique.Tenshi>().FirstOrDefault();
					var koishi = GetUnits<Unit.Blue.Unique.Koishi>().FirstOrDefault();
					if( tenshi != null )
						AttackMove( tenshi.ToSequence(), Owner.greenHome );
					if( koishi != null )
					{
						AttackMove( koishi.ToSequence(), redOrGreenHome );
						AttackMove( koishi.ToSequence(), Owner.greenHome );
					}
				}
				for( int i = 0; i < preparingTime; i++ )
				{
					if( i % createSpan == 0 )
					{
						PutAndMove( UnitKind.Low2, 3, redOrGreenHome, false );
						PutAndMove( UnitKind.Sunny, 2, redOrGreenHome, false );
						PutAndMove( UnitKind.Star, 2, redOrGreenHome, false );
						PutAndMove( UnitKind.Lunar, 2, redOrGreenHome, false );
					}
					yield return null;
				}
			}
		}
		#endregion
	}
}

#endif