﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Story
{
	public class StoryRed02 : thrts.Mission
	{
		public override string[] GetSelectableFactions()
		{
			return new string[] {
				typeof( Faction.Red ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]	{
				typeof( Faction.Red ).FullName,
				typeof( Faction.Green ).FullName,
				typeof( Faction.Blue ).FullName
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryRed02.map",
			};
		}

		public StoryRed02()
			: base()
		{
			Name = "ストーリー赤02";
			Description = "赤2話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		int[] Aya = null;
		int[] Mokou = null;
		int[] Kasen = null;
		int[] Minoriko = null;
		int[] Tenshi = null;
		int[] Koishi = null;

		Area[] areas = null;
		IBuilding enemyLow = null;
		IBuilding blueHigh = null;

		IUnit mokouUnit = null;
		IUnit ayaUnit = null;
		IUnit kasenUnit = null;
		IUnit minorikoUnit = null;
		IUnit tenshiUnit = null;
		IUnit koishiUnit = null;

		int enemyspan = 1800;

		public override void OnInitialize( thrts.IWorld world )
		{
			world.Factions[1].PlayerName = "緑勢力";
			world.Factions[2].PlayerName = "青勢力";

			// 最初から負けないように
			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;
			world.Factions[2].DoesNotLose = true;

			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Green.Low02).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Green.Low03).FullName).HP /= 2;
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Iku).FullName).HP /= 2;
				world.Factions[2].GetUnitParameter(typeof(LotusCraft.Unit.Blue.Lunar).FullName).HP /= 2;
				world.Factions[1].GetUnitParameter(typeof(LotusCraft.Unit.Green.Komachi).FullName).HP /= 2;
			}
			if (world.StoryUtilities.GetDifficulty() == 1) {
				enemyspan = 1300;
			}

			var mokou = GetUnits(world, world.Factions[0], typeof(LotusCraft.Unit.Red.Unique.Moko)).First();

			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D((int)(mokou.Pos.X), (int)(mokou.Pos.Y));
			base.OnInitialize( world );
		}

		public override System.Collections.IEnumerable Update( IWorld world )
		{
			var ui = world.StoryUI;

			enemyLow = GetBuildings<Building.Green.Barrack>(world).Where(_ => _.Faction == world.Factions[1]).First();
			blueHigh = GetBuildings<Building.Blue.Factory>(world).Where(_ => _.Faction == world.Factions[2]).First();

			Aya = LoadTachies(ui, "LotusCraft/Image/Tachie/Aya_N", "LotusCraft/Image/Tachie/Aya_S", "LotusCraft/Image/Tachie/Aya_W");
			Mokou = LoadTachies(ui, "LotusCraft/Image/Tachie/Mokou_N", "LotusCraft/Image/Tachie/Mokou_S", "LotusCraft/Image/Tachie/Mokou_W");
			Kasen = LoadTachies(ui, "LotusCraft/Image/Tachie/Kasen_N", "LotusCraft/Image/Tachie/Kasen_S", "LotusCraft/Image/Tachie/Kasen_W");
			Minoriko = LoadTachies(ui, "LotusCraft/Image/Tachie/Minoriko_N", "LotusCraft/Image/Tachie/Minoriko_S", "LotusCraft/Image/Tachie/Minoriko_W");
			Tenshi = LoadTachies(ui, "LotusCraft/Image/Tachie/Tenshi_N", "LotusCraft/Image/Tachie/Tenshi_S", "LotusCraft/Image/Tachie/Tenshi_W");
			Koishi = LoadTachies(ui, "LotusCraft/Image/Tachie/Koishi_N", "LotusCraft/Image/Tachie/Koishi_S", "LotusCraft/Image/Tachie/Koishi_W");

			mokouUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Moko)).First();
			mokouUnit.AddExperience(3000);
			ayaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Red.Unique.Aya)).First();
			ayaUnit.AddExperience(3000);

			kasenUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Kasen)).First();
			minorikoUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Minoriko)).First();
			tenshiUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Blue.Unique.Tenshi)).First();
			koishiUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Blue.Unique.Koishi)).First();
			
			// 難易度調整
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				kasenUnit.AddExperience(3000);
				minorikoUnit.AddExperience(1500);
				tenshiUnit.AddExperience(2000);
				koishiUnit.AddExperience(2000);
			}
			else if (world.StoryUtilities.GetDifficulty() == 1)
			{
				kasenUnit.AddExperience(5000);
				minorikoUnit.AddExperience(3000);
				tenshiUnit.AddExperience(4000);
				koishiUnit.AddExperience(4000);
			}
			else if (world.StoryUtilities.GetDifficulty() == 2)
			{
				kasenUnit.AddExperience(8000);
				minorikoUnit.AddExperience(6000);
				tenshiUnit.AddExperience(6000);
				koishiUnit.AddExperience(6000);
			}

			world.ChangeBGM(BGMList.Boushuen_Story1);

			//*
			foreach (var item in Talk会話冒頭(world))
			{
				yield return null;
			}
			//*/

			world.ChangeBGM(BGMList.Battle_Hakurei);

			ui.ShowConditionText("目的：華扇の撃破");
			
			
			int i = 0;
			while (true)
			{
				if (!kasenUnit.Exists())
				{
					foreach (var item2 in Talk勝利(world))
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();
					world.Factions[2].DoesNotLose = false;
					world.Factions[2].Lose();
					
					world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.02", "Clear");
					if (world.StoryUtilities.GetDifficulty() == 0)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.02.Easy", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 1)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.02.Normal", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 2)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.02.Hard", "Clear");
					}
					world.ChangeBGM(BGMList.Boushuen_Win);

					break;
				}
				if (!ayaUnit.Exists() && !mokouUnit.Exists())
				{
					foreach (var item2 in Talk敗北(world))
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();
					world.Factions[2].DoesNotLose = false;
					world.Factions[2].Lose();

					world.ChangeBGM(BGMList.Boushuen_Lose);

					break;
				}

				kasenUnit.DoFindEnemyAutomatically = false;
				kasenUnit.HoldPosition(true);
				if (minorikoUnit.Exists()) {
					minorikoUnit.HoldPosition(true);
				}

				// ざこ
				if (i % enemyspan == 0)
				{
					if (i < enemyspan * 2)
					{
						// 初期配置の敵に攻撃命令
						{
							var uniqueIDs = new List<int>();
							var lunars = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Blue.Lunar)).Take(2);
							foreach (var l in lunars)
							{
								uniqueIDs.Add(l.UniqueID);
							}
							world.UnitManager.AttackMove(uniqueIDs, ayaUnit, false);
						}

						{
							var uniqueIDs = new List<int>();
							var ikus = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Blue.Iku));
							foreach (var l in ikus)
							{
								uniqueIDs.Add(l.UniqueID);
							}
							if (minorikoUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, minorikoUnit, false);
							}
						}
					}
					if (i == enemyspan * 6)
					{
						// 青ユニークに攻撃命令
						{
							var uniqueIDs = new List<int>();
							uniqueIDs.Add(tenshiUnit.UniqueID);

							if (mokouUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
							}
							else {
								world.UnitManager.AttackMove(uniqueIDs, kasenUnit, false);
							}
						}

						{
							var uniqueIDs = new List<int>();
							uniqueIDs.Add(koishiUnit.UniqueID);
							if (minorikoUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, minorikoUnit, false);
							}
							else if(mokouUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
							}
						}
					}
					if (i < enemyspan * 2)
					{
						if (enemyLow.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low03).FullName, enemyLow);
							uniqueIDs.Add(l1.UniqueID);
							if (mokouUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
							}
						}
					}
					else
					{
						if (enemyLow.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low02).FullName, enemyLow);
							uniqueIDs.Add(l1.UniqueID);
							var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Komachi).FullName, enemyLow);
							uniqueIDs.Add(l3.UniqueID);
							if (mokouUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
							}
							else if (ayaUnit.Exists()) {
								world.UnitManager.AttackMove(uniqueIDs, ayaUnit, false);
							}

							uniqueIDs = new List<int>();
							var h1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low03).FullName, enemyLow);
							uniqueIDs.Add(l1.UniqueID);
							if (tenshiUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, tenshiUnit, false);
							}
							else if (blueHigh.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, blueHigh, false);
							}
						}

						if (blueHigh.Exists())
						{
							List<int> uniqueIDs = new List<int>();
							var h1 = Utl.PutUnitAroundBuilding(world, world.Factions[2], typeof(LotusCraft.Unit.Blue.Iku).FullName, blueHigh);
							uniqueIDs.Add(h1.UniqueID);
							if (minorikoUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, minorikoUnit, false);
							}
							else if (enemyLow.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, enemyLow, false);
							}
							else {
								if (mokouUnit.Exists())
								{
									world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
								}
								else if (ayaUnit.Exists())
								{
									world.UnitManager.AttackMove(uniqueIDs, ayaUnit, false);
								}
							}

							uniqueIDs = new List<int>();
							var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[2], typeof(LotusCraft.Unit.Blue.Lunar).FullName, blueHigh);
							uniqueIDs.Add(l1.UniqueID);

							if (ayaUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, ayaUnit, false);
							}
							else if (mokouUnit.Exists())
							{
								world.UnitManager.AttackMove(uniqueIDs, mokouUnit, false);
							}
							else {
								if (minorikoUnit.Exists())
								{
									world.UnitManager.AttackMove(uniqueIDs, minorikoUnit, false);
								}
								else if (enemyLow.Exists())
								{
									world.UnitManager.AttackMove(uniqueIDs, enemyLow, false);
								}
							}
							
						}
					}
				}
				

				
				
				i++;
				yield return null;
			}



			yield return base.Update(world);
		}

		#region Story
		private System.Collections.IEnumerable Talk会話冒頭( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk1
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowText("～博麗神社～");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n霊夢さんたちの勢力が気になります");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【妹紅】\nまぁ、博麗の巫女を抜きにして幻想郷を語るのは難しいからね");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nてっきり一緒に組んでいると思っていたアリスさんや萃香さんがああやって別に居るのならば、一体誰と組んでいるのかなと");
				ui.ShowText("【妹紅】\nこっちに引き込めばいいんじゃない？");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nうんにゃ、敵として戦いたいと雇用主の希望で");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\n難儀なもんね");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nプライドの問題なんだそうですよ。わからないでもないですが");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\n周囲も大変そうだなぁ\nやんごとなき方々の世話は大変そうだ、輝夜とか");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nはてさて、博麗神社に来てみましたがどうもお留守のようで\n端っこまで来るのはそこそこ面倒なんですが");
				ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\n空振りになったらなんでもそんなものね");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\nしかし、この間に本丸に攻め込まれている可能性は？");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nあー。無理やり押し入ろうとするなんてやるの、氷精ぐらいでしょうよ");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\nあー");
				ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nでしょう");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nしかし一度紅魔館に戻っておきましょうか");
				ui.ShowText("【文】\n妹紅さんを引き込めたことも報告したいですし");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【妹紅】\nできれば丁寧に紹介してくれるとありがたいね");
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowText("【？？？】\nそうはさせないわ！");
				ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\n誰だ！");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n鳥だ！　飛行機だ！");
				ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nいや！");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Minoriko[1], StoryUIImagePosition.Front, true, false);
				ui.ShowText("【穣子】\n豊穣神様だ！");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\n解散");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\nお疲れ様でした");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Front, false, false);
				ui.ShowText("【穣子】\nWhat the f○ck!");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\n書けないような言葉を使うのやめてくれませんかー");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nていうか焼き芋がこんなところで何をしているの？　山に帰りなさい");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【穣子】\n聞いてくださいよ");
				ui.ShowImage(Aya[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\nさあ妹紅さん紅魔館が待っていますよ");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nああ、なんだか無性にそうしたい気分なんだ");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【穣子】\n聞けよ！");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\nじゃあ五行ぐらいまでなら待ちます");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nあと三行");
				ui.ShowImage(Minoriko[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【穣子】\n実はかくかくしかじかで");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n成程わからん");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nさあ行くかー");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【穣子】\nどうかお慈悲を！");
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowText("～事情説明中～");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Front, true);
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\nつまり自分のところでも台詞すらないというわけですね");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nビッグ・ボンバーズみたいなものかしら");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【穣子】\nうう……自分たちだけで話を進めて行くんですもん");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【妹紅】\nしかし、なぜ博麗神社に？");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n霊夢さんでも買収するつもりでしたか？\n買収したところで何かが変わるとは思えませんが");
				ui.ShowImage(Minoriko[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【穣子】\n飛び出して来たら来たで行く当てもなくて、神社に行けば誰かに会えるかなと");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\n適当だなぁ");
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowText("【？？？】\nやっと見つけましたよ！");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【穣子】\nひっ！？");
				ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【華扇】\n自分の扱いが少し悪いからと逃げ出す有様。\nそれでも神に連なる者ですか");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\nあ、某裁判長様の近縁の方だ");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\n知り合い？");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n歩く説教マシーンです");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nなんだ慧音か。");
				ui.ShowImage(Mokou[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【妹紅】\nおい慧音、なんで髪の毛がピンクだし包帯なんか手に巻いてるんだ？");
				ui.ShowImage(Kasen[2], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【華扇】\n変なキャラ付けをするなと！");
				ui.ShowImage(Mokou[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【妹紅】\nいや私には見えるぞ\nお前みたいな奴はどうせ弄られるんだ、私にゃわかるね");
				ui.ShowImage(Minoriko[1], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【穣子】\n私のほうへようこそ");
				ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\nしかしこれで幽香さんたちの陣容もわかりましたね\n思わぬ収穫です");
				ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【華扇】\n待ちなさい。\n人を人とも思わぬその所業、野放しにする道理はありませんよ");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nあらら、私は事を構えるつもりはないんですが");
				ui.ShowImage(Mokou[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【妹紅】\nあちらさんがやる気なら受けないわけにもいかないからね");
				ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【華扇】\n少しお灸を据えさせていただきますよ\nいきますよ、穣子さん");
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【穣子】\nえっ、私も？");
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);

				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
					yield return null;

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			world.Factions[0].GetSite(500, (int)(kasenUnit.Pos.X), (int)(kasenUnit.Pos.Y), 6000);
			world.ForceToUpdateVision();
			world.Factions[0].MoveScreenPosition(new Vector2D((int)kasenUnit.Pos.X, (int)kasenUnit.Pos.Y));

			#region Talk2
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Minoriko[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【穣子】\nあのー、なぜこんなことに");
				ui.ShowImage(Kasen[1], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【華扇】\n黙らっしゃい！　出番がほしいんでしょう！？");
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);

				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
					yield return null;

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			world.Factions[0].MoveScreenPosition(new Vector2D((int)mokouUnit.Pos.X, (int)mokouUnit.Pos.Y));

			#region Talk3
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, true, false);
				ui.ShowText("【文】\n適当にあしらって満足してもらいましょう、陣地も作らないで今ある兵で\nここで長々と戦っても益はありませんし");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false);
				ui.ShowText("【妹紅】\n私は雇用主の方針に従うまでさ");
				ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nやる気満々なのが一人だけですからね\n説教好きの仙人にはさっさとご退場いただきましょう");
				ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, false, false);
				ui.ShowText("【妹紅】\n将を射んとせば先ず馬を射よ、と言うけれど？");
				ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false);
				ui.ShowText("【文】\nそんなじっくりしてる暇あればいいのですが…ほらあそこ");
				ui.ShowImage(-1, StoryUIImagePosition.Left, true);
				ui.ShowImage(-1, StoryUIImagePosition.Right, true);

				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
					yield return null;

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			world.Factions[0].GetSite(500, (int)(tenshiUnit.Pos.X), (int)(tenshiUnit.Pos.Y), 6000);
			world.ForceToUpdateVision();
			world.Factions[0].MoveScreenPosition(new Vector2D((int)tenshiUnit.Pos.X, (int)tenshiUnit.Pos.Y));

			#region Talk4
			{
				ui.PauseGame();
				ui.Show();
				ui.ShowImage(Tenshi[1],StoryUIImagePosition.Left,true,false);
				ui.ShowText("【天子】\n楽しそうなことをやってるじゃない\n巫女はいないけど荒らさせて貰うわ");
				ui.ShowImage(Koishi[1],StoryUIImagePosition.Right,true,false);
				ui.ShowText("【こいし】\nここに居るのは皆地上の妖怪かしら？\nあ、人間モドキも混ざってる？\n剥製にして飾ったら、お姉ちゃんは喜ぶかしら");
				ui.ShowImage(-1,StoryUIImagePosition.Left,true);
				ui.ShowImage(-1,StoryUIImagePosition.Right,true);
				ui.ShowImage(Mokou[2],StoryUIImagePosition.Right,true,false);
				ui.ShowText("【妹紅】\nうわぁ……");
				ui.ShowImage(Aya[2],StoryUIImagePosition.Left,true,false);
				ui.ShowText("【文】\n思わぬ闖入者ですね\nお互いに食い合ってくれれば楽なんですが");
				ui.ShowImage(-1,StoryUIImagePosition.Right,true);
				ui.ShowImage(-1,StoryUIImagePosition.Left,true);
				ui.ShowText("", false);


				var talkEnd = ui.ShowText("", false);

				while (!ui.IsCompleted(talkEnd))
					yield return null;

				ui.Hide();
				ui.ResumeGame();
			}
			#endregion

			world.Factions[0].MoveScreenPosition(new Vector2D((int)mokouUnit.Pos.X, (int)mokouUnit.Pos.Y));

			yield return null;
		}

		private System.Collections.IEnumerable Talk勝利( IWorld world )
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Kasen[2],StoryUIImagePosition.Left,false,false);
			ui.ShowText("【華仙】\nふぎゃ！");
			ui.ShowImage(-1,StoryUIImagePosition.Left,true);
			ui.ShowImage(Minoriko[2],StoryUIImagePosition.Right,true,false);
			ui.ShowText("【穣子】\nえ？あ、ちょ、ちょっと！私ひとりでどうしろっていうの！？");
			ui.ShowImage(-1,StoryUIImagePosition.Right,true);
			ui.ShowImage(Aya[0],StoryUIImagePosition.Left,true,false);
			ui.ShowText("【文】\n退きましょう。これ以上は時間の無駄です");
			ui.ShowImage(Mokou[2],StoryUIImagePosition.Right,true,false);
			ui.ShowText("【妹紅】\n了解。うまく撒けるといいんだがね");
			ui.ShowImage(Aya[1],StoryUIImagePosition.Left,false,false);
			ui.ShowText("【文】\n本気になった天狗に追いつけるとでも？　捕まってください。飛ばしますよ");
			ui.ShowImage(-1,StoryUIImagePosition.Front,true);
			ui.ShowImage(-1,StoryUIImagePosition.Right,true);
			ui.ShowImage(-1,StoryUIImagePosition.Left,true);
			ui.ShowImage(ui.LoadImage("LotusCraft/Image/Back/Black"), StoryUIImagePosition.Front, true);
			ui.ShowImage(ui.LoadImage("LotusCraft/Image/Back/Black"), StoryUIImagePosition.Back2, true);
			ui.ShowImage(-1,StoryUIImagePosition.Front, true);
			ui.ShowText("\n～紅魔館～");
			ui.ShowImage(Aya[2],StoryUIImagePosition.Left,false,false);
			ui.ShowText("【文】\n一時はどうなることかと思いました");
			ui.ShowImage(Mokou[0],StoryUIImagePosition.Right,false,false);
			ui.ShowText("【妹紅】\n私は面識がない連中だけど、顔見知り？");
			ui.ShowImage(Aya[0],StoryUIImagePosition.Left,false,false);
			ui.ShowText("【文】\n悟り妖怪の妹に、不良天人ですよ");
			ui.ShowImage(Aya[0],StoryUIImagePosition.Left,false,false);
			ui.ShowText("【文】\nどこが飼っているかはわからないけれど、無軌道で厄介なのは間違いないでしょうね\nもしかすると、御しきれていないのかもしれません");
			ui.ShowImage(Mokou[1],StoryUIImagePosition.Right,false,false);
			ui.ShowText("【妹紅】\nいずれにしても、戦わなければいけない時がくるさ。おお怖い怖い");
			
			var talkEnd0 = ui.ShowText( "", false );
			while( !ui.IsCompleted( talkEnd0 ) )
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北( IWorld world )
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}
		#endregion

		private int[] LoadTachies( IStoryUI ui, string path_N, string path_S, string path_W)
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		protected IEnumerable<IUnit> PutEnemy<TUnit>(Area area, int num, IWorld World) where TUnit : UnitParameter
		{
			var rect = area as AreaRectangle;
			if (rect == null)
				return new IUnit[0];

			List<IUnit> units = new List<IUnit>();
			var type = typeof(TUnit);

			var sx = Math.Min(rect.Start.X, rect.End.X);
			var sy = Math.Min(rect.Start.Y, rect.End.Y);
			var ex = Math.Max(rect.Start.X, rect.End.X);
			var ey = Math.Max(rect.Start.Y, rect.End.Y);

			for (var y = sy; y < ey; y++)
			{
				for (var x = sx; x < ex; x++)
				{
					if (units.Count >= num)
						goto LoopEnd;

					var u = World.UnitManager.Create(
						type.FullName,
						World.Factions[1],
						(x + 0.5f) * thrts.Def.MapTipLengthOnField,
						(y + 0.5f) * thrts.Def.MapTipLengthOnField,
						World.Factions[1]);
					if (u != null)
						units.Add(u);
				}
			}
		LoopEnd:

			return units;
		}

		/// <summary>
		/// ユニットを取得する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> GetUnits(IWorld world, IFaction faction, Type paramType)
		{
			return world.UnitManager.GetUnits(_ => _.Faction == faction && _.Parameter.GetType() == paramType);
		}

		/// <summary>
		/// ユニットをエリア上に生成する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <param name="area"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> PutUnitsOnArea( IWorld world, IFaction faction, Type paramType, Area area )
		{
			var ar = area as AreaRectangle;
			if( ar == null ) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min( ar.Start.X, ar.End.X );
			var sy = Math.Min( ar.Start.Y, ar.End.Y );
			var ex = Math.Max( ar.Start.X, ar.End.X );
			var ey = Math.Max( ar.Start.Y, ar.End.Y );

			for( var y = sy; y <= ey; y++ )
			{
				for( var x = sx; x <= ex; x++ )
				{
					var u =
						world.UnitManager.Create( paramType.FullName, faction, ( x + 0.5f ) * thrts.Def.MapTipLengthOnField, ( y + 0.5f ) * thrts.Def.MapTipLengthOnField, faction );
					if( u != null ) units.Add( u );
				}
			}

			return units;
		}

		public IEnumerable<IUnit> PutEnemy( IWorld world, Type unitType, Area area, int num, IBuilding target )
		{
			var rect = area as AreaRectangle;
			if( rect == null ) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min( rect.Start.X, rect.End.X );
			var sy = Math.Min( rect.Start.Y, rect.End.Y );
			var ex = Math.Max( rect.Start.X, rect.End.X );
			var ey = Math.Max( rect.Start.Y, rect.End.Y );

			for( var y = sy; y < ey; y++ )
			{
				for( var x = sx; x < ex; x++ )
				{
					if( units.Count >= num )
						goto LoopEnd;

					var u = world.UnitManager.Create(
						unitType.FullName,
						world.Factions[1],
						( x + 0.5f ) * thrts.Def.MapTipLengthOnField,
						( y + 0.5f ) * thrts.Def.MapTipLengthOnField,
						world.Factions[1] );
					if( u != null )
						units.Add( u );
				}
			}
		LoopEnd:

			if( target != null )
				world.UnitManager.SelectTarget( units, new Vector2D( target.Pos.X, target.Pos.Y ), false );

			return units;
		}

		bool IsUnitOnArea( AreaRectangle region, IUnit unit )
		{
			var startx = Math.Min( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
			var starty = Math.Min( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
			var endx = Math.Max( region.Start.X, region.End.X ) * Def.MapTipLengthOnField;
			var endy = Math.Max( region.Start.Y, region.End.Y ) * Def.MapTipLengthOnField;
			if( unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y )
			{
				return true;
			}
			return false;
		}

		IEnumerable<IBuilding> GetBuildings<T>( IWorld world ) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings( _ => _.Parameter.GetType() == typeof( T ) );
		}
	}
}

#endif