﻿#if !_TRIAL

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;

namespace LotusCraft.Story
{
	public class StoryRed04 : thrts.Mission
	{
		public override string[] GetSelectableFactions()
		{
			return new string[] {
				typeof( Faction.Red ).FullName
			};
		}

		public override string[] GetEnableFactions()
		{
			return new string[]	{
				typeof( Faction.Red ).FullName,
				typeof( Faction.Green ).FullName,
				typeof( Faction.Blue ).FullName,
			};
		}

		public override string[] GetSelectableMaps()
		{
			return new string[] {
				"LotusCraft/Story/StoryRed04.map",
			};
		}

		public StoryRed04()
			: base()
		{
			Name = "ストーリー赤04";
			Description = "赤4話のマップ";
			Mode = thrts.MissionMode.Story;

			this.TheNumberOfMaximumSelectablePlayersInTeam = 0;
			this.TheNumberOfSelectableTeams = 0;
		}

		int[] Remilia = null;
		int[] Sanae = null;
		int[] Mokou = null;
		int[] Miko = null;
		int[] Aya = null;
		int[] Patchouli = null;
		int[] Sakuya = null;
		int[] Yuka = null;
		int[] Suika = null;
		int[] Lyrica = null;

		int Back1 = 0;
		Area[] areas = null;
		IBuilding redHome = null;
		IBuilding greenHome = null;
		IBuilding blackHome = null;
		IBuilding greenLow = null;

		const int enemySpan = 1800;

		/// <summary>
		/// スペルを使用されたかどうか
		/// </summary>
		bool executedMikoSpall = false;
		bool executedSuikaSpall = false;
		bool executedLylicaSpall = false;

		public override void OnInitialize(thrts.IWorld world)
		{
			// 最初から負けないように
			world.Factions[0].DoesNotLose = true;
			world.Factions[1].DoesNotLose = true;
			world.Factions[2].DoesNotLose = true;

			// 初期資源を配布
			world.Factions[0].IncResource(0, 200);
			world.Factions[0].IncResource(1, 200);

			// 本拠の位置を初期視点にする
			var home = world.BuildingManager.GetBuildings(_ => _.Parameter.GetType() == typeof(LotusCraft.Building.Red.HomeBase)).First();
			var basePos = home.Pos;

			// 初期視点位置
			world.Factions[0].Parameter.DefaultScreenPosition = new Point2D((int)basePos.X, (int)basePos.Y);

			world.Factions[1].PlayerName = "緑勢力";
			world.Factions[2].PlayerName = "黒勢力";

			base.OnInitialize(world);
		}

		public override System.Collections.IEnumerable Update(IWorld world)
		{
			var ui = world.StoryUI;
			
			redHome = GetBuildings<Building.Red.HomeBase>(world).First();
			greenHome = GetBuildings<Building.Green.HomeBase>(world).First();
			blackHome = GetBuildings<Building.Black.HomeBase>(world).First();
			greenLow = GetBuildings<Building.Green.Barrack>(world).First();

			Remilia = LoadTachies(ui, "LotusCraft/Image/Tachie/Remilia_N", "LotusCraft/Image/Tachie/Remilia_S", "LotusCraft/Image/Tachie/Remilia_W");
			Mokou = LoadTachies(ui, "LotusCraft/Image/Tachie/Mokou_N", "LotusCraft/Image/Tachie/Mokou_S", "LotusCraft/Image/Tachie/Mokou_W");
			Sanae = LoadTachies(ui, "LotusCraft/Image/Tachie/Sanae_N", "LotusCraft/Image/Tachie/Sanae_S", "LotusCraft/Image/Tachie/Sanae_W");
			Miko = LoadTachies(ui, "LotusCraft/Image/Tachie/Miko_N", "LotusCraft/Image/Tachie/Miko_S", "LotusCraft/Image/Tachie/Miko_W");
			Aya = LoadTachies(ui, "LotusCraft/Image/Tachie/Aya_N", "LotusCraft/Image/Tachie/Aya_S", "LotusCraft/Image/Tachie/Aya_W");
			Patchouli = LoadTachies(ui, "LotusCraft/Image/Tachie/Patchouli_N", "LotusCraft/Image/Tachie/Patchouli_S", "LotusCraft/Image/Tachie/Patchouli_W");
			Sakuya = LoadTachies(ui, "LotusCraft/Image/Tachie/Sakuya_N", "LotusCraft/Image/Tachie/Sakuya_S", "LotusCraft/Image/Tachie/Sakuya_W");

			Yuka = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Yuka_N"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_S"), ui.LoadImage("LotusCraft/Image/Tachie/Yuka_W") };
			Suika = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Suika_N"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_S"), ui.LoadImage("LotusCraft/Image/Tachie/Suika_W") };
			Lyrica = new int[3] { ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_N"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_S"), ui.LoadImage("LotusCraft/Image/Tachie/Lyrica_W") };

			Back1 = ui.LoadImage("LotusCraft/Image/Back/Black");

			int experience = 0;
			if (world.StoryUtilities.GetDifficulty() == 0)
			{
				experience = 3000;
			}
			else if (world.StoryUtilities.GetDifficulty() == 1)
			{
				experience = 5000;
			}
			else if (world.StoryUtilities.GetDifficulty() > 1)
			{
				experience = 7000;
			}


			var lyricaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Lyrica)).First();
			lyricaUnit.AddExperience(experience);

			var suikaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Suika)).First();
			suikaUnit.AddExperience(experience);

			var yukaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Green.Unique.Yuka)).First();
			yukaUnit.AddExperience(experience);

			var yuyukoUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Black.Unique.Yuyuko)).First();
			yuyukoUnit.AddExperience(experience);

			var seigaUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Black.Unique.Seiga)).First();
			seigaUnit.AddExperience(experience);

			var mikoUnit = world.UnitManager.GetUnits(_ => _.Parameter.GetType() == typeof(Unit.Black.Unique.Miko)).First();
			mikoUnit.AddExperience(experience);


			world.ChangeBGM(BGMList.Boushuen_Story3);

			//*
			foreach (var item in Talk会話冒頭(world))
			{
				yield return null;
			}
			//*/

			world.ChangeBGM(BGMList.Battle_Kouma);

			ui.ShowConditionText("目的：敵拠点の撃破");

			int c = 0;
			
			while (true)
			{
				// 勝利条件
				if (!greenHome.Exists() && !blackHome.Exists())
				{
					world.ChangeBGM(BGMList.Boushuen_Win);

					foreach (var item2 in Talk勝利(world))
					{
						yield return null;
					}
					world.Factions[1].DoesNotLose = false;
					world.Factions[1].Lose();
					world.Factions[2].DoesNotLose = false;
					world.Factions[2].Lose();

					world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.04", "Clear");

					if (world.StoryUtilities.GetDifficulty() == 0)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.04.Easy", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 1)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.04.Normal", "Clear");
					}
					else if (world.StoryUtilities.GetDifficulty() == 2)
					{
						world.StoryUtilities.SetDataToSaveData("LotusCraft.Red.04.Hard", "Clear");
					}
					break;
				}

				// 敗北条件
				if (!redHome.Exists())
				{
					foreach (var item2 in Talk敗北(world))
					{
						yield return null;
					}
					world.Factions[0].DoesNotLose = false;
					world.Factions[0].Lose();
					break;
				}

				// スペル発動
				if (mikoUnit.Exists() && !executedMikoSpall)
				{
					TryCallMikoSpell(world, mikoUnit);
				}

				if (suikaUnit.Exists() && !executedSuikaSpall)
				{
					TryCallSuikaSpell(world, suikaUnit);
				}
				if (lyricaUnit.Exists() && !executedLylicaSpall)
				{
					TryCallLylicaSpell(world, lyricaUnit);
				}

				// 萃香突撃
				if (c > 0 && c % (thrts.Def.FPS * 60 * 5) == 0)
				{
					if (suikaUnit.Exists())
					{
						var uniqueIDs = new List<int>();
						uniqueIDs.Add(suikaUnit.UniqueID);

						world.UnitManager.AttackMove(uniqueIDs, redHome, false);
					}
				}

				// 幽々子突撃
				if (c > 0 && c % (thrts.Def.FPS * 60 * 10) == 0)
				{
					if (yuyukoUnit.Exists())
					{
						var uniqueIDs = new List<int>();
						uniqueIDs.Add(yuyukoUnit.UniqueID);

						world.UnitManager.AttackMove(uniqueIDs, redHome, false);
					}
				}

				// ざこ
				if (c % enemySpan == 0)
				{
					if (c >= enemySpan * 5)
					{
						if (c == enemySpan * 5)
						{
							if (greenLow.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l2.UniqueID);
								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l4.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}

							if (greenHome.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l2.UniqueID);

								if (world.StoryUtilities.GetDifficulty() > 1)
								{
									var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
									var l5 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l6 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
								}

								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}
						}
						else if (c == enemySpan * 8)
						{
							if (greenLow.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l2.UniqueID);

								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l4.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}

							if (greenHome.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l2.UniqueID);

								if (world.StoryUtilities.GetDifficulty() > 1)
								{
									var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
									var l5 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l6 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
								}

								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}
						}
						else
						{
							if (greenLow.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l2.UniqueID);
								var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l3.UniqueID);
								var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenLow);
								uniqueIDs.Add(l4.UniqueID);
								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}

							if (greenHome.Exists())
							{
								List<int> uniqueIDs = new List<int>();
								var l1 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l1.UniqueID);
								var l2 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
								uniqueIDs.Add(l2.UniqueID);

								if (world.StoryUtilities.GetDifficulty() > 1)
								{
									var l3 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l4 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
									var l5 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l3.UniqueID);
									var l6 = Utl.PutUnitAroundBuilding(world, world.Factions[1], typeof(LotusCraft.Unit.Green.Low01).FullName, greenHome);
									uniqueIDs.Add(l4.UniqueID);
								}

								world.UnitManager.AttackMove(uniqueIDs, redHome, false);
							}
						}
					}
				}

				c++;
				yield return null;
			}
			
			yield return base.Update(world);
		}

		private void TryCallMikoSpell(IWorld world, IUnit mikoUnit)
		{
			if (executedMikoSpall) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - mikoUnit.Pos.X) * (u.Pos.X - mikoUnit.Pos.X) + (u.Pos.Y - mikoUnit.Pos.Y) * (u.Pos.Y - mikoUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[2].GetUnitSpellButtonParameter(mikoUnit.Parameter.GetType().FullName, new SpellButtonKey("神光「逆らう事なきを宗とせよ」", 1));
					spell.SelectPlaceButtonEvent(mikoUnit, new Vector2D(u.Pos.X, u.Pos.Y), false);
					executedMikoSpall = true;
				}
			}
		}

		private void TryCallSuikaSpell(IWorld world, IUnit suikaUnit)
		{
			if (executedSuikaSpall) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - suikaUnit.Pos.X) * (u.Pos.X - suikaUnit.Pos.X) + (u.Pos.Y - suikaUnit.Pos.Y) * (u.Pos.Y - suikaUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[1].GetUnitSpellButtonParameter(suikaUnit.Parameter.GetType().FullName, new SpellButtonKey("鬼神「ミッシングパープルパワー」", 1));
					spell.NormalButtonEvent(suikaUnit, false);
					executedSuikaSpall = true;
				}
			}
		}

		private void TryCallLylicaSpell(IWorld world, IUnit lyUnit)
		{
			if (executedLylicaSpall || !lyUnit.Exists()) return;

			var units = world.UnitManager.GetUnits(_ => _.Faction == world.Factions[0]);
			foreach (var u in units)
			{
				var radius = 6 * thrts.Def.MapTipLengthOnField;

				if (radius * radius > (u.Pos.X - lyUnit.Pos.X) * (u.Pos.X - lyUnit.Pos.X) + (u.Pos.Y - lyUnit.Pos.Y) * (u.Pos.Y - lyUnit.Pos.Y))
				{
					var targetPos = u.Pos;
					var spell = world.Factions[1].GetUnitSpellButtonParameter(lyUnit.Parameter.GetType().FullName, new SpellButtonKey("騒符「リリカ・ソロライブ」", 1));
					spell.NormalButtonEvent(lyUnit, false);
					executedLylicaSpall = true;
				}
			}
		}


		#region Story
		private System.Collections.IEnumerable Talk会話冒頭(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk1
			ui.PauseGame();
			ui.Show();
			ui.ShowText("紅魔館");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【文】\n思った以上に穏やかですね。全体的に");
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n仕事が少ないのは何よりだ。面倒じゃないし");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n例えばですよ\nばばーんと黒幕が登場すれば、私だって新聞に大見出しで記事が書けますが、");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nぶっちゃけ今の規模じゃ三面記事に載る程度ですよ");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n平和でいいな");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nだからといって、妖怪の山で石油が算出されたからそれの取り合いガーだの、\n天狗の子供が地底に迷い込んでからの全面戦争だのと、そういうのを望んでいるわけではないんですが");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\nそもそも、そんな戦争になる前にきっちり手打ちするだろ\nそもそも、こんなところで野暮を考えるのなんていないさ");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\n個々人の小競り合いは毎日のように起きていますけどね");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【妹紅】\n誰のことを言いたいんだ？");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【文】\nさあ");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【早苗】\nでーばーんー！\n出番が必要なんですよ！");
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【早苗】\nこの私には！");
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【早苗】\nわからないんですか！");
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【早苗】\nこの東風谷早苗が力を貸しているというのに！");
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Front, false, false, true);
			ui.ShowText("【早苗】\nやったことと言えばちまちまとお茶を濁してきた程度。こんなはずでは！");
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Patchouli[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【パチュリー】\n基本的にはお前が来いだから、レミィは。\n私だって図書館から出たくないわ");
			ui.ShowImage(Patchouli[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【パチュリー】\nそもそも一日咲夜が出払ったらうちがどれだけ荒廃すると思ってるのよ");
			ui.ShowImage(Sakuya[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【咲夜】\nもちろん、お嬢様が行くのであればついていきますが");
			ui.ShowImage(Remilia[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\n出向くんだったらせいぜい博麗神社までね。それ以上は面倒だわ");
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【早苗】\nもっと見敵滅殺していかないと！　");
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\nああもう！\n自分から売り込まなくても一定の需要が見込める層はこれだから！\nその余裕が命取りなんですよ！？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【文】\n早苗さんの言うことにも一理ありますよ\nせっかく雇ってもらったのに、規模の小さい戦いしかしていないのも癪ですし");
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\nそうそう！　血沸き肉踊るような戦いを欲しているのですよ！");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【文】\nああでも、私は取材ができればそれでいいのでやっぱり積極的に行かなくても良い気が\nモケーレ・ムベンベゴッコでもしていれば暇は潰れますし");
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\n馬鹿なっ");
			ui.ShowImage(Remilia[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\n意外と出不精なんだよ、うちは。それにちょくちょくお客様には来てもらっているし");
			ui.ShowText("【早苗】\nそうですよ霊夢さんたちを撃滅しましょう。なんだったら私が呼んできますから！");
			ui.ShowText("【レミリア】\nでもなー");
			ui.ShowText("【早苗】\nでももメケーモもないですから！　今すぐ連れてきますから準備しといてください！");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\nで、その結果が");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, true);
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【幽香】\n来ちゃった♪");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\n面白そうな催しがあると聞いて");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\n霊夢さんは面倒らしいです");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n余計な連中ばっかり連れてきたのな");
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\nぶっ倒せれば誰だっていいんです");
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\n頭痛くなってきた");
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\n風邪ですか？");
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【レミリア】\n風邪だったらいいんだけど、\nこう、自分の出番のためなら手段を選ばない緑色の奴のせいで心因性の頭痛を患ったらしく");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\n永琳の奴が居れば良かったのにな");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\n飲むか？");
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\n後で、後で");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Aya[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【文】\n(いつからうちの代表が早苗さんになったんだろう)");
			ui.ShowImage(Yuka[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【幽香】\nこっちが勝ったら宴会奢ってくれるって約束してくれたのよ");
			ui.ShowImage(Miko[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nというわけで、手は抜きませんよ");
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\nお前何約束してきてんの！？");
			ui.ShowImage(Sanae[0], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【早苗】\n勝てば……問題ない……！");
			ui.ShowImage(Remilia[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【レミリア】\n万が一負けたらあんたのところの神様に仔細報告しとくから、覚悟しときなさい");
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【早苗】\n勝てば官軍負ければ賊軍。上等じゃないですか！");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【妹紅】\nその謎の自信はどこから来るんだろうか");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("", false);


			var talkEnd = ui.ShowText(""); 
			while (!ui.IsCompleted(talkEnd))
				yield return null;
			
			ui.Hide();
			ui.ResumeGame();
			#endregion


			yield return null;
		}

	

		private System.Collections.IEnumerable Talk勝利(IWorld world)
		{
			var ui = world.StoryUI;

			#region Talk12
			ui.PauseGame();
			ui.Show();
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【早苗】\n大勝利！");
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\nあーはいはい。お疲れ様でした");
			ui.ShowImage(Sanae[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【早苗】\nこれは明日の新聞の一面は決まりですね。\n東風谷早苗、平成のジャンヌダルク。劣勢を跳ね返し勝利を導く");
			ui.ShowImage(Aya[0], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【文】\nどう考えてもマッチポンプで劣勢に導いていましたけどね");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【萃香】\nじゃあ飲むか");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【文】\n前後が全く繋がっていないんですけど");
			ui.ShowImage(Suika[2], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\nせっかくこの人数が集まっているんだ\n飲む以外の選択肢があるとでも？　");
			ui.ShowImage(Suika[0], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\nなあ？　天狗");
			ui.ShowImage(Aya[1], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【文】\n私はちょっと、\n一身上の都合で冠婚葬祭が一挙にやってきてしまった感じなので帰らせていただけないかなーとか");
			ui.ShowImage(Suika[1], StoryUIImagePosition.Left, false, false, true);
			ui.ShowText("【萃香】\n飲むよな？");
			ui.ShowImage(Aya[2], StoryUIImagePosition.Right, false, false, true);
			ui.ShowText("【文】\nハイ");
			ui.ShowImage(Mokou[2], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【妹紅】\nお前、鬼が苦手なのか？");
			ui.ShowText("【文】\nイエ、ソンナコトハ……");
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(Remilia[2], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\nこいつらに何言っても無駄か……\n咲夜、すまないが、お客様をもてなすように");
			ui.ShowImage(Sakuya[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【咲夜】\n畏まりました");
			ui.ShowImage(Miko[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【神子】\nいやぁ悪いですね。押しかけて宴席まで設けて頂くだなんて");
			ui.ShowImage(Remilia[1], StoryUIImagePosition.Right, true, false, true);
			ui.ShowText("【レミリア】\n勝っても負けてもこうなるって決まってたのよ。");
			ui.ShowImage(Lyrica[1], StoryUIImagePosition.Left, true, false, true);
			ui.ShowText("【レミリア】\nチンドン屋も居るんだから、気の利いた曲でも演奏しなさい");
			ui.ShowImage(Back1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Right, true, false, false);
			ui.ShowImage(-1, StoryUIImagePosition.Left, true, false, true);
			ui.ShowImage(-1, StoryUIImagePosition.Front, true, false, true);
			ui.ShowImage(Mokou[0], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("妹紅は思う。わざわざ自分が竹林から引っ張り出された理由について\n戦力として自分を欲しがったという理屈は最もらしい理由ではあったけれど、\nそれが建前に過ぎないのは明らかで。");
			ui.ShowText("要するに、一人ぼっちなのを気遣った誰かさんが、遊びましょうとわざわざ誘ってくれたのだろう。");
			ui.ShowText("そのお節介焼きがどこの誰なのか考えるのは、とりあえず横に置くことにした。");
			ui.ShowImage(Sanae[2], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("【早苗】\nどうして私が主人公じゃないんですか！");
			ui.ShowImage(Mokou[1], StoryUIImagePosition.Front, true, false, true);
			ui.ShowText("虚しい叫びは聞き流して、自分も、騒がしい輪の内へ加わることにした。");
			ui.ShowText("", false);

			var talkEnd0 = ui.ShowText("", false);
			while (!ui.IsCompleted(talkEnd0))
				yield return null;
			ui.Hide();
			ui.ResumeGame();
			#endregion

			yield return null;
		}

		private System.Collections.IEnumerable Talk敗北(IWorld world)
		{
			var ui = world.StoryUI;

			ui.PauseGame();
			ui.Show();
			ui.Hide();
			ui.ResumeGame();

			yield return null;
		}
		#endregion

		private int[] LoadTachies(IStoryUI ui, string path_N, string path_S, string path_W)
		{
			return new int[3]{
				ui.LoadImage( path_N ),
				ui.LoadImage( path_S ),
				ui.LoadImage( path_W )
			};
		}

		/// <summary>
		/// ユニットをエリア上に生成する。
		/// </summary>
		/// <param name="world"></param>
		/// <param name="faction"></param>
		/// <param name="paramType"></param>
		/// <param name="area"></param>
		/// <returns></returns>
		public IEnumerable<IUnit> PutUnitsOnArea(IWorld world, IFaction faction, Type paramType, Area area)
		{
			var ar = area as AreaRectangle;
			if (ar == null) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min(ar.Start.X, ar.End.X);
			var sy = Math.Min(ar.Start.Y, ar.End.Y);
			var ex = Math.Max(ar.Start.X, ar.End.X);
			var ey = Math.Max(ar.Start.Y, ar.End.Y);

			for (var y = sy; y <= ey; y++)
			{
				for (var x = sx; x <= ex; x++)
				{
					var u =
						world.UnitManager.Create(paramType.FullName, faction, (x + 0.5f) * thrts.Def.MapTipLengthOnField, (y + 0.5f) * thrts.Def.MapTipLengthOnField, faction);
					if (u != null) units.Add(u);
				}
			}

			return units;
		}

		public IEnumerable<IUnit> PutEnemy(IWorld world, Type unitType, Area area, int num, IBuilding target)
		{
			var rect = area as AreaRectangle;
			if (rect == null) return new IUnit[0];

			List<IUnit> units = new List<IUnit>();

			var sx = Math.Min(rect.Start.X, rect.End.X);
			var sy = Math.Min(rect.Start.Y, rect.End.Y);
			var ex = Math.Max(rect.Start.X, rect.End.X);
			var ey = Math.Max(rect.Start.Y, rect.End.Y);

			for (var y = sy; y < ey; y++)
			{
				for (var x = sx; x < ex; x++)
				{
					if (units.Count >= num)
						goto LoopEnd;

					var u = world.UnitManager.Create(
						unitType.FullName,
						world.Factions[1],
						(x + 0.5f) * thrts.Def.MapTipLengthOnField,
						(y + 0.5f) * thrts.Def.MapTipLengthOnField,
						world.Factions[1]);
					if (u != null)
						units.Add(u);
				}
			}
		LoopEnd:

			if (target != null)
				world.UnitManager.SelectTarget(units, new Vector2D(target.Pos.X, target.Pos.Y), false);

			return units;
		}

		bool IsUnitOnArea(AreaRectangle region, IUnit unit)
		{
			var startx = Math.Min(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var starty = Math.Min(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			var endx = Math.Max(region.Start.X, region.End.X) * Def.MapTipLengthOnField;
			var endy = Math.Max(region.Start.Y, region.End.Y) * Def.MapTipLengthOnField;
			if (unit.Pos.X < endx && startx < unit.Pos.X && unit.Pos.Y < endy && starty < unit.Pos.Y)
			{
				return true;
			}
			return false;
		}

		IEnumerable<IBuilding> GetBuildings<T>(IWorld world) where T : BuildingParameter
		{
			return world.BuildingManager.GetBuildings(_ => _.Parameter.GetType() == typeof(T));
		}
	}
}

#endif