﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Black;
using LotusCraft.Building.Black;
using LotusCraft.Effect.Black;

namespace LotusCraft.Unit.Black
{
	public class BombSpirit  : General.Special
	{
		public BombSpirit ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Black/BombSpirit";
#endregion
			
			#region UnitParams
			Name = "怨霊";
			HP = 60;
			Radius = (float)Utls.UnitSize.Small;
			Speed = 1.6f*BasisSpeed;
			
			PopulationWeight = 0;
			Exp = 0;
			IsSummon = true;
			CanState = false;

			MakingTime = 25 * Def.FPS;
			ConcurrentProductions = 2;
			Costs = new []{0,0,0,0};
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};
#endregion
			
			#region Animation
			#region Animation
			this.SetBoushuenSizeAnimation("LotusCraft/Image/Unit/Black/BombSpirit");
			Animation.Width = 32;
			Animation.Height = 64;
			Animation.Count.X = 30;
			Animation.Count.Y = 1;
			Animation.FrameTime = 2;
			Animation.FrameCount = 30;
			Animation.HasDirection = false;
			AnimationWalk.Width = 32;
			AnimationWalk.Height = 64;
			AnimationWalk.Count.X = 30;
			AnimationWalk.Count.Y = 1;
			AnimationWalk.FrameTime = 1;
			AnimationWalk.FrameCount = 30;
			AnimationWalk.HasDirection = false;
#endregion
#endregion
			
			#region BuildableObjcets
#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(BombSpiritBullet).FullName };
#endregion
			
			#region Requires
#endregion
		}
	}
}

namespace LotusCraft.Bullet.Black
{
	public class BombSpiritBullet : General.CannonBullet
	{
		public BombSpiritBullet()
		{
			Type = BulletType.ShortRange;
			SpaceType = BulletSpaceType.Earth;
			
			HasSplash = true;
			SplashDamageRates.Add(0.4f*Def.MapTipLengthOnField,1.0f);
			SplashDamageRates.Add(0.7f*Def.MapTipLengthOnField,0.8f);
			SplashDamageRates.Add(1.2f*Def.MapTipLengthOnField,0.5f);
			SplashDamageRates.Add(1.8f*Def.MapTipLengthOnField,0.25f);
			this.Attack = 80;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			
			MaxRange = 0.5f * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(1.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			OnHitSplash = OnHitSplashEffect;
			
		}
		static void OnHitEffect (IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			var offset = new Vector3D (1, 2, 8);
			var effect = faction.World.EffectManager.CreateEffect (typeof(NyumaAreaEffect).FullName, target.Pos + offset);
			var scale = new Vector3D (8, 8, 8);
			effect.SetScale (scale);
			faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.Bom01).FullName, target.Pos);

			// change damage if hit farmer
			{
				var u = target as IUnit;
				if(null != u && u.Parameter.IsFarmer)
				{
					// 農民に当たった時は50%ダメージ
					bullet.Attack /= 2;
				}
			}

			// suicide unit
			if(bullet.Generator is IUnit)
				(bullet.Generator as IUnit).HP = 0;
			
		}

		int OnHitSplashEffect (IBullet bullet, ITargetObject target, int damage)
		{
			var u = target as IUnit;
			if (null != u && u.Parameter.IsFarmer) {
				damage /= 2;
			}
			return damage;
		}

	}
}
