﻿ using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Spell.Black;

namespace LotusCraft.Unit.Black
{
	public class Kokoro : General.Special
	{
		public Kokoro()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Black/Kokoro";
			#endregion

			#region UnitParams
			Name = "秦こころ(式神)";
			HP = 300;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 1.0f * BasisSpeed;
			UnitType = new TargetObjectType[]{TargetObjectType.Clay,};

			ViewRange = 12 * Def.MapTipLengthOnField;

			MakingTime = 32 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {125,125,0,0};

			PopulationWeight = 2;
			
			MoveType |= UnitMoveType.Air;

			CanState = false;

			 #endregion

			#region Description
			ButtonDescription += "飛行ユニット\n様々なタイプに切り替わることができる";

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Black/Kokoro");
			#endregion

			RequireBuildings = new string[]{
				typeof(Building.Black.Airport).FullName,
			};
			
			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(KokoroBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{ 
			#region Spells
			SpellParams = new SpellParameter<IUnit>[]
			{
				new KokoroKurumaSkill(),
				new KokoroShishiSkill(),
				new KokoroKirouSkill(),
			};
			#endregion
			#region AutoCasts
			AutocasterParams = null;
			#endregion
			#region Buttons
			ButtonParams = new ButtonParameterHolder<IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[1].DefaultSpellButton,SpellParams[1].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[2].DefaultSpellButton,SpellParams[2].SpellButtons),
			};
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Black
{
	public class KokoroBullet : General.LongRangeBullet
	{
		public KokoroBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = true;
			this.Attack = 18;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0f;
			MaxRange = 6 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(1.2f * Def.FPS);

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 1;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Spell.Black
{
	class KokoroKirouSkill : KokoroSkill
	{
		const float decRange = 2;
		const int incAtk = 6;
		const int incDef = 2;
		public KokoroKirouSkill()
			:base(ButtonPosition._2,ButtonPosition._2,2)
		{
			string basePath = "Black/Spell/KokoroSkill/Kirou/";
			this.Description = "射程が減少し、攻撃力と防御力が上昇する\n更に攻撃が必中になる";
			/// レベル別の能力説明
			this.EffectDescriptions = new string[]
			{
				"射程-"+decRange+"、防御力+"+incDef+"、攻撃力+"+incAtk,
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
			};
		}
		
		public override IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			foreach (var n in spellNames) {
				var spell = selfUnit.GetSpellWithName (n);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}
			// stop dec cooltime counter
			{
				var spell = selfUnit.GetSpellWithName (spellNames[2]);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}

			
			var world = selfUnit.Faction.World;
			
				{
					/// show effect
					if (selfUnit.AbilityManager.ContainsAbility (KokoroKirouSkillAbility .AbilityName)) {
						selfUnit.AbilityManager.ExtendLifetimeWithName (duration, KokoroKirouSkillAbility .AbilityName, duration);
					} else {
						var offset = new Vector3D (2, 2, 2);
						var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.KokoroKirouEffect ).FullName, selfUnit.Pos + offset);
						var scale = 8;
						effect.SetScale (new Vector3D (scale, scale, scale));
						var ability = new KokoroKirouSkillAbility (decRange,incAtk,incDef,effect, offset, duration);
						if (!selfUnit.AbilityManager.AppendAbility (ability))
							effect.Stop ();
					}
			}
			
			yield return false;
		}
	}
	class KokoroShishiSkill : KokoroSkill
	{
		const float incRange = 0;
		public KokoroShishiSkill()
			:base(ButtonPosition._1,ButtonPosition._2,1)
		{
			string basePath = "Black/Spell/KokoroSkill/Shishi/";
			this.Description = "対空を得る。";
			/// レベル別の能力説明
			this.EffectDescriptions = new string[]
			{
				"対地対空",
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
			};
		}
		
		public override IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			/// reset cooltime
			foreach (var n in spellNames) {
				var spell = selfUnit.GetSpellWithName (n);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}
			// stop dec cooltime counter
			{
				var spell = selfUnit.GetSpellWithName (spellNames[1]);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}

			var world = selfUnit.Faction.World;
			
				
				{
					/// show effect
					if (selfUnit.AbilityManager.ContainsAbility (KokoroShishiSkillAbility .AbilityName)) {
						selfUnit.AbilityManager.ExtendLifetimeWithName (duration, KokoroShishiSkillAbility .AbilityName, duration);
					} else {
						var offset = new Vector3D (2, 2, 2);
						var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.KokoroShishiEffect ).FullName, selfUnit.Pos + offset);
						var scale = 8;
						effect.SetScale (new Vector3D (scale, scale, scale));
						var ability = new KokoroShishiSkillAbility (incRange,effect, offset, duration);
						if (!selfUnit.AbilityManager.AppendAbility (ability))
							effect.Stop ();
					}
				}

			yield return false;
		}
	}
	class KokoroKurumaSkill : KokoroSkill
	{
		public KokoroKurumaSkill()
			:base(ButtonPosition._0,ButtonPosition._2,0)
		{
			string basePath = "Black/Spell/KokoroSkill/Kuruma/";

			this.Description = "攻撃が出来なくなる代わり、移動速度と視界が上昇する。またHPが毎秒2回復する。";
			/// レベル別の能力説明
			this.EffectDescriptions = new string[]
			{
				"攻撃不可、移動速度+0.3、視界+4",
			};
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
				new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
			};
		}
		
		public override IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			/// reset cooltime
			foreach (var n in spellNames) {
				var spell = selfUnit.GetSpellWithName (n);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}
			// stop dec cooltime counter
			{
				var spell = selfUnit.GetSpellWithName (spellNames[0]);
				if (null != spell) {
					spell.ResetCoolTime ();
					spell.BeDecCooltime = true;
				}
			}

			var world = selfUnit.Faction.World;
			
			
			{
				/// show effect
				if (selfUnit.AbilityManager.ContainsAbility (KokoroSkillAbility .AbilityName)) {
					selfUnit.AbilityManager.ExtendLifetimeWithName (duration, KokoroSkillAbility .AbilityName, duration);
				} else {
					var offset = new Vector3D (2, 2, 2);
					var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.KokoroKurumaEffect ).FullName, selfUnit.Pos + offset);
					var scale = 8;
					effect.SetScale (new Vector3D (scale, scale, scale));
					var ability = new KokoroSkillAbility (effect, offset, duration);
					if (!selfUnit.AbilityManager.AppendAbility (ability))
						effect.Stop ();
				}
			}
			
			yield return false;
		}
	}

	class KokoroSkill:thrts.SpellParameter<thrts.IUnit>
	{
		protected const int duration = 15*Def.FPS;
		protected string[] spellNames = new [] {"憂符「憂き世は憂しの小車」","喜符「昂揚の神楽獅子」","怒符「怒れる忌狼の面」",};
		public KokoroSkill (ButtonPosition x, ButtonPosition y, int id)
		{

			DefaultLevel = 1;
			MaxLevel = 1;
			if(id >= 0 && id < spellNames.Length)
				Name = spellNames [id];
			else
				Name = spellNames [0];

			string basePath = "Black/Spell/KokoroSkill/";

			if (id == 0) {
				basePath = "Black/Spell/KokoroSkill/Kuruma/";
			} else if (id == 1) {
				basePath = "Black/Spell/KokoroSkill/Shishi/";
			} else if (id == 2) {
				basePath = "Black/Spell/KokoroSkill/Kirou/";
			}
//			string basePath = unitParam.IconPath;

			CoolTimes = new int[MaxLevel + 1];
			CoolTimes [0] = 99 * Def.FPS;
			CoolTimes [1] = 15 * Def.FPS;

			Description = "攻撃が出来なくなる代わり、移動速度と視界が上昇する。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"攻撃不可、移動速度+0.3、視界+4",
			};

			/// ボタンの位置
			ButtonPosX = x;
			ButtonPosY = y;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;

			if(id < 0 || id >= spellNames.Length)
			{
				/// スペルのボタンを生成
				SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
				{
					new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "0", 0),
					new LotusCraft.Button.SpellButton<thrts.IUnit> (this, basePath + "1", PrivilegeModeSpelling, 1),
				};
			}
		}

		public virtual IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			foreach(var n in spellNames)
				selfUnit.GetSpellWithName (n).ResetCoolTime ();
			
			var world = selfUnit.Faction.World;

				{
					/// show effect
					if (selfUnit.AbilityManager.ContainsAbility (KokoroSkillAbility .AbilityName)) {
						selfUnit.AbilityManager.ExtendLifetimeWithName (duration, KokoroSkillAbility .AbilityName, duration);
					} else {
						var offset = new Vector3D (2, 2, 2);
						var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.KokoroKurumaEffect ).FullName, selfUnit.Pos + offset);
						var scale = selfUnit.ViewRange * 5;
						effect.SetScale (new Vector3D (scale, scale, scale));
						var ability = new KokoroSkillAbility (effect, offset, duration);
						if (!selfUnit.AbilityManager.AppendAbility (ability))
							effect.Stop ();
					}
				}

			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class KokoroKirouSkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "怒符「怒れる忌狼の面」";
		public float DecRange = 0;
		public int IncAtk = 0;
		public int IncArmor = 0;
		public KokoroKirouSkillAbility(float decRange,int incAtk,int incDef,IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
			this.DecRange = decRange;
			this.IncAtk = incAtk;
			this.IncArmor = incDef;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.Attack += IncAtk;
					b.SpaceType = BulletSpaceType.Earth;
					b.Type = BulletType.ShortRange;
					b.MaxRange -= DecRange*Def.MapTipLengthOnField;
				}

				unit.Armor += IncArmor;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		
	}
	class KokoroShishiSkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "喜符「昂揚の神楽獅子」";
		public float IncRange = 0;
		public KokoroShishiSkillAbility(float incRange,IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
			this.IncRange = incRange;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				foreach(var b in unit.ModifiedBulletParameters)
				{
					b.SpaceType = BulletSpaceType.Both;
					b.MaxRange += IncRange*Def.MapTipLengthOnField;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
		
	}
	
	class KokoroSkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "憂符「憂き世は憂しの小車」";
		int counter = 0;
		int HealDelay = Def.FPS;
		public KokoroSkillAbility(IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.SelectBullet = (u,t,d) => -1;

				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.ViewRange += 4*Def.MapTipLengthOnField;
				unit.Speed += 0.3f*UnitParameter.BasisSpeed;
				//回復
				counter++;
				if (counter > HealDelay && unit.HP < unit.MaxHP)
				{
					counter = 0;
					unit.HP += 2;
				}
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
}