﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.Black;
using LotusCraft.Building.Black;
using LotusCraft.Bullet.Black;

namespace LotusCraft.Unit.Black
{
	public class Low02 : General.Water
	{
		public Low02 ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Black/Low02";
#endregion
			
			#region UnitParams
			Name = "陰陽玉(水)";
			ButtonDescription += "射撃ユニット\n対空攻撃と建築を行う事が出来る。";
			HP = 125;
			Armor = 0;
			Speed = 1.0f * BasisSpeed;
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			ViewRange = 10 * Def.MapTipLengthOnField;

			PopulationWeight = 2;
			
			MakingTime = 34 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {100,0,0,0};
			RequireBuildings = new[]{
				typeof(ProduceResType1).FullName,
			};

			#endregion
			
			
			#region Animation
			this.SetOnmyouAnimation("LotusCraft/Image/Unit/Black/Low02");
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Count.X = 12;
			Animation.Count.Y = 1;
			Animation.FrameTime = 5;
			Animation.FrameCount = 12;
			Animation.HasDirection = false;
			AnimationWalk.Width = 32;
			AnimationWalk.Height = 32;
			AnimationWalk.Count.X = 12;
			AnimationWalk.Count.Y = 1;
			AnimationWalk.FrameTime = 3;
			AnimationWalk.FrameCount = 12;
			AnimationWalk.HasDirection = false;
#endregion
			
			#region BuildableObjcets
			BuildingsMade = new[] { 
				typeof(HomeBase).FullName,
				typeof(ProduceResType0).FullName,
				typeof(ProduceResType1).FullName,
				typeof(BlackSmith).FullName,
//				typeof(Tower).FullName,
				typeof(Factory).FullName,
				typeof(Airport).FullName,

				typeof(Chireiden).FullName,
			};
			BuildType =BuildType.WorkAt;
#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(Low02Bullet).FullName, };
//			SelectBullet = (unit,target,distance)=>{return (target is IBuilding) ? -1 : 0;};
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(HomeBase).FullName),
					ButtonPosition._0,
					ButtonPosition._1
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(ProduceResType0).FullName),
					ButtonPosition._1,
					ButtonPosition._1
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(ProduceResType1).FullName),
					ButtonPosition._2,
					ButtonPosition._1
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(BlackSmith).FullName),
					ButtonPosition._3,
					ButtonPosition._1
					)),
//				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
//					faction.GetBuildingParameter(typeof(Tower).FullName),
//					ButtonPosition._4,
//					ButtonPosition._1
//					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Factory).FullName),
					ButtonPosition._1,
					ButtonPosition._2
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Airport).FullName),
					ButtonPosition._2,
					ButtonPosition._2
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Chireiden).FullName),
					ButtonPosition._0,
					ButtonPosition._3
					)),
			};
#endregion
			
			base.OnLoad (faction);
		}
		
		public override void OnVanish(IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			base.OnVanish(unit);
		}

		#region 建築
		public override void OnStartBuilding(IUnit unit, IBuilding building)
		{
			// エフェクト生成
			IEffect effect = null;
			effect = unit.Faction.World.EffectManager.CreateEffect(
				typeof(Effect.Common.UnitCollectPowerStoneEffect).FullName,
				unit.Pos);
			
			effect.SetScale(new Vector3D(3.0f, 3.0f, 3.0f));
			
			// 死なない、操作不可アビリティ生成
			var ability = new UncontrolableAbility();
			ability.ExtendLifetime(2);
			
			Action<IUnit> vanished = null;
			Action<IUnit, IBuilding> completed = null;
			Action<IUnit, IBuilding> canceled = null;
			
			// 命令
			unit.DeniningAcceptingNextOrderCount = 2;
			unit.ChangeActionToPrivilegeMode(
				() =>
				{
				return BuildingStructure(unit, building, ability, effect);
			},
			() =>
			{
				effect.Stop();
			});
			
			// 死亡時の条件追加
			
			vanished = (u) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};
			
			unit.Vanished += vanished;
			
			
			completed = (u, b) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};
			
			canceled = (u, b) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};
			
			unit.CompletedBuilding += completed;
			unit.CanceledBuilding += canceled;
		}
		
		
		IEnumerable<object> BuildingStructure(IUnit unit, IBuilding building, UncontrolableAbility ability, IEffect effect)
		{
			unit.AbilityManager.AppendAbility(ability);
			
			while (true)
			{
				unit.IsBuildingIconShowing = true;
				
				if (!building.Exists() || building.IsComplete)
				{
					effect.Stop();
					yield break;
				}
				
				ability.ExtendLifetime(1);
				yield return true;
			}
		}
		
		class UncontrolableAbility : AbilityTemplate
		{
			public UncontrolableAbility()
			{
				Name = "Uncontrolable";
			}
			
			public override bool ExecUnitAbility(IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.OtherExecs)
				{
					unit.IsntFoundByEnemy = true;
					unit.IsntHitWithBullet = true;
					unit.DeniningAcceptingNextOrderCount = 2;
				}
				
				return true;
			}
		}
		
#endregion

	}
}

namespace LotusCraft.Bullet.Black
{
	public class Low02Bullet : General.LongRangeBullet
	{
		public Low02Bullet()
		{
			Type = BulletType.ShortRange;
			SpaceType = BulletSpaceType.Both;
			DoesHitOnlyTarget = true;
			this.Attack = 11;
//			DamageBonus[(int)TargetObjectType.Structure] = 0.0f;

			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			MaxRange = 5 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(0.8f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 3;
			TextureAnimation.Offset.Y = 128 * 0 + 16 * 0;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunFire).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}
