﻿ using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability;
using LotusCraft.Ability.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Bullet.Black;
using LotusCraft.Spell.Black;

namespace LotusCraft.Unit.Black
{
	public class Mamizou : General.Special
	{
		public Mamizou()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Black/Mamizou";
			#endregion

			#region UnitParams
			Name = "二ッ岩マミゾウ(式神)";
			HP = 320;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 1.0f * BasisSpeed;

			ViewRange = 14 * Def.MapTipLengthOnField;

			MakingTime = 35 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {125,150,0,0};

			PopulationWeight = 2;
			UnitType = new TargetObjectType[]{TargetObjectType.Clay,};
			MoveType |= UnitMoveType.Air;

			CanState = false;

			 #endregion

			#region Description
			ButtonDescription += "飛行・対地ユニット\n自軍ユニットの幻影を作り出すことが出来る";

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Black/Mamizou");
			#endregion

			RequireBuildings = new string[]{
				typeof(Building.Black.Airport).FullName,
			};
			
			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(MamizouBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{ 
			var mirrorUnitParams = new []
			{
				faction.GetUnitParameter(typeof(Youmu).FullName),
				faction.GetUnitParameter(typeof(Satori).FullName),
				faction.GetUnitParameter(typeof(Orin).FullName),
				faction.GetUnitParameter(typeof(Oku).FullName),
				faction.GetUnitParameter(typeof(Mamizou).FullName),
			};
			#region Spells
			SpellParams = new SpellParameter<IUnit>[]
			{
				new MamizouSkill(ButtonPosition._0,ButtonPosition._2,mirrorUnitParams,0),
				new MamizouSkill(ButtonPosition._1,ButtonPosition._2,mirrorUnitParams,1),
				new MamizouSkill(ButtonPosition._2,ButtonPosition._2,mirrorUnitParams,2),
				new MamizouSkill(ButtonPosition._3,ButtonPosition._2,mirrorUnitParams,3),
				new MamizouSkill(ButtonPosition._0,ButtonPosition._3,mirrorUnitParams,4),
			};
			#endregion
			#region AutoCasts
			AutocasterParams = null;
			#endregion
			#region Buttons
			ButtonParams = new ButtonParameterHolder<IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[1].DefaultSpellButton,SpellParams[1].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[2].DefaultSpellButton,SpellParams[2].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[3].DefaultSpellButton,SpellParams[3].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[4].DefaultSpellButton,SpellParams[4].SpellButtons),
			};
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Black
{
	public class MamizouBullet : General.LongRangeBullet
	{
		public MamizouBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = true;
			this.Attack = 18;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0f;
			MaxRange = 5 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(1.2f * Def.FPS);

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 2;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Spell.Black
{
	class MamizouSkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int duration = 90*Def.FPS;
		public UnitParameter[] UnitParams = null;
		public int UnitParamPos = 0;
		const string spellBaseName = "変化「百鬼妖界の門」: ";
		public MamizouSkill(ButtonPosition x,ButtonPosition y,UnitParameter[] unitParams,int pos)
		{

			UnitParams = unitParams;
			UnitParamPos = pos;
			DefaultLevel = 1 ;
			MaxLevel = 1;
			Name = spellBaseName+unitParams[pos].Name;
			string basePath = "Black/Spell/MamizouSkill/";
//			string basePath = unitParam.IconPath;

			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 60*Def.FPS;
			CoolTimes[1] = 60*Def.FPS;

			Description = "自軍ユニットの幻影を作り出す。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				unitParams[pos].Name + "の幻影を作成",
			};
			
			/// ボタンの位置
			ButtonPosX = x;
			ButtonPosY = y;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;

			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,unitParams[pos].IconPath.Replace("LotusCraft/Image/Button/",""),PrivilegeModeSpelling,1),
			};
		}
		

		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime

			foreach(var param in UnitParams)
				selfUnit.GetSpellWithName (spellBaseName+param.Name).ResetCoolTime ();
			
			var world = selfUnit.Faction.World;

			if (null != UnitParams && UnitParamPos < UnitParams.Length) {

				var unit = world.PutUnitAroundPoint(selfUnit.Faction,UnitParams[UnitParamPos],new Vector2D(selfUnit.Pos.X,selfUnit.Pos.Y),selfUnit.Faction);
				if(null != unit && unit.Exists())
				{
					/// show effect
					if (unit.AbilityManager.ContainsAbility (MamizouSkillAbility .AbilityName)) {
						unit.AbilityManager.ExtendLifetimeWithName (duration, MamizouSkillAbility .AbilityName, duration);
					} else {
						var offset = new Vector3D (2, 2, 2);
						var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.MamizouEffect).FullName, selfUnit.Pos + offset);
						var scale = 9;
						effect.SetScale (new Vector3D (scale, scale, scale));
						var ability = new MamizouSkillAbility (effect, offset, duration);
						if (!unit.AbilityManager.AppendAbility (ability))
							effect.Stop ();
					}
				}
			}

			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Black
{
	class MamizouSkillAbility  : FollowEffectAbility
	{
		public const string AbilityName =  "変化「百鬼妖界の門」";
		public MamizouSkillAbility(int durationTime,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = durationTime;
			this.Effect = effect;
		}
		public MamizouSkillAbility(IEffect effect,Vector3D offset,int durationTime)
		{
			this.Name = AbilityName;
			this.Effect = effect;
			this.Offset = offset;
			this.DurationTime = durationTime;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				unit.IsForbiddenSpelling = true;
				unit.DamageRatio = 2.0f;
				unit.Exp = (int)Math.Ceiling (unit.Exp/4.0f);
				break;
			case thrts.AbilityOrder.RateDecParam:
				break;
			case thrts.AbilityOrder.ConstIncParam:
				unit.IsSummon = true;
				unit.CanStateNow = false;
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}

		public override bool ModifyBullet (IBullet bullet)
		{
			bullet.Attack = 0;  
			return base.ModifyBullet (bullet);
		}

		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists()) {
				unit.HP = 0;
			}

			return base.OnVanish (unit);
		}
	}
	
}