using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Black;
using LotusCraft.Building.Black;
using LotusCraft.Effect.Black;
using LotusCraft.Spell.Black;

namespace LotusCraft.Unit.Black
{
	public class Satori : General.Special
	{
		public Satori ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Black/Satori";
			#endregion

			#region UnitParams
			Name = "古明地 さとり(式神)";
			ButtonDescription += "飛行・対空ユニット\n広い視界とスプラッシュする対空攻撃を持つ。";
			HP = 300;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.0f * BasisSpeed;

			PopulationWeight = 2;

			ViewRange = 14*Def.MapTipLengthOnField;
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			MoveType |= UnitMoveType.Air;
			
			Costs = new [] {125,125,0,0};
			MakingTime = 40 * Def.FPS;

			RequireBuildings = new string[]{
				typeof(Chireiden).FullName,
				typeof(Airport).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Black/Satori");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.BuildingButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.AttackStateButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.HoldPositionButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.StationButton()),
			};
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(SatoriBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new SatoriSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
#endregion
			
			base.OnLoad(faction);
		}
	}
}


namespace LotusCraft.Bullet.Black
{
	public class SatoriBullet : General.LongRangeBullet
	{
		public SatoriBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Sky;

			this.Attack = 30;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0;
			MaxRange = 6 * Def.MapTipLengthOnField;

			HasSplash = true;
			SplashDamageRates = new Dictionary<float, float>(){{0.8f*Def.MapTipLengthOnField,1.0f},{1.2f*Def.MapTipLengthOnField,0.8f},{1.6f*Def.MapTipLengthOnField,0.5f},{2.0f*Def.MapTipLengthOnField,0.25f}};
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(2.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			OnHitSplash = OnHitSplashEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 1;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunLong).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect2(bullet, target, Unit.Utls.HitEffectType.ColumnBlue);
		}
		static int OnHitSplashEffect(IBullet bullet, ITargetObject target,int damage)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.ColumnBlue);
			return damage;
		}
	}
}
