﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.AutoCast;
using LotusCraft.Ability;
using LotusCraft.Ability.Blue;
using LotusCraft.Building.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Bullet.Blue;
using LotusCraft.Spell.Blue;

namespace LotusCraft.Unit.Blue
{
	public class Lety : General.Special
	{
		public Lety()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Blue/Lety";
			#endregion

			#region UnitParams
			Name = "レティ・ホワイトロック(式神)";
			HP = 150;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 1.0f * BasisSpeed;

			ViewRange = 8 * Def.MapTipLengthOnField;

			MakingTime = 20 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {75,0,0,0};

			PopulationWeight = 1;

			IsDefence = true;
			MaximumExists = 1;

			MoveType |= UnitMoveType.Air;

			CanState = false;

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
				typeof(BigTree).FullName,
			};
			

			 #endregion

			#region Description
			ButtonDescription += "魔法陣無しで生産出来る飛行遠距離攻撃ユニットです。 \n 拠点数まで作成でき、敵ユニットの移動速度を下げる事ができます";

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Blue/Lety");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(LetyBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{

			#region Spells
			SpellParams = new SpellParameter<IUnit>[]
			{
				new LetySkill(),
			};
			#endregion
			#region AutoCasts
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#endregion
			#region Buttons
			ButtonParams = new ButtonParameterHolder<IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			ButtonParams = ButtonParams.Concat(faction.Parameter.CommonUnitButton).ToArray();
			#endregion
		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Blue
{
	public class LetyBullet : General.LongRangeBullet
	{
		public LetyBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = true;
			this.Attack = 10;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0f;
			MaxRange = 5 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(1.0f * Def.FPS);

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 1;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			// change damage if hit farmer
			/*
			{
				var u = target as IUnit;
				if(null != u && u.Parameter.IsFarmer)
				{
					// 農民に当たった時は50%ダメージ
					bullet.Attack /= 2;
				}
			}
			*/
			var faction = bullet.Generator.Faction;
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalRed);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}

namespace LotusCraft.Spell.Blue
{
	class LetySkill:thrts.SpellParameter<thrts.IUnit>
	{
		const int interval = 1;
		const float affectRadius = 2;
		const int duration = 20*Def.FPS;
		const float decMove = 0.5f;
		public LetySkill()
		{
			DefaultLevel = 1 ;
			MaxLevel = 1;
			Name = "寒符「コールドスナップ」";
			string basePath = "Blue/Spell/LetySkill/";
			
			CoolTimes = new int[MaxLevel+1];
			CoolTimes[0] = 99*Def.FPS;
			CoolTimes[1] = 75*Def.FPS;

			MaxRanges = new float[MaxLevel+1];
			MaxRanges[0] = 6*Def.MapTipLengthOnField;
			MaxRanges[1] = 6*Def.MapTipLengthOnField;

			Description = "目標地点付近の敵の移動速度を低下させる。";
			/// レベル別の能力説明
			EffectDescriptions = new string[]
			{
				"移動速度-"+decMove.ToString("%"),
			};
			
			/// ボタンの位置
			ButtonPosX = thrts.ButtonPosition._0;
			ButtonPosY = thrts.ButtonPosition._2;
			LvUpButtonPosX = ButtonPosX;
			LvUpButtonPosY = thrts.ButtonPosition._1;
			
			/// オートキャストの生成
			SpellAutoCasts = null;
			/// スペルのボタンを生成
			SpellButtons = new thrts.ButtonParameter<thrts.IUnit>[]
			{
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "0",0),
				new LotusCraft.Button.SpellButton<thrts.IUnit>(this,basePath + "1",PrivilegeModeSpelling,1),
			};
			LevelUpSpellButton = new LotusCraft.Button.LevelUpSpellButton(this, basePath + "LvUp");
		}
		
		public bool Spelling (thrts.IUnit selfUnit,Vector2D targetPos, int level)
		{
			var selfFaction = selfUnit.Faction;
			var world = selfFaction.World;
			
			/// search targets
			var targets = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius * Def.MapTipLengthOnField, targetPos)).Where (_ => _ != null);
			foreach (IUnit u in targets.OfType<IUnit>().Where(_=>!selfFaction.World.IsSameTeam(selfFaction,_.Faction))) {
				if(u.AbilityManager.ContainsAbility(LetySkillAbility.AbilityName))
				{
					u.AbilityManager.ExtendLifetimeWithName(interval,LetySkillAbility.AbilityName,2*Def.FPS);
				}
				else
				{
					var debuffEffect = world.EffectManager.CreateEffect(typeof(Effect.Common.DebuffEffect).FullName,u.Pos);
					debuffEffect.SetScale(new Vector3D(15,15,15));
					if(!u.AbilityManager.AppendAbility(new LetySkillAbility(decMove,debuffEffect)))
					{
						debuffEffect.Stop();
					}
				}
			}

			return true;
		}
		public IEnumerable<object> PrivilegeModeSpelling (thrts.IUnit selfUnit,Vector2D targetPos, int level)
		{
			if (level <= 0)
				yield return false;
			
			/// reset cooltime
			selfUnit.GetSpellWithName (Name).ResetCoolTime ();

			var world = selfUnit.Faction.World;

			/// show effect
			if (selfUnit.AbilityManager.ContainsAbility (this.Name + "Effect")) {
				selfUnit.AbilityManager.ExtendLifetimeWithName(duration,this.Name + "Effect",duration);
			} else {
				var offset = new Vector3D (2, 2, 2);
				var effect = world.EffectManager.CreateEffect (typeof(Effect.Common.LetyEffect).FullName, world.GetGroundAtPos(targetPos) + offset);
				var scale = affectRadius * 5;
				effect.SetScale (new Vector3D (scale, scale, scale));
				var ability = new Ability.SpellContinuousAbility ((IUnit u)=>Spelling(u,targetPos,level),effect, offset, duration, this.Name + "Effect",interval);
				ability.FollowEffect = false;
				if (!selfUnit.AbilityManager.AppendAbility (ability))
					effect.Stop ();
			}

			yield return false;
		}
	}
}

namespace LotusCraft.Ability.Blue
{
	class LetySkillAbility  : FollowEffectAbility
	{
		public float DecMove;
		public const string AbilityName =  "寒符「コールドスナップ」";
		public LetySkillAbility(float decMove,IEffect effect = null)
		{
			this.Name = AbilityName;
			this.DurationTime = Def.FPS;
			this.Effect = effect;
			this.DecMove = decMove;
		}
		public override bool ExecUnitAbility(thrts.IUnit unit, thrts.AbilityOrder order)
		{
			switch (order)
			{
			case thrts.AbilityOrder.ConstDecParam:
				break;
			case thrts.AbilityOrder.RateDecParam:
				unit.Speed *= DecMove;
				break;
			case thrts.AbilityOrder.ConstIncParam:
				break;
			case thrts.AbilityOrder.RateIncParam:
				break;
			case AbilityOrder.OtherExecs:
				break;
			default:
				break;
			}
			
			return base.ExecUnitAbility(unit,order);
		}
	}
	
}