﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Blue;
using LotusCraft.Building.Blue;
using LotusCraft.Effect.Blue;
using LotusCraft.Spell.Blue;

namespace LotusCraft.Unit.Blue
{
	public class Star : General.Special
	{
		public Star ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Blue/Star";
			#endregion

			#region UnitParams
			Name = "スターサファイア(式神)";
			HP = 280;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.1f * BasisSpeed;

			ViewRange = 12*Def.MapTipLengthOnField;
			PopulationWeight = 2;
			ButtonDescription = "飛行・対地対空ユニット\n対地対空の万能攻撃と索敵能力を持つ";

			MoveType |= UnitMoveType.Air;

			UnitType = new TargetObjectType[]{TargetObjectType.Water,};
			
			Costs = new [] {100,100,0,0};
			MakingTime = 40 * Def.FPS;

			RequireBuildings = new string[]{
				typeof(BigTree).FullName,
				typeof(Factory).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Blue/Star");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.BuildingButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.AttackStateButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.HoldPositionButton()),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new thrts.Unit.Button.StationButton()),
			};
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(StarBullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new StarSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,

				/// mode change autocast
				SpellParams[0].SpellAutoCasts[2],
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
#endregion
			
			base.OnLoad(faction);
		}
	}
}


namespace LotusCraft.Bullet.Blue
{
	public class StarBullet : General.LongRangeBullet
	{
		public StarBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			DoesHitOnlyTarget = true;
			this.Attack = 21;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			MaxRange = 6 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(1.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 0;
			TextureAnimation.Offset.Y = 128 * 1 + 16 * 5;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunFire).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}
