﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability;
using LotusCraft.Bullet.Common;
using LotusCraft.Effect.Common;

namespace LotusCraft.Unit.Common
{
	public class Mercenary01 : General.Clay
	{
		int duration = 15*Def.FPS;
		public Mercenary01()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Common/Mercenary01";
			#endregion

			#region UnitParams
			Name = "傭兵(岩)";
			HP = 500;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 1.2f * BasisSpeed;

			IsSummon = true;
			CanState = false;

			ViewRange = 9 * 32;

			MakingTime = 25 * 30;
			/// mine,gas,sup,?
			Costs = new [] {0,0,0,0};

			Exp = 0;

 			#endregion

			#region Description
			ButtonDescription += "一定時間共に戦う増援ユニットです。\nこのユニットは近くに居る敵を攻撃することができます。";

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Common/Mercenary01");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(Mercenary01Bullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			ButtonParams = faction.Parameter.CommonUnitButton.ToArray();
			#endregion

		}
		public override void OnAppear (IUnit unit)
		{
			if(unit.Faction.World.CurrentTime > 10)
				unit.Faction.World.SoundManager.PlaySoundEffect (typeof(SoundEffect.CreatedUnit).FullName,unit.Pos);

			/// 自動死亡のアビリティを追加
			unit.AbilityManager.AppendAbility(new SuicideAbility(null,duration,this.Name));

			base.OnAppear (unit);
		}
		public override void OnVanish (IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			unit.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.PCH).FullName, unit.Pos);
			base.OnVanish (unit);
		}
	}
}
namespace LotusCraft.Bullet.Common
{
	public class Mercenary01Bullet : General.ShortRangeBullet
	{
		public Mercenary01Bullet()
		{
			Type = BulletType.LongRange;
			DoesHitOnlyTarget = true;
			this.Attack = 50;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0f;
			MaxRange = 4 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(2.0f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullet";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunFire).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalOrange);
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
		}
	}
}
