﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Ability.Green;
using LotusCraft.Button;
using LotusCraft.Bullet.Green;
using LotusCraft.Building.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Unit.Green
{
	public class Low01 : General.Water
	{
		public Low01 ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Green/Low01";
			#endregion

			#region UnitParams
			Name = "毛玉";
			HP = 75;
			Radius = (float)Utls.UnitSize.VerySmall;
			Armor = 0;
			Speed = 1.3f * BasisSpeed;
//			Exp /= 2;
			ButtonDescription = "近接ユニット\n一度の生産で2匹作られる";

			ViewRange = 9*Def.MapTipLengthOnField;

			PopulationWeight = 1;

			MakingTime = 30 * Def.FPS;
			ConcurrentProductions = 2;
			/// mine,gas,sup,?
			Costs = new [] {50,0,0,0};
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
			#endregion

			#region Animation
			this.SetKedamaAnimation("LotusCraft/Image/Unit/Green/Low01");
			Animation.FrameCount = 3;
			AnimationWalk.FrameCount = 3; 
			#endregion

			#region BuildableObjcets
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(Low01Bullet).FullName };
			#endregion
			
		}
		public override void OnVanish(IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(-2, -2, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(DeadUnitEffect).FullName, unit.Pos + offset, faction);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
			base.OnVanish(unit);
		}
		public override void OnAppear (IUnit unit)
		{
			unit.AbilityManager.AppendAbility(new Ability.Green.OnGardenEffectAbility());
			base.OnAppear (unit);
		}

		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new ButtonParameterHolder<IUnit>(new MetamolToSmallSuika(faction.GetUnitParameter(typeof(SuikaSmall).FullName),faction.GetUnitParameter(typeof(SuikaSmallCocoon).FullName),ButtonPosition._2,ButtonPosition._2)),
			};
#endregion
			
			base.OnLoad(faction);
		}
	}
}
namespace LotusCraft.Button
{
	class MetamolToSmallSuika : thrts.ButtonParameter<IUnit>
	{
		public MetamolToSmallSuika(UnitParameter unitParam,
								   UnitParameter progressUnitParam,
								   thrts.ButtonPosition row,
								   thrts.ButtonPosition column,
								   string iconPath = null,
								   string shortcutKeyword = null)
		{
			Position = Tuple.Create (row, column);
			if (null == iconPath)
				ButtonImagePath = unitParam.IconPath;
			
			ShortcutKeyword = shortcutKeyword != null ? shortcutKeyword : string.Empty;
			
			/// 説明に人口条件や時間条件を追記
			string buttonDescription = string.Empty;
			
			buttonDescription += unitParam.ButtonDescription;
			
			var text = buttonDescription.Split (new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
				.Select (_ => new ColorText (_))
					.Select (_ => new ColorTextLine (_))
					.ToArray ();
			List<ColorText> unitNameText = new List<ColorText> ();
			unitNameText.Add (new ColorText (unitParam.Name + " ["));
			foreach (var t in unitParam.UnitType) {
				switch (t) {
				case TargetObjectType.Clay:
					unitNameText.Add (new ColorText (" 岩", new Color (1, 0, 0)));
					break;
				case TargetObjectType.Water:
					unitNameText.Add (new ColorText (" 水", new Color (0, 191/255f,243/255f)));
					break;
				case TargetObjectType.Structure:
					unitNameText.Add (new ColorText (" 鋼", new Color (0.5f, 0.5f, 0.5f)));
					break;
				case TargetObjectType.Hero:
					unitNameText.Add (new ColorText (" 白", new Color (1, 1, 1)));
					break;
				}
			}
			unitNameText.Add (new ColorText (" ]"));
			var unitNameTextLine = new[]{new ColorTextLine(unitNameText)};
			
			MouseOverTextFunc = faction => unitNameTextLine
				.EndWith(faction.GetAttackBonusText(unitParam))
					.EndWith(text)
					.EndWith(
						faction.Parameter.GetType().FullName != typeof(Faction.Green).FullName || faction.World.UnitManager.DontUseInitCost(faction, unitParam.GetType().FullName)
						? faction.GetCostInfoText(unitParam.Costs, unitParam.PopulationWeight, unitParam.MakingTime / Def.FPS)
						: faction.GetCostInfoText(unitParam.InitCosts, unitParam.PopulationWeight, unitParam.MakingTime / Def.FPS))
					.EndWith(faction.GetRequireBuildingsInfoText(unitParam.RequireBuildings))
					.EndWith(faction.GetRequireUnitsInfoText(unitParam.RequireUnits))
					.EndWith(unitParam.IsUnique
							 ? (faction.IsSatisfyUnits(new string[] { unitParam.GetType().FullName }, false)
					   ? new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない", new Color(1, 0, 0))) }
					: new ColorTextLine[] { new ColorTextLine(new ColorText("このユニットは同時に1体しか存在できない")) }
					)
							 : new ColorTextLine[0])
					;
			
			Kind = ButtonKind.Normal;
			NormalButtonEvent = (unit, isAppend) =>
			{
				var faction = unit.Faction;

				if (Enumerable.Range(0, unitParam.Costs.Length)
					.Any(i => faction.GetResource(i) < unitParam.Costs[i]))
				{
					return;
				}

				for(int i=0;i< unitParam.Costs.Length;++i)
				{
					faction.DecResource(i,unitParam.Costs[i]);
				}
				var hpratio = unit.HP / (float)unit.MaxHP;

				unit.SetParameter(progressUnitParam);

				unit.HP = (int)Math.Ceiling(unit.MaxHP*hpratio);
				unit.AbilityManager.AppendAbility(new MetamolProgressAbility(unitParam.GetType().FullName,unitParam.MakingTime));

			};
			CanExecute = (faction, _, holder) =>
			{
				var cantExecuteMessage = new List<ColorTextLine>();

				if(faction != holder.Parameter.Faction)
				{
					cantExecuteMessage.Add(new ColorTextLine(new ColorText("このユニットは生産出来ません", new Color(1, 0, 0))));
				}
				else
				{
					if (unitParam.IsUnique && faction.IsSatisfyUnits(new string[] { unitParam.GetType().FullName }, false)&& faction.IsSatisfyUnits(new string[] { progressUnitParam.GetType().FullName }, false))
					{
						cantExecuteMessage.Add(new ColorTextLine(new ColorText("既に " + unitParam.Name + " を生産しています", new Color(1, 0, 0))));
					}
					if (faction.World.UnitManager.DontUseInitCost(faction, unitParam.GetType().FullName) || unitParam.InitCosts == null)
					{
						if (Enumerable.Range(0, unitParam.Costs.Length)
							.Any(i => faction.GetResource(i) < unitParam.Costs[i]))
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("リソースが足りません", new Color(1, 0, 0))));
						}
					}
					if (unitParam.RequireBuildings != null && unitParam.RequireBuildings.Any())
					{
						if (!faction.IsSatisfyBuildings(unitParam.RequireBuildings))
						{
							cantExecuteMessage.Add(new ColorTextLine(new ColorText("前提建物が存在しません", new Color(1, 0, 0))));
						}
					}
				}
				return new ExecutableInfo(!cantExecuteMessage.Any(), cantExecuteMessage);
			};
		}
	}
}

namespace LotusCraft.Ability.Green
{
	public class MetamolProgressAbility : thrts.AbilityTemplate
	{
		string unitParamName;
		bool remaintimeInMpBar;
		public MetamolProgressAbility (string paramName, int duration)
		{
			this.DurationTime = duration;
			this.unitParamName = paramName;
			this.remaintimeInMpBar = false;
		}
		public override bool ExecUnitAbility (IUnit unit, AbilityOrder order)
		{
			switch(order)
			{
				case AbilityOrder.OtherExecs:
				if(remaintimeInMpBar)
					unit.MPBarCurrent = DurationTime;
				break;
			}
			return base.ExecUnitAbility (unit, order);
		}
		public override bool OnAdd (IUnit unit)
		{
			if (!unit.IsMPBarShown) {
				unit.IsMPBarShown = true;
				unit.MPBarMax = DurationTime;
				remaintimeInMpBar = true;
			}
			return base.OnAdd (unit);
		}
		public override bool OnVanish (IUnit unit)
		{
			if (null != unit && unit.Exists () && unit.HP > 0) {
				// change to metamol unit
				var unitParam = unit.Faction.GetUnitParameter(unitParamName);
				if(unitParam != null)
				{
					var hpratio = unit.HP / (float)unit.MaxHP;
					
					unit.SetParameter(unitParam);

					unit.HP = (int)Math.Ceiling(unit.MaxHP*hpratio);
					if(remaintimeInMpBar)
						unit.IsMPBarShown = false;
				}
				else
					unit.HP = 0;
			}
			return base.OnVanish(unit);
		}

	}
}
namespace LotusCraft.Bullet.Green
{
	public class Low01Bullet : General.ShortRangeBullet
	{
		public Low01Bullet()
		{
			Type = BulletType.ShortRange;
			SpaceType  = BulletSpaceType.Earth;
			this.Attack = 6;
			Speed = 0.0f;
//			MaxRange = 0.5f * Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(0.4f * Def.FPS);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}
