﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Green;
using LotusCraft.Spell.Green;
using LotusCraft.Building.Green;
using LotusCraft.Effect.Green;

namespace LotusCraft.Unit.Green
{
	public class Medicine : General.Special
	{
		public Medicine ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Green/Medicine";
			#endregion

			#region UnitParams
			Name = "メディスン・メランコリー(式神)";
			HP = 360;
			Speed = 0.8f*BasisSpeed;
			MoveType |= UnitMoveType.Air;

			ViewRange = 14*Def.MapTipLengthOnField;

			PopulationWeight = 4;

			MakingTime = 48 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {150,175,0,0};
			ButtonDescription = "飛行・対地ユニット。\n防御力を無視してダメージを与える毒付与能力を持つ。";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));
			UnitType = new TargetObjectType[] { TargetObjectType.Clay, };
			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Green/Medicine");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(MedicineBullet).FullName };
			#endregion

			#region Requires
			RequireBuildings = new string[]{
				typeof(Cherry).FullName,
				typeof(SuzuranField).FullName,
			};
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new MedicineSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion
			
			base.OnLoad(faction);
		}
	}
}

namespace LotusCraft.Bullet.Green
{
	public class MedicineBullet : General.LongRangeBullet
	{
		public MedicineBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			this.Attack = 30;
			Speed = BulletUtls.SlowSpeed * BasisSpeed;
			
			MaxRange = 6 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.8f * 30);
			AttackRigor = (int)(0.4f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/kome";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.taihou).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalBlue);
		}
	}
}
