﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Neutral;
using LotusCraft.Effect.Neutral;

namespace LotusCraft.Unit.Neutral
{
	public class High02 : NeutralBase
	{
		public High02()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Neutral/angel03_n";
			#endregion

			#region UnitParams
			Name = "中立・天女";
			HP = 450;
			Armor = 2;
			Radius = (float)Utls.UnitSize.Medium;
			Speed = 0.8f * BasisSpeed;

			ViewRange = 7 * 32;

			MakingTime = 25 * 30;
			Exp = 120;
			/// mine,gas,sup,?
			Costs = new [] {200,25,0,0};
			#endregion

			#region Description
			ButtonDescription += "中立の天女。強めで、遠くの敵を攻撃することができる。\n周囲の中立ユニットを回復する能力を持つ。";

			#endregion

			#region Animation
//			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Neutral/fluff02");
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Neutral/angel03_n");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(High02Bullet).FullName };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			this.AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				new AutoCast.AutoCast<IUnit>(this.Name,2*Def.FPS,skillAutocast,true),
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.MoveButton ()),
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.StopButton ()),
				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.HoldPositionButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.PatrolButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.AttackButton ()),
//				new thrts.ButtonParameterHolder<thrts.IUnit> (new LotusCraft.Button.StationButton ()),
			};
			#endregion

		}

		public bool skillAutocast (thrts.IUnit unit)
		{
			var affectRadius =4;
			var healHP = 5;
			var selfFaction = unit.Faction;
			// get target from world
			var target = selfFaction.World.GetObjectFromCircle (new Circle2D (affectRadius*Def.MapTipLengthOnField, new Vector2D(unit.Pos.X,unit.Pos.Y)))
					.OfType<IUnit> ()
					.Where(_ => _!=unit && _.HP < _.MaxHP &&  selfFaction.World.IsSameTeam(_.Faction,selfFaction))
					.OrderBy(_=>_.HP/_.MaxHP);
			// heal team unit
			if(target.Count() > 0)
			{
				// HPが最も減っている自分以外ユニット1体を選択
				var u = target.First();
				// heal unit
				u.HP += healHP;
				if(u.HP > u.MaxHP)
					u.HP = u.MaxHP;

				// add effect
//				var effect  = unit.Faction.World.EffectManager.CreateEffect(typeof(HealEffect).FullName, unit.Faction, u.Pos);
//				if(!u.AbilityManager.AppendAbility(new FollowEffectAbility(effect, new Vector3D(0,0,1), 1*Def.FPS, this.Name+"HealEffect")))
//					effect.Stop();
			}
			return true;
		}
	}
}


namespace LotusCraft.Bullet.Neutral
{
	public class High02Bullet : General.LongRangeBullet
	{
		public High02Bullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;
			DoesHitOnlyTarget = true;
			this.Attack = 24;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			
			MaxRange = 7 * 32;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(1.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/kome";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunFire).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet,ITargetObject target)
		{
			Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.NormalOrange);
		}
	}
}
