using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Red;
using LotusCraft.Building.Red;
//
using LotusCraft.Spell.Common;

namespace LotusCraft.Unit.Red
{
	public class Flandre : General.Special
	{
		public Flandre ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Red/Flandre";
			#endregion

			#region UnitParams
			Name = "フランドール・スカーレット(式神)";
			HP = 240;
			Radius = (float)Utls.UnitSize.Small;
			Armor = 0;
			Speed = 1.3f * BasisSpeed;

			PopulationWeight = 2;
			StationWeight = 3;
			ViewRange = 9 * 32;

			MakingTime = 28 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {150,50 ,0,0};

			ButtonDescription = "近接ユニット\n体力が低い代わりに高い攻撃力と移動速度を持つ";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			RequireBuildings = new string[]{
				typeof(StairsOfKoumakan).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Red/Flandre");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(Bullet.Red.FlandreBullet).FullName, };
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new LotusCraft.Spell.Red.FlandreSkill (),
				new ShinsokuSkill(),
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
				new ButtonParameterHolder<IUnit>(SpellParams[1].DefaultSpellButton,SpellParams[1].SpellButtons),
			};
			#endregion

			base.OnLoad(faction);
		}
	}

	public class FlandreImage : General.Special
	{
		public FlandreImage ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Red/Flandre";
#endregion
			
			#region UnitParams
			Name = "フランドール・スカーレット(式神)";
			HP = 240;
			Radius = (float)Utls.UnitSize.Small;
			Armor = 0;
			Speed = 1.3f * BasisSpeed;
			
			PopulationWeight = 0;
			
			ViewRange = 9 * 32;
			
			MakingTime = 40 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {75,150 ,0,0};
			
			ButtonDescription = "近接ユニット。\n体力が低い代わりに高い攻撃力と移動速度を持つ。";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};
			
			RequireBuildings = new string[]{
				typeof(StairsOfKoumakan).FullName,
			};

			// image setting
			DamageRatio = 2;
			CanState = false;

#endregion
			
			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Red/Flandre");
#endregion
			
			#region BuildableObjcets
			//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
#endregion
			
			#region Bullets
			Bullets = new string[] { typeof(Bullet.Red.FlandreImageBullet).FullName, };
#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			#region Buttons
			#endregion
			base.OnLoad(faction);
			// image setting
			Exp = 0;
		}
		public override void OnAppear (IUnit unit)
		{
//			base.OnAppear (unit);
		}
	}
}

namespace LotusCraft.Bullet.Red
{
	public class FlandreBullet : General.ShortRangeBullet
	{
		public FlandreBullet()
		{
			Type = BulletType.ShortRange;
			this.Attack = 45;
			Speed = 0.0f;
			
			MaxRange = 0.5f * 32;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(0.0f * 30);
			AttackSpeed = (int)(1.5f * 30);
			
			OnHit =OnHitEffect;
			
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target);
		}
	}
	public class FlandreImageBullet : FlandreBullet
	{
		public FlandreImageBullet()
		{
			OnHit = OnHitImageBullet;
		}
		static void OnHitImageBullet(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target);
			bullet.Attack = 0;
		}
	}

}
