using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Red;
using LotusCraft.Building.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Spell.Red;

namespace LotusCraft.Unit.Red
{
	public class Koakuma : General.Special
	{
		public Koakuma ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Red/Koakuma";
			#endregion

			#region UnitParams
			Name = "小悪魔(式神)";
			ButtonDescription += "飛行・対地ユニット\n長い射程と広い視界、ヒロインへのボーナスを持つ";
			HP = 360;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 0.7f * BasisSpeed;

			PopulationWeight = 4;

			ViewRange = 14*Def.MapTipLengthOnField;
			UnitType = new TargetObjectType[]{TargetObjectType.Clay,};

			MoveType |= UnitMoveType.Air;
			
			Costs = new [] {200,200,0,0};
			MakingTime = 40 * Def.FPS;

			RequireBuildings = new string[]{
				typeof(Airport).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Red/Koakuma");
			#endregion

			#region BuildableObjcets
//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Buttons
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(KoakumaBullet).FullName };
			#endregion

			#region Requires
			RequireBuildings = new string[]{
				typeof(Barrack).FullName,
			};
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			/*
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new KoakumaSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit> (SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion
			*/
			base.OnLoad(faction);
		}
	}
}


namespace LotusCraft.Bullet.Red
{
	public class KoakumaBullet : General.LongRangeBullet
	{
		public KoakumaBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;

			DoesHitOnlyTarget = true;

			this.Attack = 39;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Gravity = 0;
			MaxRange = 9 * Def.MapTipLengthOnField;

			AttackDelay = (int)(0.0f * 30);
			AttackRigor = 0;
			AttackSpeed = (int)(1.5f * 30);

			DamageBonus[(int)TargetObjectType.Hero] =  2.0f;

			OnHit = OnHitEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/bullets";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 4;
			TextureAnimation.Count.Y = 8;
			TextureAnimation.Offset.X = 128 * 2 + 16 * 3;
			TextureAnimation.Offset.Y = 128 * 0 + 16 * 0;
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunLong).FullName, bullet.Position);
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.ColumnBlue);
		}
	}
}
