 using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Bullet.Red;
using LotusCraft.Building.Red;
using LotusCraft.Effect.Red;
using LotusCraft.Spell.Red;

namespace LotusCraft.Unit.Red
{
	public class Meirin : General.Special
	{
		public Meirin()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/Red/Meirin";
			#endregion

			#region UnitParams
			Name = "紅美鈴(式神)";
			ButtonDescription += "砲撃ユニット\n迎撃モード時、敵ヒロイン以外から発見されず、高火力の範囲攻撃能力を得る\n通常モードでは攻撃不可、迎撃モードでは移動不可";
			HP = 160;
			Radius = (float)Utls.UnitSize.Large;
			Speed = 1.0f * BasisSpeed;

			PopulationWeight = 2;

			ViewRange = 14 * Def.MapTipLengthOnField;
			UnitType = new TargetObjectType[] { TargetObjectType.Clay, };

			//			MoveType |= UnitMoveType.Air;

			Costs = new[] { 75, 50, 0, 0 };
			MakingTime = 28 * Def.FPS;

			RequireBuildings = new string[]{
//				typeof(Airport).FullName,
			};

			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/Red/Meirin");
			#endregion

			#region BuildableObjcets
			//			BuildingsMade = new[] { "LotusCraft/Building/Test" };
			#endregion

			#region Buttons
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(MeirinMobBigBullet).FullName };
			SelectBullet = (IUnit u, ITargetObject t, float d) => -1;
			#endregion
		}
		public override void OnLoad(IFaction faction)
		{
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new LotusCraft.Spell.Red.MeirinSkill (),
			};
			AutocasterParams = new AutoCastParameter<IUnit>[]
			{
				SpellParams[0].DefaultAutoCast,
			};
			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion

			base.OnLoad(faction);
		}
	}
}


namespace LotusCraft.Bullet.Red
{
	public class MeirinMobBigBullet : General.CannonBullet
	{
		public MeirinMobBigBullet()
		{
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Both;

			DoesHitOnlyTarget = false;
			
			HasSplash = true;
			IsSplashMissHit = true;
			SplashDamageRates.Add(1.75f * Def.MapTipLengthOnField, 0.25f);
			Attack = 160;
			DamageBonus[(int)TargetObjectType.Clay] = 2.0f;
			DamageBonus[(int)TargetObjectType.Water] = 2.0f;
			Speed = 7.0f * BasisSpeed;
			//Gravity = 5;
			MaxRange = 8 * Def.MapTipLengthOnField;
			MinRange = 0 * Def.MapTipLengthOnField;
			
			AttackDelay = (int)(0.0f * Def.FPS);
			AttackRigor = (int)(0.0f * Def.FPS);
			AttackSpeed = (int)(32.0f * Def.FPS);
			
			OnHit += OnHitEffect;
			OnMiss += OnMissEffect;

			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/ball_orange_big";
			TextureAnimation.AlphaBlend = AlphaBlend.Add;
			TextureAnimation.Width = 64;
			TextureAnimation.Height = 64;
			TextureAnimation.Center.X = 0;
			TextureAnimation.Center.Y = 0;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = false;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 1;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.taihou).FullName, bullet.Position);
		}
		void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, target.Pos, scale);
		}
		
		void OnMissEffect(IBullet bullet)
		{
			var faction = bullet.Generator.Faction;
			float scale = (SplashDamageRates.Max(_ => _.Key) / Def.MapTipLengthOnField) * 5;
			CreateEffect(faction, bullet.Position, scale);
		}
		
		public static void CreateEffect(IFaction faction, Vector3D pos, float scale)
		{
			var offset = new Vector3D(0, 0, 4);
			var effect = faction.World.EffectManager.CreateEffect(typeof(MeirinMobHitBigEffect).FullName, pos + offset);
			effect.SetScale(new Vector3D(scale,scale,scale));
			faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom12).FullName, pos);
		}
	}
}
