﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using thrts;

namespace LotusCraft.Unit
{
	static class Utls
	{
		const int clickable_32 = 32;
		const int clickable_book = 64;
		const int clickable_ufo = 40;
		public enum UnitSize
		{
			VerySmall = 6,
			Small = 8,
			Medium = 10,
			Large = 13,
			XL = 18,
		};

		public enum HitEffectType
		{
			NormalOrange,
			NormalGreen,
			NormalBlue,
			NormalPurple,
			NormalRed,
			NormalYellow,
			ColumnRed,
			ColumnGreen,
			ColumnBlue,
			Bomb,
			BombRed,
			BombGreen,
			BombBlue,
		}

		public static void ShowDeadEffect(IUnit unit)
		{
			var faction = unit.Faction;
			var offset = new Vector3D(0, 0, 8);
			var effect = faction.World.EffectManager.CreateEffect(typeof(Effect.Common.DeadUnitEffect).FullName, unit.Pos + offset);
			var scale = new Vector3D(8, 8, 8);
			effect.SetScale(scale);
		}

		public static void SetCutinAnimation(this thrts.UnitParameter unitParam, string basePath)
		{
			unitParam.Cutin.Path800 = basePath + "/Main800";
			unitParam.Cutin.BackgroundPath800 = basePath + "/Back800";
			unitParam.Cutin.ForegroundPath800 = basePath + "/Fore800";

			unitParam.Cutin.Count800.X = 5;
			unitParam.Cutin.Count800.Y = 3;

			unitParam.Cutin.Path1280 = basePath + "/Main1280";
			unitParam.Cutin.BackgroundPath1280 = basePath + "/Back1280";
			unitParam.Cutin.ForegroundPath1280 = basePath + "/Fore1280";

			unitParam.Cutin.Count1280.X = 5;
			unitParam.Cutin.Count1280.Y = 3;

			unitParam.Cutin.BackgroundBlack = 0.5f;
		}

		public static void SetBoushuenSizeAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -6;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 4;
			Animation.FrameTime = 8;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 4;
			Animation.Count.Y = 8;

			//			unitParameter.Radius = 7;
			unitParameter.Animation = Animation;

			unitParameter.AnimationFrame = Animation.Clone();
			unitParameter.AnimationFrame.Offset.X = 128;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = unitParameter.AnimationWalk.Clone();
			unitParameter.AnimationWalkFrame.Offset.X = 128;
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetBigSizeAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 64;
			Animation.Height = 64;
			Animation.Center.X = 0;
			Animation.Center.Y = -6;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 4;
			Animation.FrameTime = 8;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 4;
			Animation.Count.Y = 8;

			//			unitParameter.Radius = 7;
			unitParameter.Animation = Animation;

			unitParameter.AnimationFrame = Animation.Clone();
			unitParameter.AnimationFrame.Offset.X = 256;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = unitParameter.AnimationWalk.Clone();
			unitParameter.AnimationWalkFrame.Offset.X = 256;
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitBig";

			unitParameter.ClickedWidth = 56;
			unitParameter.ClickedHeight = 64;
		}

		public static void SetBigSizeAnimation2(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 4;
			Animation.FrameTime = 8;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 4;
			Animation.Count.Y = 8;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 32 * 4;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -4;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 4;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = true;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 4;
			AnimationFrame.Count.Y = 8;
			AnimationFrame.Scale.X = 2.0f;
			AnimationFrame.Scale.Y = 2.0f;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetBookUnitAnimation(this thrts.UnitParameter unitParameter, string path) {
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 64;
			Animation.Height = 64;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 16;
			Animation.FrameTime = 4;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 16;
			Animation.Count.Y = 8;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 1024;
			AnimationFrame.Width = 64;
			AnimationFrame.Height = 64;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -2;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 16;
			AnimationFrame.FrameTime = 4;
			AnimationFrame.HasDirection = true;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 16;
			AnimationFrame.Count.Y = 4;
			AnimationFrame.Scale.X = 1.0f;
			AnimationFrame.Scale.Y = 1.0f;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_book;
			unitParameter.ClickedHeight = clickable_book;
			
		}

		public static void SetMiniSuikaAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 20;
			Animation.Height = 20;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 4;
			Animation.FrameTime = 8;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 4;
			Animation.Count.Y = 8;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 20 * 4;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -2;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 4;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = true;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 4;
			AnimationFrame.Count.Y = 8;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetKedamaAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 3;
			Animation.FrameTime = 8;
			Animation.HasDirection = true;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 3;
			Animation.Count.Y = 8;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 32 * 4;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -4;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 3;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = true;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 3;
			AnimationFrame.Count.Y = 8;
			AnimationFrame.Scale.X = 2.0f;
			AnimationFrame.Scale.Y = 2.0f;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetBearAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 8;
			Animation.FrameTime = 8;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 8;
			Animation.Count.Y = 1;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 32 * 8;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -4;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 8;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = false;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 8;
			AnimationFrame.Count.Y = 1;
			AnimationFrame.Scale.X = 2.0f;
			AnimationFrame.Scale.Y = 2.0f;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetUFOAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 8;
			Animation.FrameTime = 8;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 8;
			Animation.Count.Y = 1;

			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 32 * 8;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -4;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 8;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = false;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 8;
			AnimationFrame.Count.Y = 1;
			AnimationFrame.Scale.X = 2.0f;
			AnimationFrame.Scale.Y = 2.0f;

			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;

			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;

			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;

			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";

			unitParameter.ClickedWidth = clickable_ufo;
			unitParameter.ClickedHeight = clickable_ufo;
		}

		public static void SetOnmyouAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 32;
			Animation.Height = 32;
			Animation.Center.X = 0;
			Animation.Center.Y = -2;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 12;
			Animation.FrameTime = 8;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 12;
			Animation.Count.Y = 1;
			
			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 32 * 12;
			AnimationFrame.Width = 20;
			AnimationFrame.Height = 20;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -4;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 12;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = false;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 12;
			AnimationFrame.Count.Y = 1;
			AnimationFrame.Scale.X = 2.0f;
			AnimationFrame.Scale.Y = 2.0f;
			
			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;
			
			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;
			
			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;
			
			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitSmall";
			
			unitParameter.ClickedWidth = clickable_32;
			unitParameter.ClickedHeight = clickable_32;
		}

		public static void SetFlowerAnimation(this thrts.UnitParameter unitParameter, string path)
		{
			var Animation = new TextureAnimation();
			Animation.Path = path;
			Animation.Width = 64;
			Animation.Height = 64;
			Animation.Center.X = 0;
			Animation.Center.Y = -6;
			Animation.CenterPosition = CenterPosition.LowerMiddle;
			Animation.FrameCount = 1;
			Animation.FrameTime = 8;
			Animation.HasDirection = false;
			Animation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			Animation.Count.X = 1;
			Animation.Count.Y = 1;
			
			var AnimationFrame = new TextureAnimation();
			AnimationFrame.Path = path;
			AnimationFrame.Offset.X = 64 * 1;
			AnimationFrame.Width = 64;
			AnimationFrame.Height = 64;
			AnimationFrame.Center.X = 0;
			AnimationFrame.Center.Y = -6;
			AnimationFrame.CenterPosition = CenterPosition.LowerMiddle;
			AnimationFrame.FrameCount = 1;
			AnimationFrame.FrameTime = 8;
			AnimationFrame.HasDirection = false;
			AnimationFrame.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			AnimationFrame.Count.X = 1;
			AnimationFrame.Count.Y = 1;
			AnimationFrame.Scale.X = 1.0f;
			AnimationFrame.Scale.Y = 1.0f;
			
			unitParameter.Animation = Animation;
			unitParameter.AnimationFrame = AnimationFrame;
			
			unitParameter.AnimationWalk = Animation.Clone();
			unitParameter.AnimationWalk.FrameTime = 4;
			
			unitParameter.AnimationWalkFrame = AnimationFrame.Clone();
			unitParameter.AnimationWalkFrame.FrameTime = 4;
			
			unitParameter.ShadowImagePath = "LotusCraft/Image/Shadow/ShadowUnitBig";
			
			unitParameter.ClickedWidth = 56;
			unitParameter.ClickedHeight = 64;
		}

		public static void ShowGunFire(IBullet bullet)
		{
			var bpos = bullet.Position;
			Vector3D tpos = bpos;

			if (bullet.Target != null && bullet.Target.Exists())
			{
				tpos = bullet.Target.Pos;
			}
			else
			{
				return;
			}

			var d = tpos - bpos;
			d.Normalize();
			d = d * 10;
			bpos = bpos + d;

			bullet.Generator.Faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.GunFireTextureEffect).FullName, bpos);
		}

		public static void ShowCommonHitEffect(IBullet bullet, ITargetObject target, HitEffectType hitEffectType = HitEffectType.NormalOrange)
		{
			var faction = bullet.Generator.Faction;
			float bonus = 0;

			if (target is IUnit)
			{
				var u = target as IUnit;
				var type = u.Parameter.UnitType[0];
				bonus = bullet.Parameter.DamageBonus[(int)type];
			}
			else if (target is IBuilding)
			{
				var b = target as IBuilding;
				bonus = bullet.Parameter.DamageBonus[(int)TargetObjectType.Structure];
			}

			// 効果音
			if (bullet.Parameter.Type == BulletType.ShortRange)
			{
				faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitShortRangeAttack).FullName, target.Pos);
			}

			if (bullet.Parameter.Type == BulletType.LongRange)
			{
				faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitLongRangeAttack).FullName, target.Pos);
			}

			//if(bullet.Parameter.DamageBonus[
			var pos = target.Pos;
			pos.Z += 6.0f;

			if (target is IBuilding && bullet.Parameter.Type == BulletType.LongRange)
			{
				pos = bullet.Position;
			}

			if (target is IBuilding && bullet.Parameter.Type == BulletType.ShortRange)
			{
				if (bullet.Generator != null)
				{
					pos = (target.Pos + bullet.Generator.Pos) / 2.0f;
					pos.Z += 6.0f;
				}
			}

			if (hitEffectType == HitEffectType.NormalOrange)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitOrangeTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalPurple)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitPurpleTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalYellow)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitYellowTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.Bomb)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitBlueTextureEffect).FullName, pos);
			}
		}

		public static void ShowHitEffect(ITargetObject target, HitEffectType hitEffectType = HitEffectType.NormalOrange)
		{
			IFaction faction = null;
			if (target is IUnit)
			{
				var u = target as IUnit;
				faction = u.Faction;
			}
			else if (target is IBuilding)
			{
				var b = target as IBuilding;
				faction = b.Faction;
			}

			var pos = target.Pos;
			pos.Z += 6.0f;


			if (hitEffectType == HitEffectType.NormalOrange)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitOrangeTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalPurple)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitPurpleTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalYellow)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitYellowTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.Bomb)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitBlueTextureEffect).FullName, pos);
			}
		}
		
//追加
		public static void ShowCommonHitEffect2(IBullet bullet, ITargetObject target, HitEffectType hitEffectType = HitEffectType.NormalOrange)
		{
			var faction = bullet.Generator.Faction;
			float bonus = 0;

			if (target is IUnit)
			{
				var u = target as IUnit;
				var type = u.Parameter.UnitType[0];
				bonus = bullet.Parameter.DamageBonus[(int)type];
			}
			else if (target is IBuilding)
			{
				var b = target as IBuilding;
				bonus = bullet.Parameter.DamageBonus[(int)TargetObjectType.Structure];
			}

			// 効果音
			if (bullet.Parameter.Type == BulletType.ShortRange)
			{
				faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.HitShortRangeAttack).FullName, target.Pos);
			}

			if (bullet.Parameter.Type == BulletType.LongRange)
			{
				faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.Bom01).FullName, target.Pos);
			}
			
			//if(bullet.Parameter.DamageBonus[
			var pos = target.Pos;
			pos.Z += 6.0f;

			if (target is IBuilding && bullet.Parameter.Type == BulletType.LongRange)
			{
				pos = bullet.Position;
			}

			if (target is IBuilding && bullet.Parameter.Type == BulletType.ShortRange)
			{
				if (bullet.Generator != null)
				{
					pos = (target.Pos + bullet.Generator.Pos) / 2.0f;
					pos.Z += 6.0f;
				}
			}

			if (hitEffectType == HitEffectType.NormalOrange)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitOrangeTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalPurple)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitPurpleTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalYellow)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitYellowTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.Bomb)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitBlueTextureEffect).FullName, pos);
			}
		}

		public static void ShowHitEffect2(ITargetObject target, HitEffectType hitEffectType = HitEffectType.NormalOrange)
		{
			IFaction faction = null;
			if (target is IUnit)
			{
				var u = target as IUnit;
				faction = u.Faction;
			}
			else if (target is IBuilding)
			{
				var b = target as IBuilding;
				faction = b.Faction;
			}

			var pos = target.Pos;
			pos.Z += 6.0f;


			if (hitEffectType == HitEffectType.NormalOrange)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitOrangeTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalPurple)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitPurpleTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.NormalYellow)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.NormalHitYellowTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitBlueTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.ColumnGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.ColumnHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.Bomb)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombRed)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitRedTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombGreen)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitGreenTextureEffect).FullName, pos);
			}
			else if (hitEffectType == HitEffectType.BombBlue)
			{
				faction.World.EffectManager.CreateTextureEffect(typeof(Effect.Common.BombHitBlueTextureEffect).FullName, pos);
			}
		}
	}
}
