﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Building.White;
using LotusCraft.Effect.White;
using LotusCraft.Unit;
using LotusCraft.Faction;

namespace LotusCraft.Unit.White
{
	public class Farmer : General.Farmer
	{
		public Farmer()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/White/Farmer";
			#endregion

			#region UnitParams
			Name = "白・"+Name;
//			HP = 1000;
//			Speed = 1.0f * BasisSpeed;

			ViewRange = ViewRange;

			MakingTime = MakingTime;
			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/White/Farmer");
			#endregion

			#region BuildableObjcets
			BuildingsMade = new[] { 
				typeof(HomeBase).FullName,
				typeof(ProduceResType0).FullName,
				typeof(ProduceResType1).FullName,
				typeof(Barrack).FullName,
				typeof(BlackSmith).FullName,
				typeof(Bunker).FullName,
				typeof(Factory).FullName,
				typeof(Airport).FullName,

				typeof(Houtou).FullName,
				typeof(Seirensen).FullName,
				typeof(Hospital).FullName,
			};
			BuildType =BuildType.WorkAt;
			#endregion

			#region Bullets
			Bullets = new string[] { typeof(Bullet.White.FarmerBullet).FullName,};
			#endregion

			#region Requires
			#endregion
		}
		public override void OnLoad (IFaction faction)
		{
			#region Buttons

			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
//				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CancelBuildingButton()),

				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(HomeBase).FullName),
				   ButtonPosition._0,
					ButtonPosition._1
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(ProduceResType0).FullName),
					ButtonPosition._1,
					ButtonPosition._1
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(ProduceResType1).FullName),
					ButtonPosition._2,
					ButtonPosition._1
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Barrack).FullName),
					ButtonPosition._0,
					ButtonPosition._2
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(BlackSmith).FullName),
					ButtonPosition._3,
					ButtonPosition._1
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Bunker).FullName),
					ButtonPosition._4,
					ButtonPosition._1
				)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Factory).FullName),
					ButtonPosition._1,
					ButtonPosition._2
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Airport).FullName),
					ButtonPosition._2,
					ButtonPosition._2
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Houtou).FullName),
					ButtonPosition._0,
					ButtonPosition._3
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Seirensen).FullName),
					ButtonPosition._1,
					ButtonPosition._3
					)),
				new thrts.ButtonParameterHolder<thrts.IUnit>(new LotusCraft.Button.CreateBuildingByUnitButton(
					faction.GetBuildingParameter(typeof(Hospital).FullName),
					ButtonPosition._2,
					ButtonPosition._3
					)),
			};

			#endregion

			base.OnLoad(faction);
		}

		public override void OnVanish(IUnit unit)
		{
			Utls.ShowDeadEffect(unit);
			base.OnVanish(unit);
		}

		#region 建築
		public override void OnStartBuilding(IUnit unit, IBuilding building)
		{
			// エフェクト生成
			IEffect effect = null;
			effect = unit.Faction.World.EffectManager.CreateEffect(
					typeof(Effect.Common.UnitCollectPowerStoneEffect).FullName,
					unit.Pos);

			effect.SetScale(new Vector3D(3.0f, 3.0f, 3.0f));

			// 死なない、操作不可アビリティ生成
			var ability = new UncontrolableAbility();
			ability.ExtendLifetime(2);

			Action<IUnit> vanished = null;
			Action<IUnit, IBuilding> completed = null;
			Action<IUnit, IBuilding> canceled = null;

			// 命令
			unit.DeniningAcceptingNextOrderCount = 2;
			unit.ChangeActionToPrivilegeMode(
				() =>
				{
					return BuildingStructure(unit, building, ability, effect);
				},
				() =>
				{
					effect.Stop();
				});

			// 死亡時の条件追加

			vanished = (u) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};

			unit.Vanished += vanished;


			completed = (u, b) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};

			canceled = (u, b) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
				unit.CompletedBuilding -= completed;
				unit.CanceledBuilding -= canceled;
			};

			unit.CompletedBuilding += completed;
			unit.CanceledBuilding += canceled;
		}


		IEnumerable<object> BuildingStructure(IUnit unit, IBuilding building, UncontrolableAbility ability, IEffect effect)
		{
			unit.AbilityManager.AppendAbility(ability);

			while (true)
			{
				unit.IsBuildingIconShowing = true;

				if (!building.Exists() || building.IsComplete)
				{
					effect.Stop();
					yield break;
				}

				ability.ExtendLifetime(1);
				yield return true;
			}
		}

		class UncontrolableAbility : AbilityTemplate
		{
			public UncontrolableAbility()
			{
				Name = "Uncontrolable";
			}

			public override bool ExecUnitAbility(IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.OtherExecs)
				{
					unit.IsntFoundByEnemy = true;
					unit.IsntHitWithBullet = true;
					unit.DeniningAcceptingNextOrderCount = 2;
				}

				return true;
			}
		}

		#endregion

		public override bool OnStartingGettingResource(IUnit unit, IResource resource)
		{
			if(resource.ResourceType == 0)
			{
				IBuilding newBuilding = null;
				var param = unit.Faction.GetBuildingParameter(typeof(ProduceResType0).FullName);

				// 転向されてるか何かで存在しない
				if (param == null)
				{ 
					return false;
				}

				// 資源不足
				if (param.Costs[0] > unit.Faction.GetResource(0)) return false;
				if (param.Costs[1] > unit.Faction.GetResource(1)) return false;
				if (param.Costs[2] > unit.Faction.GetResource(2)) return false;
				if (param.Costs[3] > unit.Faction.GetResource(3)) return false;

				// 建築開始
				unit.StartBuilding(
					param,
					new Point2D((int)(resource.Pos.X / Def.MapTipLengthOnField), (int)(resource.Pos.Y / Def.MapTipLengthOnField)),
					out newBuilding);

				if (newBuilding == null)
				{
					// 既に建っているか調べる
					var building = unit.Faction.World.GetObjectFromCircle(new Circle2D(20.0f, new Vector2D(resource.Pos.X, resource.Pos.Y))).
						OfType<IBuilding>().
						Where( _ => _.Parameter == param).
						FirstOrDefault();

					if (building == null)
					{
						return false;
					}
					else if (building.Faction != unit.Faction)
					{
						return false;
					}
					else if (building != null && building.IsComplete)
					{
						// 完成済み
						ChangeActionToCollectResource(unit, building);
					}
					else if (building != null && !building.IsComplete)
					{
						// 未完成
						ChangeActionToWaitToComplete(unit, building);
					}
					else
					{
						return false;
					}
				}

				// 上手く行った
				return true;
			}

			if (resource.ResourceType == 1)
			{
				IBuilding newBuilding = null;
				var param = unit.Faction.GetBuildingParameter(typeof(ProduceResType1).FullName);

				// 転向されてるか何かで存在しない
				if (param == null)
				{
					return false;
				}

				// 資源不足
				if (param.Costs[0] > unit.Faction.GetResource(0)) return false;
				if (param.Costs[1] > unit.Faction.GetResource(1)) return false;
				if (param.Costs[2] > unit.Faction.GetResource(2)) return false;
				if (param.Costs[3] > unit.Faction.GetResource(3)) return false;

				// 建築開始
				unit.StartBuilding(
					param,
					new Point2D((int)(resource.Pos.X / Def.MapTipLengthOnField), (int)(resource.Pos.Y / Def.MapTipLengthOnField)),
					out newBuilding);

				if (newBuilding == null)
				{
					// 既に建っているか調べる
					var building = unit.Faction.World.GetObjectFromCircle(new Circle2D(20.0f, new Vector2D(resource.Pos.X, resource.Pos.Y))).
						OfType<IBuilding>().
						Where(_ => _.Parameter == param).
						FirstOrDefault();

					if (building == null)
					{
						return false;
					}
					else if (building.Faction != unit.Faction)
					{
						return false;
					}
					else if (building != null && building.IsComplete)
					{
						// 完成済み
						ChangeActionToCollectResource(unit, building);
					}
					else if(building != null && !building.IsComplete)
					{
						// 未完成
						ChangeActionToWaitToComplete(unit, building);
					}
					else
					{
						return false;
					}
				}

				// 上手く行った
				return true;
			}


			return false;
		}

		public override bool OnTouchedBuildingWhichRequiresWorkerToTouch(IUnit unit, IBuilding building)
		{
			if (building.IsComplete)
			{
				return ChangeActionToCollectResource(unit, building);
			}
			else
			{
				return ChangeActionToWaitToComplete(unit, building);
			}
		}

		bool ChangeActionToWaitToComplete(IUnit unit, IBuilding building)
		{
			// 資源収集開始
			if (building.Parameter.GetType() != typeof(ProduceResType0) &&
				building.Parameter.GetType() != typeof(ProduceResType1)) return false;

			if (building.IsComplete)
			{
				return ChangeActionToCollectResource(unit, building);
			}

			unit.ChangeActionToPrivilegeMode(
				() =>
				{
					return WaitingToCompleteBuilding(unit, building);
				},
				() =>
				{
				});

			return true;
		}

		bool ChangeActionToCollectResource(IUnit unit, IBuilding building)
		{
			// 資源収集開始
			if (building.Parameter.GetType() != typeof(ProduceResType0) &&
				building.Parameter.GetType() != typeof(ProduceResType1)) return false;

			// エフェクト生成
			IEffect effect = null;
			int resourceType = 0;

			AbilityTemplate ability = null;
			if (building.Parameter.GetType() == typeof(ProduceResType0))
			{
				effect = unit.Faction.World.EffectManager.CreateEffect(
					typeof(Effect.Common.UnitCollectAlcoholEffect).FullName,
					unit.Pos);
				resourceType = 0;

				// 回収アビリティ生成
				ability = new Collecting0Ability();
				ability.ExtendLifetime(2);
			}

			if (building.Parameter.GetType() == typeof(ProduceResType1))
			{
				effect = unit.Faction.World.EffectManager.CreateEffect(
					typeof(Effect.Common.UnitCollectPowerStoneEffect).FullName,
					unit.Pos);
				resourceType = 1;

				// 回収アビリティ生成
				ability = new Collecting1Ability();
				ability.ExtendLifetime(2);
			}

			effect.SetScale(new Vector3D(3.0f, 3.0f, 3.0f));

			Action<IUnit> vanished = null;
			
			// 命令
			unit.ChangeActionToPrivilegeMode(
				() =>
				{
					return TouchingResourceCollector(unit, building, ability, effect, resourceType);
				},
				() =>
				{
					effect.Stop();
					unit.Vanished -= vanished;
				});

			// 死亡時の条件追加
			vanished = (u) =>
			{
				effect.Stop();
				unit.Vanished -= vanished;
			};

			unit.Vanished += vanished;

			return true;
		}

		IEnumerable<object> WaitingToCompleteBuilding(IUnit unit, IBuilding building)
		{
			while (true) 
			{
				// 建築物が破棄
				if (!building.Exists())
				{
					unit.StopOrder();
					yield break;
				}

				if (building.IsComplete)
				{
					if (ChangeActionToCollectResource(unit, building))
					{
						// 命令継続
						yield return true;
					}
					else
					{
						// 命令終了
						yield break;
					}
				}
				
				yield return true;
			}
		}

		IEnumerable<object> TouchingResourceCollector(IUnit unit, IBuilding building, AbilityTemplate ability, IEffect effect, int resourceType)
		{
			unit.AbilityManager.AppendAbility(ability);

			bool first = true;

			while (true)
			{
				if (!building.Exists())
				{
					effect.Stop();
					yield break;
				}

				if (resourceType == 0)
				{
					unit.IsAlcoholIconShowing = true;
				}
				else if (resourceType == 1)
				{
					unit.IsReiryokuIconShowing = true;
				}

				if (first)
				{
					var previousWorkerCount = (int)building.UserValues["PreviousWorkerCount"];

					// 最大数超過 or 資源切れ(既に回収中の農民を追い出すのを防止)
					if (building.WorkerCountRequired <= previousWorkerCount || previousWorkerCount < 0)
					{
						effect.Stop();
						yield break;
					}
				}

				var workerCount = (int)building.UserValues["WorkerCount"];

				// 最大数超過 or 資源切れ(1フレームに2体以上張り付くのを防止)
				if (building.WorkerCountRequired <= workerCount || workerCount < 0)
				{
					effect.Stop();
					yield break;
				}


				if (first)
				{
					first = false;

					// ユニットの移動
					var positions = building.UserValues[Building.General.ProduceResources<HomeBase, int>.WorkerPositionsKey] as Vector2D[];

					for (int i = 0; i < positions.Length; i++)
					{
						if (unit.Warp(positions[i], false))
						{
							effect.SetPosition(unit.Pos);
							break;
						}
					}
				}

				workerCount++;
				building.UserValues["WorkerCount"] = workerCount;

				ability.ExtendLifetime(1);
				yield return true;
			}
		}

		class Collecting0Ability : AbilityTemplate
		{
			public Collecting0Ability()
			{
				Name = "Collecting0";
			}

			public override bool ExecUnitAbility(IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.OtherExecs)
				{
					//unit.IsntFoundByEnemy = true;
					//unit.IsntHitWithBullet = true;
					unit.IsAlcoholIconShowing = true;
				}

				return true;
			}
		}

		class Collecting1Ability : AbilityTemplate
		{
			public Collecting1Ability()
			{
				Name = "Collecting1";
			}

			public override bool ExecUnitAbility(IUnit unit, AbilityOrder order)
			{
				if (order == AbilityOrder.OtherExecs)
				{
					unit.IsReiryokuIconShowing = true;
				}

				return true;
			}
		}
	}
}

namespace LotusCraft.Bullet.White
{
	public class FarmerBullet : General.ShortRangeBullet
	{
		public FarmerBullet()
		{
			Type = BulletType.ShortRange;
			
			this.Attack = 6;
			Speed = 0.0f;
			DamageBonus[(int)TargetObjectType.Hero] = 2.5f;
			DamageBonus[(int)TargetObjectType.Structure] = 1.5f;
			MaxRange = 0.5f * 32;
			
			AttackDelay = 0;
			AttackRigor = (int)(1.5f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
		}
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target);
		}
	}
}
