﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using thrts;
using LotusCraft.Effect.White;
using LotusCraft.Building.White;
using LotusCraft.Spell.White;

namespace LotusCraft.Unit.White
{
	public class Murasa : LotusCraft.Unit.General.Special
	{
		public Murasa ()
		{
			#region Icon
			IconPath = "LotusCraft/Image/Button/White/Murasa";
			#endregion

			#region UnitParams
			Name = "村紗 水蜜(式神)";
			HP = 200;
			Armor = 0;
			Speed = 1.4f * BasisSpeed;

			PopulationWeight = 3;

			ViewRange = 12 * 32;

			MakingTime = 28 * Def.FPS;
			/// mine,gas,sup,?
			Costs = new [] {125,0,0,0};

			ButtonDescription = "射撃ユニット\n高い機動力と水属性に高いボーナスを持った貫通弾を持つ";
			Description = _ => ButtonDescription.Split('\n').Select(s => new ColorTextLine(s));
			UnitType = new TargetObjectType[]{TargetObjectType.Water,};

			RequireBuildings = new string[]{
				typeof(Seirensen).FullName,
			};
			#endregion

			#region Animation
			this.SetBigSizeAnimation2("LotusCraft/Image/Unit/White/Murasa");
			#endregion

			#region BuildableObjcets
			#endregion

			#region Bullets
			Bullets = new string[] { 
				typeof(Bullet.White.MurasaBullet).FullName, 
				typeof(Bullet.White.MurasaBlueBullet).FullName, 
			};
			#endregion

			SelectBullet = SelectAnchorBullet;
			
		}
		public override void OnLoad (IFaction faction)
		{
			#region Spells
			SpellParams = new thrts.SpellParameter<thrts.IUnit>[]
			{
				new LotusCraft.Spell.White.MurasaSkill (),
			};
			#endregion

			#region Buttons
			ButtonParams = new thrts.ButtonParameterHolder<thrts.IUnit>[]
			{
				new thrts.ButtonParameterHolder<thrts.IUnit>(SpellParams[0].DefaultSpellButton,SpellParams[0].SpellButtons),
			};
			#endregion

			base.OnLoad(faction);
		}
		static int SelectAnchorBullet (IUnit unit, ITargetObject target, float distance)
		{
			var spell = null != unit? unit.GetSpellWithName(MurasaSkill.SpellName):null;
			if(spell != null && spell.Level > 0)
				return 1;
			else
				return 0;
		}
	}

}

namespace LotusCraft.Bullet.White
{
	public class MurasaBullet : General.MiddleRangeBullet
	{
		public MurasaBullet()
		{
			
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			
			HitType = BulletHitType.Pierce;
			
			this.Attack = 15;
			DamageBonus[(int)TargetObjectType.Water] = 1.8f;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;
			Radius /= 2;

			MaxRange = 5 * Def.MapTipLengthOnField;
			Gravity = 0f;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(2.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/anchor_red";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 8;
		}
		
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.ColumnRed);
		}
	}
	public class MurasaBlueBullet :MurasaBullet
	{
		public MurasaBlueBullet()
		{
			
			Type = BulletType.LongRange;
			SpaceType = BulletSpaceType.Earth;
			DoesHitOnlyTarget = false;

			HitType = BulletHitType.Pierce;
			
			this.Attack = 15;
			DamageBonus[(int)TargetObjectType.Water] = 2.0f;
			Speed = BulletUtls.NormalSpeed * BasisSpeed;

			MaxRange = 4 * Def.MapTipLengthOnField;
			Gravity = 0f;
			
			AttackDelay = (int)(0.0f * 30);
			AttackRigor = (int)(2.0f * 30);
			AttackSpeed = AttackDelay + AttackRigor; AttackRigor=0;
			
			OnHit = OnHitEffect;
			
			TextureAnimation = new TextureAnimation();
			TextureAnimation.Path = "LotusCraft/Image/Bullet/anchor";
			TextureAnimation.Width = 16;
			TextureAnimation.Height = 16;
			TextureAnimation.Center.X = 8;
			TextureAnimation.Center.Y = 8;
			TextureAnimation.CenterPosition = CenterPosition.MiddleMiddle;
			TextureAnimation.FrameCount = 1;
			TextureAnimation.FrameTime = 1;
			TextureAnimation.HasDirection = true;
			TextureAnimation.LoopType = TextureAnimation.AnimetionLoopType.Loop;
			TextureAnimation.Count.X = 1;
			TextureAnimation.Count.Y = 8;
			OnCreated = OnCreate;
		}
		static void OnCreate(IBullet bullet)
		{
			Unit.Utls.ShowGunFire(bullet);
			bullet.Generator.Faction.World.SoundManager.PlaySoundEffect(typeof(SoundEffect.GunLong).FullName, bullet.Position);
		}
		
		static void OnHitEffect(IBullet bullet, ITargetObject target)
		{
			LotusCraft.Unit.Utls.ShowCommonHitEffect(bullet, target, Unit.Utls.HitEffectType.ColumnBlue);
		}
	}
}
